@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -100,6 +100,9 @@ static const std::vector<ImplicitArg> IMPLICIT_ARGS = {
     ImplicitArg(ImplicitArg::IMPLICIT_ARG_BUFFER_PTR, "implicitArgBuffer", ImplicitArg::GLOBALPTR, WIAnalysis::UNIFORM_GLOBAL, 1, ImplicitArg::ALIGN_PTR, true),
 
     ImplicitArg(ImplicitArg::ASSERT_BUFFER_POINTER, "assertBufferPointer", ImplicitArg::GLOBALPTR, WIAnalysis::UNIFORM_GLOBAL, 1, ImplicitArg::ALIGN_GRF, false, GenISAIntrinsic::GenISA_getAssertBufferPtr),
+
+    // BufferBoundsChecking
+    ImplicitArg(ImplicitArg::BUFFER_SIZE, "bufferSize", ImplicitArg::LONG, WIAnalysis::UNIFORM_GLOBAL, 1, ImplicitArg::ALIGN_QWORD, true),
 };
 
 ImplicitArg::ImplicitArg(
@@ -436,7 +439,7 @@ void ImplicitArgs::addImplicitArgs(llvm::Function& F, const SmallVectorImpl<Impl
     {
         if (!isImplicitArgExist(F, arg, pMdUtils))
         {
-            ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(ArgInfoMetaData::get());
+            ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(new ArgInfoMetaData());
             argMD->setArgId(arg);
             funcInfo->addImplicitArgInfoListItem(argMD);
         }
@@ -451,7 +454,7 @@ void ImplicitArgs::addImplicitArgsTotally(llvm::Function& F, const SmallVectorIm
     {
         if (!isImplicitArgExist(F, arg, pMdUtils))
         {
-            ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(ArgInfoMetaData::get());
+            ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(new ArgInfoMetaData());
             argMD->setArgId(arg);
             funcInfo->addImplicitArgInfoListItem(argMD);
             argAdded = true;
@@ -480,7 +483,7 @@ void ImplicitArgs::addImageArgs(llvm::Function& F, const ImplicitArg::ArgMap& ar
         {
             for (const auto& argI : argMapIter->second)
             {
-                ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(ArgInfoMetaData::get());
+                ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(new ArgInfoMetaData());
                 argMD->setArgId(argType);
                 argMD->setExplicitArgNum(argI);
                 funcInfo->addImplicitArgInfoListItem(argMD);
@@ -498,7 +501,7 @@ void ImplicitArgs::addStructArgs(llvm::Function& F, const Argument* A, const Imp
         unsigned int id = argI.first;
         unsigned int offset = argI.second;
 
-        ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(ArgInfoMetaData::get());
+        ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(new ArgInfoMetaData());
         argMD->setExplicitArgNum(A->getArgNo());
         argMD->setArgId(id);
         argMD->setStructArgOffset(offset);
@@ -514,7 +517,7 @@ void ImplicitArgs::addNumberedArgs(llvm::Function& F, const ImplicitArg::ArgMap&
         ImplicitArg::ArgType argId = argPair.first;
         for (const auto& argNum : argPair.second)
         {
-            ArgInfoMetaDataHandle argMD(ArgInfoMetaData::get());
+            ArgInfoMetaDataHandle argMD(new ArgInfoMetaData());
             argMD->setArgId(argId);
             argMD->setExplicitArgNum(argNum);
             funcInfo->addImplicitArgInfoListItem(argMD);
@@ -610,6 +613,21 @@ void ImplicitArgs::addBindlessOffsetArgs(llvm::Function& F, const IGCMD::MetaDat
 
         OffsetArgs[ImplicitArg::BINDLESS_OFFSET].insert(argNo);
     }
+
+    // Create a bindless offset for the implicit args CONST_BASE and GLOBAL_BASE as well.
+    // At this step, they are not yet added to the function signature, but since implicit args
+    // are added sequentially, we can calculate the associated argNo here.
+    unsigned argNo = F.arg_size();
+    for (auto AI = funcInfoMD->begin_ImplicitArgInfoList(), AE = funcInfoMD->end_ImplicitArgInfoList(); AI != AE; AI++, argNo++)
+    {
+        ArgInfoMetaDataHandle argInfo = *AI;
+        ImplicitArg::ArgType argId = static_cast<ImplicitArg::ArgType>(argInfo->getArgId());
+        if (argId == ImplicitArg::CONSTANT_BASE || argId == ImplicitArg::GLOBAL_BASE)
+        {
+            OffsetArgs[ImplicitArg::BINDLESS_OFFSET].insert(argNo);
+        }
+    }
+
     if (OffsetArgs.size() > 0)
     {
         ImplicitArgs::addNumberedArgs(F, OffsetArgs, pMdUtils);
