@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2019-2022 Intel Corporation
+Copyright (C) 2019-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -56,6 +56,7 @@ private:
     //TODO: this is hardcoded string, we might want to put all "printf" of different adaptors to one place eventually
     static constexpr char *PrintfFuncName = "printf";
 
+    bool isChildOfXe2 = false;
 
     // Field for explicit GlobalBufferPtr - used on OpenCL path.
     Value* GlobalBufferPtr = nullptr;
@@ -72,8 +73,9 @@ private:
                 enabledSlices++;
             }
         }
+        isChildOfXe2 = Ctx.platform.isCoreChildOf(IGFX_XE2_HPG_CORE);
 
-        if (Ctx.platform.isProductChildOf(IGFX_PVC))
+        if (isChildOfXe2 || Ctx.platform.isProductChildOf(IGFX_PVC))
         {
             NumDSSPerSlice = SysInfo.MaxSubSlicesSupported / std::max(SysInfo.MaxSlicesSupported, enabledSlices);
             EuCountPerDSS = SysInfo.MaxEuPerSubSlice;
@@ -213,6 +215,8 @@ public:
     }
 
     ~RTBuilder() {}
+    RTBuilder(const RTBuilder&) = delete;
+    RTBuilder& operator=(const RTBuilder&) = delete;
 
 
     inline Value* get32BitLaneID(void)
@@ -242,6 +246,8 @@ public:
         const Twine &JoinBBName = "");
 
     Value* getRtMemBasePtr(void);
+    Value* getStackSizePerRay(void);
+    Value* getNumDSSRTStacks(void);
     Value* getMaxBVHLevels(void);
     Value* getStatelessScratchPtr(void);
     Value* getPrimLeafIndex(StackPointerVal* StackPointer, bool CommittedHit);
@@ -250,14 +256,14 @@ public:
 
     Value* CreateSyncStackPtrIntrinsic(Value* Addr, Type* PtrTy, bool AddDecoration);
 
+    RayQueryCheckIntrinsic* CreateRayQueryCheckIntrinsic(Value* predicate = nullptr);
+    RayQueryReleaseIntrinsic* CreateRayQueryReleaseIntrinsic(Value* predicate = nullptr);
+
+    PreemptionDisableIntrinsic* CreatePreemptionDisableIntrinsic(Value* Flag = nullptr);
+    PreemptionEnableIntrinsic* CreatePreemptionEnableIntrinsic(Value* Flag = nullptr);
 
     SyncStackPointerVal* getSyncStackPointer();
 
-    // Note: 'traceRayCtrl' should be already by 8 bits to its location
-    // in the payload before passing as an argument to this function.
-    // getTraceRayPayload() just ORs together the bvh, ctrl, and stack id.
-    TraceRayIntrinsic* createTraceRay(Value* bvhLevel, Value* traceRayCtrl, bool isRayQuery, const Twine& PayloadName = "");
-
     void createReadSyncTraceRay(Value* val);
 
     TraceRaySyncIntrinsic* createSyncTraceRay(Value* bvhLevel, Value* traceRayCtrl, const Twine& PayloadName = "");
@@ -351,11 +357,11 @@ public:
     Value* getHitValid(StackPointerVal* StackPointer, bool CommittedHit);
     void   setHitValid(StackPointerVal* StackPointer, bool CommittedHit);
     Value* getSyncTraceRayControl(Value* ptrCtrl);
-    void   setSyncTraceRayControl(Value* ptrCtrl, unsigned ctrl);
+    void   setSyncTraceRayControl(Value* ptrCtrl, RTStackFormat::TraceRayCtrl ctrl);
     Value* getHitBaryCentric(StackPointerVal* StackPointer, uint32_t idx, bool CommittedHit);
 
 
-    Value* getGlobalBufferPtr();
+    Value* getGlobalBufferPtr(IGC::ADDRESS_SPACE Addrspace = IGC::ADDRESS_SPACE_CONSTANT);
     Value* getSyncStackID();
     Value* getSyncStackOffset(bool rtMemBasePtr = true);
     SpillValueIntrinsic* getSpillValue(Value* Val, uint64_t Offset);
@@ -384,6 +390,7 @@ public:
     Value* getGlobalSyncStackID();
 
 private:
+    TraceRayIntrinsic* createTraceRay(Value* bvhLevel, Value* traceRayCtrl, bool isRayQuery, const Twine& PayloadName = "");
 
     Value* canonizePointer(Value* Ptr);
     Value* getSyncRTStackSize();
@@ -430,6 +437,7 @@ private:
     Value* emitStateRegID(uint32_t BitStart, uint32_t BitEnd);
     std::pair<uint32_t, uint32_t> getSliceIDBitsInSR0() const;
     std::pair<uint32_t, uint32_t> getDualSubsliceIDBitsInSR0() const;
+    std::pair<uint32_t, uint32_t> getSubsliceIDBitsInSR0() const;
 
 
     Value* getGlobalDSSID();
@@ -446,10 +454,16 @@ public:
     Type* getSMStack2Ty() const;
     Type* getRTStack2Ty() const;
     Type* getRTStack2PtrTy(RTMemoryAccessMode Mode, bool async = true) const;
-    Type* getRayDispatchGlobalDataPtrTy(Module &M);
+    Type* getRayDispatchGlobalDataPtrTy(Module &M, IGC::ADDRESS_SPACE Addrspace);
 
+    ConstantInt* supportStochasticLod();
+
+
+
+    GenIntrinsicInst* createDummyInstID(Value* pSrcVal);
 
     CallInst* ctlz(Value* V);
+    CallInst* cttz(Value* V);
 
     void createPotentialHit2CommittedHit(StackPointerVal* StackPtr);
 
@@ -459,6 +473,7 @@ public:
         Value* RootNodePtr,
         Value* RayFlags,
         Value* InstanceInclusionMask,
+        Value* ComparisonValue,
         Value* TMax);
 
     void emitSingleRQMemRayWrite(
