@@ -8,13 +8,8 @@ SPDX-License-Identifier: MIT
 
 //===----------------------------------------------------------------------===//
 ///
-/// \file
-/// \brief This file contains a definition of the structure of the ray tracing
-/// stack as seen by a ray.  Each ray will have its own private copy of the
-/// stack. Through a combination of offsetof() and sizeof(), passes will find
-/// offsets to fields within the structs and update themselves automatically
-/// when this file changes.
-///
+/// This file contains a definition of the structure of the ray tracing
+/// stack as seen by a ray.
 ///
 //===----------------------------------------------------------------------===//
 #pragma once
@@ -23,7 +18,7 @@ SPDX-License-Identifier: MIT
 
 #include <stdint.h>
 #include <stddef.h>
-#if !defined(__clang__) || (__clang_major__ >= 10)
+#if !defined(__clang__) || (__clang_major__ >= 15)
 #include <type_traits>
 #endif
 
@@ -99,7 +94,7 @@ namespace IGC {
 // will be patched with the global root signature at compile time
 struct alignas(8) TypeHoleGlobalRootSig
 {
-    char __Padding[8];
+    char __Padding[16];
 };
 
 // This is currently all of the cross-thread constant data that will be populated
@@ -242,6 +237,7 @@ struct MemTravStack
     };
 };
 
+// Define structs Xe, used to specialize other template structs, like MemHit or RTStack, via GenT arg
 #define STYLE(X) struct X;
 #include "RayTracingMemoryStyle.h"
 #undef STYLE
@@ -330,7 +326,7 @@ struct MemRay<Xe>
         instLeafPtr           = 48,
         rayMask               = 8,
 
-        reserved2             = 7,
+        ComparisonValue       = 7,
         pad2                  = 8,
     };
 
@@ -402,7 +398,7 @@ struct MemRay<Xe>
     // DG2
     //uint64_t pad                 : 8;                              // explicit padding bits
     uint64_t pad                   : (T)Bits::pad;                   // explicit padding bits
-    uint64_t reserved2             : (T)Bits::reserved2;
+    uint64_t ComparisonValue       : (T)Bits::ComparisonValue;       // to be compared with Instance.ComparisonValue
     uint64_t shaderIndexMultiplier : (T)Bits::shaderIndexMultiplier; // shader index multiplier
 
     union
@@ -531,8 +527,8 @@ struct InstanceLeaf<Xe>
             geomFlags             = 2,
             startNodePtr          = 48,
             instFlags             = 8,
-            reserved1             = 1,
-            reserved2             = 7,
+            ComparisonMode        = 1,
+            ComparisonValue       = 7,
         };
 
         uint32_t shaderIndex : (T)Bits::shaderIndex;  // shader index used to calculate instancing shader in case of software instancing
@@ -547,8 +543,9 @@ struct InstanceLeaf<Xe>
         uint64_t startNodePtr : (T)Bits::startNodePtr;  // start node where to continue traversal of the instanced object
         uint64_t instFlags    : (T)Bits::instFlags;     // flags for the instance (see InstanceFlags)
 
-        uint64_t reserved1       : (T)Bits::reserved1; // 0 for less than or equal, 1 for greater
-        uint64_t reserved2       : (T)Bits::reserved2; // to be compared with ray.ComparisonValue
+        // Xe2+
+        uint64_t ComparisonMode  : (T)Bits::ComparisonMode;  // 0 for less than or equal, 1 for greater
+        uint64_t ComparisonValue : (T)Bits::ComparisonValue; // to be compared with ray.ComparisonValue
 
 
         // Note that the hardware swaps the translation components of the
@@ -590,10 +587,26 @@ struct RTStack
     MemHit<GenT> committedHit;    // stores committed hit
     MemHit<GenT> potentialHit;    // stores potential hit that is passed to any hit shader
 
-    MemRay<GenT> ray[MaxBVHLevels];       // stores a ray for each instancing level
+    MemRay<GenT> ray0;
+    MemRay<GenT> ray1;
+    MemRay<GenT> ray[MaxBVHLevels - 2];       // stores a ray for each instancing level
     MemTravStack travStack[MaxBVHLevels]; // spill location for the internal stack state per instancing level
+
+    static_assert(MaxBVHLevels > 2);
 };
 
+template <typename GenT>
+struct RTStack<GenT, MAX_BVH_LEVELS>
+{
+    MemHit<GenT> committedHit;    // stores committed hit
+    MemHit<GenT> potentialHit;    // stores potential hit that is passed to any hit shader
+
+    MemRay<GenT> ray0;
+    MemRay<GenT> ray1;
+    MemTravStack travStack[MAX_BVH_LEVELS]; // spill location for the internal stack state per instancing level
+};
+
+
 // This is the ShadowMemory that we maintain if we DONOT need spill/fill it to/from HW RTStack.
 // We keep this data structure as small as possible to reduce the size.
 template <typename GenT, uint32_t MaxBVHLevels>
@@ -602,7 +615,22 @@ struct SMStack
     MemHit<GenT> committedHit;    // stores committed hit
     MemHit<GenT> potentialHit;    // stores potential hit that is passed to any hit shader
 
-    MemRay<GenT> ray[MaxBVHLevels];       // stores a ray for each instancing level
+    MemRay<GenT> ray0;
+    MemRay<GenT> ray1;
+    MemRay<GenT> ray[MaxBVHLevels - 2];       // stores a ray for each instancing level
+    //MemTravStack travStack[MaxBVHLevels]; // spill location for the internal stack state per instancing level
+
+    static_assert(MaxBVHLevels > 2);
+};
+
+template <typename GenT>
+struct SMStack<GenT, MAX_BVH_LEVELS>
+{
+    MemHit<GenT> committedHit;    // stores committed hit
+    MemHit<GenT> potentialHit;    // stores potential hit that is passed to any hit shader
+
+    MemRay<GenT> ray0;
+    MemRay<GenT> ray1;
     //MemTravStack travStack[MaxBVHLevels]; // spill location for the internal stack state per instancing level
 };
 
@@ -613,7 +641,10 @@ using RTStack2 = RTStack<GenT, MAX_BVH_LEVELS>;
 template <typename GenT>
 using SMStack2 = SMStack<GenT, MAX_BVH_LEVELS>;
 
-#if !defined(__clang__) || (__clang_major__ >= 10)
+template <typename RTStack2T>
+constexpr size_t sizeofRTStack2() { return sizeof(RTStack2T); }
+
+#if !defined(__clang__) || (__clang_major__ >= 15)
 static_assert(std::is_standard_layout_v<RTStack2<Xe>>);
 static_assert(std::is_standard_layout_v<SMStack2<Xe>>);
 #endif
@@ -623,10 +654,12 @@ static_assert(std::is_standard_layout_v<SMStack2<Xe>>);
 // documentation purposes.
 static_assert(offsetof(RTStack2<Xe>, committedHit) == 0,   "unexpected offset!");
 static_assert(offsetof(RTStack2<Xe>, potentialHit) == 32,  "unexpected offset!");
-static_assert(offsetof(RTStack2<Xe>, ray)          == 64,  "unexpected offset!");
+static_assert(offsetof(RTStack2<Xe>, ray0)         == 64,  "unexpected offset!");
+static_assert(offsetof(RTStack2<Xe>, ray1)         == 64 + 64, "unexpected offset!");
 static_assert(offsetof(RTStack2<Xe>, travStack)    == 192, "unexpected offset!");
 
 
+
 /////////////// BVH structures ///////////////
 
 enum NodeType : uint8_t
@@ -744,7 +777,7 @@ enum class InstanceFlags : uint8_t
     TRIANGLE_CULL_DISABLE = 0x1,
     TRIANGLE_FRONT_COUNTERCLOCKWISE = 0x2,
     FORCE_OPAQUE = 0x4,
-    FORCE_NON_OPAQUE = 0x8
+    FORCE_NON_OPAQUE = 0x8,
 };
 
 template <typename GenT>
@@ -904,22 +937,33 @@ struct alignas(LSC_WRITE_GRANULARITY) SWHotZone_v2
     // pad to LSC write granularity (16B on Gen12)
 };
 
+struct alignas(LSC_WRITE_GRANULARITY) SWHotZone_v3
+{
+    uint32_t StackOffset;
+    uint32_t DispatchRaysIndex[3];
+
+    uint32_t Complete;
+
+    // pad to LSC write granularity (16B on Gen12)
+    uint32_t Pad[3];
+};
+
 constexpr uint32_t StackFrameAlign = 16;
 static_assert(IGC::RTStackAlign % LSC_WRITE_GRANULARITY == 0, "not aligned to write granularity?");
 
 // This is the structure of memory that will be allocated by the UMD:
 template <typename HotZoneTy, typename SyncStackTy, typename AsyncStackTy, typename SWStackTy,
-          uint64_t DSS_COUNT, uint64_t NumDSSRTStacks, uint64_t SIMD_LANES_PER_DSS>
+          uint32_t RTStackAlign, uint64_t DSS_COUNT, uint64_t NumDSSRTStacks, uint64_t SIMD_LANES_PER_DSS>
 struct RTMemory
 {
     // Packed SW hot-zones
     alignas(IGC::RTStackAlign) HotZoneTy HotZones[DSS_COUNT * NumDSSRTStacks];
 
     // sync stacks for synchronous ray tracing
-    alignas(IGC::RTStackAlign) SyncStackTy SyncStacks[DSS_COUNT * SIMD_LANES_PER_DSS];
+    alignas(RTStackAlign) SyncStackTy SyncStacks[DSS_COUNT * SIMD_LANES_PER_DSS];
 
     // RTMemBasePointer points here <----
-    alignas(IGC::RTStackAlign) AsyncStackTy AsyncStacks[DSS_COUNT * NumDSSRTStacks];
+    alignas(RTStackAlign) AsyncStackTy AsyncStacks[DSS_COUNT * NumDSSRTStacks];
 
     // Align to L3 sector size, or LSC sector size if stack is LSC-cached
     alignas(IGC::RTStackAlign) SWStackTy SWStacks[DSS_COUNT * NumDSSRTStacks];
@@ -936,14 +980,15 @@ constexpr uint64_t calcRTMemoryAllocSize(
     uint64_t SIMD_LANES_PER_DSS)
 {
     // SIMD_LANES_PER_DSS = EUCount * ThreadCount * SIMD16
-    return IGC::Align(SWHotZoneSize * DSS_COUNT * NumDSSRTStacks, IGC::RTStackAlign)     +
-           IGC::Align(SyncStackSize * DSS_COUNT * SIMD_LANES_PER_DSS, IGC::RTStackAlign) +
-           IGC::Align(AsyncStackSize * DSS_COUNT * NumDSSRTStacks, IGC::RTStackAlign)    +
-           IGC::Align(SWStackSize * DSS_COUNT * NumDSSRTStacks, IGC::RTStackAlign);
+    return IGC::Align(SWHotZoneSize * DSS_COUNT * NumDSSRTStacks, IGC::RTStackAlign) +
+        IGC::Align(SyncStackSize * DSS_COUNT * SIMD_LANES_PER_DSS, IGC::RTStackAlign) +
+        IGC::Align(AsyncStackSize * DSS_COUNT * NumDSSRTStacks, IGC::RTStackAlign) +
+        IGC::Align(SWStackSize * DSS_COUNT * NumDSSRTStacks, IGC::RTStackAlign);
 }
 
+
 constexpr uint32_t getSyncStackSize() {
-#define STYLE(X) static_assert(sizeof(RTStack2<Xe>) == sizeof(RTStack2<X>));
+#define STYLE(X) static_assert(sizeofRTStack2<RTStack2<Xe>>() == sizeofRTStack2<RTStack2<X>>());
 #include "RayTracingMemoryStyle.h"
 #undef STYLE
     return sizeof(RTStack2<Xe>);
@@ -951,7 +996,7 @@ constexpr uint32_t getSyncStackSize() {
 
 // As per this:
 // The rtMemBasePtr points to the top of the async stacks.  After having computed
-// the full allocation with 'calcRTMemoryAllocSize()', This returns the offset
+// the full allocation with 'calcRTMemoryAllocSize()', this returns the offset
 // of the async stacks, hot zone, and sw stack from the base of the allocation.
 constexpr uint64_t calcRTMemoryOffsets(
     uint64_t SWHotZoneSize,
@@ -976,33 +1021,68 @@ constexpr uint64_t calcRTMemoryOffsets(
     return AsyncOffset;
 }
 
+
 // unit tests:
 static_assert(
     calcRTMemoryAllocSize(
         sizeof(SWHotZone_v1), sizeof(RTStack2<Xe>), sizeof(RTStack2<Xe>), 128,
         32, 2048, 16 * 8 * 16) ==
-    sizeof(RTMemory<SWHotZone_v1, RTStack2<Xe>, RTStack2<Xe>, uint8_t[128],
+    sizeof(RTMemory<SWHotZone_v1, RTStack2<Xe>, RTStack2<Xe>, uint8_t[128], IGC::RTStackAlign,
         32, 2048, 16 * 8 * 16>), "mismatch?");
 
 static_assert(
     calcRTMemoryAllocSize(
         8, sizeof(RTStack2<Xe>), sizeof(RTStack2<Xe>), 136,
         32, 2048, 16 * 8 * 16) ==
-    sizeof(RTMemory<uint8_t[8], RTStack2<Xe>, RTStack2<Xe>, uint8_t[136],
+    sizeof(RTMemory<uint8_t[8], RTStack2<Xe>, RTStack2<Xe>, uint8_t[136], IGC::RTStackAlign,
         32, 2048, 16 * 8 * 16>), "mismatch?");
 
+
+/* a list of commands for the ray tracing hardware */
+enum class TraceRayCtrl : uint8_t
+{
+    TRACE_RAY_INITIAL = 0,              // Initializes hit and initializes traversal state
+    TRACE_RAY_INSTANCE = 1,             // Loads committed hit and initializes traversal state
+    TRACE_RAY_COMMIT = 2,               // Loads potential hit and loads traversal state
+    TRACE_RAY_CONTINUE = 3,             // Loads committed hit and loads traversal state
+    TRACE_RAY_NONE = 5                  // Illegal!
+};
+
+struct TraceRayPayload
+{
+    enum class PayloadBits : uint8_t
+    {
+        bvhLevel     = 3,
+        reserved     = 5,
+        traceRayCtrl = 3,
+        reserved_1   = 5,
+        stackID      = 12,
+        reserved_2   = 4,
+    };
+
+    // This is the offset within the bitfield
+    enum class PayloadOffsets : uint8_t
+    {
+        bvhLevel     = 0,
+        reserved     = (uint8_t)PayloadBits::bvhLevel,
+        traceRayCtrl = PayloadOffsets::reserved + (uint8_t)PayloadBits::reserved,
+        reserved_1   = PayloadOffsets::traceRayCtrl + (uint8_t)PayloadBits::traceRayCtrl,
+        stackID      = PayloadOffsets::reserved_1 + (uint8_t)PayloadBits::reserved_1,
+    };
+    // Trace Ray Payload (per SIMD lane)
+    struct TraceRayPayloadData
+    {
+        uint32_t        bvhLevel     : (uint32_t)PayloadBits::bvhLevel;     // bits[  2:0]: tells the hardware which ray to process
+        uint32_t        reserved     : (uint32_t)PayloadBits::reserved;     // bits[  7:3]: reserved MBZ
+        uint32_t        traceRayCtrl : (uint32_t)PayloadBits::traceRayCtrl; // bits[ 10:8]: trace ray operation to perform
+        uint32_t        reserved_1   : (uint32_t)PayloadBits::reserved_1;   // bits[15:11]: reserved MBZ
+        uint32_t        stackID      : (uint32_t)PayloadBits::stackID;      // bits[27:16]: the ID of the stack of this thread
+        uint32_t        reserved_2   : (uint32_t)PayloadBits::reserved_2;   // bits[31:28]: reserved MBZ
+    } payload;
+};
+
 struct TraceRayMessage
 {
-    /* a list of commands for the ray tracing hardware */
-    enum TraceRayCtrl : uint8_t
-    {
-        TRACE_RAY_INITIAL = 0,              // Initializes hit and initializes traversal state
-        TRACE_RAY_INSTANCE = 1,             // Loads committed hit and initializes traversal state
-        TRACE_RAY_COMMIT = 2,               // Loads potential hit and loads traversal state
-        TRACE_RAY_CONTINUE = 3,             // Loads committed hit and loads traversal state
-        TRACE_RAY_NONE = 5                  // Illegal!
-    };
-
     // This data is sent per message and is uniform across all rays in a dispatch.
     struct Header
     {
@@ -1013,21 +1093,17 @@ struct TraceRayMessage
             uint64_t rayQueryLocation;
             // bit 128
             uint64_t rayQuery         : 1;   // indicates a ray query message
+            uint64_t rayQueryCheck    : 1;   // indicates a ray query check message
+            uint64_t rayQueryRelease  : 1;   // indicates a ray query release message
         };
     } header;
 
-    // this data is sent per ray (SIMD LANE)
-    struct Payload
-    {
-        uint8_t      bvhLevel;     // the level tells the hardware which ray to process
-        TraceRayCtrl traceRayCtrl; // the command the hardware should perform
-        uint16_t     stackID;      // the ID of the stack of this thread
-    } payload;
+    TraceRayPayload payload;
 
-    static_assert(sizeof(Payload) == 4, "Payload must be 4 bytes");
+    static_assert(sizeof(TraceRayPayload::payload) == 4, "Payload must be 4 bytes");
 };
 
-// TraceRayInline enums
+// RayQuery enums
 
 enum COMMITTED_STATUS : uint32_t
 {
