@@ -83,7 +83,8 @@ private:
 
     //return m_ShMemRTCtrls[index]
     Value* getShMemRTCtrl(RTBuilder& builder, Value* queryIndex) {
-        return builder.CreateGEP(m_ShMemRTCtrls, { builder.getInt32(0), queryIndex }, VALUE_NAME("&shadowMem.RTCtrl"));
+        return builder.CreateGEP(cast<llvm::AllocaInst>(m_ShMemRTCtrls)->getAllocatedType(),
+            m_ShMemRTCtrls, { builder.getInt32(0), queryIndex }, VALUE_NAME("&shadowMem.RTCtrl"));
     }
 
     //return rtStacks[index]
@@ -94,7 +95,8 @@ private:
     //return rtStacks[index]
     RTBuilder::SyncStackPointerVal* getShMemRayQueryRTStack(RTBuilder& builder, Value* queryIndex) {
         return static_cast<RTBuilder::SyncStackPointerVal*>(
-            builder.CreateGEP(m_ShMemRTStacks, { builder.getInt32(0), queryIndex }, VALUE_NAME("&shadowMem.RTStack")));
+            builder.CreateGEP(cast<llvm::AllocaInst>(m_ShMemRTStacks)->getAllocatedType(),
+                m_ShMemRTStacks, { builder.getInt32(0), queryIndex }, VALUE_NAME("&shadowMem.RTStack")));
     }
 
     std::pair<BasicBlock*, BasicBlock*> branchOnPotentialHitDone(
@@ -104,6 +106,7 @@ private:
     void emitSingleRQMemRayWrite(RTBuilder& builder, Value* queryObjIndex);
     bool analyzeSingleRQMemRayWrite(const Function& F) const;
     std::pair<bool, unsigned> analyzeSingleRQProceed(const Function& F) const;
+    void HandleAcceptHitAndEndSearch(RTBuilder& builder, RTBuilder::SyncStackPointerVal* ShadowMemStackPointer, Instruction* IP);
 
     Value* emitProceedMainBody(RTBuilder& builder, Value* queryObjIndex);
 
@@ -297,6 +300,7 @@ void TraceRayInlineLoweringPass::LowerTraceRayInline(Function& F)
                 builder.getRootNodePtr(trace->getBVH()),
                 trace->getFlag(),
                 trace->getMask(),
+                trace->getComparisonValue(),
                 trace->getTMax());
         }
 
@@ -304,7 +308,7 @@ void TraceRayInlineLoweringPass::LowerTraceRayInline(Function& F)
         // RayQueryObject->stateInfo.traceRayCtrl = TRACE_RAY_INITIAL
         builder.setSyncTraceRayControl(
             getShMemRTCtrl(builder, QueryObjIndex),
-            TraceRayMessage::TraceRayCtrl::TRACE_RAY_INITIAL);
+            TraceRayCtrl::TRACE_RAY_INITIAL);
 
         if (singleRQMemRayStore)
         {
@@ -463,6 +467,37 @@ void TraceRayInlineLoweringPass::LowerSyncStackToShadowMemory(Function& F)
             ShadowMemRTCtrlPtr);
 
         SS2SM->replaceAllUsesWith(Proceed);
+
+        if (IGC_IS_FLAG_DISABLED(DisableInvalidateRTStackAfterLastRead))
+        {
+            auto* LSCInvalidate = GenISAIntrinsic::getDeclaration(
+                F.getParent(),
+                GenISAIntrinsic::GenISA_LSCInvalidate,
+                { builder.getInt32Ty(), HWStackPointer->getType() }
+            );
+
+            auto [InvalidateAfterProceedBB, _] = builder.createTriangleFlow(builder.CreateNot(Proceed), SS2SM, VALUE_NAME("InvalidateAfterProceed"));
+            builder.SetInsertPoint(InvalidateAfterProceedBB->getTerminator());
+
+            LSC_L1_L3_CC CacheCtrl =
+                m_CGCtx->platform.isSupportedLSCCacheControlsEnum(LSC_L1IAR_L3IAR, true) ?
+                LSC_L1IAR_L3IAR :
+                LSC_L1IAR_WB_L3C_WB;
+
+            for (uint i = 0; i < getSyncStackSize() / m_CGCtx->platform.LSCCachelineSize(); i++)
+            {
+                builder.CreateCall(
+                    LSCInvalidate,
+                    {
+                        HWStackPointer,
+                        builder.getInt32(i * m_CGCtx->platform.LSCCachelineSize()),
+                        builder.getInt32(LSC_DATA_SIZE_32b), // doesn't matter what we put here because the entire cacheline is invalidated
+                        builder.getInt32(LSC_DATA_ELEMS_1),
+                        builder.getInt32(CacheCtrl)
+                    }
+                );
+            }
+        }
     }
 
     for (auto SS2SM : SS2SMs)
@@ -798,7 +833,9 @@ void TraceRayInlineLoweringPass::LowerCommitNonOpaqueTriangleHit(Function& F)
         auto* const ShadowMemStackPointer = getShMemRayQueryRTStack(builder, CH->getQueryObjIndex());
 
         builder.createPotentialHit2CommittedHit(ShadowMemStackPointer);
-        builder.setSyncTraceRayControl(getShMemRTCtrl(builder, CH->getQueryObjIndex()), TraceRayMessage::TraceRayCtrl::TRACE_RAY_COMMIT);
+        builder.setSyncTraceRayControl(getShMemRTCtrl(builder, CH->getQueryObjIndex()), TraceRayCtrl::TRACE_RAY_COMMIT);
+
+        HandleAcceptHitAndEndSearch(builder, ShadowMemStackPointer, CH);
     }
 
     for (auto CH : CommitHits)
@@ -925,7 +962,9 @@ void TraceRayInlineLoweringPass::LowerCommitProceduralPrimitiveHit(Function& F)
 
         builder.setSyncTraceRayControl(
             getShMemRTCtrl(builder, CH->getQueryObjIndex()),
-            TraceRayMessage::TraceRayCtrl::TRACE_RAY_COMMIT);
+            TraceRayCtrl::TRACE_RAY_COMMIT);
+
+        HandleAcceptHitAndEndSearch(builder, ShadowMemStackPointer, CH);
     }
 
     for (auto CH : CommitHits)
@@ -934,6 +973,21 @@ void TraceRayInlineLoweringPass::LowerCommitProceduralPrimitiveHit(Function& F)
     }
 }
 
+void TraceRayInlineLoweringPass::HandleAcceptHitAndEndSearch(RTBuilder& builder, RTBuilder::SyncStackPointerVal* ShadowMemStackPointer, Instruction* IP)
+{
+    auto* rayFlags = builder.getRayFlags(ShadowMemStackPointer);
+    auto* AcceptHitAndEndSearch = builder.getInt16(static_cast<uint32_t>(RTStackFormat::RayFlags::ACCEPT_FIRST_HIT_AND_END_SEARCH));
+
+    Value* V;
+
+    V = builder.CreateAnd(rayFlags, AcceptHitAndEndSearch);
+    V = builder.CreateICmpNE(V, builder.getInt16(0));
+
+    auto [accepthitBB, _] = builder.createTriangleFlow(V, IP, VALUE_NAME("AcceptHitAndEndSearch"), VALUE_NAME("Default"));
+    builder.SetInsertPoint(accepthitBB->getTerminator());
+    builder.setDoneBit(ShadowMemStackPointer, false);
+}
+
 //For 3D/Compute Shaders The RTGlobals Pointer comes from a different location.
 class RTGlobalsPointerLoweringPass : public FunctionPass
 {
@@ -1034,7 +1088,7 @@ bool RTGlobalsPointerLoweringPass::runOnFunction(Function& F)
             // structures to `RTGlobalsAlign`.
             constexpr uint32_t Offset =
                 IGC::Align(sizeof(RayDispatchGlobalData), IGC::RTGlobalsAlign);
-            Ptr = builder.CreateGEP(Ptr, builder.getInt32(Offset));
+            Ptr = builder.CreateGEP(builder.getInt8Ty(), Ptr, builder.getInt32(Offset));
             auto* rtGlobalsPtrHi = builder.CreateBitCast(
                 Ptr, rtGlobalsPtr->getType());
             rtGlobalsPtrSplit = builder.CreateSelect(
