@@ -16,6 +16,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 
 #include <iomanip>
+#include <iostream>
 #include <fstream>
 #include "Probe/Assertion.h"
 
@@ -51,7 +52,7 @@ bool CGen8OpenCLProgram::CLProgramCtxProvider::isProgramDebuggable() const {
     return m_Context.m_InternalOptions.KernelDebugEnable;
 }
 bool CGen8OpenCLProgram::CLProgramCtxProvider::hasProgrammableBorderColor() const {
-    return m_Context.m_DriverInfo.ProgrammableBorderColorInCompute();
+    return false;
 }
 
 bool CGen8OpenCLProgram::CLProgramCtxProvider::useBindlessMode() const {
@@ -265,9 +266,16 @@ RETVAL CGen8OpenCLProgramBase::GetProgramDebugData(char* dstBuffer, size_t dstBu
     return retValue;
 }
 
-static void dumpZEInfo(
-    const IGC::CodeGenContext &Ctx, ZEBinaryBuilder &ZEBuilder)
-{
+static void dumpZEInfo(const IGC::CodeGenContext &Ctx,
+                       ZEBinaryBuilder &ZEBuilder, bool toConsole = false) {
+    if (toConsole) {
+        llvm::SmallVector<char, 1024> buf;
+        llvm::raw_svector_ostream os(buf);
+        ZEBuilder.printZEInfo(os);
+        std::cout << os.str().str() << std::endl;
+        return;
+    }
+
     auto filename = IGC::Debug::DumpName(
         IGC::Debug::GetShaderOutputName())
         .Hash(Ctx.hash)
@@ -509,7 +517,7 @@ bool CGen8OpenCLProgram::GetZEBinary(
         // FIXME: We actually expect only one simd mode per kernel. There should not be multiple SIMD mode available
         // for one kernel (runtime cannot support that). So these check can be simplified
         std::vector<IGC::COpenCLKernel*> kernelVec;
-        if ((m_Context.m_DriverInfo.sendMultipleSIMDModes() || m_Context.m_enableSimdVariantCompilation)
+        if ((m_Context.m_enableSimdVariantCompilation)
             && (m_Context.getModuleMetaData()->csInfo.forcedSIMDSize == 0))
         {
             // For multiple SIMD modes, send SIMD modes in descending order
@@ -789,7 +797,7 @@ bool CGen8OpenCLProgram::GetZEBinary(
         if (IGC_IS_FLAG_DISABLED(ElfTempDumpEnable))
         {
             // Remove all temporary input ELF files
-            for (auto elfFile : elfVecNames)
+            for (const auto &elfFile : elfVecNames)
             {
                 if (elfFile.compare(elfLinkerLogName.c_str()))
                 {
@@ -814,6 +822,8 @@ bool CGen8OpenCLProgram::GetZEBinary(
     // dump .ze_info to a file
     if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
         dumpZEInfo(m_Context, zebuilder);
+    if (IGC_IS_FLAG_ENABLED(DumpZEInfoToConsole))
+        dumpZEInfo(m_Context, zebuilder, true);
 
     return retValue;
 }
@@ -828,8 +838,7 @@ void CGen8OpenCLProgram::CreateKernelBinaries()
 
         // Determine how many simd modes we have per kernel
         std::vector<IGC::COpenCLKernel*> kernelVec;
-        if ((m_Context.m_DriverInfo.sendMultipleSIMDModes() || m_Context.m_enableSimdVariantCompilation)
-            && (m_Context.getModuleMetaData()->csInfo.forcedSIMDSize == 0))
+        if (m_Context.m_enableSimdVariantCompilation && m_Context.getModuleMetaData()->csInfo.forcedSIMDSize == 0)
         {
             // For multiple SIMD modes, send SIMD modes in descending order
             if (IGC::COpenCLKernel::IsValidShader(simd32Shader))
