@@ -285,7 +285,7 @@ void ZEBinaryBuilder::addSPIRV(const uint8_t* data, uint32_t size)
 
 void ZEBinaryBuilder::addMiscInfoSection(std::string sectName, const uint8_t* data, uint32_t size)
 {
-    mBuilder.addSectionMisc(sectName, data, size);
+    mBuilder.addSectionMisc(std::move(sectName), data, size);
 }
 
 void ZEBinaryBuilder::addMetrics(const uint8_t* data, uint32_t size)
@@ -421,17 +421,17 @@ void ZEBinaryBuilder::addProgramSymbols(const IGC::SOpenCLProgramInfo& annotatio
 
     // add symbols defined in global constant section
     IGC_ASSERT(symbols.globalConst.empty() || mGlobalConstSectID != -1);
-    for (auto sym : symbols.globalConst)
+    for (const auto& sym : symbols.globalConst)
         addSymbol(sym, llvm::ELF::STB_GLOBAL, mGlobalConstSectID);
 
     // add symbols defined in global string constant section
     IGC_ASSERT(symbols.globalStringConst.empty() || mConstStringSectID != -1);
-    for (auto sym : symbols.globalStringConst)
+    for (const auto& sym : symbols.globalStringConst)
         addSymbol(sym, llvm::ELF::STB_GLOBAL, mConstStringSectID);
 
     // add symbols defined in global section, mGlobalSectID may be unallocated
     // at this point if symbols are undef
-    for (auto sym : symbols.global)
+    for (const auto& sym : symbols.global)
         addSymbol(sym, llvm::ELF::STB_GLOBAL, mGlobalSectID);
 }
 
@@ -453,13 +453,13 @@ void ZEBinaryBuilder::addKernelSymbols(
         annotations.m_kernelProgram);
 
     // add local symbols of this kernel binary
-    for (auto sym : symbols.local) {
+    for (const auto& sym : symbols.local) {
         IGC_ASSERT(sym.s_type != vISA::GenSymType::S_UNDEF);
         addSymbol(sym, llvm::ELF::STB_LOCAL, kernelSectId);
     }
 
     // add function symbols defined in kernel text
-    for (auto sym : symbols.function)
+    for (const auto &sym : symbols.function)
         addSymbol(sym, llvm::ELF::STB_GLOBAL, kernelSectId);
 
     // we do not support sampler symbols now
@@ -473,11 +473,11 @@ void ZEBinaryBuilder::addProgramRelocations(const IGC::SOpenCLProgramInfo& annot
     // FIXME: For r_type, zebin::R_TYPE_ZEBIN should have the same enum value as visa::GenRelocType.
     // Take the value directly
     IGC_ASSERT(relocs.globalConstReloc.empty() || mGlobalConstSectID != -1);
-    for (auto reloc : relocs.globalConstReloc)
+    for (const auto &reloc : relocs.globalConstReloc)
         mBuilder.addRelRelocation(reloc.r_offset, reloc.r_symbol, static_cast<zebin::R_TYPE_ZEBIN>(reloc.r_type), mGlobalConstSectID);
 
     IGC_ASSERT(relocs.globalReloc.empty() || mGlobalSectID != -1);
-    for (auto reloc : relocs.globalReloc)
+    for (const auto& reloc : relocs.globalReloc)
         mBuilder.addRelRelocation(reloc.r_offset, reloc.r_symbol, static_cast<zebin::R_TYPE_ZEBIN>(reloc.r_type), mGlobalSectID);
 }
 
@@ -500,7 +500,7 @@ void ZEBinaryBuilder::addKernelRelocations(
     // FIXME: For r_type, zebin::R_TYPE_ZEBIN should have the same enum value as visa::GenRelocType.
     // Take the value directly
     if (!relocs.empty())
-        for (auto reloc : relocs)
+        for (const auto& reloc : relocs)
             mBuilder.addRelRelocation(reloc.r_offset, reloc.r_symbol, (zebin::R_TYPE_ZEBIN)reloc.r_type, targetId);
 }
 
@@ -565,6 +565,8 @@ void ZEBinaryBuilder::addKernelExecEnv(const SOpenCLKernelInfo& annotations,
     env.simd_size = annotations.m_executionEnvironment.CompiledSIMDSize;
     // set slm size to inline local size
     env.slm_size = annotations.m_executionEnvironment.SumFixedTGSMSizes ;
+    env.private_size = annotations.m_executionEnvironment.PerThreadPrivateMemoryUsage;
+    env.spill_size = annotations.m_executionEnvironment.PerThreadSpillMemoryUsage;
     env.subgroup_independent_forward_progress = annotations.m_executionEnvironment.SubgroupIndependentForwardProgressRequired;
     if (annotations.m_executionEnvironment.WorkgroupWalkOrder[0] ||
         annotations.m_executionEnvironment.WorkgroupWalkOrder[1] ||
@@ -838,7 +840,7 @@ void ZEBinaryBuilder::addElfSections(void* elfBin, size_t elfSize)
 
                                 // Avoid symbol duplications - check whether a current symbol has been previously added.
                                 bool isSymbolAdded = false;
-                                for (auto zeBinSym : zeBinSymbols)
+                                for (const auto& zeBinSym : zeBinSymbols)
                                 {
                                     if (!zeBinSym.compare(zeSym.s_name))
                                     {
@@ -913,12 +915,17 @@ void ZEBinaryBuilder::printBinaryObject(const std::string& filename)
         mBuilder.finalize(os);
 }
 
+void ZEBinaryBuilder::printZEInfo(raw_ostream &os)
+{
+    mZEInfoBuilder.printZEInfoInYaml(os);
+}
+
 void ZEBinaryBuilder::printZEInfo(const std::string& filename)
 {
     std::error_code EC;
     llvm::raw_fd_ostream os(filename, EC);
     if (!EC)
-        mZEInfoBuilder.printZEInfoInYaml(os);
+        printZEInfo(os);
 }
 
 void ZEBinaryBuilder::addKernelDebugEnv(const SOpenCLKernelInfo& annotations,
