@@ -55,8 +55,6 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/BuiltInFuncImport.h"
 #include "Compiler/Optimizer/CodeAssumption.hpp"
 #include "Compiler/Optimizer/Scalarizer.h"
-#include "Compiler/Optimizer/OpenCLPasses/DebuggerSupport/ImplicitGIDPass.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/DebuggerSupport/ImplicitGIDRestoring.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ExtensionFuncs/ExtensionArgAnalysis.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ExtensionFuncs/ExtensionFuncsAnalysis.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ExtensionFuncs/ExtensionFuncResolution.hpp"
@@ -104,6 +102,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/SPIRMetaDataTranslation.h"
 #include "Compiler/Optimizer/OpenCLPasses/ErrorCheckPass/ErrorCheckPass.h"
 #include "Compiler/Optimizer/OpenCLPasses/PoisonFP64KernelsPass/PoisonFP64KernelsPass.h"
+#include "Compiler/Optimizer/OpenCLPasses/BfloatFuncs/BfloatFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/DpasFuncs/DpasFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/LSCFuncs/LSCFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/NamedBarriers/NamedBarriersResolution.hpp"
@@ -128,6 +127,8 @@ SPDX-License-Identifier: MIT
 #include "LowerInvokeSIMD.hpp"
 #include "ResolveConstExprCalls.h"
 #include "Compiler/Optimizer/IGCInstCombiner/IGCInstructionCombining.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/BufferBoundsChecking/BufferBoundsChecking.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/BufferBoundsChecking/BufferBoundsCheckingPatcher.hpp"
 
 #include "common/debug/Debug.hpp"
 #include "common/igc_regkeys.hpp"
@@ -140,6 +141,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/TimeStatsCounter.h"
 #include "Compiler/DebugInfo/ScalarVISAModule.h"
 #include "Compiler/DebugInfo/Utils.h"
+#include "Compiler/Builtins/BIFFlagCtrl/BIFFlagCtrlResolution.hpp"
 #include "DebugInfo/VISADebugEmitter.hpp"
 
 #include <string>
@@ -304,6 +306,9 @@ static void CommonOCLBasedPasses(
     CompilerOpts.IsLibraryCompilation =
         pContext->m_Options.IsLibraryCompilation;
 
+    CompilerOpts.LibraryCompileSIMDSize =
+        pContext->m_Options.LibraryCompileSIMDSize;
+
     CompilerOpts.ExcludeIRFromZEBinary =
         pContext->m_InternalOptions.ExcludeIRFromZEBinary;
 
@@ -314,6 +319,9 @@ static void CommonOCLBasedPasses(
         pContext->m_InternalOptions.EnableFP64GenEmu ||
         pContext->m_Options.EnableFP64GenEmu;
 
+    CompilerOpts.FP64GenConvEmulationEnabled =
+        pContext->m_Options.EnableFP64GenConvEmu;
+
     CompilerOpts.LoadCacheDefault =
         pContext->m_InternalOptions.LoadCacheDefault;
 
@@ -327,7 +335,6 @@ static void CommonOCLBasedPasses(
 #endif // IGC_SCALAR_USE_KHRONOS_SPIRV_TRANSLATOR
     mpmSPIR.add(new TypesLegalizationPass());
     mpmSPIR.add(new TargetLibraryInfoWrapperPass());
-    mpmSPIR.add(createDeadCodeEliminationPass());
     mpmSPIR.add(new MetaDataUtilsWrapper(pMdUtils, pContext->getModuleMetaData()));
     mpmSPIR.add(new CodeGenContextWrapper(pContext));
     mpmSPIR.add(new SPIRMetaDataTranslation());
@@ -335,6 +342,7 @@ static void CommonOCLBasedPasses(
     mpmSPIR.add(new ConvertUserSemanticDecoratorOnFunctions());
 #endif // IGC_SCALAR_USE_KHRONOS_SPIRV_TRANSLATOR
     mpmSPIR.add(new HandleSpirvDecorationMetadata());
+    mpmSPIR.add(createDeadCodeEliminationPass());
     mpmSPIR.run(*pContext->getModule());
 
     bool isOptDisabled = CompilerOpts.OptDisable;
@@ -366,11 +374,6 @@ static void CommonOCLBasedPasses(
         mpm.add(createBIFTransformsPass());
     }
 
-    if(pContext->m_InternalOptions.KernelDebugEnable)
-    {
-        mpm.add(new ImplicitGlobalId());
-    }
-
     if (IGC_IS_FLAG_ENABLED(EnableCodeAssumption))
     {
         mpm.add(new CodeAssumption());
@@ -387,7 +390,12 @@ static void CommonOCLBasedPasses(
 
     mpm.add(new HandleDevicelibAssert());
     if (IGC_IS_FLAG_ENABLED(StackOverflowDetection)) {
-        mpm.add(new StackOverflowDetectionPass());
+        mpm.add(new StackOverflowDetectionPass(StackOverflowDetectionPass::Mode::Initialize));
+    }
+
+    if (IGC_IS_FLAG_ENABLED(BufferBoundsChecking) || pContext->isBufferBoundsChecking())
+    {
+        mpm.add(new BufferBoundsChecking());
     }
 
     mpm.add(new NamedBarriersResolution(pContext->platform.getPlatformInfo().eRenderCoreFamily));
@@ -395,6 +403,7 @@ static void CommonOCLBasedPasses(
     mpm.add(createTimeStatsCounterPass(pContext, TIME_Unify_BuiltinImport, STATS_COUNTER_START));
     mpm.add(createBuiltInImportPass(std::move(BuiltinGenericModule), std::move(BuiltinSizeModule)));
     mpm.add(createTimeStatsCounterPass(pContext, TIME_Unify_BuiltinImport, STATS_COUNTER_END));
+    mpm.add(new BIFFlagCtrlResolution(pContext));
 
     if (IGC_GET_FLAG_VALUE(AllowMem2Reg))
     {
@@ -439,9 +448,6 @@ static void CommonOCLBasedPasses(
 
         // Check after GlobalDCE in case of doubles in dead functions
         mpm.add(new ErrorCheck());
-        if (pContext->m_Options.EnableUnsupportedFP64Poisoning) {
-            mpm.add(new PoisonFP64Kernels());
-        }
 
         mpm.add(new LowerInvokeSIMD());
 
@@ -494,6 +500,11 @@ static void CommonOCLBasedPasses(
 
         mpm.add(createGASRetValuePropagatorPass());
 
+        // this pass is intended to inline the remaining always inline functions that had issues
+        // with argument address spaces (byVal addrspace(4)) in the previous attempt
+        mpm.add(createAlwaysInlinerLegacyPass());
+        mpm.add(new PurgeMetaDataUtils());
+
         // Run another round of constant breaking as GAS resolving may generate constants (constant address)
         mpm.add(new BreakConstantExpr());
     }
@@ -534,6 +545,12 @@ static void CommonOCLBasedPasses(
     mpm.add(createRayTracingIntrinsicAnalysisPass());
     // Adding implicit args based on Analysis passes
     mpm.add(new AddImplicitArgs());
+
+    if (IGC_IS_FLAG_ENABLED(BufferBoundsChecking) || pContext->isBufferBoundsChecking())
+    {
+        mpm.add(new BufferBoundsCheckingPatcher());
+    }
+
     mpm.add(createRayTracingIntrinsicResolutionPass());
 
     // Resolution passes
@@ -542,6 +559,7 @@ static void CommonOCLBasedPasses(
     mpm.add(new ResolveOCLAtomics());
     mpm.add(new ResourceAllocator());
     mpm.add(new SubGroupFuncsResolution());
+    mpm.add(new BfloatFuncsResolution());
     mpm.add(createDpasFuncsResolutionPass());
     mpm.add(createLSCFuncsResolutionPass());
 
@@ -566,6 +584,7 @@ static void CommonOCLBasedPasses(
     mpm.add(new ErrorCheck());
     if (pContext->m_Options.EnableUnsupportedFP64Poisoning) {
         mpm.add(new PoisonFP64Kernels());
+        mpm.add(new PurgeMetaDataUtils());
     }
 
     mpm.add(new ImageFuncResolution());
@@ -603,10 +622,6 @@ static void CommonOCLBasedPasses(
     // TODO: Run CheckInstrTypes after builtin import to determine if builtins have allocas.
     mpm.add(createSROAPass());
     mpm.add(createIGCInstructionCombiningPass());
-    if (pContext->m_InternalOptions.KernelDebugEnable)
-    {
-        mpm.add(new ImplicitGIDRestoring());
-    }
     // See the comment above (it's copied as is).
     // Instcombine can create constant expressions, which are not handled by the program scope constant resolution pass.
     // For example, in InsertDummyKernelForSymbolTablePass addresses of indirectly called functions
