@@ -456,7 +456,7 @@ bool TranslateSPIRVToLLVM(
     // This option has to be enabled since SPIRV-Translator for LLVM13 because of:
     // https://github.com/KhronosGroup/SPIRV-LLVM-Translator/commit/835eb7e. This change
     // has been also backported to SPIRV-Translator for LLVM11.
-#if LLVM_VERSION_MAJOR >= 13 || LLVM_VERSION_MAJOR == 11
+#if LLVM_VERSION_MAJOR >= 13
     Opts.setPreserveOCLKernelArgTypeMetadataThroughString(true);
 #endif
 
@@ -469,7 +469,7 @@ bool TranslateSPIRVToLLVM(
             Opts.setSpecConst(SC.first, SC.second);
     }
 
-    // Actual translation from SPIR-V to LLLVM
+    // Actual translation from SPIR-V to LLVM
     success = llvm::readSpirv(Context, Opts, IS, LLVMModule, stringErrMsg);
 #else // IGC Legacy SPIRV Translator
     success = igc_spv::ReadSPIRV(Context, IS, LLVMModule, stringErrMsg, &specIDToSpecValueMap);
@@ -541,7 +541,7 @@ bool ProcessElfInput(
                 const std::string suffix = spvHashSuffix.str();
                 const char* pOutputFolder = IGC::Debug::GetShaderOutputFolder();
 
-                // Remove any already exising SPIR-V dumps from GetSpecConstantsInfo
+                // Remove any already existing SPIR-V dumps from GetSpecConstantsInfo
                 // and dump new ones with correct names
                 std::string spvHashString = suffix.c_str();
                 spvHashString.erase(0, 1);
@@ -912,7 +912,7 @@ bool ParseInput(
 
     // IGC does not handle legacy ocl binary for now (legacy ocl binary
     // is the binary that contains text LLVM IR (2.7 or 3.0).
-    if (strInput.size() > 1 && !(strInput[0] == 'B' && strInput[1] == 'C'))
+    if (!strInput.startswith("BC"))
     {
         bool isLLVM27IR = false, isLLVM30IR = false;
 
@@ -1002,11 +1002,11 @@ void RebuildGlobalAnnotations(IGC::OpenCLProgramContext& oclContext, Module* pKe
     };
 
     std::vector<Constant*> newGlobalAnnotations;
-    auto annotations_array = dyn_cast<ConstantArray>(globalAnnotations->getOperand(0));
+    auto annotations_array = cast<ConstantArray>(globalAnnotations->getOperand(0));
     for (const auto& op : annotations_array->operands())
     {
-        auto annotation_struct = dyn_cast<ConstantStruct>(op.get());
-        auto annotated_function = dyn_cast<Function>(annotation_struct->getOperand(0)->getOperand(0));
+        auto annotation_struct = cast<ConstantStruct>(op.get());
+        auto annotated_function = cast<Function>(annotation_struct->getOperand(0)->getOperand(0));
 
         if (requiresRecompilation(annotated_function))
         {
@@ -1110,21 +1110,23 @@ void overrideOCLProgramBinary(
     binarySize = newBinarySize;
 }
 
-void dumpOCLProgramBinary(
-    OpenCLProgramContext& Ctx,
-    const char* binaryOutput,
-    size_t binarySize)
-{
-    auto name = DumpName(IGC::Debug::GetShaderOutputName())
-        .Hash(Ctx.hash)
-        .Type(ShaderType::OPENCL_SHADER)
-        .Extension("progbin");
+void dumpOCLProgramBinary(const char* fileName, const char *binaryOutput,
+                          size_t binarySize) {
+  std::error_code EC;
+  llvm::raw_fd_ostream f(fileName, EC);
 
-    std::error_code EC;
-    llvm::raw_fd_ostream f(name.str(), EC);
+  if (!EC)
+    f.write(binaryOutput, binarySize);
+}
 
-    if (!EC)
-        f.write(binaryOutput, binarySize);
+void dumpOCLProgramBinary(OpenCLProgramContext &Ctx, const char *binaryOutput,
+                          size_t binarySize) {
+  auto name = DumpName(IGC::Debug::GetShaderOutputName())
+                  .Hash(Ctx.hash)
+                  .Type(ShaderType::OPENCL_SHADER)
+                  .Extension("progbin");
+
+  dumpOCLProgramBinary(name.str().data(), binaryOutput, binarySize);
 }
 
 static std::unique_ptr<llvm::MemoryBuffer> GetGenericModuleBuffer()
@@ -1162,21 +1164,73 @@ bool TranslateBuildSPMD(
     // Setting mutex to ensure that single thread will enter and setup this flag.
     {
         const std::lock_guard<std::mutex> lock(llvm_mutex);
+        std::vector<const char*> args;
+        args.push_back("igc");
+        auto optionsMap = llvm::cl::getRegisteredOptions();
+
         // Disable code sinking in instruction combining.
         // This is a workaround for a performance issue caused by code sinking
         // that is being done in LLVM's instcombine pass.
         // This code will be removed once sinking is removed from instcombine.
-        auto optionsMap = llvm::cl::getRegisteredOptions();
         llvm::StringRef instCombineFlag = "-instcombine-code-sinking=0";
         auto instCombineSinkingSwitch = optionsMap.find(instCombineFlag.trim("-=0"));
         if (instCombineSinkingSwitch != optionsMap.end())
         {
             if (instCombineSinkingSwitch->getValue()->getNumOccurrences() == 0)
             {
-                const char* const args[] = { "igc", instCombineFlag.data() };
-                llvm::cl::ParseCommandLineOptions(sizeof(args) / sizeof(args[0]), args);
+                args.push_back(instCombineFlag.data());
             }
         }
+
+        // With the default (250) maximum number of accesses allowed for memory
+        // promotion when using MemorySSA we lack the performance for some
+        // applications. Setting the number of accesses for memory promotion
+        // cap to 500 solves this issue.
+        llvm::StringRef licmMSSAPromotionFlag = "-licm-mssa-max-acc-promotion=500";
+        auto licmMSSAPromotionSwitch = optionsMap.find(licmMSSAPromotionFlag.trim("-=500"));
+        if (licmMSSAPromotionSwitch != optionsMap.end())
+        {
+            if (licmMSSAPromotionSwitch->getValue()->getNumOccurrences() == 0)
+            {
+                args.push_back(licmMSSAPromotionFlag.data());
+            }
+        }
+
+        // Avoid stack overflow in AliasAnalysis for expansive loop unrolling cases.
+        llvm::StringRef aaQueryDepthFlag = "-basic-aa-max-query-depth=192";
+        auto aaQueryDepthSwitch = optionsMap.find(aaQueryDepthFlag.trim("-=192"));
+        if (aaQueryDepthSwitch != optionsMap.end())
+        {
+            if (aaQueryDepthSwitch->getValue()->getNumOccurrences() == 0)
+            {
+                args.push_back(aaQueryDepthFlag.data());
+            }
+        }
+
+        llvm::StringRef dsePartialOverwriteTrackingFlag = "-enable-dse-partial-overwrite-tracking=1";
+        auto dsePartialOverwriteTrackingSwitch = optionsMap.find(dsePartialOverwriteTrackingFlag.trim("-=1"));
+        if (dsePartialOverwriteTrackingSwitch != optionsMap.end())
+        {
+            if (dsePartialOverwriteTrackingSwitch->getValue()->getNumOccurrences() == 0)
+            {
+                args.push_back(dsePartialOverwriteTrackingFlag.data());
+            }
+        }
+
+        llvm::StringRef dseMSSAStepLimitFlag = "-dse-memoryssa-walklimit=150";
+        auto dseMSSAStepLimitSwitch = optionsMap.find(dseMSSAStepLimitFlag.trim("-=150"));
+        if (dseMSSAStepLimitSwitch != optionsMap.end())
+        {
+            if (dseMSSAStepLimitSwitch->getValue()->getNumOccurrences() == 0)
+            {
+                args.push_back(dseMSSAStepLimitFlag.data());
+            }
+        }
+
+        if (std::size(args) > 1)
+        {
+            llvm::cl::ParseCommandLineOptions(std::size(args), &args[0]);
+        }
     }
 
     if (IGC_IS_FLAG_ENABLED(QualityMetricsEnable))
@@ -1224,10 +1278,13 @@ bool TranslateBuildSPMD(
         // dump cmd file that has igcstandalone command to compile this kernel.
         std::ostringstream cmdline;
         cmdline << "IGCStandalone -api ocl"
+                << " -llvm " << LLVM_VERSION_MAJOR
                 << std::hex
                 << " -device 0x" << IGCPlatform.GetProductFamily()
                 << ".0x" << IGCPlatform.GetDeviceId()
                 << ".0x" << IGCPlatform.GetRevId()
+                << " -gmd_render 0x" << GFX_GET_GMD_RELEASE_VERSION_RENDER(IGCPlatform.getPlatformInfo())
+                << ".0x" << GFX_GET_GMD_REV_ID_RENDER(IGCPlatform.getPlatformInfo())
                 << std::dec
                 << " -inputcs " << getBaseFilename(inputf);
         if (isbc)
@@ -1313,6 +1370,10 @@ bool TranslateBuildSPMD(
         oclContext.m_floatDenormMode32 = FLOAT_DENORM_RETAIN;
         oclContext.m_floatDenormMode64 = FLOAT_DENORM_RETAIN;
     }
+    if (oclContext.platform.hasBFTFDenormMode())
+    {
+        oclContext.m_floatDenormModeBFTF = FLOAT_DENORM_RETAIN;
+    }
 
     unsigned PtrSzInBits = pKernelModule->getDataLayout().getPointerSizeInBits();
     // TODO: Again, this should not happen on each compilation
@@ -1376,7 +1437,7 @@ bool TranslateBuildSPMD(
                 // When llvm reads in multiple modules, say, M0, M1, under the same llvmcontext, if both
                 // M0 and M1 has the same struct type,  M0 will have the original name and M1 the derived
                 // name for that type.  For example, clk_event_t,  M0 will have clk_event_t, while M1 will
-                // have clk_event_t.2 (number is arbitary). After linking, those two named types should be
+                // have clk_event_t.2 (number is arbitrary). After linking, those two named types should be
                 // mapped to the same type, otherwise, we could have type-mismatch (for example, OCL GAS
                 // builtin_functions tests will assertion fail during inlining due to type-mismatch).  Furthermore,
                 // when linking M1 into M0 (M0 : dstModule, M1 : srcModule), the final type is the type
@@ -1484,6 +1545,10 @@ bool TranslateBuildSPMD(
                     oclContext.m_floatDenormMode16 = FLOAT_DENORM_FLUSH_TO_ZERO;
                     oclContext.m_floatDenormMode32 = FLOAT_DENORM_FLUSH_TO_ZERO;
                 }
+                if (modMD->compOpt.BFTFDenormsAreZero)
+                {
+                    oclContext.m_floatDenormModeBFTF = FLOAT_DENORM_FLUSH_TO_ZERO;
+                }
                 if (IGC_GET_FLAG_VALUE(ForceFastestSIMD))
                 {
                     oclContext.m_retryManager.AdvanceState();
@@ -1546,6 +1611,10 @@ bool TranslateBuildSPMD(
                         oclContext.m_retryManager.kernelSet.find(pFunc->getName().str()) == oclContext.m_retryManager.kernelSet.end())
                     {
                         pFunc->eraseFromParent();
+                        // TODO: Consider running a proper cleanup of
+                        // !opencl.kernels metadata entries here instead of
+                        // deferring 'null' entries to the "retried"
+                        // unification phase.
                     }
                 }
             }
@@ -1595,7 +1664,7 @@ bool TranslateBuildSPMD(
     }
     else
     {
-        // ze binary foramt
+        // ze binary format
         llvm::SmallVector<char, 64> buf;
         llvm::raw_svector_ostream llvm_os(buf);
         const bool excludeIRFromZEBinary = IGC_IS_FLAG_ENABLED(ExcludeIRFromZEBinary) || oclContext.getModuleMetaData()->compOpt.ExcludeIRFromZEBinary;
@@ -1623,6 +1692,9 @@ bool TranslateBuildSPMD(
     if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
         dumpOCLProgramBinary(oclContext, binaryOutput, binarySize);
 
+    if (const char *progbinCustomFN = IGC_GET_REGKEYSTRING(ProgbinDumpFileName))
+      dumpOCLProgramBinary(progbinCustomFN, binaryOutput, binarySize);
+
     if (IGC_IS_FLAG_ENABLED(ShaderOverride))
         overrideOCLProgramBinary(oclContext, binaryOutput, binarySize);
 
@@ -1643,6 +1715,7 @@ bool TranslateBuildSPMD(
 
     COMPILER_TIME_END(&oclContext, TIME_TOTAL);
 
+    COMPILER_TIME_PER_PASS_PRINT(&oclContext, ShaderType::OPENCL_SHADER, oclContext.hash);
     COMPILER_TIME_PRINT(&oclContext, ShaderType::OPENCL_SHADER, oclContext.hash);
 
     COMPILER_TIME_DEL(&oclContext, m_compilerTimeStats);
@@ -1711,7 +1784,6 @@ bool TranslateBuild(
 
     // set g_CurrentShaderHash in igc_regkeys.cpp
     SetCurrentDebugHash(inputShHash);
-
     // on wrong spec constants, vc::translateBuild may fail
     // so lets dump those early
     if (pInputArgs->SpecConstantsSize > 0 && IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
@@ -1812,9 +1884,7 @@ static constexpr STB_TranslationCode g_cICBETranslationCodes[] =
 TRANSLATION_BLOCK_API void Register(STB_RegisterArgs* pRegisterArgs)
 {
     pRegisterArgs->Version = TC::STB_VERSION;
-    pRegisterArgs->NumTranslationCodes =
-        sizeof(g_cICBETranslationCodes) /
-        sizeof(g_cICBETranslationCodes[0]);
+    pRegisterArgs->NumTranslationCodes = std::size(g_cICBETranslationCodes);
 
     if (pRegisterArgs->pTranslationCodes)
     {
