@@ -86,6 +86,9 @@ CIF_DEFINE_INTERFACE_VER_WITH_COMPATIBILITY(IgcFeaturesAndWorkarounds, 2, 1) {
   virtual void SetMaxOCLParamSize(uint32_t s);
   virtual uint32_t GetMaxOCLParamSize() const;
 };
+CIF_DEFINE_INTERFACE_VER_WITH_COMPATIBILITY(IgcFeaturesAndWorkarounds, 3, 2) {
+    CIF_INHERIT_CONSTRUCTOR();
+};
 
 
 CIF_GENERATE_VERSIONS_LIST(IgcFeaturesAndWorkarounds);
