@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2022 Intel Corporation
+Copyright (C) 2022-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -12,6 +12,8 @@ SPDX-License-Identifier: MIT
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/IR/Type.h"
+#include "llvmWrapper/IR/Function.h"
+#include "llvmWrapper/IR/Attributes.h"
 #include "llvmWrapper/Support/Alignment.h"
 #include "llvmWrapper/Transforms/Utils/Cloning.h"
 #include <llvm/IR/Module.h>
@@ -500,6 +502,56 @@ Value* PromoteBools::getOrCreatePromotedValue(Value* value)
     return newValue;
 }
 
+void PromoteBools::setPromotedAttributes(Function* newFunction, AttributeList& attributeList)
+{
+#if LLVM_VERSION_MAJOR >= 12
+    auto getPromoted = [this, &newFunction](llvm::Attribute attr)
+        {
+            if (attr.isTypeAttribute())
+            {
+                return attr.getWithNewType(newFunction->getContext(),
+                    getOrCreatePromotedType(attr.getValueAsType()));
+            }
+            else
+            {
+                return attr;
+            }
+        };
+
+    // set function attributes
+    AttrBuilder attrBuilder(newFunction->getContext());
+    for (const auto& attr : IGCLLVM::getFnAttrs(attributeList))
+    {
+        attrBuilder.addAttribute(getPromoted(attr));
+    }
+    IGCLLVM::addFnAttrs(newFunction, attrBuilder);
+
+    // set return attributes
+    attrBuilder.clear();
+    for (const auto &attr : IGCLLVM::getRetAttrs(attributeList))
+    {
+        attrBuilder.addAttribute(getPromoted(attr));
+    }
+    IGCLLVM::addRetAttrs(newFunction, attrBuilder);
+
+    // set params' attributes
+    for (size_t i = 0; i < newFunction->arg_size(); i++)
+    {
+        if (!attributeList.hasParamAttrs(i))
+        {
+            continue;
+        }
+
+        attrBuilder.clear();
+        for (const auto& attr : IGCLLVM::getParamAttrs(attributeList, i))
+        {
+            attrBuilder.addAttribute(getPromoted(attr));
+        }
+        newFunction->addParamAttrs(i, attrBuilder);
+    }
+#endif // LLVM_VERSION_MAJOR >= 12
+}
+
 Function* PromoteBools::promoteFunction(Function* function)
 {
     if (!function
@@ -516,8 +568,11 @@ Function* PromoteBools::promoteFunction(Function* function)
         function->getName() + ".promoted",
         function->getParent()
     );
+
     newFunction->setCallingConv(function->getCallingConv());
-    newFunction->setAttributes(function->getAttributes());
+
+    AttributeList attributeList = function->getAttributes();
+    setPromotedAttributes(newFunction, attributeList);
 
     // Clone and fix function body
     if (!function->isDeclaration())
@@ -532,9 +587,17 @@ Function* PromoteBools::promoteFunction(Function* function)
             newFunctionArgIt->setName(functionArgIt->getName());
             argsMap[&*functionArgIt++] = newFunctionArgIt++;
         }
+
+        // Create a copy of the promoted attributes list so that
+        // we can reset it after CloneFunctionInto
+        AttributeList newAttributeList = newFunction->getAttributes();
+
         IGCLLVM::CloneFunctionInto(newFunction, function, argsMap,
             IGCLLVM::CloneFunctionChangeType::GlobalChanges, returns);
 
+        // CloneFunctionInto set the potentially unpromoted attrs again.
+        newFunction->setAttributes(newAttributeList);
+
         // Fix body
         for (auto& arg : function->args())
         {
@@ -595,7 +658,7 @@ GlobalVariable* PromoteBools::promoteGlobalVariable(GlobalVariable* globalVariab
 
     auto newGlobalVariable = new GlobalVariable(
         *globalVariable->getParent(),
-        getOrCreatePromotedType(IGCLLVM::getNonOpaquePtrEltTy(globalVariable->getType())),
+        getOrCreatePromotedType(globalVariable->getValueType()),
         globalVariable->isConstant(),
         globalVariable->getLinkage(),
         promoteConstant(globalVariable->getInitializer()),
@@ -850,7 +913,7 @@ CallInst* PromoteBools::promoteCall(CallInst* call)
     {
         if (wasPromoted(newFunction) || typeNeedsPromotion(functionType))
         {
-            newFunction = dyn_cast<Function>(getOrCreatePromotedValue(function));
+            newFunction = cast<Function>(getOrCreatePromotedValue(function));
         }
 
         for (auto& arg : call->args())
@@ -1069,9 +1132,12 @@ StoreInst* PromoteBools::promoteStore(StoreInst* store)
         return store;
     }
 
+    auto promotedSrc = convertI1ToI8(getOrCreatePromotedValue(src), store);
+    auto promotedDst = convertI1ToI8(getOrCreatePromotedValue(dst), store);
+
     auto newStore = new StoreInst(
-        getOrCreatePromotedValue(src),
-        getOrCreatePromotedValue(dst),
+        promotedSrc,
+        promotedDst,
         store->isVolatile(),
         store
     );
