@@ -115,7 +115,7 @@ namespace IGC
     class CEncoder
     {
     public:
-        void InitEncoder(bool canAbortOnSpill, bool hasStackCall, bool hasInlineAsmCall, bool hasAdditionalVisaAsmToLink, int numThreadsPerEU, VISAKernel* prevKernel);
+        void InitEncoder(bool canAbortOnSpill, bool hasStackCall, bool hasInlineAsmCall, bool hasAdditionalVisaAsmToLink, int numThreadsPerEU, uint lowerBoundGRF, VISAKernel* prevKernel);
         void InitBuildParams(llvm::SmallVector<std::unique_ptr< char, std::function<void(char*)>>, 10> & params);
         void InitVISABuilderOptions(TARGET_PLATFORM VISAPlatform, bool canAbortOnSpill, bool hasStackCall, bool enableVISA_IR);
         SEncoderState CopyEncoderState();
@@ -140,7 +140,6 @@ namespace IGC
         std::string GetShaderName();
 
         CEncoder();
-        ~CEncoder();
         void SetProgram(CShader* program);
         void Jump(CVariable* flag, uint label);
         void AddLabel(uint label);
@@ -170,6 +169,8 @@ namespace IGC
         void Send(CVariable* dst, CVariable* src, uint exDesc, CVariable* messDescriptor, bool isSendc = false);
         void Send(CVariable* dst, CVariable* src, uint ffid, CVariable* exDesc, CVariable* messDescriptor, bool isSendc = false);
         void Sends(CVariable* dst, CVariable* src0, CVariable* src1, uint ffid, CVariable* exDesc, CVariable* messDescriptor, bool isSendc = false, bool hasEOT = false);
+
+
         void RenderTargetWrite(CVariable* var[],
             bool isUndefined[],
             bool lastRenderTarget,
@@ -191,6 +192,7 @@ namespace IGC
             uint writeMask,
             CVariable* offset,
             const ResourceDescriptor& bindingTableIndex,
+            const ResourceDescriptor& pairedesource,
             const SamplerDescriptor& SamplerIdx,
             uint numSources,
             CVariable* dst,
@@ -204,6 +206,7 @@ namespace IGC
             uint writeMask,
             CVariable* offset,
             const ResourceDescriptor& resource,
+            const ResourceDescriptor& pairedesource,
             uint numSources,
             CVariable* dst,
             llvm::SmallVector<CVariable*, 4> & payload,
@@ -216,6 +219,7 @@ namespace IGC
             EOPCODE subOpcode,
             CVariable* offset,
             const ResourceDescriptor& resource,
+            const ResourceDescriptor& pairedesource,
             const SamplerDescriptor& sampler,
             uint numSources,
             CVariable* dst,
@@ -243,7 +247,7 @@ namespace IGC
         void MediaBlockMessage(ISA_Opcode subOpcode,
             CVariable* dst,
             e_predefSurface surfaceType,
-            CVariable* bufId,
+            CVariable* buf,
             CVariable* xOffset,
             CVariable* yOffset,
             uint modifier,
@@ -285,23 +289,34 @@ namespace IGC
             ResourceDescriptor * resource,
             LSC_ADDR_SIZE addrSize, int addrImmOffset,
             LSC_CACHE_OPTS cacheOpts);
-        void LSC_AtomicRaw(
-            AtomicOp atomic_op, CVariable * dst, CVariable * offset,
-            CVariable * src0, CVariable * src1,
-            unsigned short bitwidth, ResourceDescriptor * resource,
-            LSC_ADDR_SIZE addr_size,
-            int immOff, LSC_CACHE_OPTS cacheOpts);
+        void LSC_AtomicRaw(AtomicOp atomic_op, CVariable *dst,
+                           CVariable *offset, CVariable *src0, CVariable *src1,
+                           unsigned short bitwidth,
+                           ResourceDescriptor *resource,
+                           LSC_ADDR_SIZE addr_size, int immOff, int immScale,
+                           LSC_CACHE_OPTS cacheOpts);
         void LSC_Fence(LSC_SFID sfid, LSC_SCOPE scope, LSC_FENCE_OP op);
         void LSC_2DBlockMessage(
             LSC_OP subOp, ResourceDescriptor* resource,
             CVariable* dst, CVariable* bufId,
             CVariable* xOffset, CVariable* yOffset,
-            unsigned char blockWidth,
-            unsigned char blockHeight,
+            unsigned blockWidth,
+            unsigned blockHeight,
             unsigned elemSize, unsigned numBlocks,
             bool isTranspose, bool isVnni,
             CVariable* flatImageBaseoffset, CVariable* flatImageWidth,
-            CVariable* flatImageHeight, CVariable* flatImagePitch);
+            CVariable* flatImageHeight, CVariable* flatImagePitch,
+            LSC_CACHE_OPTS cacheOpts = { LSC_CACHING_DEFAULT, LSC_CACHING_DEFAULT });
+        void LSC_2DBlockMessage(
+            LSC_OP subOp,
+            CVariable* Dst,
+            CVariable* AddrPayload,
+            CVariable* Src,
+            uint32_t ImmX, uint32_t ImmY,
+            uint32_t elemSize,
+            uint32_t blockWidth, uint32_t blockHeight, uint32_t numBlocks,
+            bool isTranspose, bool isVnni,
+            LSC_CACHE_OPTS cacheOpts = { LSC_CACHING_DEFAULT, LSC_CACHING_DEFAULT });
         void NamedBarrier(e_barrierKind BarrierKind, CVariable* src0, CVariable* src1);
         void LSC_TypedReadWrite(
             LSC_OP subOp, ResourceDescriptor* resource,
@@ -312,6 +327,28 @@ namespace IGC
             LSC_ADDR_SIZE addr_size, int chMask,
             LSC_CACHE_OPTS cacheOpts = { LSC_CACHING_DEFAULT, LSC_CACHING_DEFAULT });
 
+
+        void LSC_TypedAtomic(
+            AtomicOp subOp, ResourceDescriptor* resource,
+            CVariable* pU, CVariable* pV, CVariable* pR,
+            CVariable* pSrc0, CVariable* pSrc1, CVariable* pSrcDst,
+            unsigned elemSize, LSC_ADDR_SIZE addr_size);
+
+
+        void LSC_Typed2dBlock(
+            LSC_OP subOpcode,
+            CVariable* dst,
+            e_predefSurface surfaceType,
+            CVariable* buf,
+            CVariable* xOffset,
+            CVariable* yOffset,
+            int blockWidth,
+            int blockHeight);
+        void LSC_UntypedAppendCounterAtomic(
+            LSC_OP lscOp,
+            ResourceDescriptor* resource,
+            CVariable* dst,
+            CVariable* src0);
         void AppendBreakpoint();
         void ScatterA64(CVariable* val, CVariable* offset, unsigned elementSize, unsigned numElems);
         void ByteGather(CVariable* dst, const ResourceDescriptor& resource, CVariable* offset, unsigned elementSize, unsigned numElems);
@@ -523,9 +560,10 @@ namespace IGC
         VISA_VectorOpnd* GetSourceOperandNoModifier(CVariable* var);
         VISA_VectorOpnd* GetDestinationOperand(CVariable* var, const SModifier& mod);
         VISA_RawOpnd* GetRawSource(CVariable* var, uint offset = 0);
+        VISA_RawOpnd* GetPairedResourceOperand(const ResourceDescriptor& pairedResource);
         VISA_RawOpnd* GetRawDestination(CVariable* var, unsigned offset = 0);
         VISA_PredOpnd* GetFlagOperand(const SFlag& flag);
-        VISA_StateOpndHandle* GetVISASurfaceOpnd(e_predefSurface surfaceType, CVariable* bti);
+        VISA_StateOpndHandle* GetVISASurfaceOpnd(e_predefSurface surfaceType, CVariable* var);
         VISA_StateOpndHandle* GetVISASurfaceOpnd(const ResourceDescriptor& resource);
         VISA_LabelOpnd* GetOrCreateLabel(uint label, VISA_Label_Kind kind = LABEL_BLOCK);
         VISA_LabelOpnd* GetFuncLabel(llvm::Function* F);
@@ -698,6 +736,8 @@ namespace IGC
         void SaveOption(vISAOptions option, const char* val);
         void SetBuilderOptions(VISABuilder* pbuilder);
 
+        unsigned int GetSpillThreshold(SIMDMode simdmode);
+
     private:
         // helper functions for compile flow
         /// shaderOverrideVISAFirstPass - pre-process shader overide dir for visa shader override.
@@ -721,7 +761,8 @@ namespace IGC
         /// create symbol tables and GlobalHostAccessTable according to if it's zebin
         /// or patch-token formats
         void createSymbolAndGlobalHostAccessTables(bool hasSymbolTable,
-                                                   VISAKernel &pMainKernel);
+                                                   VISAKernel &pMainKernel,
+                                                   unsigned int scratchOffset);
 
         /// create relocation according to if it's zebin or patch-token formats
         void createRelocationTables(VISAKernel &pMainKernel);
@@ -781,6 +822,7 @@ namespace IGC
         VISAKernel* vKernelTmp;
         bool m_hasPrevKernel = false;
         unsigned int m_payloadEnd = 0;
+        unsigned int m_argumentStackSize = 0;
 
         bool m_isCodePatchCandidate = false;
 
@@ -837,7 +879,8 @@ namespace IGC
         }
         void SetFunctionMaxArgumentStackSize(llvm::Function* F, unsigned size) {
             if (funcAttributeMap.find(F) != funcAttributeMap.end())
-                funcAttributeMap[F].argumentStackSize = MAX(funcAttributeMap[F].argumentStackSize, size);
+                m_argumentStackSize = funcAttributeMap[F].argumentStackSize = MAX(funcAttributeMap[F].argumentStackSize, size);
+
         }
         void SetFunctionAllocaStackSize(llvm::Function* F, unsigned size) {
             if (funcAttributeMap.find(F) != funcAttributeMap.end())
@@ -1068,11 +1111,13 @@ namespace IGC
         const ResourceDescriptor& resource, uint numSources, CVariable* dst,
         llvm::SmallVector<CVariable*, 4> & payload, bool feedbackEnable)
     {
+        ResourceDescriptor pairedResource{};
         Load(
             subOpcode,
             writeMask,
             offset,
             resource,
+            pairedResource,
             numSources,
             dst,
             payload,
