@@ -1,7 +1,7 @@
 
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -194,7 +194,7 @@ void CheckInstrTypes::checkGlobalLocal(llvm::Instruction& I)
     BasicBlock* dBB = I.getParent();
 
     for (auto U : I.users()) {
-        auto UI = dyn_cast<Instruction>(U);
+        auto UI = cast<Instruction>(U);
         BasicBlock* uBB = UI->getParent();
         if (uBB != dBB)
         {
@@ -298,6 +298,11 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
         case GenISAIntrinsic::GenISA_floatatomicraw:
         case GenISAIntrinsic::GenISA_floatatomicrawA64:
         case GenISAIntrinsic::GenISA_floatatomicstructured:
+        case GenISAIntrinsic::GenISA_floatatomictyped:
+        case GenISAIntrinsic::GenISA_fcmpxchgatomictyped:
+        case GenISAIntrinsic::GenISA_LSCAtomicFP64:
+        case GenISAIntrinsic::GenISA_LSCAtomicFP32:
+        case GenISAIntrinsic::GenISA_LSCAtomicInts:
             g_InstrTypes.hasAtomics = true;
             g_InstrTypes.numAtomics++;
             break;
@@ -589,6 +594,44 @@ void CheckInstrTypes::visitSelectInst(SelectInst& I)
     g_InstrTypes.hasSel = true;
 }
 
+void CheckInstrTypes::visitIntrinsicInst(IntrinsicInst& I)
+{
+    auto isGenericPtr = [](Value* V) {
+        IGC_ASSERT(V->getType()->isPointerTy());
+        PointerType* PTy = cast<PointerType>(V->getType());
+        return PTy->getAddressSpace() == ADDRESS_SPACE_GENERIC;
+    };
+
+    switch (I.getIntrinsicID())
+    {
+        case Intrinsic::memcpy:
+        case Intrinsic::memmove:
+        {
+            auto MT = dyn_cast<MemTransferInst>(&I);
+            Value* Dst = MT->getRawDest();
+            Value* Src = MT->getRawSource();
+            if (isGenericPtr(Dst) || isGenericPtr(Src))
+            {
+                g_InstrTypes.hasGenericAddressSpacePointers = true;
+                g_InstrTypes.hasDynamicGenericLoadStore = true;
+            }
+            break;
+        }
+        case Intrinsic::memset:
+        {
+            auto MS = dyn_cast<MemSetInst>(&I);
+            Value* Dst = MS->getRawDest();
+            if (isGenericPtr(Dst))
+            {
+                g_InstrTypes.hasGenericAddressSpacePointers = true;
+                g_InstrTypes.hasDynamicGenericLoadStore = true;
+            }
+            break;
+        }
+        default: break;
+    }
+}
+
 void CheckInstrTypes::visitGetElementPtrInst(llvm::GetElementPtrInst& I)
 {
     g_InstrTypes.numInsts++;
