@@ -205,6 +205,9 @@ namespace IGC
         /// support predicate add pattern match
         virtual bool SupportMatchPredAdd() const { return false; }
 
+        /// Support passing globally accessed pointers implicitly to callees using argument registers
+        virtual bool SupportGlobalStackArgs() const { return false; }
+
         /// Adjust adapter to adjust the loop unrolling threshold
         virtual unsigned int GetLoopUnrollThreshold() const
         {
@@ -313,7 +316,6 @@ namespace IGC
         virtual bool UseScratchSpaceForATSPlus() const { return false; }
         /// Enables HWGenerateThreadID from API level. To help debug, we must enable it from both API level AND IGC Core level.
         virtual bool SupportHWGenerateTID() const { return false; }
-        virtual bool supportHWDispatchWalkOrder() const { return false; }
         // Enables the use of simple push constants when on platforms with local (device) memory
         virtual bool supportsSimplePushForLocalMem() const { return false; }
         // disable dual8 with discard
@@ -323,12 +325,6 @@ namespace IGC
         // If enabled, IGC must provide the corresponding UMD info on how much
         // memory to allocate for the RTGlobals + global root signature.
         virtual bool supportsExpandedRTGlobals() const { return false; }
-        // If enabled, UMD must support setting up threadgroup according to
-        // RayTracingCustomTileXDim* and RayTracingCustomTileYDim*. If you want
-        // to experiment with non-power-of-2 x dimensions, you also have to
-        // support filling local IDs in the indirect state for the shader to
-        // read (not required otherwise).
-        virtual bool supportsRaytracingTiling() const { return false; }
         // Enables the use of scratch space in raytracing shaders when possible
         virtual bool supportsRTScratchSpace() const { return false; }
         // Enables Raytracing printf
@@ -373,6 +369,16 @@ namespace IGC
         // If enabled IGC must not hoist convergent instructions.
         virtual bool DisableConvergentInstructionsHoisting() const { return false; }
 
+        // Each API can define its own preferred values for the spill threshold
+        virtual unsigned getSIMD8_SpillThreshold() const    { return IGC_GET_FLAG_VALUE(SIMD8_SpillThreshold);    }
+        virtual unsigned getSIMD16_SpillThreshold() const   { return IGC_GET_FLAG_VALUE(SIMD16_SpillThreshold);   }
+        virtual unsigned getSIMD32_SpillThreshold() const   { return IGC_GET_FLAG_VALUE(SIMD32_SpillThreshold);   }
+        virtual unsigned getCSSIMD16_SpillThreshold() const { return IGC_GET_FLAG_VALUE(CSSIMD16_SpillThreshold); }
+        virtual unsigned getCSSIMD32_SpillThreshold() const { return IGC_GET_FLAG_VALUE(CSSIMD32_SpillThreshold); }
+
+        // Informs if it supports to use a logical SSID from msg0.0 instead of a physical ID from sr0
+        virtual bool supportsLogicalSSIDInHWTID() const { return false; }
+
 protected:
     bool autoGRFSelection = false;
     };
