@@ -148,17 +148,31 @@ public:
     void emitReturn(llvm::ReturnInst* inst);
     void EmitInsertValueToStruct(llvm::InsertValueInst* II);
     void EmitExtractValueFromStruct(llvm::ExtractValueInst* EI);
-    void EmitInsertValueToLayoutStruct(llvm::InsertValueInst* II);
+    void EmitInsertValueToLayoutStruct(llvm::InsertValueInst* IVI);
+    void EmitExtractValueFromLayoutStruct(llvm::ExtractValueInst* EVI);
     void emitVectorCopyToAOS(uint32_t AOSBytes,
         CVariable* Dst, CVariable* Src, uint32_t nElts,
-        uint32_t DstSubRegOffset = 0, uint32_t SrcSubRegOffset = 0);
-    void emitCopyFromLayoutStruct(llvm::Value* D, llvm::Value* S);
+        uint32_t DstSubRegOffset = 0, uint32_t SrcSubRegOffset = 0) {
+        emitVectorCopyToOrFromAOS(AOSBytes, Dst, Src, nElts,
+            DstSubRegOffset, SrcSubRegOffset, true);
+    }
+    void emitVectorCopyFromAOS(uint32_t AOSBytes,
+        CVariable* Dst, CVariable* Src, uint32_t nElts,
+        uint32_t DstSubRegOffset = 0, uint32_t SrcSubRegOffset = 0) {
+        emitVectorCopyToOrFromAOS(AOSBytes, Dst, Src, nElts,
+            DstSubRegOffset, SrcSubRegOffset, false);
+    }
+    void emitVectorCopyToOrFromAOS(uint32_t AOSBytes,
+        CVariable* Dst, CVariable* Src, uint32_t nElts,
+        uint32_t DstSubRegOffset, uint32_t SrcSubRegOffset,
+        bool IsToAOS);
+    void emitCopyToOrFromLayoutStruct(llvm::Value* D, llvm::Value* S);
 
     /// stack-call code-gen functions
     void emitStackCall(llvm::CallInst* inst);
     void emitStackFuncEntry(llvm::Function* F);
     void emitStackFuncExit(llvm::ReturnInst* inst);
-    void InitializeKernelStack(llvm::Function* pKernel);
+    void InitializeKernelStack(llvm::Function* pKernel, CVariable* stackBufferBase = nullptr);
 
     /// stack-call functions for reading and writing argument/retval data to stack
     typedef SmallVector<std::tuple<CVariable*, uint32_t, uint32_t, uint32_t, bool>, 8> StackDataBlocks;
@@ -250,6 +264,10 @@ public:
     void emitThreadGroupBarrier(llvm::Instruction* inst);
     void emitThreadGroupNamedBarriersInit(llvm::Instruction* inst);
     void emitThreadGroupNamedBarriersBarrier(llvm::Instruction* inst);
+    void emitLSCTypedRead(llvm::Instruction* inst);
+    void emitLSCTypedWrite(llvm::Instruction* inst);
+    void emitLSCAtomicTyped(llvm::GenIntrinsicInst* inst);
+    void emitLscUniformAtomicCounter(llvm::GenIntrinsicInst* pInst);
     void emitCastSelect(CVariable* flag, CVariable* dst, CVariable* src0, CVariable* src1);
     void emitMemoryFence(llvm::Instruction* inst);
     void emitMemoryFence(void);
@@ -257,34 +275,24 @@ public:
     void emitFlushSamplerCache();
     void emitSurfaceInfo(llvm::GenIntrinsicInst* intrinsic);
 
-    static uint64_t getFPOffset() { return SIZE_OWORD; }
     void emitStackAlloca(llvm::GenIntrinsicInst* intrinsic);
     void emitVLAStackAlloca(llvm::GenIntrinsicInst* intrinsic);
 
     void emitUAVSerialize();
 
-    void emitScalarAtomics(
-        llvm::Instruction* pInst,
-        ResourceDescriptor& resource,
-        AtomicOp atomic_op,
-        CVariable* pDstAddr,
-        CVariable* pU,
-        CVariable* pV,
-        CVariable* pR,
-        CVariable* pSrc,
-        bool isA64,
-        int bitSize);
+    void emitScalarAtomics(llvm::Instruction *pInst,
+                           ResourceDescriptor &resource, AtomicOp atomic_op,
+                           CVariable *pDstAddr, CVariable *pU, CVariable *pV,
+                           CVariable *pR, CVariable *pSrc, bool isA64,
+                           int bitSize, int immOffset, int immScale,
+                           LSC_ADDR_SIZE addrSize);
 
-    void emitScalarAtomicLoad(
-        llvm::Instruction* pInst,
-        ResourceDescriptor& resource,
-        CVariable* pDstAddr,
-        CVariable* pU,
-        CVariable* pV,
-        CVariable* pR,
-        CVariable* pSrc,
-        bool isA64,
-        int bitSize);
+    void emitScalarAtomicLoad(llvm::Instruction *pInst,
+                              ResourceDescriptor &resource,
+                              CVariable *pDstAddr, CVariable *pU, CVariable *pV,
+                              CVariable *pR, CVariable *pSrc, bool isA64,
+                              int bitWidth, int immOffset, int immScale,
+                              LSC_ADDR_SIZE addrSize);
 
     /// wave/subgroup support
     /// reduction and prefix/postfix facilities
@@ -338,14 +346,19 @@ public:
         bool isPrefix);
 
     bool IsUniformAtomic(llvm::Instruction* pInst);
-    void emitAtomicRaw(llvm::GenIntrinsicInst* pInst);
+    void emitAtomicRaw(llvm::GenIntrinsicInst *pInst, Value *varOffset,
+                       ConstantInt *immOffset = nullptr,
+                       ConstantInt *immScale = nullptr
+    );
     void emitAtomicTyped(llvm::GenIntrinsicInst* pInst);
     void emitAtomicCounter(llvm::GenIntrinsicInst* pInst);
     void emitFastClear(llvm::LoadInst* inst);
     void emitFastClearSend(llvm::Instruction* pInst);
     void setRovCacheCtrl(llvm::GenIntrinsicInst* inst);
     llvm::Optional<LSC_CACHE_OPTS>
-        setCacheOptionsForConstantBufferLoads(Instruction& inst) const;
+        cacheOptionsForConstantBufferLoads(Instruction* inst, LSC_L1_L3_CC Ctrl) const;
+    llvm::Optional<LSC_CACHE_OPTS>
+        cacheOptionsForConstantBufferLoads(Instruction* inst) const;
     bool useRasterizerOrderedByteAddressBuffer(llvm::GenIntrinsicInst* inst);
     void emitUniformAtomicCounter(llvm::GenIntrinsicInst* pInst);
 
@@ -357,6 +370,7 @@ public:
     void emitExtract(llvm::Instruction* inst);
     void emitBitCast(llvm::BitCastInst* btCst);
     void emitBitcastfromstruct(llvm::GenIntrinsicInst* BCFromStruct);
+    void emitBitcasttostruct(llvm::GenIntrinsicInst* BCToStruct);
     void emitPtrToInt(llvm::PtrToIntInst* p2iCst);
     void emitIntToPtr(llvm::IntToPtrInst* i2pCst);
     void emitAddrSpaceCast(llvm::AddrSpaceCastInst* addrSpaceCast);
@@ -367,6 +381,7 @@ public:
     CVariable* GetVMaskPred(CVariable*& predicate);
     void createVMaskPred(CVariable*& predicate);
     void UseVMaskPred();
+    CVariable* GetCombinedVMaskPred(CVariable* basePredicate = nullptr);
     CVariable* m_vMaskPredForSubplane = nullptr;
 
     void emitGradientX(const SSource& source, const DstModifier& modifier);
@@ -445,7 +460,7 @@ public:
         uint32_t DstSubRegOffset = 0, uint32_t SrcSubRegOffset = 0);
     void emitCopyAll(CVariable* Dst, CVariable* Src, llvm::Type* Ty);
 
-    void emitPushFrameToStack(unsigned& pushSize);
+    void emitPushFrameToStack(Function* ParentFunction, unsigned& pushSize);
     // emitMul64 - emulate 64bit multiply by 32-bit operations.
     // Dst must be a 64-bit type variable.
     // Src0 and Src1 must be in 32-bit type variable/immediate
@@ -478,23 +493,42 @@ public:
     LSC_CACHE_OPTS getDefaultRaytracingCachePolicy(bool isLoad) const;
     void emitAsyncStackID(llvm::GenIntrinsicInst* I);
     void emitTraceRay(llvm::TraceRayIntrinsic *I, bool RayQueryEnable);
+
+
     void emitReadTraceRaySync(llvm::GenIntrinsicInst* I);
 
+    void emitRayQueryCheckRelease(
+        llvm::GenIntrinsicInst* I,
+        bool RayQueryCheckEnable = false,
+        bool RayQueryReleaseEnable = false
+    );
+
+    void emitPreemptionDisable(llvm::PreemptionDisableIntrinsic* PDI);
+    void emitPreemptionEnable(llvm::PreemptionEnableIntrinsic* PEI);
+
+    enum PreemptionEncoding
+    {
+        PreemptionDisabled = (0 << 14),
+        PreemptionEnabled  = (1 << 14)
+    };
+
+    static PreemptionEncoding getEncoderPreemptionMode(EPreemptionMode preemptionMode);
 
     void emitBTD(
         CVariable* GlobalBufferPtr,
         CVariable* StackID,
         CVariable* ShaderRecord,
         CVariable* Flag,
-        bool releaseStackID);
+        bool releaseStackID
+        );
+
+
     void emitBindlessThreadDispatch(llvm::BTDIntrinsic *I);
     void emitStackIDRelease(llvm::StackIDReleaseIntrinsic *I);
     void emitGetShaderRecordPtr(llvm::GetShaderRecordPtrIntrinsic *I);
     void emitGlobalBufferPtr(llvm::GenIntrinsicInst *I);
     void emitLocalBufferPtr(llvm::GenIntrinsicInst *I);
     void emitInlinedDataValue(llvm::GenIntrinsicInst *I);
-    void emitTileXOffset(llvm::TileXIntrinsic *I);
-    void emitTileYOffset(llvm::TileYIntrinsic *I);
     void emitDpas(llvm::GenIntrinsicInst *GII,
                   const SSource* source,
                   const DstModifier& modifier);
@@ -505,6 +539,8 @@ public:
     void emitHDCuncompressedwrite(llvm::GenIntrinsicInst* I);
     ////////////////////////////////////////////////////////////////////
     // LSC related functions
+    bool tryOverrideCacheOpts(LSC_CACHE_OPTS& cacheOpts, bool isLoad, bool isTGM, const llvm::Value* warningContextValue) const;
+    LSC_CACHE_OPTS translateLSCCacheControlsEnum(LSC_L1_L3_CC l1l3cc, bool isLoad, const llvm::Value* warningContextValue) const;
     LSC_CACHE_OPTS translateLSCCacheControlsFromValue(
         llvm::Value *value, bool isLoad) const;
     LSC_CACHE_OPTS translateLSCCacheControlsFromMetadata(
@@ -532,10 +568,17 @@ public:
 
     void emitLscIntrinsicLoad(llvm::GenIntrinsicInst* GII);
     void emitLscIntrinsicPrefetch(llvm::GenIntrinsicInst* GII);
+    void emitLscSimdBlockPrefetch(llvm::GenIntrinsicInst* GII);
     void emitLscIntrinsicStore(llvm::GenIntrinsicInst* GII);
+    void emitLscIntrinsicLoadCmask(llvm::GenIntrinsicInst* inst);
+    void emitLscIntrinsicStoreCmask(llvm::GenIntrinsicInst* GII);
 
     void emitLSCFence(llvm::GenIntrinsicInst* inst);
     void emitLSC2DBlockOperation(llvm::GenIntrinsicInst* inst);
+    void emitLSC2DBlockAddrPayload(llvm::GenIntrinsicInst* GII);
+    void emitLSC2DBlockReadWriteWithAddrPayload(llvm::GenIntrinsicInst* GII);
+    void emitLSC2DBlockSetAddrPayloadField(llvm::GenIntrinsicInst* GII);
+
     void emitLSCAtomic(llvm::GenIntrinsicInst* inst);
     void emitLSCIntrinsic(llvm::GenIntrinsicInst* GII);
     void emitLSCLoad(
@@ -598,7 +641,7 @@ public:
     bool shouldGenerateLSC(llvm::Instruction* vectorLdStInst = nullptr, bool isTGM = false);
     bool forceCacheCtrl(llvm::Instruction* vectorLdStInst = nullptr);
     uint32_t totalBytesToStoreOrLoad(llvm::Instruction* vectorLdStInst);
-    void emitsrnd(llvm::GenIntrinsicInst* GII);
+    void emitSrnd(llvm::GenIntrinsicInst* GII);
     void emitStaticConstantPatchValue(
         llvm::StaticConstantPatchIntrinsic* staticConstantPatch32);
     // Debug Built-Ins
@@ -614,13 +657,19 @@ public:
     void emitLaunder(llvm::GenIntrinsicInst* GII);
     void emitImplicitArgIntrinsic(llvm::GenIntrinsicInst* I);
     void emitStoreImplBufferPtr(llvm::GenIntrinsicInst* I);
+    void emitSetStackCallsBaseAddress(llvm::GenIntrinsicInst* I);
+    void emitSaveInReservedArgSpace(llvm::SaveInReservedArgSpaceIntrinsic* I);
+    void emitReadFromReservedArgSpace(llvm::ReadFromReservedArgSpaceIntrinsic* I);
     void emitStoreLocalIdBufferPtr(llvm::GenIntrinsicInst* I);
+    void emitStoreGlobalBufferArg(llvm::GenIntrinsicInst* I);
     void emitLoadImplBufferPtr(llvm::GenIntrinsicInst* I);
     void emitLoadLocalIdBufferPtr(llvm::GenIntrinsicInst* I);
+    void emitLoadGlobalBufferArg(llvm::GenIntrinsicInst* I);
 
     void emitMayUnalignedVectorCopy(
         CVariable* D, uint32_t D_off, CVariable* S, uint32_t S_off, llvm::Type* Ty);
-    void emitStackOverflowDetectionCall(llvm::Function* ParentFunction);
+    Function* findStackOverflowDetectionFunction(Function* ParentFunction, bool FindInitFunction);
+    void emitStackOverflowDetectionCall(llvm::Function* ParentFunction, bool EmitInitFunction);
 
     std::pair<llvm::Value*, llvm::Value*> getPairOutput(llvm::Value*) const;
 
@@ -758,10 +807,11 @@ public:
     bool IsIndirectAccess(llvm::Value* value);
 
     CVariable* GetSrcVariable(const SSource& source, bool fromConstPool = false);
-    void SetSourceModifiers(unsigned int sourceIndex, const SSource& source);
+    void SetSourceModifiers(unsigned int sourceIndex, const SSource& source) const;
 
     SBasicBlock* getCurrentBlock() const {
-        return m_currentBlock < 0 ? nullptr : &(m_pattern->m_blocks[m_currentBlock]);
+        // if m_currentBlock is not set to initial value then return current block
+        return m_currentBlock == UINT32_MAX ? nullptr : &(m_pattern->m_blocks[m_currentBlock]);
     }
 
     CodeGenContext* m_pCtx = nullptr;
@@ -793,7 +843,7 @@ public:
     inline void ContextSwitchShaderBody(bool last = true);
 
 private:
-    uint m_labelForDMaskJmp;
+    uint m_labelForDMaskJmp = 0;
 
     llvm::DenseMap<llvm::Instruction*, bool> instrMap;
 
@@ -833,7 +883,7 @@ private:
     ERoundingMode m_roundingMode_FP;
     ERoundingMode m_roundingMode_FPCvtInt;
 
-    uint m_currentBlock = (uint) -1;
+    uint m_currentBlock = UINT32_MAX;
 
     bool m_currFuncHasSubroutine = false;
 
@@ -870,7 +920,7 @@ private:
     // computation.
     CVariable* getOrCreatePerLaneOffsetVariable(unsigned TypeSizeInBytes)
     {
-        for (auto Item : PerLaneOffsetVars)
+        for (const auto& Item : PerLaneOffsetVars)
         {
             if (Item.first == TypeSizeInBytes)
             {
@@ -914,16 +964,10 @@ private:
     int getGRFSize() const { return m_currShader->getGRFSize(); }
 
     void initDefaultRoundingMode();
-    ERoundingMode GetRoundingMode_FPCvtInt(llvm::Instruction* pInst);
-    ERoundingMode GetRoundingMode_FP(llvm::Instruction* inst);
     void SetRoundingMode_FP(ERoundingMode RM_FP);
     void SetRoundingMode_FPCvtInt(ERoundingMode RM_FPCvtInt);
-    bool setRMExplicitly(llvm::Instruction* inst);
     void ResetRoundingMode(llvm::Instruction* inst);
 
-    // returns true if the instruction does not care about the rounding mode settings
-    bool ignoreRoundingMode(llvm::Instruction* inst) const;
-
     // A64 load/store with HWA that make sure the offset hi part is the same per LS call
     // addrUnifrom: if the load/store address is uniform, we can skip A64 WA
     void emitGatherA64(llvm::Value* loadInst, CVariable* dst, CVariable* offset, unsigned elemSize, unsigned numElems, bool addrUniform);
@@ -994,6 +1038,8 @@ private:
     bool m_isDuplicate;
     CVariable *m_tmpDest = nullptr;
     std::set<CoalescingEngine::CCTuple *> lifetimeStartAdded;
+    tuple<CVariable*, CVariable*, CVariable*> addToCachedPayloadUVR(CVariable* pU, CVariable* pV, CVariable* pR);
+    std::map<tuple<Value*, Value*, Value*>, tuple<CVariable*, CVariable*, CVariable*>> atomic_shared_pUVR;
 };
 
 } // namespace IGC
