@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -49,10 +49,15 @@ namespace IGC {
         void processSCC(std::vector<llvm::CallGraphNode*>* SCCNodes);
         void setFuncProperties(llvm::CallGraph& CG);
         void copyFuncProperties(llvm::Function* To, llvm::Function* From);
+
         GenXFunctionGroupAnalysis* FGA;
         IGC::IGCMD::MetaDataUtils* pMdUtils;
         bool Modified;
         unsigned m_FunctionCloningThreshold = 0;
+
+        // Find the set of functions that cannot be promoted to indirect, must be cloned
+        void detectUnpromotableFunctions(llvm::Module* pM);
+        std::set<llvm::Function*> m_UnpromotableFuncs;
     };
 
     /// \brief A collection of functions that are reachable from a kernel.
@@ -122,6 +127,9 @@ namespace IGC {
                 delete F;
             }
         }
+        FunctionGroup() = default;
+        FunctionGroup(const FunctionGroup&) = delete;
+        FunctionGroup& operator=(const FunctionGroup&) = delete;
 
         /// \brief The entry kernel function of group.
         llvm::Function* getHead() const {
@@ -220,6 +228,8 @@ namespace IGC {
         static char ID;
         explicit GenXFunctionGroupAnalysis();
         ~GenXFunctionGroupAnalysis() { clear(); }
+        GenXFunctionGroupAnalysis(const GenXFunctionGroupAnalysis&) = delete;
+        GenXFunctionGroupAnalysis& operator=(const GenXFunctionGroupAnalysis&) = delete;
 
         virtual llvm::StringRef getPassName() const  override { return "FunctionGroup analysis"; }
 
