@@ -138,13 +138,13 @@ bool LdShrink::runOnFunction(Function& F) {
                 = PointerType::get(ScalarTy, PtrTy->getAddressSpace());
             Value* ScalarPtr = Builder.CreatePointerCast(Ptr, ScalarPtrTy);
             if (Offset)
-                ScalarPtr = Builder.CreateInBoundsGEP(ScalarPtr, Builder.getInt32(Offset));
+                ScalarPtr = Builder.CreateInBoundsGEP(ScalarTy, ScalarPtr, Builder.getInt32(Offset));
 
             alignment_t alignment
                 = (alignment_t)MinAlign(IGCLLVM::getAlignmentValue(LI),
                     DL->getTypeStoreSize(ScalarTy) * Offset);
 
-            LoadInst* NewLoad = Builder.CreateAlignedLoad(ScalarPtr, IGCLLVM::getAlign(alignment));
+            LoadInst* NewLoad = Builder.CreateAlignedLoad(ScalarTy, ScalarPtr, IGCLLVM::getAlign(alignment));
             NewLoad->setDebugLoc(LI->getDebugLoc());
 
             ExtractElementInst* EEI = cast<ExtractElementInst>(*LI->user_begin());
