@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -17,13 +17,15 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPush.hpp"
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/IRBuilder.h"
+#include "llvm/IR/GetElementPtrTypeIterator.h"
 #include <llvm/IR/Function.h>
-#include <llvm/IR/Instructions.h>
 #include <llvm/ADT/SmallVector.h>
 #include <llvm/Transforms/Utils/Local.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "Probe/Assertion.h"
 
+#include <algorithm>
+
 #define MAX_ALLOCA_PROMOTE_GRF_NUM      48
 #define MAX_PRESSURE_GRF_NUM            90
 
@@ -174,6 +176,7 @@ bool LowerGEPForPrivMem::runOnFunction(llvm::Function& F)
     m_pRegisterPressureEstimate->buildRPMapPerInstruction();
 
     m_allocasToPrivMem.clear();
+    m_promotedLiveranges.clear();
     visit(F);
 
     std::vector<llvm::AllocaInst*>& allocaToHande = m_allocasToPrivMem;
@@ -273,6 +276,9 @@ bool LowerGEPForPrivMem::IsNativeType(Type* type)
         return false;
     }
 
+    if (type->isStructTy())
+        return false;
+
     return true;
 }
 
@@ -302,18 +308,18 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
         SubGroupSizeMetaDataHandle subGroupSize = funcInfoMD->getSubGroupSize();
         if (subGroupSize->hasValue())
         {
-            SIMDSize = std::max((uint32_t)subGroupSize->getSIMD_size(), SIMDSize);
+            SIMDSize = std::max((uint32_t)subGroupSize->getSIMDSize(), SIMDSize);
         }
 
         allowedAllocaSizeInBytes = (allowedAllocaSizeInBytes * 8) / SIMDSize;
     }
-    Type* baseType = nullptr;
-    bool allUsesAreVector = false;
-    if (!CanUseSOALayout(pAlloca, baseType, allUsesAreVector))
+    SOALayoutChecker checker(*pAlloca, m_ctx->type == ShaderType::OPENCL_SHADER);
+    SOALayoutInfo SOAInfo = checker.getOrGatherInfo();
+    if (!SOAInfo.canUseSOALayout)
     {
         return StatusPrivArr2Reg::CannotUseSOALayout;
     }
-    if (!IsNativeType(baseType))
+    if (!IsNativeType(SOAInfo.baseType))
     {
         return StatusPrivArr2Reg::IsNotNativeType;
     }
@@ -335,6 +341,15 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
         return StatusPrivArr2Reg::OutOfAllocSizeLimit;
     }
 
+    // Multiple indirect byte access could be harmful for performance
+    if (SOAInfo.baseType->getScalarType()->isIntegerTy(8) && !isUniformAlloca &&
+        m_ctx->platform.isCoreChildOf(IGFX_XE_HPG_CORE))
+    {
+        // Limit promotable alloca size with byte indirect access by 4 GRF vector size
+        if (allocaSize * SIMDSize / m_ctx->platform.getGRFSize() > 4)
+            return StatusPrivArr2Reg::IsNotNativeType;
+    }
+
     // get all the basic blocks that contain the uses of the alloca
     // then estimate how much changing this alloca to register adds to the pressure at that block.
     unsigned int lowestAssignedNumber = 0xFFFFFFFF;
@@ -351,7 +366,7 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
         // subtract impact from GEP operations related to alloca from the register pressure
         // since after promotion alloca to register these GEPs will be eliminated
         unsigned int GEPImpact = 0;
-        for (auto GEPinst : GEPliveranges)
+        for (const auto& GEPinst : GEPliveranges)
         {
             if (GEPinst.LR->contains(i))
                 GEPImpact += GEPinst.varSize;
@@ -361,7 +376,7 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
         pressure = std::max(pressure, RPinst - GEPImpact);
     }
 
-    for (auto it : m_promotedLiveranges)
+    for (const auto& it : m_promotedLiveranges)
     {
         // check interval intersection
         if ((it.lowId < lowestAssignedNumber && it.highId > lowestAssignedNumber) ||
@@ -384,112 +399,240 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
     return StatusPrivArr2Reg::OK;
 }
 
-static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA, bool& allUsesAreVector)
+SOALayoutChecker::SOALayoutChecker(AllocaInst& allocaToCheck, bool isOCL)
+    : allocaRef(allocaToCheck)
 {
-    for (Value::user_iterator use_it = I->user_begin(), use_e = I->user_end(); use_it != use_e; ++use_it)
+    auto F = allocaToCheck.getParent()->getParent();
+    pDL = &F->getParent()->getDataLayout();
+    newAlgoControl = IGC_GET_FLAG_VALUE(EnablePrivMemNewSOATranspose);
+    if (IGC_IS_FLAG_ENABLED(NewSOATransposeForOpenCL) && !isOCL) {
+        newAlgoControl = 0;
+    }
+}
+
+SOALayoutInfo SOALayoutChecker::getOrGatherInfo()
+{
+    if (pInfo)
+        return *pInfo;
+    pInfo = std::make_unique<SOALayoutInfo>(false, nullptr, false, 4);
+
+    // Do not allow SOA layout for vla which will be stored on the stack.
+    // We don't support SOA layout for privates on stack at all so this is just to make
+    // the implementation simpler.
+    if (LowerGEPForPrivMem::IsVariableSizeAlloca(allocaRef))
+        return *pInfo;
+
+    // Don't even look at non-array allocas.
+    // (extractAllocaDim can not handle them anyway, causing a crash)
+    llvm::Type* pType = allocaRef.getAllocatedType();
+    if (pType->isStructTy() && pType->getStructNumElements() == 1)
     {
-        if (GetElementPtrInst * gep = dyn_cast<GetElementPtrInst>(*use_it))
-        {
-            if (CheckUsesForSOAAlyout(gep, vectorSOA, allUsesAreVector))
-                continue;
-        }
-        if (llvm::LoadInst * pLoad = llvm::dyn_cast<llvm::LoadInst>(*use_it))
-        {
-            bool isVectorLoad = pLoad->getType()->isVectorTy();
-            vectorSOA &= isVectorLoad;
-            allUsesAreVector &= isVectorLoad;
-            if (!pLoad->isSimple())
-                return false;
-        }
-        else if (llvm::StoreInst * pStore = llvm::dyn_cast<llvm::StoreInst>(*use_it))
-        {
-            if (!pStore->isSimple())
-                return false;
-            llvm::Value* pValueOp = pStore->getValueOperand();
-            bool isVectorStore = pStore->getValueOperand()->getType()->isVectorTy();
-            vectorSOA &= isVectorStore;
-            allUsesAreVector &= isVectorStore;
-            if (pValueOp == I)
-            {
-                // GEP instruction is the stored value of the StoreInst (not supported case)
-                return false;
+        pType = pType->getStructElementType(0);
+    }
+    if ((!pType->isArrayTy() && !pType->isVectorTy()) || allocaRef.isArrayAllocation())
+        return *pInfo;
+
+    // Enable transpose for array of struct
+    pInfo->baseType = GetBaseType(pType, newAlgoControl > 0 ? true : false);
+    if (!pInfo->baseType)
+        return *pInfo;
+
+    if (useNewAlgo(pInfo->baseType)) {
+        SOAPartitionBytes = selectPartitionSize(pInfo->baseType);
+
+        StructType* STy = dyn_cast<StructType>(pInfo->baseType);
+        if (STy != nullptr) {
+            if (!isPowerOf2_32(SOAPartitionBytes) || !checkStruct(STy)) {
+                if (newAlgoControl < 3)
+                    return *pInfo;
+
+                // newAlgoControl = 3
+                // check if partition size can be the entire struct
+                uint32_t sz = (uint32_t)pDL->getTypeStoreSize(STy);
+                if (sz > 16 || !isPowerOf2_32(sz)) {
+                    return *pInfo;
+                }
+                SOAPartitionBytes = std::max(4u, sz);
             }
         }
-        else if (llvm::BitCastInst * pBitCast = llvm::dyn_cast<llvm::BitCastInst>(*use_it))
-        {
-            Type* baseT = GetBaseType(IGCLLVM::getNonOpaquePtrEltTy(pBitCast->getType()));
-            Type* sourceType = GetBaseType(IGCLLVM::getNonOpaquePtrEltTy(pBitCast->getOperand(0)->getType()));
-            if (pBitCast->use_empty())
-            {
-                continue;
-            }
-            else if (baseT != nullptr &&
-                baseT->getScalarSizeInBits() != 0 &&
-                baseT->getScalarSizeInBits() == sourceType->getScalarSizeInBits())
-            {
-                vectorSOA &= (unsigned int)baseT->getPrimitiveSizeInBits() == sourceType->getPrimitiveSizeInBits();
-                if (CheckUsesForSOAAlyout(pBitCast, vectorSOA, allUsesAreVector))
-                    continue;
-            }
-            else if (IsBitCastForLifetimeMark(pBitCast))
-            {
-                continue;
-            }
-            // Not a candidate.
+
+        // Skip for non-power-of-2 partition size
+        if (isPowerOf2_32(SOAPartitionBytes)) {
+            pInfo->canUseSOALayout = checkUsers(allocaRef);
+            pInfo->SOAPartitionBytes = SOAPartitionBytes;
+        }
+        return *pInfo;
+    }
+    // only handle case with a simple base type
+    if (!(pInfo->baseType->getScalarType()->isFloatingPointTy() ||
+        pInfo->baseType->getScalarType()->isIntegerTy()))
+        return *pInfo;
+
+    // Now that we've confirmed our alloca to be a valid candidate, assume that
+    // all memory instructions are vector unless proven otherwise.
+    pInfo->allUsesAreVector = true;
+    // Start the traversal: each specified visitor function will delegate its
+    // checked instruction to the same method if need be.
+    pInfo->canUseSOALayout = checkUsers(allocaRef);
+    if (!isVectorSOA)
+    {
+        pInfo->baseType = pInfo->baseType->getScalarType();
+    }
+    return *pInfo;
+}
+
+uint32_t SOALayoutChecker::selectPartitionSize(Type* Ty)
+{
+    uint32_t size = 4;
+    if (StructType* StTy = dyn_cast<StructType>(Ty)) {
+        int nElts = (int)StTy->getNumElements();
+        for (int ix = 0; ix < nElts; ++ix) {
+            Type* eTy = StTy->getElementType(ix);
+            uint32_t sz = selectPartitionSize(eTy);
+            size = std::max(sz, size);
+        }
+        return size;
+    }
+    if (Ty->isArrayTy()) {
+        // Don't split vector
+        uint32_t sz = selectPartitionSize(Ty->getArrayElementType());
+        size = std::max(sz, size);
+        return size;
+    }
+
+    uint32_t sz = (uint32_t)pDL->getTypeStoreSize(Ty);
+    size = std::max(sz, size);
+    return size;
+}
+
+bool SOALayoutChecker::checkStruct(StructType* StTy)
+{
+    if (!StTy->isSized())
+        return false;
+
+    uint32_t StTyBytes = (uint32_t)pDL->getTypeStoreSize(StTy);
+
+    // Larger struct shall be multiple of partition size
+    if (StTyBytes > SOAPartitionBytes &&
+        (StTyBytes % SOAPartitionBytes) != 0)
+        return false;
+    // Partition shall be multiple of smaller struct size
+    if (StTyBytes < SOAPartitionBytes &&
+        (SOAPartitionBytes % StTyBytes) != 0)
+        return false;
+
+    const StructLayout* SL = pDL->getStructLayout(StTy);
+    int32_t nElts = (int)StTy->getNumElements();
+    for (int ix = 0; ix < nElts; ++ix) {
+        Type* ty = StTy->getElementType(ix);
+        uint32_t eTyBytes = (uint32_t)pDL->getTypeStoreSize(ty);
+        IGC_ASSERT(SOAPartitionBytes >= eTyBytes);
+        if (!isPowerOf2_32(eTyBytes))
             return false;
-        }
-        else if (IntrinsicInst * intr = dyn_cast<IntrinsicInst>(*use_it))
-        {
-            llvm::Intrinsic::ID  IID = intr->getIntrinsicID();
-            if (IID == llvm::Intrinsic::lifetime_start ||
-                IID == llvm::Intrinsic::lifetime_end)
-            {
-                continue;
+
+        if (newAlgoControl == 1) {
+            // only handle struct with members being same-sized scalars
+            if (SOAPartitionBytes != eTyBytes ||
+                !ty->isSingleValueType() ||
+                ty->isVectorTy()) {
+                return false;
             }
-            return false;
         }
-        else
-        {
-            // This is some other instruction. Right now we don't want to handle these
+        // newAlgoControl=2 is handled in other places
+        else if (newAlgoControl > 2) {
+            // May handle nested struct/array, etc.
+            if (!ty->isSingleValueType()) {
+                return false;
+            }
+        }
+        uint32_t byteOffset = (uint32_t)SL->getElementOffset(ix);
+        uint32_t chunkOff = (byteOffset % SOAPartitionBytes);
+        // check alignment
+        if (MinAlign(eTyBytes, chunkOff) < eTyBytes) {
             return false;
         }
     }
     return true;
 }
 
-
-bool IGC::CanUseSOALayout(AllocaInst* I, Type*& base, bool& allUsesAreVector)
+// TODO: Consider a worklist-based implementation instead.
+bool SOALayoutChecker::checkUsers(Instruction& I)
 {
-    // Do not allow SOA layout for vla which will be stored on the stack.
-    // We don't support SOA layout for privates on stack at all so this is just to make
-    // the implementation simpler.
-    if (LowerGEPForPrivMem::IsVariableSizeAlloca(*I))
-        return false;
-
-    // Don't even look at non-array allocas.
-    // (extractAllocaDim can not handle them anyway, causing a crash)
-    llvm::Type* pType = IGCLLVM::getNonOpaquePtrEltTy(I->getType());
-    if (pType->isStructTy() && pType->getStructNumElements() == 1)
+    parentLevelInst = &I;
+    for (Value::user_iterator userIt = I.user_begin(), userE = I.user_end(); userIt != userE; ++userIt)
     {
-        pType = pType->getStructElementType(0);
+        auto& userInst = *cast<Instruction>(*userIt);
+        if (!visit(userInst))
+            return false;
     }
-    if ((!pType->isArrayTy() && !pType->isVectorTy()) || I->isArrayAllocation())
-        return false;
+    return true;
+}
 
-    base = GetBaseType(pType);
-    if (base == nullptr)
-        return false;
-    // only handle case with a simple base type
-    if (!(base->getScalarType()->isFloatingPointTy() || base->getScalarType()->isIntegerTy()))
-        return false;
-    bool vectorSOA = true;
-    allUsesAreVector = true;
-    bool useSOA = CheckUsesForSOAAlyout(I, vectorSOA, allUsesAreVector);
-    if (!vectorSOA)
+bool SOALayoutChecker::visitBitCastInst(BitCastInst& BI)
+{
+    if (BI.use_empty() || IsBitCastForLifetimeMark(&BI))
     {
-        base = base->getScalarType();
+        return true;
     }
-    return useSOA;
+
+    Type* baseT = GetBaseType(IGCLLVM::getNonOpaquePtrEltTy(BI.getType()), true);
+    Type* sourceType = GetBaseType(IGCLLVM::getNonOpaquePtrEltTy(BI.getOperand(0)->getType()), true);
+    if (baseT->isStructTy() || sourceType->isStructTy()) {
+        StructType* bSTy = dyn_cast<StructType>(baseT);
+        StructType* sSTy = dyn_cast<StructType>(sourceType);
+        IGC_ASSERT(bSTy || sSTy);
+        return bSTy && sSTy && (bSTy == sSTy || bSTy->isLayoutIdentical(sSTy));
+    }
+    if (baseT->getScalarSizeInBits() != 0 &&
+        baseT->getScalarSizeInBits() == sourceType->getScalarSizeInBits())
+    {
+        const bool sameSize = ((uint32_t)baseT->getPrimitiveSizeInBits() ==
+            (uint32_t)sourceType->getPrimitiveSizeInBits());
+        isVectorSOA &= sameSize;
+        if (newAlgoControl > 1 && baseT->isVectorTy() && !sameSize) {
+            return false;
+        }
+        return checkUsers(BI);
+    }
+
+    // Not a candidate.
+    return false;
+}
+
+bool SOALayoutChecker::visitGetElementPtrInst(GetElementPtrInst& GEP)
+{
+    return checkUsers(GEP);
+}
+
+bool SOALayoutChecker::visitIntrinsicInst(IntrinsicInst& II)
+{
+    llvm::Intrinsic::ID IID = II.getIntrinsicID();
+    return IID == llvm::Intrinsic::lifetime_start ||
+           IID == llvm::Intrinsic::lifetime_end;
+}
+
+bool SOALayoutChecker::visitLoadInst(LoadInst& LI)
+{
+    bool isVectorLoad = LI.getType()->isVectorTy();
+    isVectorSOA &= isVectorLoad;
+    pInfo->allUsesAreVector &= isVectorLoad;
+    return LI.isSimple();
+}
+
+bool SOALayoutChecker::visitStoreInst(StoreInst& SI)
+{
+    if (!SI.isSimple())
+        return false;
+    llvm::Value* pValueOp = SI.getValueOperand();
+    bool isVectorStore = pValueOp->getType()->isVectorTy();
+    isVectorSOA &= isVectorStore;
+    pInfo->allUsesAreVector &= isVectorStore;
+    if (pValueOp == parentLevelInst)
+    {
+        // GEP instruction is the stored value of the StoreInst (unsupported case)
+        return false;
+    }
+    return true;
 }
 
 void LowerGEPForPrivMem::MarkNotPromtedAllocas(llvm::AllocaInst& I, IGC::StatusPrivArr2Reg status)
@@ -524,7 +667,7 @@ void LowerGEPForPrivMem::MarkNotPromtedAllocas(llvm::AllocaInst& I, IGC::StatusP
 
     UserAddrSpaceMD& userASMD = m_ctx->getUserAddrSpaceMD();
     std::function<void(Instruction*, MDNode*)> markAS_PRIV;
-    markAS_PRIV = [&markAS_PRIV, &userASMD](Instruction* instr, MDNode* node) -> void
+    markAS_PRIV = [&markAS_PRIV, &userASMD](Instruction* instr, MDNode* node)
     {
         // Avoid instruction which has already md set
         if (!userASMD.Has(instr, LSC_DOC_ADDR_SPACE::PRIVATE))
@@ -644,13 +787,17 @@ public:
     AllocaInst* pVecAlloca;
     // location of lifetime starts
     llvm::SmallPtrSet<Instruction*, 4> pStartPoints;
-    TransposeHelperPromote(AllocaInst* pAI) : TransposeHelper(false) { pVecAlloca = pAI; }
+    TransposeHelperPromote(AllocaInst* pAI, const DataLayout& DL)
+        : TransposeHelper(DL, false)
+    {
+        pVecAlloca = pAI;
+    }
 };
 
 void LowerGEPForPrivMem::handleAllocaInst(llvm::AllocaInst* pAlloca)
 {
     // Extract the Alloca size and the base Type
-    Type* pType = IGCLLVM::getNonOpaquePtrEltTy(pAlloca->getType());
+    Type* pType = pAlloca->getAllocatedType();
     Type* pBaseType = GetBaseType(pType)->getScalarType();
     IGC_ASSERT(pBaseType);
     llvm::AllocaInst* pVecAlloca = createVectorForAlloca(pAlloca, pBaseType);
@@ -661,7 +808,7 @@ void LowerGEPForPrivMem::handleAllocaInst(llvm::AllocaInst* pAlloca)
 
     IRBuilder<> IRB(pVecAlloca);
     Value* idx = IRB.getInt32(0);
-    TransposeHelperPromote helper(pVecAlloca);
+    TransposeHelperPromote helper(pVecAlloca, *m_pDL);
     helper.HandleAllocaSources(pAlloca, idx);
     IGC_ASSERT(nullptr != pAlloca);
     // for uniform alloca, we need to insert an initial definition
@@ -693,6 +840,11 @@ void TransposeHelper::handleGEPInst(
     llvm::GetElementPtrInst* pGEP,
     llvm::Value* idx)
 {
+    if (useNewAlgo()) {
+        handleGEPInstNew(pGEP, idx);
+        return;
+    }
+
     IGC_ASSERT(nullptr != pGEP);
     IGC_ASSERT(static_cast<ADDRESS_SPACE>(pGEP->getPointerAddressSpace()) == ADDRESS_SPACE_PRIVATE);
     // Add GEP instruction to remove list
@@ -710,7 +862,7 @@ void TransposeHelper::handleGEPInst(
     //
     IRBuilder<> IRB(pGEP);
     Value* pScalarizedIdx = IRB.getInt32(0);
-    Type* T = IGCLLVM::getNonOpaquePtrEltTy(pGEP->getPointerOperandType());
+    Type* T = pGEP->getSourceElementType();
     for (unsigned i = 0, e = pGEP->getNumIndices(); i < e; ++i)
     {
         auto GepOpnd = IRB.CreateZExtOrTrunc(pGEP->getOperand(i + 1), IRB.getInt32Ty());
@@ -765,10 +917,75 @@ void TransposeHelper::handleGEPInst(
         }
         pScalarizedIdx = IRB.CreateNUWMul(pScalarizedIdx, IRB.getInt32(arr_sz));
     }
-    pScalarizedIdx = IRB.CreateNUWAdd(pScalarizedIdx, idx);
+    ConstantInt* CIidx = dyn_cast<ConstantInt>(idx);
+    if (CIidx && CIidx->isNegative())
+    {
+        pScalarizedIdx = IRB.CreateAdd(pScalarizedIdx, idx);
+    }
+    else
+    {
+        pScalarizedIdx = IRB.CreateNUWAdd(pScalarizedIdx, idx);
+    }
     HandleAllocaSources(pGEP, pScalarizedIdx);
 }
 
+void TransposeHelper::handleGEPInstNew(
+    llvm::GetElementPtrInst* pGEP,
+    llvm::Value* idx)
+{
+    IGC_ASSERT(nullptr != pGEP);
+    IGC_ASSERT(static_cast<ADDRESS_SPACE>(pGEP->getPointerAddressSpace()) == ADDRESS_SPACE_PRIVATE);
+    // Add GEP instruction to remove list
+    m_toBeRemovedGEP.push_back(pGEP);
+    if (pGEP->use_empty())
+    {
+        // GEP has no users, do nothing.
+        return;
+    }
+
+    auto isIntZero = [](Value* V) {
+        ConstantInt* CI = dyn_cast<ConstantInt>(V);
+        return (CI && CI->isZero());
+    };
+
+    IRBuilder<> IRB(pGEP);
+    // linearOffset : byte offset of int32 (could be negative, no overflow).
+    Value* linearOffset = IRB.getInt32(0);
+    gep_type_iterator GTI = gep_type_begin(pGEP);
+    for (auto OI = pGEP->op_begin() + 1, E = pGEP->op_end(); OI != E; ++OI, ++GTI) {
+        Value* ix = *OI;
+        Value* OffsetVal;
+        if (StructType* StTy = GTI.getStructTypeOrNull()) {
+            unsigned Field = int_cast<unsigned>(cast<ConstantInt>(ix)->getZExtValue());
+            uint64_t Offset = m_DL.getStructLayout(StTy)->getElementOffset(Field);
+            OffsetVal = IRB.getInt32((uint32_t)Offset);
+        }
+        else {
+            Value* NewIx = IRB.CreateSExtOrTrunc(ix, linearOffset->getType());
+            // OffsetVal = NewIx * tyBytes
+            if (isIntZero(NewIx)) {
+                OffsetVal = NewIx;
+            }
+            else {
+                Type* Ty = GTI.getIndexedType();
+                uint64_t tyBytes = m_DL.getTypeAllocSize(Ty);
+                OffsetVal = IRB.CreateNSWMul(NewIx, IRB.getInt32((uint32_t)tyBytes));
+            }
+        }
+        // linearOffset += OffsetVal
+        if (!isIntZero(OffsetVal)) {
+            if (isIntZero(linearOffset))
+                linearOffset = OffsetVal;
+            else
+                linearOffset = IRB.CreateNSWAdd(linearOffset, OffsetVal);
+        }
+    }
+    // linearOffset += idx
+    if (!isIntZero(idx))
+        linearOffset = IRB.CreateAdd(linearOffset, idx);
+    HandleAllocaSources(pGEP, linearOffset);
+}
+
 // Load N elements from a vector alloca, Idx, ... Idx + N - 1. Return a scalar
 // or a vector value depending on N.
 static Value* loadEltsFromVecAlloca(
@@ -777,7 +994,7 @@ static Value* loadEltsFromVecAlloca(
     IGCLLVM::IRBuilder<>& IRB,
     Type* scalarType)
 {
-    Value* pLoadVecAlloca = IRB.CreateLoad(pVecAlloca);
+    Value* pLoadVecAlloca = IRB.CreateLoad(pVecAlloca->getAllocatedType(), pVecAlloca);
     if (N == 1)
     {
         return IRB.CreateBitCast(
@@ -833,7 +1050,7 @@ void TransposeHelperPromote::handleStoreInst(
 
     IGCLLVM::IRBuilder<> IRB(pStore);
     llvm::Value* pStoreVal = pStore->getValueOperand();
-    llvm::Value* pLoadVecAlloca = IRB.CreateLoad(pVecAlloca);
+    llvm::Value* pLoadVecAlloca = IRB.CreateLoad(pVecAlloca->getAllocatedType(), pVecAlloca);
     llvm::Value* pIns = pLoadVecAlloca;
     IGC_ASSERT(nullptr != pStoreVal);
     IGC_ASSERT(nullptr != pStoreVal->getType());
