@@ -20,6 +20,7 @@ SPDX-License-Identifier: MIT
 #include "skuwa/iacm_g12_rev_id.h"
 
 #include "iStdLib/utility.h"
+#include "visa_igc_common_header.h"
 
 namespace IGC
 {
@@ -45,7 +46,8 @@ private:
 bool hasQWAddSupport() const
 {
     return (m_platformInfo.eProductFamily == IGFX_PVC &&
-        m_platformInfo.usRevId >= REVISION_D);  // true from PVC XT B0 RevID==0x5==REVISION_D
+        m_platformInfo.usRevId >= REVISION_D)  // true from PVC XT B0 RevID==0x5==REVISION_D
+        || isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 //all the platforms which DONOT support 64 bit int operations
@@ -146,11 +148,44 @@ bool enablePSsimd32() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GE
 
 bool supportSimd32PerPixelPSWithNumSamples16() const
 {
-    return false;
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool canSupportWMTPWithoutBTD() const {
+    // Returns true if the platform has the capability of supporting
+    // WMTP but WMTP isn't supported for all shader types
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 
-bool supportDisableMidThreadPreemptionSwitch() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN10_CORE; }
+bool canSupportWMTP() const
+{
+    // Returns true if the platform has the capability of supporting
+    // WMTP but WMTP isn't supported for all shader types
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool supportsWMTPForShaderType(ShaderType type) const {
+
+    if (isCoreChildOf(IGFX_XE2_HPG_CORE)) {
+        switch(type) {
+        case ShaderType::COMPUTE_SHADER:
+        case ShaderType::OPENCL_SHADER:
+            return true;
+        default:
+            return false;
+        }
+    }
+
+    return false;
+}
+
+bool supportDisableMidThreadPreemptionSwitch() const
+{
+    return
+    isCoreChildOf(IGFX_GEN10_CORE)
+    ;
+}
 
 bool needSWStencil() const
 {
@@ -420,6 +455,13 @@ bool HDCCoalesceSLMAtomicINCWithNoReturn() const
     return m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE;
 }
 
+
+int LSCCachelineSize() const
+{
+    return 64;
+}
+
+
 bool HDCCoalesceAtomicCounterAccess() const
 {
     return (m_platformInfo.eRenderCoreFamily < IGFX_GEN12_CORE) && IGC_IS_FLAG_DISABLED(ForceSWCoalescingOfAtomicCounter);
@@ -458,7 +500,17 @@ bool hasSamplerSupport() const
         (IGC_IS_FLAG_ENABLED(EnableSamplerSupport)); // flag for IGFX_PVC
 }
 
-bool hasLogicalSSID() const { return false; }
+bool hasSamplerFeedbackSurface() const
+{
+    return m_platformInfo.eProductFamily >= IGFX_BMG;
+}
+
+// logical subslice id
+bool hasLogicalSSID() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
 uint32_t getMinPushConstantBufferAlignment() const
 {
     return 8; // DWORDs
@@ -509,6 +561,15 @@ unsigned getSlmSizePerSsOrDss() const
             slmSizePerSsOrDss = 131072;
         }
     }
+    else if (isProductChildOf(IGFX_BMG))
+    {
+        // SLMSizeInKb is reported per SS for BMG+
+        slmSizePerSsOrDss = GetGTSystemInfo().SLMSizeInKb * 1024;
+    }
+    else if (isProductChildOf(IGFX_PVC))
+    {
+        slmSizePerSsOrDss = GetGTSystemInfo().SLMSizeInKb / GetGTSystemInfo().SubSliceCount * 1024;
+    }
     return slmSizePerSsOrDss;
 }
 
@@ -534,12 +595,21 @@ bool supportBfnInstruction() const
 
 bool supportDpasInstruction() const
 {
-    return isProductChildOf(IGFX_XE_HP_SDV) && m_platformInfo.eProductFamily != IGFX_METEORLAKE;
+    return isProductChildOf(IGFX_XE_HP_SDV) && m_platformInfo.eProductFamily != IGFX_METEORLAKE &&
+        !(m_platformInfo.eProductFamily == IGFX_PVC && GFX_IS_VG_CONFIG(m_platformInfo.usDeviceID));
+}
+
+bool supportJointMatrixOCLExtension() const
+{
+    return supportDpasInstruction() ||
+        // SPV_INTEL_joint_matrix extension is partially supported on PVC-VG. Only OpJointMatrixMadINTEL
+        // opcodes are not supported and proper error is emitted in case any of them is used in a kernel.
+        (m_platformInfo.eProductFamily == IGFX_PVC && GFX_IS_VG_CONFIG(m_platformInfo.usDeviceID));
 }
 
 bool hasPackedRestrictedFloatVector() const
 {
-    return true;
+    return !isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool supportLoadThreadPayloadForCompute() const
@@ -567,6 +637,7 @@ bool hasScratchSurface() const
     return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
+
 bool hasAtomicPreDec() const
 {
     return !isProductChildOf(IGFX_XE_HP_SDV);
@@ -628,7 +699,7 @@ bool supportInlineData() const
 // TODO: temporary solution, remove this once it's not needed
 bool supportInlineDataOCL() const
 {
-    if (m_platformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE)
+    if (m_platformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE && IGC_IS_FLAG_DISABLED(ForceInlineDataForXeHPC))
     {
         return false;
     }
@@ -667,7 +738,10 @@ bool supportDynamicBTIsAllocation() const
 
 bool loosenSimd32occu() const
 {
-    return (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE && m_platformInfo.eProductFamily != IGFX_DG2);
+    if (IGC_GET_FLAG_VALUE(ForceLoosenSimd32Occu) == 2)
+        return (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE && m_platformInfo.eProductFamily != IGFX_DG2);
+    else
+        return IGC_GET_FLAG_VALUE(ForceLoosenSimd32Occu);
 }
 
 // Local memory here refers to memory on the device- e.g. HBM for PVC.
@@ -691,6 +765,14 @@ bool supportHWGenerateTID() const
     return IGC_IS_FLAG_ENABLED(EnableHWGenerateThreadID) && isProductChildOf(IGFX_DG2);
 }
 
+bool supportHWDispatchWalkOrder() const
+{
+    const bool hasHWDispatchWalkOrder =
+        (m_platformInfo.eProductFamily == IGFX_ARROWLAKE && !(GFX_IS_ARL_S(m_platformInfo.usDeviceID)))
+        || isCoreChildOf(IGFX_XE2_HPG_CORE);
+    return hasHWDispatchWalkOrder;
+}
+
 bool hasHalfSIMDLSC() const
 {
     return (m_platformInfo.eProductFamily == IGFX_DG2 && SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0)) ||
@@ -699,7 +781,8 @@ bool hasHalfSIMDLSC() const
         // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
         (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B) ||
         m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
-        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
+        isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool NeedsLSCFenceUGMBeforeEOT() const
@@ -710,7 +793,8 @@ bool NeedsLSCFenceUGMBeforeEOT() const
 bool hasPartialInt64Support() const
 {
     // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
-    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B);
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B) ||
+        isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool hasInt64Add() const
@@ -733,7 +817,9 @@ bool hasFullInt64() const
 
 bool hasInt64DstMul() const
 {
-    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId < REVISION_B);
+
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId < REVISION_B)
+            || isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool hasExecSize16DPAS() const
@@ -756,6 +842,11 @@ bool hasNoFullI64Support() const
     return (hasNoInt64Inst() || hasPartialInt64Support());
 }
 
+bool hasBFTFDenormMode() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
 bool WaPredicatedStackIDRelease() const
 {
     return m_WaTable.Wa_22014559856 &&
@@ -772,11 +863,20 @@ SIMDMode getMaxRayQuerySIMDSize() const
     }
     else
     {
-        IGC_ASSERT_MESSAGE(0, "Change code here to support new platform!");
+        IGC_ASSERT_MESSAGE(0, "Unsupported platform!");
         return SIMDMode::UNKNOWN;
     }
 }
 
+SIMDMode getPreferredRayQuerySIMDSize() const
+{
+    const SIMDMode ret = isCoreChildOf(IGFX_XE_HPC_CORE) ? SIMDMode::SIMD16 : SIMDMode::SIMD8;
+
+    IGC_ASSERT_MESSAGE(ret <= getMaxRayQuerySIMDSize(), "Preferred SIMD size for RayQuery must not be greater than MaxRayQuerySIMDSize!");
+
+    return ret;
+}
+
 SIMDMode getPreferredRayTracingSIMDSize() const
 {
     return isCoreChildOf(IGFX_XE_HPC_CORE) ? SIMDMode::SIMD16 : SIMDMode::SIMD8;
@@ -801,7 +901,9 @@ bool supports3DAndCubeSampleD() const
         m_platformInfo.eProductFamily != IGFX_XE_HP_SDV &&
         m_platformInfo.eProductFamily != IGFX_DG2 &&
         m_platformInfo.eProductFamily != IGFX_METEORLAKE &&
-        m_platformInfo.eProductFamily != IGFX_ARROWLAKE
+        m_platformInfo.eProductFamily != IGFX_ARROWLAKE &&
+        m_platformInfo.eProductFamily != IGFX_BMG &&
+        m_platformInfo.eProductFamily != IGFX_LUNARLAKE
         ) ||
         IGC_IS_FLAG_DISABLED(EnableSampleDEmulation);
 }
@@ -836,6 +938,9 @@ bool LSCEnabled(SIMDMode m = SIMDMode::UNKNOWN) const
                      GFX_IS_DG2_G12_CONFIG(m_platformInfo.usDeviceID)
                 ) && m == SIMDMode::SIMD8) ||
                 m == SIMDMode::SIMD16;
+        case IGFX_METEORLAKE:
+        case IGFX_ARROWLAKE:
+            return m == SIMDMode::UNKNOWN || m == SIMDMode::SIMD8 || m == SIMDMode::SIMD16;
         default:
             return true;
         }
@@ -851,9 +956,10 @@ uint32_t getMaxLSCBlockMsgSize(bool isD64 = true) const
 
 bool hasURBFence() const
 {
-    return m_platformInfo.eProductFamily == IGFX_DG2 ||
+    return (m_platformInfo.eProductFamily == IGFX_DG2 ||
         m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
-        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
+        isCoreChildOf(IGFX_XE2_HPG_CORE));
 }
 
 bool hasMultiTile() const
@@ -875,7 +981,6 @@ bool L3CacheCoherentCrossTiles() const {
     return isCoreChildOf(IGFX_XE_HPC_CORE);
 }
 
-
 //if RayTracing fence WA for LSCBackupMode (DG2.A0 only actually) is enabled.
 //In case of RayQuery of DG2.A0, we have to flush L1 after RTWrite and before shader read.
 //The reason is, for RT writes, write - completion must guarantee that
@@ -897,6 +1002,11 @@ bool hasLSCTypedMessage() const
     return isCoreChildOf(IGFX_XE_HPC_CORE);
 }
 
+bool LSC2DSupportImmXY() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
 SIMDMode getMaxLSCTypedMessageSize() const
 {
     switch (m_platformInfo.eRenderCoreFamily)
@@ -906,7 +1016,14 @@ SIMDMode getMaxLSCTypedMessageSize() const
     case IGFX_XE_HPC_CORE:
         return SIMDMode::SIMD16;
     default:
+        if (IGC_IS_FLAG_ENABLED(DisableLSCSIMD32TGMMessages))
+        {
             return SIMDMode::SIMD16;
+        }
+        else
+        {
+            return SIMDMode::SIMD32;
+        }
     }
 }
 
@@ -918,13 +1035,14 @@ unsigned getAccChNumUD() const
 bool hasInt64SLMAtomicCAS() const
 {
     // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
-    return m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B;
-
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B)
+            || isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool hasFP64GlobalAtomicAdd() const
 {
-    return m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId > REVISION_A0;
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId > REVISION_A0)
+            || isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool supports16BitLdMcs() const
@@ -951,7 +1069,8 @@ bool typedReadSupportsAllRenderableFormats() const
         (m_platformInfo.eProductFamily == IGFX_DG2 && isDG2G12Config) ||
         (m_platformInfo.eProductFamily == IGFX_METEORLAKE) ||
         (m_platformInfo.eProductFamily == IGFX_ARROWLAKE) ||
-        (m_platformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE))
+        (m_platformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE) ||
+        isCoreChildOf(IGFX_XE2_HPG_CORE))
     {
         return IGC_IS_FLAG_DISABLED(ForceFormatConversionDG2Plus);
     }
@@ -961,7 +1080,7 @@ bool typedReadSupportsAllRenderableFormats() const
 
 bool EnableNewTileYCheckDefault() const
 {
-    return (m_platformInfo.eProductFamily == IGFX_DG2);
+    return true;
 }
 
 bool EnableKeepTileYForFlattenedDefault() const
@@ -978,12 +1097,13 @@ bool needsWAForThreadsUtilization() const
 
 bool supportDualSimd8PS() const
 {
-    return IGC_IS_FLAG_ENABLED(EnableDualSIMD8) && (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE);
+    return IGC_IS_FLAG_ENABLED(EnableDualSIMD8) && (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE) &&
+        !isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 bool hasDualSimd8Payload() const
 {
-    return m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE;
+    return (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE) && !isCoreChildOf(IGFX_XE2_HPG_CORE);
 }
 
 unsigned int getMaxMeshShaderThreads() const {
@@ -1020,6 +1140,11 @@ unsigned int getMaxNumberHWThreadForEachWG() const
             return getMaxNumberThreadPerSubslice() * 2;
         }
     }
+    else if (m_platformInfo.eRenderCoreFamily == IGFX_XE2_HPG_CORE)
+    {
+        // Each WG is dispatched into one subslice
+        return getMaxNumberThreadPerSubslice();
+    }
     else {
         IGC_ASSERT_MESSAGE(0, "Unsupported platform!");
     }
@@ -1058,10 +1183,12 @@ bool matchImmOffsetsLSC() const
 {
     enum LscMatchImmMode {
         OFF = 0,
+        IF_HW_SUPPORTED = 1,
         ON = 2,
     };
     auto immOffsetMode = (LscMatchImmMode)IGC_GET_FLAG_VALUE(LscImmOffsMatch);
-    return hasLSC() && immOffsetMode >= ON;
+    return hasLSC() &&
+        (immOffsetMode >= ON || (immOffsetMode == IF_HW_SUPPORTED && isCoreChildOf(IGFX_XE2_HPG_CORE)));
 }
 
 bool WaCubeHFPrecisionBug() const
@@ -1072,19 +1199,28 @@ bool WaCubeHFPrecisionBug() const
 //The max size in bytes of the scratch space per thread.
 //  XeHP_SDV and above are for each physical thread: 256k.
 //  TGLLP and below are for each FFTID: 2M.
-uint32_t maxPerThreadScratchSpace() const
+uint32_t maxPerThreadScratchSpace(
+) const
 {
-    return (hasScratchSurface() ? 0x40000 : 0x200000);
+
+    return hasScratchSurface() ? 0x40000 : 0x200000;
+}
+
+bool enableSpillCompressionCheckDefault() const
+{
+    bool bEnabled = false;
+    return bEnabled;
 }
 
 bool supportAIParameterCombiningWithLODBiasEnabled() const
 {
     return IGC_IS_FLAG_ENABLED(EnableAIParameterCombiningWithLODBias) &&
-           (m_platformInfo.eProductFamily == IGFX_DG2 && SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0)) ||
+           ((m_platformInfo.eProductFamily == IGFX_DG2 && SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0)) ||
            GFX_IS_DG2_G12_CONFIG(m_platformInfo.usDeviceID) ||
            GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID) ||
            m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
-           m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
+           m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
+           isCoreChildOf(IGFX_XE2_HPG_CORE));
 }
 
 bool useScratchSpaceForOCL() const
@@ -1125,6 +1261,8 @@ bool hasNoFP64Inst() const {
 
 //all the platforms which have correctly rounded macros (INVM, RSQRTM, MADM)
 bool hasCorrectlyRoundedMacros() const {
+    if (isCoreChildOf(IGFX_XE2_HPG_CORE) && IGC_IS_FLAG_DISABLED(DisableCorrectlyRoundedMacros))
+        return true;
     return (m_platformInfo.eProductFamily != IGFX_ICELAKE_LP &&
         m_platformInfo.eProductFamily != IGFX_LAKEFIELD &&
         m_platformInfo.eProductFamily != IGFX_JASPERLAKE &&
@@ -1136,13 +1274,16 @@ bool hasCorrectlyRoundedMacros() const {
         m_platformInfo.eProductFamily != IGFX_ALDERLAKE_N &&
         m_platformInfo.eProductFamily != IGFX_DG2 &&
         m_platformInfo.eProductFamily != IGFX_METEORLAKE) &&
-        m_platformInfo.eProductFamily != IGFX_ARROWLAKE;
+        m_platformInfo.eProductFamily != IGFX_ARROWLAKE &&
+        m_platformInfo.eProductFamily != IGFX_BMG &&
+        m_platformInfo.eProductFamily != IGFX_LUNARLAKE;
 }
 
 // Has 64bit support but use 32bit for perf reasons
 bool preferFP32IntDivRemEmu() const {
-    return m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
-        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
+    return (m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
+        isCoreChildOf(IGFX_XE2_HPG_CORE));
 }
 
 // Platforms that haven't HW support for FP64 operations
@@ -1197,6 +1338,7 @@ bool emulateByteScraterMsgForSS() const
 bool has64BMediaBlockRW() const
 {
     bool hasBlockRW = (IGC_IS_FLAG_ENABLED(Enable64BMediaBlockRW) && isProductChildOf(IGFX_XE_HP_SDV));
+    hasBlockRW = hasBlockRW || isCoreChildOf(IGFX_XE2_HPG_CORE);
     return hasBlockRW;
 }
 
@@ -1253,6 +1395,47 @@ uint32_t getMaxBlockMsgSize(bool isSLM) const
     return 128;
 }
 
+bool hasLSCUrbMessage() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool hasSampleMlodMessage() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool forceSamplerHeader() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool hasDualKSPPS() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool hasLSCSamplerRouting() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool supportStochasticLod() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool supportsProgrammableOffsets() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool isDynamicRayQueryDynamicRayManagementMechanismEnabled() const
+{
+    return (isCoreChildOf(IGFX_XE2_HPG_CORE) &&
+        IGC_IS_FLAG_DISABLED(DisableRayQueryDynamicRayManagementMechanism));
+}
+
 // ***** Below go accessor methods for testing WA data from WA_TABLE *****
 
 bool WaDoNotPushConstantsForAllPulledGSTopologies() const
@@ -1403,12 +1586,12 @@ const SCompilerHwCaps& GetCaps() { return m_caps; }
 
 bool supportHeaderRTW() const
 {
-    return true;
+    return (!isCoreChildOf(IGFX_XE2_HPG_CORE));
 }
 
 bool preemptionSupported() const
 {
-    if (isCoreChildOf(IGFX_XE_HPC_CORE))
+    if (isCoreChildOf(IGFX_XE_HPC_CORE) && !isCoreChildOf(IGFX_XE2_HPG_CORE))
         return false;
 
     return GetPlatformFamily() >= IGFX_GEN9_CORE;
@@ -1445,6 +1628,8 @@ unsigned forceQwAtSrc0ForQwShlWA() const
 
 bool hasSIMD8Support() const
 {
+    if (isCoreChildOf(IGFX_XE2_HPG_CORE))
+        return false;
     return !(m_platformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE);
 }
 
@@ -1460,6 +1645,26 @@ bool WaDisableD64ScratchMessage() const
     return getWATable().Wa_15010203763 != 0;
 }
 
+bool supportLargeGRF() const
+{
+    return isCoreChildOf(IGFX_XE_HPG_CORE);
+}
+
+bool supportCheckCSThreadsLimit() const
+{
+    return (m_platformInfo.eRenderCoreFamily == IGFX_XE2_HPG_CORE);
+}
+bool supportTriggerLargeGRFRetry() const
+{
+    return (m_platformInfo.eRenderCoreFamily == IGFX_XE2_HPG_CORE);
+}
+
+
+
+bool EnableCSWalkerPass() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
 
 bool limitedBCR() const
 {
@@ -1470,6 +1675,9 @@ bool limitedBCR() const
 
 uint32_t getMaxAddressedHWThreads() const
 {
+    // TODO Calculate the value
+    if (isCoreChildOf(IGFX_XE2_HPG_CORE))
+        return 8192;
     return 4096;
 }
 
@@ -1484,15 +1692,61 @@ uint32_t getVISAABIVersion() const {
 
 bool supportsNonDefaultLSCCacheSetting() const
 {
-    return false;
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+bool isSupportedLSCCacheControlsEnum(LSC_L1_L3_CC l1l3cc, bool isLoad) const
+{
+    if (isLoad)
+    {
+        switch (l1l3cc)
+        {
+        default: break;
+        case LSC_L1UC_L3CC:
+        case LSC_L1C_L3CC:
+        case LSC_L1IAR_L3IAR:
+            return isCoreChildOf(IGFX_XE2_HPG_CORE);
+        }
+    }
+    return true;
 }
 
 unsigned int roundUpTgsmSize(DWORD size) const
 {
     const DWORD blockSize = getSharedLocalMemoryBlockSize() ? getSharedLocalMemoryBlockSize() : sizeof(KILOBYTE);
     size = iSTD::Round(size, blockSize) / blockSize;
+    if (size > 16)
+    {
+        if (isCoreChildOf(IGFX_XE2_HPG_CORE))
+        {
+            const DWORD roundSize = iSTD::RoundPower2(size) / 4;
+            return iSTD::Round(size, roundSize) * blockSize;
+        }
+    }
     return iSTD::RoundPower2(size) * blockSize;
 }
 
+
+unsigned getRayTracingTileXDim2D() const
+{
+    return 32;
+}
+
+unsigned getRayTracingTileYDim2D() const
+{
+        return 4;
+}
+
+bool preferLSCCache() const
+{
+    return isCoreChildOf(IGFX_XE2_HPG_CORE);
+}
+
+
+bool allowDivergentControlFlowRayQueryCheckRelease() const
+{
+    return false;
+}
+
 };
 }//namespace IGC
