@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -13,18 +13,23 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/GenCodeGenModule.h"
 #include "Compiler/CISACodeGen/AdvCodeMotion.h"
 #include "Compiler/CISACodeGen/RematAddressArithmetic.h"
+#include "Compiler/CISACodeGen/IGCLivenessAnalysis.h"
 #include "Compiler/CISACodeGen/AdvMemOpt.h"
 #include "Compiler/CISACodeGen/Emu64OpsPass.h"
 #include "Compiler/CISACodeGen/PullConstantHeuristics.hpp"
 #include "Compiler/CISACodeGen/PushAnalysis.hpp"
 #include "Compiler/CISACodeGen/ScalarizerCodeGen.hpp"
+#include "Compiler/CISACodeGen/HoistCongruentPhi.hpp"
 #include "Compiler/CISACodeGen/CodeSinking.hpp"
 #include "Compiler/CISACodeGen/AddressArithmeticSinking.hpp"
 #include "Compiler/CISACodeGen/AtomicOptPass.hpp"
+#include "Compiler/CISACodeGen/BlockMemOpAddrScalarizationPass.hpp"
 #include "Compiler/CISACodeGen/SinkCommonOffsetFromGEP.h"
 #include "Compiler/CISACodeGen/ConstantCoalescing.hpp"
 #include "Compiler/CISACodeGen/CheckInstrTypes.hpp"
 #include "Compiler/CISACodeGen/EstimateFunctionSize.h"
+#include "Compiler/CISACodeGen/GenerateBlockMemOpsPass.hpp"
+#include "Compiler/CISACodeGen/GenerateFrequencyData.hpp"
 #include "Compiler/CISACodeGen/PassTimer.hpp"
 #include "Compiler/CISACodeGen/FixAddrSpaceCast.h"
 #include "Compiler/CISACodeGen/FixupExtractValuePair.h"
@@ -35,9 +40,10 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/MemOpt.h"
 #include "Compiler/CISACodeGen/MemOpt2.h"
 #include "Compiler/CISACodeGen/PreRARematFlag.h"
-#include "Compiler/CISACodeGen/PreRAScheduler.hpp"
 #include "Compiler/CISACodeGen/PromoteConstantStructs.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/AlignmentAnalysis/AlignmentAnalysis.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/GenericAddressResolution/GASResolving.h"
+#include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/LowerByValAttribute.hpp"
 #include "Compiler/CISACodeGen/ResolvePredefinedConstant.h"
 #include "Compiler/CISACodeGen/Simd32Profitability.hpp"
 #include "Compiler/CISACodeGen/SimplifyConstant.h"
@@ -50,6 +56,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/LowerGEPForPrivMem.hpp"
 #include "Compiler/CISACodeGen/POSH_RemoveNonPositionOutput.h"
 #include "Compiler/CISACodeGen/RegisterEstimator.hpp"
+#include "Compiler/CISACodeGen/RegisterPressureEstimate.hpp"
 #include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "Compiler/CISACodeGen/RayTracingShaderLowering.hpp"
 #include "Compiler/CISACodeGen/RayTracingStatefulPass.h"
@@ -61,17 +68,15 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/PrepareLoadsStoresPass.h"
 #include "Compiler/CISACodeGen/EvaluateFreeze.hpp"
 #include "Compiler/CISACodeGen/DpasScan.hpp"
+#include "Compiler/CISACodeGen/FPRoundingModeCoalescing.hpp"
 
 #include "Compiler/CISACodeGen/SLMConstProp.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/DebuggerSupport/ImplicitGIDPass.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/DebuggerSupport/ImplicitGIDRestoring.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/GenericAddressResolution/GenericAddressDynamicResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryUsageAnalysis.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryToSLM.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ProgramScopeConstants/ProgramScopeConstantResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/WIFuncs/WIFuncResolution.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/RegPressureLoopControl/RegPressureLoopControl.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/BreakConstantExpr/BreakConstantExpr.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ReplaceUnsupportedIntrinsics/ReplaceUnsupportedIntrinsics.hpp"
 #include "Compiler/Optimizer/PreCompiledFuncImport.hpp"
@@ -84,6 +89,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/OpenCLPasses/WIFuncs/WIFuncResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ScalarArgAsPointer/ScalarArgAsPointer.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/GEPLoopStrengthReduction/GEPLoopStrengthReduction.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/StackOverflowDetection/StackOverflowDetection.hpp"
 #include "Compiler/Optimizer/MCSOptimization.hpp"
 #include "Compiler/Optimizer/GatingSimilarSamples.hpp"
 #include "Compiler/Optimizer/IntDivConstantReduction.hpp"
@@ -98,6 +104,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/GenUpdateCB.h"
 #include "Compiler/PromoteResourceToDirectAS.h"
 #include "Compiler/PromoteStatelessToBindless.h"
+#include "Compiler/ShrinkArrayAlloca.h"
 #if defined(_DEBUG) && !defined(ANDROID)
 #include "Compiler/VerificationPass.hpp"
 #endif
@@ -123,6 +130,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/Config/llvm-config.h"
 #include "llvm/ADT/PostOrderIterator.h"
+#include <llvm/IR/DebugInfo.h>
 #include <llvm/IR/LLVMContext.h>
 #include <llvm/IR/Verifier.h>
 #include <llvm/Analysis/CFGPrinter.h>
@@ -138,6 +146,9 @@ SPDX-License-Identifier: MIT
 #include <llvm/Linker/Linker.h>
 #include <llvm/Analysis/ScopedNoAliasAA.h>
 #include <llvm/Analysis/TargetLibraryInfo.h>
+#include <llvm/Analysis/BlockFrequencyInfo.h>
+#include <llvm/Analysis/BranchProbabilityInfo.h>
+#include <llvm/Analysis/LoopInfo.h>
 #include <llvm/ADT/StringExtras.h>
 #include <llvm/IRReader/IRReader.h>
 #include <llvm/Support/MathExtras.h>
@@ -168,6 +179,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/AnnotateUniformAllocas.h"
 #include "Probe/Assertion.h"
 #include "Compiler/CISACodeGen/PartialEmuI64OpsPass.h"
+#include "Compiler/TranslateToProgrammableOffsetsPass.hpp"
 
 
 /***********************************************************************************
@@ -227,7 +239,18 @@ void AddAnalysisPasses(CodeGenContext& ctx, IGCPassManager& mpm)
     // only limited code-sinking to several shader-type
     // vs input has the URB-reuse issue to be resolved.
     // Also need to understand the performance benefit better.
-    mpm.add(new CodeSinking(true));
+    if (!isOptDisabled)
+    {
+        mpm.add(new HoistCongruentPHI());
+        mpm.add(new CodeSinking());
+        if ((IGC_IS_FLAG_DISABLED(DisableLoopSink) || IGC_IS_FLAG_ENABLED(ForceLoopSink))
+             && ctx.type == ShaderType::OPENCL_SHADER
+             && ctx.m_instrTypes.numOfLoop > 0
+             && ctx.m_instrTypes.numInsts >= IGC_GET_FLAG_VALUE(CodeLoopSinkingMinSize))
+        {
+            mpm.add(new CodeLoopSinking());
+        }
+    }
 
 
     // Run flag re-materialization if it's beneficial.
@@ -285,10 +308,20 @@ void AddAnalysisPasses(CodeGenContext& ctx, IGCPassManager& mpm)
             }
         }
         mpm.add(createPromoteMemoryToRegisterPass());
+        if (ctx.type == ShaderType::OPENCL_SHADER &&
+            !isOptDisabled &&
+            IGC_IS_FLAG_ENABLED(EnableExplicitCopyForByVal))
+        {
+            mpm.add(new LowerByValAttribute());
+            mpm.add(createReplaceUnsupportedIntrinsicsPass());
+        }
         // Resolving private memory allocas
         mpm.add(CreatePrivateMemoryResolution());
     }
 
+    // Reorder FP instructions to minimize number of times rounding mode is switched.
+    mpm.add(createFPRoundingModeCoalescingPass());
+
     // Expected to be the last ModulePass before EmitPass at this point.
     // (Shall be after GenXCodeGenModulePass.)
     //
@@ -317,6 +350,15 @@ void AddAnalysisPasses(CodeGenContext& ctx, IGCPassManager& mpm)
     // collect stats after all the optimization. This info can be dumped to the cos file
     mpm.add(new CheckInstrTypes(true, false));
 
+    if ((IGC_GET_FLAG_VALUE(StaticProfileGuidedSpillCostAnalysis) &
+         FrequencyDataDS::PGSS_IGC_GEN) != 0) {
+        mpm.add(createGenerateFrequencyDataPass());
+    }
+
+    if (IGC_IS_FLAG_ENABLED(StackOverflowDetection)) {
+        mpm.add(new StackOverflowDetectionPass(StackOverflowDetectionPass::Mode::RemoveDummyCalls));
+    }
+
     //
     // Generally, passes that change IR should be prior to this place!
     //
@@ -404,12 +446,11 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         IGC_IS_FLAG_ENABLED(ForceAllPrivateMemoryToSLM) ||
         IGC_IS_FLAG_ENABLED(ForcePrivateMemoryToSLMOnBuffers))
     {
-        DummyPass* dummypass = new DummyPass();
         TargetIRAnalysis GenTTgetIIRAnalysis([&](const Function& F) {
-            GenIntrinsicsTTIImpl GTTI(&ctx, dummypass);
+            GenIntrinsicsTTIImpl GTTI(&ctx);
             return TargetTransformInfo(GTTI);
             });
-        mpm.add(new TargetTransformInfoWrapperPass(GenTTgetIIRAnalysis));
+        mpm.add(new TargetTransformInfoWrapperPass(std::move(GenTTgetIIRAnalysis)));
     }
 
     // Disable all target library functions.
@@ -461,7 +502,7 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
 
         if (IGC_IS_FLAG_ENABLED(allowLICM) && ctx.m_retryManager.AllowLICM())
         {
-            mpm.add(createDisableLICMForSpecificLoops());
+            mpm.add(createSpecialCasesDisableLICM());
             mpm.add(llvm::createLICMPass());
         }
         mpm.add(llvm::createLoopSimplifyPass());
@@ -482,12 +523,13 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     bool hasDPDivSqrtEmu = !ctx.platform.hasNoFP64Inst() && !ctx.platform.hasCorrectlyRoundedMacros() && ctx.m_DriverInfo.NeedFP64DivSqrt();
     uint32_t theEmuKind = (ctx.m_hasDPEmu ? EmuKind::EMU_DP : 0);
     theEmuKind |= (hasDPDivSqrtEmu ? EmuKind::EMU_DP_DIV_SQRT : 0);
+    theEmuKind |= (ctx.m_hasDPConvEmu ? EmuKind::EMU_DP_CONV : 0);
     theEmuKind |= (ctx.m_DriverInfo.NeedI64BitDivRem() ? EmuKind::EMU_I64DIVREM : 0);
     theEmuKind |= (ctx.m_DriverInfo.NeedFP64toFP16Conv() ? EmuKind::EMU_FP64_FP16_CONV : 0);
     theEmuKind |=
         ((IGC_IS_FLAG_ENABLED(ForceSPDivEmulation) ||
             (ctx.m_DriverInfo.NeedIEEESPDiv() && !ctx.platform.hasCorrectlyRoundedMacros()))
-        ? EmuKind::EMU_SP_DIV : 0);
+            ? EmuKind::EMU_SP_DIV : 0);
     if (ctx.platform.preferFP32IntDivRemEmu() && IGC_IS_FLAG_DISABLED(Force32BitIntDivRemEmu)) {
         // Prefer using FP32 emulation even though DP support is available
         theEmuKind |= EmuKind::EMU_I32DIVREM_SP;
@@ -598,10 +640,17 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     }
     else
     {
-            if (IGC_IS_FLAG_ENABLED(AllowMem2Reg))
+        if (IGC_IS_FLAG_ENABLED(AllowMem2Reg))
             mpm.add(createPromoteMemoryToRegisterPass());
     }
 
+    // There's no particular reason for this exact place, but it should be after LowerGEPForPrivMem
+    if (IGC_IS_FLAG_ENABLED(EnableSplitIndirectEEtoSel))
+    {
+        mpm.add(createSplitIndirectEEtoSelPass());
+    }
+
+
     if (ctx.type == ShaderType::OPENCL_SHADER ||
         ctx.type == ShaderType::COMPUTE_SHADER)
     {
@@ -617,7 +666,7 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
                 IGC_GET_REGKEYSTRING(ForcePrivateMemoryToSLMOnBuffers));
 
             mpm.add(new PrivateMemoryToSLM(
-                forcedBuffers,
+                std::move(forcedBuffers),
                 IGC_IS_FLAG_ENABLED(EnableOptReportPrivateMemoryToSLM)));
             mpm.add(createInferAddressSpacesPass());
         }
@@ -628,6 +677,13 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         // need to run loop simplify to canonicalize loop and merge latches
         mpm.add(createLoopCanonicalization());
         mpm.add(createLoopSimplifyPass());
+        if (!IGC_IS_FLAG_ENABLED(DisableLoopSplitWidePHIs))
+            mpm.add(createLoopSplitWidePHIs());
+    }
+
+    if (IGC_IS_FLAG_ENABLED(StackOverflowDetection)) {
+        // Cleanup stack overflow detection calls if necessary.
+        mpm.add(new StackOverflowDetectionPass(StackOverflowDetectionPass::Mode::AnalyzeAndCleanup));
     }
 
     if  (ctx.enableFunctionCall() || ctx.type == ShaderType::RAYTRACING_SHADER)
@@ -648,6 +704,13 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     if (!(IGC_IS_FLAG_ENABLED(EnableUnmaskedFunctions) &&
           IGC_IS_FLAG_ENABLED(LateInlineUnmaskedFunc)))
     {
+        if (ctx.type == ShaderType::OPENCL_SHADER &&
+            !isOptDisabled &&
+            IGC_IS_FLAG_ENABLED(EnableExplicitCopyForByVal))
+        {
+            mpm.add(new LowerByValAttribute());
+            mpm.add(createReplaceUnsupportedIntrinsicsPass());
+        }
         mpm.add(CreatePrivateMemoryResolution());
     }
     // Should help MemOpt pass to merge more loads
@@ -657,22 +720,35 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     if (!isOptDisabled &&
         ctx.m_instrTypes.hasLoadStore && IGC_IS_FLAG_DISABLED(DisableMemOpt) && !ctx.getModuleMetaData()->disableMemOptforNegativeOffsetLoads) {
 
+#if LLVM_VERSION_MAJOR <= 10
+        if (ctx.type == ShaderType::OPENCL_SHADER)
+        {
+            // Some LLVM passes may produce load/store instructions without alignment set,
+            // for example SROA. LLVM API has been changed to ensure that alignment is never
+            // missing, it has been changed in LLVM11: https://reviews.llvm.org/D77454.
+            // Merging loads/stores without alignment set results in producing loads/stores
+            // that operate on a bigger types, hence loosing an information about actual alignment.
+            // AlignmentAnalysis restores alignments based on data-flow analysis, so that MemOpt
+            // can propagate the correct alignment to the merged instructions.
+            mpm.add(new AlignmentAnalysis());
+        }
+#endif
+
         if ((ctx.type == ShaderType::RAYTRACING_SHADER || ctx.hasSyncRTCalls()) &&
             IGC_IS_FLAG_DISABLED(DisablePrepareLoadsStores))
         {
             mpm.add(createPrepareLoadsStoresPass());
         }
 
-        // run AdvMemOpt and MemOPt back-to-back so that we only
+        // run AdvMemOpt and MemOPt back-to-back so that we only
         // need to run WIAnalysis once
         if (IGC_IS_FLAG_ENABLED(EnableAdvMemOpt))
             mpm.add(createAdvMemOptPass());
 
-        if (IGC_IS_FLAG_ENABLED(EnableLdStCombine) &&
-            ctx.type == ShaderType::OPENCL_SHADER)
-        {
-            // start with OCL, will apply to others.
-            //   Once it is stable, no split 64bit store/load anymore.
+        if(IGC_IS_FLAG_SET(DumpRegPressureEstimate)) mpm.add(new IGCRegisterPressurePrinter("final"));
+
+        if (doLdStCombine(&ctx)) {
+            // Once it is stable, no split 64bit store/load anymore.
             mpm.add(createLdStCombinePass());
         }
 
@@ -695,11 +771,6 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         }
 
         mpm.add(createIGCInstructionCombiningPass());
-        if (ctx.type == ShaderType::OPENCL_SHADER &&
-            static_cast<OpenCLProgramContext&>(ctx).m_InternalOptions.KernelDebugEnable)
-        {
-            mpm.add(new ImplicitGIDRestoring());
-        }
     }
 
     if (ctx.hasSyncRTCalls())
@@ -708,19 +779,22 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     }
 
     if (ctx.type == ShaderType::OPENCL_SHADER &&
-        static_cast<OpenCLProgramContext&>(ctx).
-            m_InternalOptions.PromoteStatelessToBindless &&
-        (static_cast<OpenCLProgramContext&>(ctx).
-            m_InternalOptions.UseBindlessLegacyMode ||
-            !ctx.getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired)
-        )
+        static_cast<OpenCLProgramContext&>(ctx).m_InternalOptions.PromoteStatelessToBindless)
     {
-        mpm.add(new PromoteStatelessToBindless());
+        if (static_cast<OpenCLProgramContext&>(ctx).m_InternalOptions.UseBindlessLegacyMode)
+        {
+            mpm.add(new PromoteStatelessToBindless());
+        }
+        else if (!ctx.getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired)
+        {
+            // Advanced bindless mode used by the regular OpenCL compilation path
+            mpm.add(new StatelessToStateful(TargetAddressing::BINDLESS));
+        }
     }
 
     if (!isOptDisabled && useStatelessToStateful(ctx))
     {
-        mpm.add(new StatelessToStateful());
+        mpm.add(new StatelessToStateful(TargetAddressing::BINDFUL));
     }
 
     // Light cleanup for subroutines after cloning. Note that the constant
@@ -756,12 +830,6 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     // Since we don't support switch statements, switch lowering is needed after the last CFG simplication
     mpm.add(llvm::createLowerSwitchPass());
 
-    // There's no particular reason for this exact place, but it should be after LowerGEPForPrivMem
-    if (IGC_IS_FLAG_ENABLED(EnableSplitIndirectEEtoSel))
-    {
-        mpm.add(createSplitIndirectEEtoSelPass());
-    }
-
     // This pass can create constant expression
     if (ctx.m_DriverInfo.HasDoubleLoadStore())
     {
@@ -810,16 +878,6 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         // Last instruction combining pass needs to be before Legalization pass, as it can produce illegal instructions.
         mpm.add(new RemoveCodeAssumptions());
         mpm.add(createIGCInstructionCombiningPass());
-
-        // Optimize lower-level IR
-        if (!fastCompile && !highAllocaPressure && !isPotentialHPCKernel)
-        {
-            if (ctx.type == ShaderType::OPENCL_SHADER &&
-                static_cast<OpenCLProgramContext&>(ctx).m_InternalOptions.KernelDebugEnable)
-            {
-                mpm.add(new ImplicitGIDRestoring());
-            }
-        }
         mpm.add(new GenSpecificPattern());
         // Cases with DPDivSqrtEmu grow significantly.
         // We can disable EarlyCSE when m_hasDPDivSqrtEmu is true,
@@ -834,13 +892,8 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         }
         if (!fastCompile && !highAllocaPressure && !isPotentialHPCKernel && IGC_IS_FLAG_ENABLED(allowLICM) && ctx.m_retryManager.AllowLICM())
         {
-            mpm.add(createDisableLICMForSpecificLoops());
+            mpm.add(createSpecialCasesDisableLICM());
             mpm.add(createLICMPass());
-            if (ctx.type == ShaderType::OPENCL_SHADER ||
-                ctx.type == ShaderType::COMPUTE_SHADER)
-            {
-                mpm.add(new RegPressureLoopControl());
-            }
             mpm.add(llvm::createEarlyCSEPass());
         }
         mpm.add(createAggressiveDCEPass());
@@ -874,41 +927,42 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
 
     // Run address remat after GVN as it may hoist address calculations and
     // create PHI nodes with addresses.
+    if (IGC_IS_FLAG_ENABLED(RematEnable) || (ctx.m_retryManager.AllowCloneAddressArithmetic() && ctx.type == ShaderType::OPENCL_SHADER)) {
 
-    if (IGC_IS_FLAG_ENABLED(EnableRemat)) {
+        if(IGC_GET_FLAG_VALUE(RematInstCombineBefore)) mpm.add(createIGCInstructionCombiningPass());
+        // TODO: This is a workaround that helps to reduce amount of instructions for clone address arithmetic
+        // it helps with chain of instructions like this
+        // %remat12 = add i64 %baseArith, 100780848
+        // %remat13 = add i64 %remat12, %basePtr
+        // %remat14 = add i64 %remat13, %offsetI
+        // %remat15 = add i64 %remat14, %offsetJ
+        // load ...
+        // ....
+        // %remat21 = add i64 %baseArith, 201561696
+        // %remat22 = add i64 %remat21, %basePtr
+        // %remat23 = add i64 %remat22, %offsetI
+        // %remat24 = add i64 %remat23, %offsetJ
+        // load ...
+        // we can compress this chain of instruction into one "add" for each "load"
+        // this is achieved by combining reassoc + cse 3 times (each pair hoists one add)
+        // it should be substituted for general pass when it's implemented
+        //
+        // Now it's accessible through flag, for testing purposes
+        if (IGC_GET_FLAG_VALUE(RematReassocBefore)) {
+            mpm.add(llvm::createReassociatePass());
+            mpm.add(llvm::createEarlyCSEPass());
+            mpm.add(llvm::createReassociatePass());
+            mpm.add(llvm::createEarlyCSEPass());
+            mpm.add(llvm::createReassociatePass());
+            mpm.add(llvm::createEarlyCSEPass());
+        }
 
-      // TODO: This is a workaround that helps to reduce amount of instructions for clone address arithmetic
-      // it helps with chain of instructions like this
-      // %remat12 = add i64 %baseArith, 100780848
-      // %remat13 = add i64 %remat12, %basePtr
-      // %remat14 = add i64 %remat13, %offsetI
-      // %remat15 = add i64 %remat14, %offsetJ
-      // load ...
-      // ....
-      // %remat21 = add i64 %baseArith, 201561696
-      // %remat22 = add i64 %remat21, %basePtr
-      // %remat23 = add i64 %remat22, %offsetI
-      // %remat24 = add i64 %remat23, %offsetJ
-      // load ...
-      // we can compress this chain of instruction into one "add" for each "load"
-      // this is achieved by combining reassoc + cse 3 times (each pair hoists one add)
-      // it should be substituted for general pass when it's implemented
-      //
-      // Now it's accessible through flag, for testing purposes
-
-      if (IGC_GET_FLAG_VALUE(RematReassocBefore)) {
-        mpm.add(llvm::createReassociatePass());
-        mpm.add(llvm::createEarlyCSEPass());
-        mpm.add(llvm::createReassociatePass());
-        mpm.add(llvm::createEarlyCSEPass());
-        mpm.add(llvm::createReassociatePass());
-        mpm.add(llvm::createEarlyCSEPass());
-      }
-
-      mpm.add(createCloneAddressArithmeticPass());
-      // cloneAddressArithmetic leaves old instructions unnecessary
-      // dce pass helps to clean that up
-      mpm.add(createDeadCodeEliminationPass());
+        if(IGC_IS_FLAG_SET(DumpRegPressureEstimate)) mpm.add(new IGCRegisterPressurePrinter("before"));
+        mpm.add(createCloneAddressArithmeticPass());
+        // cloneAddressArithmetic leaves old instructions unnecessary
+        // dce pass helps to clean that up
+        mpm.add(createDeadCodeEliminationPass());
+        if(IGC_IS_FLAG_SET(DumpRegPressureEstimate)) mpm.add(new IGCRegisterPressurePrinter("after"));
     }
 
     mpm.add(createRematAddressArithmeticPass());
@@ -1094,7 +1148,7 @@ bool SimdEarlyCheck(CodeGenContext* ctx)
 
 void destroyShaderMap(CShaderProgram::KernelShaderMap& shaders)
 {
-    for (auto i : shaders)
+    for (const auto& i : shaders)
     {
         CShaderProgram* shader = i.second;
         COMPILER_SHADER_STATS_PRINT(shader->m_shaderStats,
@@ -1198,17 +1252,9 @@ void OptimizeIR(CodeGenContext* const pContext)
     IGC_ASSERT(nullptr != pContext->getModuleMetaData());
     bool NoOpt = pContext->getModuleMetaData()->compOpt.OptDisable;
 
-    alwaysInlineForNoOpt(pContext, NoOpt);
+    DumpHashToOptions(pContext->hash, pContext->type);
 
-    if (pContext->type == ShaderType::OPENCL_SHADER)
-    {
-        if (((OpenCLProgramContext*)pContext)->m_InternalOptions.KernelDebugEnable)
-        {
-            IGCPassManager mpm(pContext, "CleanImplicitId");
-            mpm.add(new CleanImplicitIds());
-            mpm.run(*pContext->getModule());
-        }
-    }
+    alwaysInlineForNoOpt(pContext, NoOpt);
 
     // Insert per-func optimization metadata
     for (auto& F : *pContext->getModule())
@@ -1249,10 +1295,8 @@ void OptimizeIR(CodeGenContext* const pContext)
         mpm.add(new MetaDataUtilsWrapper(pMdUtils, pContext->getModuleMetaData()));
 
         mpm.add(new CodeGenContextWrapper(pContext));
-        DummyPass* dummypass = new DummyPass();
-        mpm.add(dummypass);
             TargetIRAnalysis GenTTgetIIRAnalysis([&](const Function& F) {
-            GenIntrinsicsTTIImpl GTTI(pContext, dummypass);
+            GenIntrinsicsTTIImpl GTTI(pContext);
             return TargetTransformInfo(GTTI);
         });
 
@@ -1316,7 +1360,8 @@ void OptimizeIR(CodeGenContext* const pContext)
 
         if (pContext->m_DriverInfo.CodeSinkingBeforeCFGSimplification())
         {
-            mpm.add(new CodeSinking(true));
+            mpm.add(new HoistCongruentPHI());
+            mpm.add(new CodeSinking());
         }
 #if LLVM_VERSION_MAJOR >= 12
         mpm.add(llvm::createCFGSimplificationPass(SimplifyCFGOptions().hoistCommonInsts(true)));
@@ -1353,6 +1398,15 @@ void OptimizeIR(CodeGenContext* const pContext)
 
         mpm.add(new BreakConstantExpr());
         mpm.add(new IGCConstProp());
+        GFX_ONLY_PASS { mpm.add(createTranslateToProgrammableOffsetsPass()); }
+
+        // This pass needs to be extended for other devices
+        if (pContext->platform.getPlatformInfo().eProductFamily == IGFX_PVC)
+        {
+            mpm.add(new GenerateBlockMemOpsPass());
+        }
+        mpm.add(new BlockMemOpAddrScalarizationPass());
+
         mpm.add(new CustomSafeOptPass());
         if (!pContext->m_DriverInfo.WADisableCustomPass())
         {
@@ -1365,11 +1419,6 @@ void OptimizeIR(CodeGenContext* const pContext)
         }
 
         mpm.add(createIGCInstructionCombiningPass());
-        if (pContext->type == ShaderType::OPENCL_SHADER &&
-            static_cast<OpenCLProgramContext*>(pContext)->m_InternalOptions.KernelDebugEnable)
-        {
-            mpm.add(new ImplicitGIDRestoring());
-        }
         mpm.add(new FCmpPaternMatch());
         mpm.add(llvm::createDeadCodeEliminationPass()); // this should be done both before/after constant propagation
 
@@ -1406,9 +1455,19 @@ void OptimizeIR(CodeGenContext* const pContext)
                 mpm.add(llvm::createLCSSAPass());
                 mpm.add(llvm::createLoopSimplifyPass());
 
+                if (IGC_IS_FLAG_ENABLED(EnableGEPLSR) &&
+                    pContext->type == ShaderType::OPENCL_SHADER &&
+                    pContext->platform.getPlatformInfo().eProductFamily == IGFX_PVC &&
+                    !useStatelessToStateful(*pContext) &&
+                    pContext->m_retryManager.IsFirstTry())
+                {
+                    mpm.add(createGEPLoopStrengthReductionPass(IGC_IS_FLAG_ENABLED(allowLICM) &&
+                            pContext->m_retryManager.AllowLICM()));
+                }
+
                 if (IGC_IS_FLAG_ENABLED(allowLICM) && pContext->m_retryManager.AllowLICM())
                 {
-                    mpm.add(createDisableLICMForSpecificLoops());
+                    mpm.add(createSpecialCasesDisableLICM());
                     int licmTh = IGC_GET_FLAG_VALUE(LICMStatThreshold);
                     mpm.add(new InstrStatistic(pContext, LICM_STAT, InstrStatStage::BEGIN, licmTh));
                     mpm.add(llvm::createLICMPass());
@@ -1428,12 +1487,6 @@ void OptimizeIR(CodeGenContext* const pContext)
                 {
                     mpm.add(createLoopHoistConstant());
                 }
-
-                if (pContext->type == ShaderType::OPENCL_SHADER &&
-                    static_cast<OpenCLProgramContext*>(pContext)->m_InternalOptions.KernelDebugEnable)
-                {
-                    mpm.add(new ImplicitGIDRestoring());
-                }
                 if (IGC_IS_FLAG_ENABLED(EnableAdvCodeMotion) &&
                     pContext->type == ShaderType::OPENCL_SHADER &&
                     !pContext->m_instrTypes.hasSwitch)
@@ -1471,7 +1524,7 @@ void OptimizeIR(CodeGenContext* const pContext)
 
                 if (IGC_IS_FLAG_ENABLED(allowLICM) && pContext->m_retryManager.AllowLICM())
                 {
-                    mpm.add(createDisableLICMForSpecificLoops());
+                    mpm.add(createSpecialCasesDisableLICM());
                     mpm.add(llvm::createLICMPass());
                 }
 
@@ -1497,15 +1550,6 @@ void OptimizeIR(CodeGenContext* const pContext)
                         mpm.add(createSROAPass());
                     }
                 }
-
-                if (pContext->type == ShaderType::OPENCL_SHADER &&
-                    pContext->platform.getPlatformInfo().eProductFamily == IGFX_PVC &&
-                    !useStatelessToStateful(*pContext) &&
-                    pContext->m_retryManager.IsFirstTry())
-                {
-                    //mpm.add(createGEPLoopStrengthReductionPass(IGC_IS_FLAG_ENABLED(allowLICM) &&
-                    //        pContext->m_retryManager.AllowLICM()));
-                }
             }
 
             // Note:
@@ -1573,12 +1617,23 @@ void OptimizeIR(CodeGenContext* const pContext)
                 // jump threading currently causes the atomic_flag test from c11 conformance to fail.  Right now,
                 // only do jump threading if we don't have atomics as using atomics as locks seems to be the most common
                 // case of violating the no independent forward progress clause from the spec.
-                mpm.add(llvm::createJumpThreadingPass());
+
+                // We need to increase default duplication threshold since JumpThreading pass cost estimation does
+                // not consider that not all instructions need to be duplicated.
+                int BBDuplicateThreshold = (pContext->type == ShaderType::OPENCL_SHADER) ? 9 : -1;
+#if LLVM_VERSION_MAJOR >= 15 || LLVM_VERSION_MAJOR < 12
+                // In LLVM-12.x an extra parameter InsertFreezeWhenUnfoldingSelect = false was added
+                // to JumpThreading pass, but since LLVM-15.x it was removed again.
+                mpm.add(llvm::createJumpThreadingPass(BBDuplicateThreshold));
+#else // LLVM_VERSION_MAJOR
+                mpm.add(llvm::createJumpThreadingPass(false, BBDuplicateThreshold));
+#endif // LLVM_VERSION_MAJOR
             }
             mpm.add(llvm::createCFGSimplificationPass());
             mpm.add(llvm::createEarlyCSEPass());
             if (pContext->m_instrTypes.hasNonPrimitiveAlloca)
             {
+                mpm.add(new ShrinkArrayAllocaPass());
                 // run custom safe opts to potentially get rid of indirect
                 // addressing of private arrays, see visitLoadInst
                 mpm.add(new CustomSafeOptPass());
@@ -1610,11 +1665,6 @@ void OptimizeIR(CodeGenContext* const pContext)
 
             // run instruction combining to clean up the code after CFG optimizations
             mpm.add(createIGCInstructionCombiningPass());
-            if (pContext->type == ShaderType::OPENCL_SHADER &&
-                static_cast<OpenCLProgramContext*>(pContext)->m_InternalOptions.KernelDebugEnable)
-            {
-                mpm.add(new ImplicitGIDRestoring());
-            }
 
             mpm.add(llvm::createDeadCodeEliminationPass());
             mpm.add(llvm::createEarlyCSEPass());
