@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -48,6 +48,7 @@ class DeSSA;
 class CoalescingEngine;
 class GenXFunctionGroupAnalysis;
 class VariableReuseAnalysis;
+class ResourceLoopAnalysis;
 class EmitPass;
 
 struct PushInfo;
@@ -95,6 +96,8 @@ public:
 
     CShader(llvm::Function*, CShaderProgram* pProgram);
     virtual ~CShader();
+    CShader(const CShader&) = delete;
+    CShader& operator=(const CShader&) = delete;
     void        Destroy();
     virtual void InitEncoder(SIMDMode simdMode, bool canAbortOnSpill, ShaderDispatchMode shaderMode = ShaderDispatchMode::NOT_APPLICABLE);
     virtual void PreCompile() {}
@@ -105,7 +108,7 @@ public:
     virtual void PreAnalysisPass();
     virtual void ExtractGlobalVariables() {}
     void         EOTURBWrite();
-    CVariable* URBFence();
+    CVariable* URBFence(LSC_SCOPE scope = LSC_SCOPE_LOCAL);
     void         EOTGateway(CVariable* payload = nullptr);
     void         EOTRenderTarget(CVariable* r1,
         bool isPerCoarse);
@@ -151,7 +154,8 @@ public:
     CVariable* LazyCreateCCTupleBackingVariable(
         CoalescingEngine::CCTuple* ccTuple,
         VISA_Type baseType = ISA_TYPE_UD);
-    CVariable* GetSymbol(llvm::Value* value, bool fromConstantPool = false);
+    CVariable* GetSymbol(llvm::Value* value, bool fromConstantPool = false,
+                         e_alignment MinAlign = EALIGN_AUTO);
     void        AddSetup(uint index, CVariable* var);
     bool        AppendPayloadSetup(CVariable* var);
     void        AddPatchTempSetup(CVariable* var);
@@ -240,11 +244,14 @@ public:
     CVariable* GetSP();
     CVariable* GetFP();
     CVariable* GetPrevFP();
+    CVariable* GetARGVReservedVariable(ARG_SPACE_RESERVATION_SLOTS slot);
+    uint32_t GetARGVReservedVariablesTotalSize();
     CVariable* GetARGV();
     CVariable* GetRETV();
     CVariable* GetPrivateBase();
     CVariable* GetImplArgBufPtr();
     CVariable* GetLocalIdBufPtr();
+    CVariable* GetGlobalBufferArg();
     void SaveSRet(CVariable* sretPtr);
     CVariable* GetAndResetSRet();
 
@@ -255,8 +262,6 @@ public:
     void SaveStackState();
     void RestoreStackState();
 
-    void InitializeScratchSurfaceStateAddress();
-
     void RemoveBitRange(CVariable*& src, unsigned removebit, unsigned range);
 
     void        AllocateInput(CVariable* var, uint offset, uint instance = 0, bool forceLiveOut = false);
@@ -267,8 +272,8 @@ public:
     CVariable* GetScalarConstant(llvm::Value* c);
     CVariable* GetUndef(VISA_Type type);
     llvm::Constant* findCommonConstant(llvm::Constant* C, uint elts, uint currentEmitElts, bool& allSame);
-    virtual unsigned int GetGlobalMappingValue(llvm::Value* c);
-    virtual CVariable* GetGlobalMapping(llvm::Value* c);
+    virtual unsigned int GetSLMMappingValue(llvm::Value *c);
+    virtual CVariable *GetSLMMapping(llvm::Value *c);
     CVariable* BitCast(CVariable* var, VISA_Type newType);
     void        CacheArgumentsList();
     virtual void MapPushedInputs();
@@ -292,6 +297,7 @@ public:
     void        SetDominatorTreeHelper(llvm::DominatorTree* DT) { m_DT = DT; }
     void        SetDataLayout(const llvm::DataLayout* DL) { m_DL = DL; }
     void        SetVariableReuseAnalysis(VariableReuseAnalysis* VRA) { m_VRA = VRA; }
+    void        SetResourceLoopAnalysis(ResourceLoopAnalysis *RLA) { m_RLA = RLA; }
     void        SetMetaDataUtils(IGC::IGCMD::MetaDataUtils* pMdUtils) { m_pMdUtils = pMdUtils; }
     void        SetScratchSpaceSize(uint size) { m_ScratchSpaceSize = size; }
 
@@ -315,6 +321,7 @@ public:
 
     GenXFunctionGroupAnalysis* GetFGA() { return m_FGA; }
     bool HasStackCalls() const { return m_HasStackCall; }
+    void SetHasStackCalls(bool hasStackCall) { m_HasStackCall = hasStackCall; }
     bool HasNestedCalls() const { return m_HasNestedCall; }
     bool HasIndirectCalls() const { return m_HasIndirectCall; }
     bool IsIntelSymbolTableVoidProgram() const { return m_IsIntelSymbolTableVoidProgram; }
@@ -537,7 +544,7 @@ public:
 
     static unsigned GetIMEReturnPayloadSize(llvm::GenIntrinsicInst* I);
 
-    void addCVarsForVectorBC(llvm::BitCastInst* BCI, llvm::SmallVector<CVariable*, 8> CVars)
+    void addCVarsForVectorBC(llvm::BitCastInst* BCI, const llvm::SmallVector<CVariable*, 8>& CVars)
     {
         IGC_ASSERT_MESSAGE(m_VectorBCItoCVars.find(BCI) == std::end(m_VectorBCItoCVars), "a variable already exists for this vector bitcast");
         m_VectorBCItoCVars.try_emplace(BCI, CVars);
@@ -574,6 +581,15 @@ public:
 
     // In bytes
     uint32_t getGRFSize() const { return m_Platform->getGRFSize(); }
+
+    // In bytes
+    uint32_t getCVarSize(CVariable* var) const {
+        uint varNumberElement = var->GetNumberElement();
+        uint varElemSize = var->GetElemSize();
+        uint varSize = varNumberElement * varElemSize;
+        return (getGRFSize() >= varSize) ? getGRFSize() : varSize;
+    }
+
     // in DWORDs
     uint32_t getMinPushConstantBufferAlignmentInBytes() const { return m_Platform->getMinPushConstantBufferAlignment() * sizeof(DWORD); }
 
@@ -667,6 +683,7 @@ protected:
     const llvm::DataLayout* m_DL;
     GenXFunctionGroupAnalysis* m_FGA;
     VariableReuseAnalysis* m_VRA;
+    ResourceLoopAnalysis* m_RLA;
 
     uint m_numBlocks;
     IGC::IGCMD::MetaDataUtils* m_pMdUtils;
@@ -720,10 +737,13 @@ protected:
     CVariable* m_FP;
     CVariable* m_SavedFP;
     CVariable* m_ARGV;
+    std::array<CVariable*, NUM_ARG_SPACE_RESERVATION_SLOTS> m_ARGVReservedVariables{};
+    uint32_t m_ARGVReservedVariablesTotalSize = 0;
     CVariable* m_RETV;
     CVariable* m_SavedSRetPtr;
     CVariable* m_ImplArgBufPtr;
     CVariable* m_LocalIdBufPtr;
+    CVariable* m_GlobalBufferArg;
 
     std::vector<USC::SConstantGatherEntry> gatherMap;
     uint     m_ConstantBufferLength;
@@ -821,4 +841,5 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
 void AddAnalysisPasses(CodeGenContext& ctx, IGCPassManager& mpm);
 void destroyShaderMap(CShaderProgram::KernelShaderMap& shaders);
 void unify_opt_PreProcess(CodeGenContext* pContext);
-}
\ No newline at end of file
+
+}
