@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -168,7 +168,7 @@ namespace IGC {
         const llvm::BasicBlock* full_join;
         llvm::DenseSet<llvm::BasicBlock*> influence_region;
         llvm::SmallPtrSet<llvm::BasicBlock*, 4> partial_joins;
-        llvm::BasicBlock* fork_blk;
+        llvm::BasicBlock* fork_blk = nullptr;
     };
 } // namespace IGC
 
@@ -280,12 +280,14 @@ void WIAnalysisRunner::init(
     IGC::IGCMD::MetaDataUtils* MDUtils,
     IGC::CodeGenContext* CGCtx,
     IGC::ModuleMetaData* ModMD,
-    IGC::TranslationTable* TransTable)
+    IGC::TranslationTable* TransTable,
+    bool ForCodegen)
 {
     m_func = F;
     this->LI = LI;
     this->DT = DT;
     this->PDT = PDT;
+    this->ForCodegen = ForCodegen;
     m_pMdUtils = MDUtils;
     m_CGCtx = CGCtx;
     m_ModMD = ModMD;
@@ -519,9 +521,8 @@ void WIAnalysisRunner::updateArgsDependency(llvm::Function* pF)
     //    order (intel_reqd_workgroup_walk_order()) and work group size (reqd_work_group_size()),
     //    we may derive that some of local IDs are uniform.
     bool localX_uniform = false, localY_uniform = false, localZ_uniform = false;
-    // DispatchOCLWGInLinearOrder should be removed after testing the guarded code.
-    if (!IsSubroutine &&
-        IGC_IS_FLAG_ENABLED(DispatchOCLWGInLinearOrder))
+
+    if (!IsSubroutine)
     {
         checkLocalIdUniform(pF, localX_uniform, localY_uniform, localZ_uniform);
     }
@@ -847,6 +848,17 @@ void WIAnalysisRunner::calculate_dep(const Value* val)
         // Spec enforces subgroup broadcast to use thread-uniform local ID.
         if (isWaveBroadcastIndex(inst))
         {
+#ifdef _DEBUG
+            // Print warning exactly once per kernel.
+            static SmallPtrSet<Function*, 8> detectedKernels;
+            if (dep > WIAnalysis::UNIFORM_THREAD && !detectedKernels.count(m_func))
+            {
+                detectedKernels.insert(m_func);
+                std::string msg = "Detected llvm.genx.GenISA.WaveBroadcast with potentially non-uniform LocalID in kernel " + m_func->getName().str()
+                    + "; such operation doesn't meet specification of OpGroupBroadcast and can lead to unexpected results.";
+                m_CGCtx->EmitWarning(msg.c_str());
+            }
+#endif // DEBUG
             dep = WIAnalysis::UNIFORM_THREAD;
         }
 
@@ -854,7 +866,7 @@ void WIAnalysisRunner::calculate_dep(const Value* val)
         if (!hasOriginal || dep != orig)
         {
             // i1 instructions used in phi cannot be uniform as it may prevent us from removing the phi of 1
-            if (inst->getType()->isIntegerTy(1) && WIAnalysis::isDepUniform(dep) && HasPhiUse(inst))
+            if (ForCodegen && inst->getType()->isIntegerTy(1) && WIAnalysis::isDepUniform(dep) && HasPhiUse(inst))
             {
                 dep = WIAnalysis::RANDOM;
             }
@@ -956,7 +968,14 @@ void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
             if (NumPreds <= 1)
                 continue;
         }
-        auto PJDom = DT->getNode(PJ)->getIDom()->getBlock();
+        auto *PJNode = DT->getNode(PJ);
+        if (PJNode == nullptr)
+        {
+            IGC_ASSERT(!DT->isReachableFromEntry(PJ));
+            continue;
+        }
+        auto PJDom = PJNode->getIDom()->getBlock();
+
         // If both partial-join and it IDom are in partial-join region
         // there are cases in which phi-nodes in partial-joins are not
         // relevant to the cbr under the investigation
@@ -1392,6 +1411,7 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         intrinsic_name == llvm_waveAll ||
         intrinsic_name == llvm_waveClustered ||
         intrinsic_name == llvm_ld_ptr ||
+        intrinsic_name == llvm_ldlptr ||
         (IGC_IS_FLAG_DISABLED(DisableUniformTypedAccess) && intrinsic_name == llvm_typed_read) ||
         intrinsic_name == llvm_add_pair ||
         intrinsic_name == llvm_sub_pair ||
@@ -1410,11 +1430,9 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         GII_id == GenISAIntrinsic::GenISA_srnd_ftohf ||
         GII_id == GenISAIntrinsic::GenISA_srnd_hftobf8 ||
         GII_id == GenISAIntrinsic::GenISA_ftotf32 ||
-        GII_id == GenISAIntrinsic::GenISA_tf32tof ||
         GII_id == GenISAIntrinsic::GenISA_GlobalBufferPointer ||
         GII_id == GenISAIntrinsic::GenISA_LocalBufferPointer ||
         GII_id == GenISAIntrinsic::GenISA_InlinedData ||
-        GII_id == GenISAIntrinsic::GenISA_TileYOffset ||
         GII_id == GenISAIntrinsic::GenISA_GetShaderRecordPtr ||
         GII_id == GenISAIntrinsic::GenISA_URBWrite ||
         GII_id == GenISAIntrinsic::GenISA_URBRead ||
@@ -1428,6 +1446,7 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         GII_id == GenISAIntrinsic::GenISA_add_rtz  ||
         GII_id == GenISAIntrinsic::GenISA_slice_id ||
         GII_id == GenISAIntrinsic::GenISA_subslice_id  ||
+        GII_id == GenISAIntrinsic::GenISA_logical_subslice_id ||
         GII_id == GenISAIntrinsic::GenISA_dual_subslice_id ||
         GII_id == GenISAIntrinsic::GenISA_eu_id        ||
         GII_id == GenISAIntrinsic::GenISA_eu_thread_id ||
@@ -1452,11 +1471,16 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         GII_id == GenISAIntrinsic::GenISA_getStageInGridSize ||
         GII_id == GenISAIntrinsic::GenISA_getSyncBuffer ||
         GII_id == GenISAIntrinsic::GenISA_getRtGlobalBufferPtr ||
+        GII_id == GenISAIntrinsic::GenISA_GetGlobalBufferArg ||
         GII_id == GenISAIntrinsic::GenISA_GetImplicitBufferPtr ||
         GII_id == GenISAIntrinsic::GenISA_GetLocalIdBufferPtr  ||
+        GII_id == GenISAIntrinsic::GenISA_ReadFromReservedArgSpace ||
         GII_id == GenISAIntrinsic::GenISA_getAssertBufferPtr  ||
         GII_id == GenISAIntrinsic::GenISA_staticConstantPatchValue ||
-        GII_id == GenISAIntrinsic::GenISA_bitcastfromstruct)
+        GII_id == GenISAIntrinsic::GenISA_bitcastfromstruct ||
+        GII_id == GenISAIntrinsic::GenISA_bitcasttostruct ||
+        GII_id == GenISAIntrinsic::GenISA_LSC2DBlockCreateAddrPayload ||
+        GII_id == GenISAIntrinsic::GenISA_LSC2DBlockCopyAddrPayload)
     {
         switch (GII_id)
         {
@@ -1473,12 +1497,17 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             return WIAnalysis::UNIFORM_THREAD;
         case GenISAIntrinsic::GenISA_slice_id:
         case GenISAIntrinsic::GenISA_subslice_id:
+        case GenISAIntrinsic::GenISA_logical_subslice_id:
         case GenISAIntrinsic::GenISA_dual_subslice_id:
             // Make sure they are UNIFORM_WORKGROUP
             //return WIAnalysis::UNIFORM_WORKGROUP;
             return WIAnalysis::UNIFORM_THREAD;
         case GenISAIntrinsic::GenISA_GetImplicitBufferPtr:
         case GenISAIntrinsic::GenISA_GetLocalIdBufferPtr:
+        case GenISAIntrinsic::GenISA_GetGlobalBufferArg:
+        case GenISAIntrinsic::GenISA_ReadFromReservedArgSpace:
+        case GenISAIntrinsic::GenISA_LSC2DBlockCreateAddrPayload:
+        case GenISAIntrinsic::GenISA_LSC2DBlockCopyAddrPayload:
             return WIAnalysis::UNIFORM_THREAD;
         case GenISAIntrinsic::GenISA_getR0:
         case GenISAIntrinsic::GenISA_getPayloadHeader:
@@ -1511,12 +1540,6 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             }
         }
 
-        if (GII_id == GenISAIntrinsic::GenISA_TileYOffset)
-        {
-            {
-                return WIAnalysis::RANDOM;
-            }
-        }
 
         if (intrinsic_name == llvm_sgv)
         {
@@ -1529,8 +1552,7 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             case VIEWPORT_INDEX: // viewport index from PS payload
             {
                 IGC_ASSERT(m_CGCtx->type == ShaderType::PIXEL_SHADER);
-                const bool hasMultipolyDispatch =
-                    m_CGCtx->platform.supportDualSimd8PS();
+                const bool hasMultipolyDispatch = m_CGCtx->platform.hasDualKSPPS() || m_CGCtx->platform.supportDualSimd8PS();
                 return hasMultipolyDispatch ? WIAnalysis::RANDOM : WIAnalysis::UNIFORM_THREAD;
             }
             case POSITION_X: // position from VUE header in GS or pixel position X in PS
@@ -2067,22 +2089,29 @@ void WIAnalysisRunner::checkLocalIdUniform(
     auto funcMD = modMD->FuncMD.find(F);
 
     int32_t WO_0 = -1, WO_1 = -1, WO_2 = -1;
-    if (funcMD != modMD->FuncMD.end())
+    if (funcMD == modMD->FuncMD.end())
     {
-        WorkGroupWalkOrderMD workGroupWalkOrder = funcMD->second.workGroupWalkOrder;
-        if (workGroupWalkOrder.dim0 || workGroupWalkOrder.dim1 || workGroupWalkOrder.dim2)
-        {
-            WO_0 = workGroupWalkOrder.dim0;
-            WO_1 = workGroupWalkOrder.dim1;
-            WO_2 = workGroupWalkOrder.dim2;
-        }
+        return;
     }
 
+    WorkGroupWalkOrderMD workGroupWalkOrder = funcMD->second.workGroupWalkOrder;
+    if (!workGroupWalkOrder.dim0 && !workGroupWalkOrder.dim1 && !workGroupWalkOrder.dim2)
+    {
+        return;
+    }
+
+    WO_0 = workGroupWalkOrder.dim0;
+    WO_1 = workGroupWalkOrder.dim1;
+    WO_2 = workGroupWalkOrder.dim2;
+
+    // We expect that the work group walk order is always fixed.
+    IGC_ASSERT(WO_0 == 0 && WO_1 == 1 && WO_2 == 2);
+
     uint32_t simdSize = 0;
     SubGroupSizeMetaDataHandle subGroupSize = funcInfoMD->getSubGroupSize();
     if (subGroupSize->hasValue())
     {
-        simdSize = (uint32_t)subGroupSize->getSIMD_size();
+        simdSize = (uint32_t)subGroupSize->getSIMDSize();
     }
     simdSize = simdSize >= 8 ? simdSize : 32;
 
@@ -2128,22 +2157,18 @@ void WIAnalysisRunner::checkLocalIdUniform(
         IsLzUniform = true;
     }
 
-    if (IGC_IS_FLAG_ENABLED(DispatchOCLWGInLinearOrder) ||
-        (WO_0 == 0 && WO_1 == 1 && WO_2 == 2))
+    // linear order dispatch
+    uint32_t XxY = X * Y;
+    if (X > 0 && (X % simdSize) == 0)
     {
-        // linear order dispatch
-        uint32_t XxY = X * Y;
-        if (X > 0 && (X % simdSize) == 0)
-        {
-            // X is multiple of simdSize
-            IsLyUniform = true;
-            IsLzUniform = true;
-        }
-        else if (X > 0 && Y > 0 && (XxY % simdSize) == 0)
-        {
-            // X*Y is multiple of simdSize
-            IsLzUniform = true;
-        }
+        // X is multiple of simdSize
+        IsLyUniform = true;
+        IsLzUniform = true;
+    }
+    else if (X > 0 && Y > 0 && (XxY % simdSize) == 0)
+    {
+        // X*Y is multiple of simdSize
+        IsLzUniform = true;
     }
 }
 
