@@ -78,7 +78,8 @@ namespace IGC
             IGCMD::MetaDataUtils* MDUtils,
             CodeGenContext* CGCtx,
             ModuleMetaData* ModMD,
-            TranslationTable* TransTable);
+            TranslationTable* TransTable,
+            bool ForCodegen = true);
 
         WIAnalysisRunner(
             llvm::Function* F,
@@ -88,13 +89,13 @@ namespace IGC
             IGCMD::MetaDataUtils* MDUtils,
             CodeGenContext* CGCtx,
             ModuleMetaData* ModMD,
-            TranslationTable* TransTable)
+            TranslationTable* TransTable,
+            bool ForCodegen = true)
         {
-            init(F, LI, DT, PDT, MDUtils, CGCtx, ModMD, TransTable);
+            init(F, LI, DT, PDT, MDUtils, CGCtx, ModMD, TransTable, ForCodegen);
         }
 
         WIAnalysisRunner() {}
-        ~WIAnalysisRunner() {}
 
         bool run();
 
@@ -261,17 +262,17 @@ namespace IGC
 
         /// Iteratively one set holds the changed from the previous iteration and
         /// the other holds the new changed values from the current iteration.
-        std::vector<const llvm::Value*> m_changed1;
-        std::vector<const llvm::Value*> m_changed2;
+        std::vector<const llvm::Value*> m_changed1{};
+        std::vector<const llvm::Value*> m_changed2{};
         /// ptr to m_changed1, m_changed2
-        std::vector<const llvm::Value*>* m_pChangedOld;
-        std::vector<const llvm::Value*>* m_pChangedNew;
+        std::vector<const llvm::Value*>* m_pChangedOld = nullptr;
+        std::vector<const llvm::Value*>* m_pChangedNew = nullptr;
 
         /// <summary>
         ///  hold the vector-defs that are promoted from an uniform alloca
         ///  therefore, need to be forced into uniform no matter what.
         /// </summary>
-        std::vector<const llvm::Value*> m_forcedUniforms;
+        std::vector<const llvm::Value*> m_forcedUniforms{};
 
         llvm::Function* m_func = nullptr;
         llvm::LoopInfo *LI = nullptr;
@@ -291,6 +292,10 @@ namespace IGC
 
         // For dumpping WIA info per each invocation
         static llvm::DenseMap<const llvm::Function*, int> m_funcInvocationId;
+
+        // Is this analysis providing information that could be used for late
+        // stage codegen, or is it just used to determine uniformity early on?
+        bool ForCodegen = true;
     };
 
     /// @brief Work Item Analysis class used to provide information on
@@ -375,7 +380,6 @@ namespace IGC
                 Dep == WIDependancy::UNIFORM_WORKGROUP ||
                 Dep == WIDependancy::UNIFORM_THREAD;
         }
-    private:
         WIAnalysisRunner Runner;
     };
 
