@@ -23,6 +23,29 @@ DECLARE_OPCODE(GenISA_gather4ptr, GenISAIntrinsic, llvm_gather4ptr, false, false
 DECLARE_OPCODE(GenISA_gather4Cptr, GenISAIntrinsic, llvm_gather4Cptr, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_gather4POptr, GenISAIntrinsic, llvm_gather4POptr, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_gather4POCptr, GenISAIntrinsic, llvm_gather4POCptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_sampleMlodptr, GenISAIntrinsic, llvm_sample_mlodptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_sampleCMlodptr, GenISAIntrinsic, llvm_sample_c_mlodptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_sampleBCMlodptr, GenISAIntrinsic, llvm_sample_bc_mlodptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_sampleDCMlodptr, GenISAIntrinsic, llvm_sample_dc_mlodptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_samplePOptr, GenISAIntrinsic, llvm_sample_poptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_samplePOBptr, GenISAIntrinsic, llvm_sample_pobptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_samplePOLptr, GenISAIntrinsic, llvm_sample_polptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_samplePOCptr, GenISAIntrinsic, llvm_sample_pocptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_samplePODptr, GenISAIntrinsic, llvm_sample_podptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_samplePOLCptr, GenISAIntrinsic, llvm_sample_polcptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4Iptr, GenISAIntrinsic, llvm_gather4Iptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4Bptr, GenISAIntrinsic, llvm_gather4Bptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4Lptr, GenISAIntrinsic, llvm_gather4Lptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4ICptr, GenISAIntrinsic, llvm_gather4ICptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4LCptr, GenISAIntrinsic, llvm_gather4LCptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedptr, GenISAIntrinsic, llvm_gather4POPackedptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedLptr, GenISAIntrinsic, llvm_gather4POPackedLptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedBptr, GenISAIntrinsic, llvm_gather4POPackedBptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedIptr, GenISAIntrinsic, llvm_gather4POPackedIptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedCptr, GenISAIntrinsic, llvm_gather4POPackedCptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedICptr, GenISAIntrinsic, llvm_gather4POPackedICptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_gather4POPackedLCptr, GenISAIntrinsic, llvm_gather4POPackedLCptr, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_ldlptr, GenISAIntrinsic, llvm_ldlptr, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_ldmsptr, GenISAIntrinsic, llvm_ldmsptr, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_ldmsptr16bit, GenISAIntrinsic, llvm_ldmsptr16bit, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_ldmcsptr, GenISAIntrinsic, llvm_ldmcsptr, false, false, false, false, false, false, false)
@@ -62,6 +85,7 @@ DECLARE_OPCODE(floor, Intrinsic, llvm_floor, true, true, true, true, true, false
 DECLARE_OPCODE(ceil, Intrinsic, llvm_ceil, true, true, true, true, true, false, false)
 DECLARE_OPCODE(trunc, Intrinsic, llvm_round_z, true, true, true, true, true, false, false)
 DECLARE_OPCODE(ctlz, Intrinsic, llvm_ctlz, false, false, true, false, true, false, false)
+DECLARE_OPCODE(GenISA_inv, GenISAIntrinsic, llvm_inv, false, false, false, false, true, false, false)
 DECLARE_OPCODE(GenISA_ROUNDNE, GenISAIntrinsic, llvm_roundne, true, true, true, true, true, false, false)
 DECLARE_OPCODE(GenISA_frc, GenISAIntrinsic, llvm_frc, true, false, true, true, false, false, false)
 DECLARE_OPCODE(InsertElement, Instruction, llvm_insert, true, false, false, false, false, false, false)
@@ -164,11 +188,21 @@ DECLARE_OPCODE(GenISA_firstbitHi, GenISAIntrinsic, llvm_fbh, false, false, false
 DECLARE_OPCODE(GenISA_firstbitLo, GenISAIntrinsic, llvm_fbl, false, false, false, false, true, false, false)
 DECLARE_OPCODE(GenISA_firstbitShi, GenISAIntrinsic, llvm_fbh_shi, false, false, false, false, true, false, false)
 DECLARE_OPCODE(GenISA_intatomicraw, GenISAIntrinsic, llvm_int_atomic_raw, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_floatatomicraw, GenISAIntrinsic, llvm_float_atomic_raw, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_intatomicrawA64, GenISAIntrinsic, llvm_int_atomic_raw_A64, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_floatatomicrawA64, GenISAIntrinsic, llvm_float_atomic_raw_A64, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_dwordatomicstructured, GenISAIntrinsic, llvm_dword_atomic_structured, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_floatatomicstructured, GenISAIntrinsic, llvm_float_atomic_structured, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_intatomictyped, GenISAIntrinsic, llvm_int_atomic_typed, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_floatatomictyped, GenISAIntrinsic, llvm_float_atomic_typed, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_icmpxchgatomicraw, GenISAIntrinsic, llvm_icmpxchg_atomic_raw, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_fcmpxchgatomicraw, GenISAIntrinsic, llvm_fcmpxchg_atomic_raw, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_icmpxchgatomicrawA64, GenISAIntrinsic, llvm_icmpxchg_atomic_raw_A64, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_fcmpxchgatomicrawA64, GenISAIntrinsic, llvm_fcmpxchg_atomic_raw_A64, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_cmpxchgatomicstructured, GenISAIntrinsic, llvm_cmpxchg_atomic_structured, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_fcmpxchgatomicstructured, GenISAIntrinsic, llvm_fcmpxchg_atomic_structured, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_icmpxchgatomictyped, GenISAIntrinsic, llvm_icmpxchg_atomic_typed, false, false, false, false, false, true, false)
+DECLARE_OPCODE(GenISA_fcmpxchgatomictyped, GenISAIntrinsic, llvm_fcmpxchg_atomic_typed, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_atomiccounterinc, GenISAIntrinsic, llvm_atomic_counter_inc, false, false, false, false, false, true, false)
 DECLARE_OPCODE(GenISA_atomiccounterpredec, GenISAIntrinsic, llvm_atomic_counter_predec, false, false, false, false, false, true, false)
 
