@@ -33,13 +33,14 @@ namespace IGC
         bool allowConstantCoalescing;
         bool allowLargeGRF;
         bool allowLoadSinking;
+        bool forceIndirectCallsInSyncRT;
         unsigned nextState;
     };
 
     static const RetryState RetryTable[] = {
-       // adrCl  licm  codSk AdrSk  Slice  PrivM  VISAP  URBWr  Coals  GRF    loadSk
-        { false, true,  true, false, false, true,  true,  true,  true,  false, false, 1 },
-        { true,  false, true, true,  true,  false, false, false, false, true,  true, 500 }
+       // adrCl  licm   codSk AdrSk  Slice  PrivM  VISAP  URBWr  Coals  GRF    loadSk, SyncRT
+        { false, true,  true, false, false, true,  true,  true,  true,  false, false,  false, 1 },
+        { true,  false, true, true,  true,  false, false, false, false, true,  true,   true, 500 }
     };
 
     static constexpr size_t RetryTableSize = sizeof(RetryTable) / sizeof(RetryState);
@@ -152,6 +153,13 @@ namespace IGC
         return RetryTable[id].allowLargeGRF;
     }
 
+    bool RetryManager::ForceIndirectCallsInSyncRT() const
+    {
+        unsigned id = GetRetryId();
+        IGC_ASSERT(id < RetryTableSize);
+        return RetryTable[id].forceIndirectCallsInSyncRT;
+    }
+
     bool RetryManager::AllowLoadSinking(Function* F) const
     {
         unsigned id = GetPerFuncRetryStateId(F);
@@ -177,6 +185,10 @@ namespace IGC
             (stateId < RetryTableSize && RetryTable[stateId].nextState >= RetryTableSize));
     }
 
+    bool RetryManager::Trigger2xGRFRetry() const
+    {
+        return (lastSpillSize > IGC_GET_FLAG_VALUE(CSSpillThreshold2xGRFRetry));
+    }
     unsigned RetryManager::GetRetryId() const
     {
         return stateId;
@@ -276,6 +288,13 @@ namespace IGC
             IGC_ASSERT(currentShader);
             IGC_ASSERT(previousShader);
 
+            // basically a small work around, if we have high spilling kernel on our hands, we are not afraid to use
+            // pre-retry shader, when we have less spills than retry one has
+            unsigned int Threshold = IGC_GET_FLAG_VALUE(RetryRevertExcessiveSpillingKernelThreshold);
+            bool IsExcessiveSpillKernel = currentShader->m_spillSize >= Threshold;
+            float SpillCoefficient = float(IGC_GET_FLAG_VALUE(RetryRevertExcessiveSpillingKernelCoefficient)) / 100.0f;
+            if (IsExcessiveSpillKernel) threshold = SpillCoefficient;
+
             // Check if current shader spill is larger than previous shader spill
             // Threshold flag controls comparison tolerance - i.e. A threshold of 2.0 means that the
             // current shader spill must be 2x larger than previous spill to be considered "better".
@@ -370,6 +389,51 @@ namespace IGC
         }
     }
 
+    void CodeGenContext::print(llvm::raw_ostream& stream) const
+    {
+#define PRINT_CTX_MEMBER(member) stream << "\n" << #member << ": " << member;
+
+        // TODO: Automate, see comment for CodeGenContext::print's declaration.
+        PRINT_CTX_MEMBER(HdcEnableIndexSize);
+        PRINT_CTX_MEMBER(LtoUsedMask);
+        PRINT_CTX_MEMBER(m_checkFastFlagPerInstructionInCustomUnsafeOptPass);
+        PRINT_CTX_MEMBER(m_ConstantBufferCount);
+        PRINT_CTX_MEMBER(m_ConstantBufferReplaceShaderPatternsSize);
+        PRINT_CTX_MEMBER(m_ConstantBufferReplaceSize);
+        PRINT_CTX_MEMBER(m_ConstantBufferUsageMask);
+        PRINT_CTX_MEMBER(m_constantPayloadNextAvailableGRFOffset);
+        PRINT_CTX_MEMBER(m_disableICBPromotion);
+        PRINT_CTX_MEMBER(m_dxbcCount);
+        PRINT_CTX_MEMBER(m_enableFunctionPointer);
+        PRINT_CTX_MEMBER(m_enableSampleMultiversioning);
+        PRINT_CTX_MEMBER(m_enableSimdVariantCompilation);
+        PRINT_CTX_MEMBER(m_enableSubroutine);
+        PRINT_CTX_MEMBER(m_floatDenormMode16);
+        PRINT_CTX_MEMBER(m_floatDenormMode32);
+        PRINT_CTX_MEMBER(m_floatDenormMode64);
+        PRINT_CTX_MEMBER(m_floatDenormModeBFTF);
+        PRINT_CTX_MEMBER(m_ForceEarlyZMathCheck);
+        PRINT_CTX_MEMBER(m_hasDPDivSqrtEmu);
+        PRINT_CTX_MEMBER(m_hasDPEmu);
+        PRINT_CTX_MEMBER(m_hasEmu64BitInsts);
+        PRINT_CTX_MEMBER(m_hasGlobalInPrivateAddressSpace);
+        PRINT_CTX_MEMBER(m_hasLegacyDebugInfo);
+        PRINT_CTX_MEMBER(m_hasStackCalls);
+        PRINT_CTX_MEMBER(m_hasVendorExtension);
+        PRINT_CTX_MEMBER(m_highPsRegisterPressure);
+        PRINT_CTX_MEMBER(m_inputCount);
+        PRINT_CTX_MEMBER(m_numGradientSinked);
+        PRINT_CTX_MEMBER(m_NumGRFPerThread);
+        PRINT_CTX_MEMBER(m_numPasses);
+        PRINT_CTX_MEMBER(m_sampler);
+        PRINT_CTX_MEMBER(m_SIMDInfo);
+        PRINT_CTX_MEMBER(m_src1RemovedForBlendOpt);
+        PRINT_CTX_MEMBER(m_tempCount);
+        PRINT_CTX_MEMBER(m_threadCombiningOptDone);
+
+        stream << "\n\n";
+    }
+
     void CodeGenContext::initLLVMContextWrapper(bool createResourceDimTypes)
     {
         llvmCtxWrapper = new LLVMContextWrapper(createResourceDimTypes);
@@ -387,10 +451,12 @@ namespace IGC
     }
 
     IGCLLVM::Module* CodeGenContext::getModule() const { return module; }
+    std::vector<std::string> CodeGenContext::getEntryNames() const { return entry_names; }
 
     static void initCompOptionFromRegkey(CodeGenContext* ctx)
     {
         SetCurrentDebugHash(ctx->hash);
+        SetCurrentEntryPoints(ctx->entry_names);
 
         CompOptions& opt = ctx->getModuleMetaData()->compOpt;
 
@@ -403,11 +469,30 @@ namespace IGC
     void CodeGenContext::setModule(llvm::Module* m)
     {
         module = (IGCLLVM::Module*)m;
+        this->setEntryNames(module);
         m_pMdUtils = new IGC::IGCMD::MetaDataUtils(m);
         modMD = new IGC::ModuleMetaData();
         initCompOptionFromRegkey(this);
     }
 
+    // get the entry point names from the root entrypoint node
+    void CodeGenContext::setEntryNames(llvm::Module* m)
+    {
+        for (auto& func : *m)
+        {
+            if (func.isDeclaration())
+                continue;
+
+            const auto funcName = func.getName().str();
+            if (!funcName.empty())
+            {
+                this->entry_names.emplace_back(funcName);
+            }
+        }
+    }
+    void CodeGenContext::clearEntryNames() {
+        this->entry_names.clear();
+    }
     // Several clients explicitly delete module without resetting module to null.
     // This causes the issue later when the dtor is invoked (trying to delete a
     // dangling pointer again). This function is used to replace any explicit
@@ -426,7 +511,6 @@ namespace IGC
 
     IGC::ModuleMetaData* CodeGenContext::getModuleMetaData() const
     {
-        IGC_ASSERT_MESSAGE(nullptr != modMD, "Module Metadata is not initialized");
         return modMD;
     }
 
@@ -497,18 +581,24 @@ namespace IGC
             }
         }
         m_enableSubroutine = EnableSubroutine;
-        m_hasStackCalls = EnableStackFuncs;
+        m_hasStackCalls |= EnableStackFuncs;
     }
 
     // check if DP emu is required
     void CodeGenContext::checkDPEmulationEnabled()
     {
+        // TODO: the method should also check DivSqrt Emulation Mode
         if ((IGC_IS_FLAG_ENABLED(ForceDPEmulation) ||
             (m_DriverInfo.NeedFP64(platform.getPlatformInfo().eProductFamily) && platform.hasNoFP64Inst())) ||
             (getCompilerOption().FP64GenEmulationEnabled && platform.emulateFP64ForPlatformsWithoutHWSupport()))
         {
             m_hasDPEmu = true;
         }
+
+        if (getCompilerOption().FP64GenConvEmulationEnabled && platform.emulateFP64ForPlatformsWithoutHWSupport())
+        {
+            m_hasDPConvEmu = true;
+        }
     }
 
     void CodeGenContext::InitVarMetaData() {}
@@ -580,12 +670,19 @@ namespace IGC
         return MD->functionType == KernelFunction;
     }
 
-    static void findCallingKernels
-        (const CodeGenContext *ctx, const llvm::Function *F, llvm::SmallPtrSetImpl<const llvm::Function *> &kernels)
+    static void findCallingKernels( const CodeGenContext *ctx,
+                                    const llvm::Function *F,
+                                    llvm::SmallPtrSetImpl<const llvm::Function *> &kernels,
+                                    SmallPtrSet<const llvm::Function*, 32>& visited)
     {
         if (F == nullptr || kernels.count(F))
             return;
 
+        // Check if function was already visited during search
+        if (visited.find(F) != visited.end())
+            return;
+        visited.insert(F);
+
         for (const llvm::User *U : F->users()) {
             auto *CI = llvm::dyn_cast<llvm::CallInst>(U);
             if (CI == nullptr)
@@ -601,7 +698,7 @@ namespace IGC
             }
             // Caller is not a kernel, try to check which kerneles might
             // be calling it:
-            findCallingKernels(ctx, caller, kernels);
+            findCallingKernels(ctx, caller, kernels, visited);
         }
     }
 
@@ -673,10 +770,9 @@ namespace IGC
 #endif
     }
 
-    void CodeGenContext::EmitError(std::ostream &OS, const char* errorstr, const llvm::Value* context) const
+    void CodeGenContext::EmitMessage(std::ostream &OS, const char* messagestr, const llvm::Value* context) const
     {
-        OS << "\nerror: ";
-        OS << errorstr;
+        OS << messagestr;
         // Try to get debug location to print out the relevant info.
         if (const llvm::Instruction *I = llvm::dyn_cast_or_null<llvm::Instruction>(context)) {
             if (const llvm::DILocation *DL = I->getDebugLoc()) {
@@ -693,7 +789,8 @@ namespace IGC
             // might be using this function.
             } else {
                 llvm::SmallPtrSet<const llvm::Function *, 16> kernels;
-                findCallingKernels(this, F, kernels);
+                llvm::SmallPtrSet<const llvm::Function*, 32> visited;
+                findCallingKernels(this, F, kernels, visited);
 
                 const size_t kernelsCount = kernels.size();
                 OS << "\nin function: '" << demangleFuncName(std::string(F->getName())) << "' ";
@@ -710,18 +807,19 @@ namespace IGC
                 }
             }
         }
-        OS << "\nerror: backend compiler failed build.\n";
     }
 
     void CodeGenContext::EmitError(const char* errorstr, const llvm::Value *context)
     {
-        EmitError(this->oclErrorMessage, errorstr, context);
+        this->oclErrorMessage << "\nerror: ";
+        EmitMessage(this->oclErrorMessage, errorstr, context);
+        this->oclErrorMessage << "\nerror: backend compiler failed build.\n";
     }
 
-    void CodeGenContext::EmitWarning(const char* warningstr)
+    void CodeGenContext::EmitWarning(const char* warningstr, const llvm::Value* context)
     {
         this->oclWarningMessage << "\nwarning: ";
-        this->oclWarningMessage << warningstr;
+        EmitMessage(this->oclWarningMessage, warningstr, context);
         this->oclWarningMessage << "\n";
     }
 
@@ -754,8 +852,6 @@ namespace IGC
     /// there is no user-forced setting
     uint32_t CodeGenContext::getNumGRFPerThread(bool returnDefault)
     {
-        constexpr uint32_t DEFAULT_TOTAL_GRF_NUM = 128;
-
         if (m_NumGRFPerThread)
             return m_NumGRFPerThread;
 
@@ -766,20 +862,18 @@ namespace IGC
         }
         if (getModuleMetaData()->csInfo.forceTotalGRFNum != 0)
         {
-            {
-                m_NumGRFPerThread = getModuleMetaData()->csInfo.forceTotalGRFNum;
-                return m_NumGRFPerThread;
-            }
+            m_NumGRFPerThread = getModuleMetaData()->csInfo.forceTotalGRFNum;
+            return m_NumGRFPerThread;
         }
 
-
         // read value from CompOptions first
-        DWORD GRFNum4RQToUse = getModuleMetaData()->compOpt.ForceLargeGRFNum4RQ ? 0x100 : 0;
+        DWORD GRFNum4RQToUse = getModuleMetaData()->compOpt.ForceLargeGRFNum4RQ ? 256 : 0;
 
         // override if reg key value is set
-        GRFNum4RQToUse = IGC_IS_FLAG_ENABLED( TotalGRFNum4RQ ) ? IGC_GET_FLAG_VALUE( TotalGRFNum4RQ ) : GRFNum4RQToUse;
+        GRFNum4RQToUse = IGC_IS_FLAG_ENABLED( TotalGRFNum4RQ ) ?
+            IGC_GET_FLAG_VALUE( TotalGRFNum4RQ ) : GRFNum4RQToUse;
 
-        if (hasSyncRTCalls() && (this->type != ShaderType::RAYTRACING_SHADER) && GRFNum4RQToUse != 0)
+        if (hasSyncRTCalls() && GRFNum4RQToUse != 0)
         {
             m_NumGRFPerThread = GRFNum4RQToUse;
             return m_NumGRFPerThread;
@@ -838,10 +932,47 @@ namespace IGC
             "IGC::PositionOnlyVertexShader") != nullptr;
     }
 
+
+
+    bool CodeGenContext::isBufferBoundsChecking() const
+    {
+        return false;
+    }
+
     void CodeGenContext::setFlagsPerCtx()
     {
     }
 
+    // Returns the SIMD mode of a kernel based on a platform and settings flags.
+    // VS, DS, HS, GS currently supported only.
+    SIMDMode CodeGenContext::GetSIMDMode()
+    {
+        SIMDMode simdMode = SIMDMode::UNKNOWN;
+
+        uint32_t forcedSIMDSize = 0;
+        bool isGeomFF = (type == ShaderType::VERTEX_SHADER) ||
+            (type == ShaderType::HULL_SHADER) ||
+            (type == ShaderType::DOMAIN_SHADER) ||
+            (type == ShaderType::GEOMETRY_SHADER);
+
+        if (isGeomFF)
+        {
+            // Step 1: get platform-dependent default mode.
+            {
+                simdMode = platform.getMinDispatchMode();
+            }
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(0, "Incorrect shader type");
+        }
+
+        IGC_ASSERT_MESSAGE(!hasSyncRTCalls() || (simdMode <= platform.getPreferredRayQuerySIMDSize()),
+            "Unsupported SIMD mode for RayQuery");
+
+        return simdMode;
+    }
+
     // [used by shader dump] create unqiue id, starting from 1, for each
     // entry function.
     // Each entry function has 1-1 map b/w its name and its dump name.
