@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -69,7 +69,6 @@ namespace llvm
 }
 
 #define MAX_VSHADER_INPUT_REGISTERS_PACKAGEABLE 32
-static const unsigned int g_c_Max_PS_attributes = 32;
 
 namespace IGCOpts
 {
@@ -136,10 +135,7 @@ namespace IGC
         unsigned int    m_funcRelocationTableSize = 0;
         unsigned int    m_funcRelocationTableEntries = 0;
         RelocListTy     m_relocs;                  // duplicated information of m_funcRelocationTable, for zebin
-        void* m_funcAttributeTable = nullptr;
-        unsigned int    m_funcAttributeTableSize = 0;
-        unsigned int    m_funcAttributeTableEntries = 0;
-        FuncAttrListTy  m_funcAttrs;               // duplicated information of m_funcAttributeTable, for zebin
+        FuncAttrListTy  m_funcAttrs;
         void* m_globalHostAccessTable = nullptr;
         unsigned int    m_globalHostAccessTableSize = 0;
         unsigned int    m_globalHostAccessTableEntries = 0;
@@ -164,6 +160,8 @@ namespace IGC
 
         unsigned int m_numThreads = 0;
 
+        unsigned int m_perThreadArgumentStackSize = 0;
+
         void Destroy()
         {
             if (m_programBin)
@@ -178,10 +176,6 @@ namespace IGC
             {
                 IGC::aligned_free(m_debugDataGenISA);
             }
-            if (m_funcAttributeTable)
-            {
-                IGC::aligned_free(m_funcAttributeTable);
-            }
             if (m_funcSymbolTable)
             {
                 free(m_funcSymbolTable);
@@ -309,65 +303,65 @@ namespace IGC
 
     struct SInstrTypes
     {
-        bool CorrelatedValuePropagationEnable;
-        bool hasMultipleBB;
-        bool hasCmp;
-        bool hasSwitch;
-        bool hasPhi;
-        bool hasLoadStore;
-        bool hasIndirectCall;
-        bool hasInlineAsm;
-        bool hasInlineAsmPointerAccess;
-        bool hasIndirectBranch;
-        bool hasFunctionAddressTaken;
-        bool hasSel;
-        bool hasPointer;
-        bool hasLocalLoadStore;
-        bool hasGlobalLoad; // has (stateless) loads from global addresspace
-        bool hasGlobalStore; // has (stateless) stores to global addresspace
-        bool hasStorageBufferLoad; // has (stateful) loads from storage buffers (UAV/SSBO)
-        bool hasStorageBufferStore; // has (stateful) stores to storage buffers (UAV/SSBO)
-        bool hasSubroutines;
-        bool hasPrimitiveAlloca;
-        bool hasNonPrimitiveAlloca;
-        bool hasReadOnlyArray;
-        bool hasBuiltin;
-        bool hasFRem;
-        bool psHasSideEffect;     //<! only relevant to pixel shader, has other memory writes besides RTWrite
-        bool hasGenericAddressSpacePointers;
-        bool hasDebugInfo;        //<! true only if module contains debug info !llvm.dbg.cu
-        bool hasAtomics;
-        bool hasDiscard;
-        bool hasTypedRead;
-        bool hasTypedwrite;
-        bool mayHaveIndirectOperands;  //<! true if code may have indirect operands like r5[a0].
+        bool CorrelatedValuePropagationEnable{};
+        bool hasMultipleBB{};
+        bool hasCmp{};
+        bool hasSwitch{};
+        bool hasPhi{};
+        bool hasLoadStore{};
+        bool hasIndirectCall{};
+        bool hasInlineAsm{};
+        bool hasInlineAsmPointerAccess{};
+        bool hasIndirectBranch{};
+        bool hasFunctionAddressTaken{};
+        bool hasSel{};
+        bool hasPointer{};
+        bool hasLocalLoadStore{};
+        bool hasGlobalLoad{}; // has (stateless) loads from global addresspace
+        bool hasGlobalStore{}; // has (stateless) stores to global addresspace
+        bool hasStorageBufferLoad{}; // has (stateful) loads from storage buffers (UAV/SSBO)
+        bool hasStorageBufferStore{}; // has (stateful) stores to storage buffers (UAV/SSBO)
+        bool hasSubroutines{};
+        bool hasPrimitiveAlloca{};
+        bool hasNonPrimitiveAlloca{};
+        bool hasReadOnlyArray{};
+        bool hasBuiltin{};
+        bool hasFRem{};
+        bool psHasSideEffect{};     //<! only relevant to pixel shader, has other memory writes besides RTWrite
+        bool hasGenericAddressSpacePointers{};
+        bool hasDebugInfo{};        //<! true only if module contains debug info !llvm.dbg.cu
+        bool hasAtomics{};
+        bool hasDiscard{};
+        bool hasTypedRead{};
+        bool hasTypedwrite{};
+        bool mayHaveIndirectOperands{};  //<! true if code may have indirect operands like r5[a0].
         // true if shader may have indirect texture or buffer.
         // Note: does not check for indirect sampler
-        bool mayHaveIndirectResources;
-        bool hasUniformAssumptions;
-        bool hasPullBary;
-        bool sampleCmpToDiscardOptimizationPossible;
-        bool hasRuntimeValueVector;
-        bool hasDynamicGenericLoadStore;
-        bool hasUnmaskedRegion;
-        unsigned int numCall;
-        unsigned int numBarrier;
-        unsigned int numLoadStore;
-        unsigned int numWaveIntrinsics;
-        unsigned int numAtomics;
-        unsigned int numTypedReadWrite;
-        unsigned int numAllInsts;
-        unsigned int sampleCmpToDiscardOptimizationSlot;
-        unsigned int numSample;
-        unsigned int numBB;
-        unsigned int numLoopInsts;
-        unsigned int numOfLoop;
-        unsigned int numInsts;    //<! measured after optimization, used as a compiler heuristic
-        unsigned int numAllocaInsts;
-        unsigned int numPsInputs;
-        unsigned int numGlobalInsts;
-        unsigned int numLocalInsts;
-        unsigned int numSamplesVaryingResource; //<! measured before CodeGen for scheduling heuristic
+        bool mayHaveIndirectResources{};
+        bool hasUniformAssumptions{};
+        bool hasPullBary{};
+        bool sampleCmpToDiscardOptimizationPossible{};
+        bool hasRuntimeValueVector{};
+        bool hasDynamicGenericLoadStore{};
+        bool hasUnmaskedRegion{};
+        unsigned int numCall{};
+        unsigned int numBarrier{};
+        unsigned int numLoadStore{};
+        unsigned int numWaveIntrinsics{};
+        unsigned int numAtomics{};
+        unsigned int numTypedReadWrite{};
+        unsigned int numAllInsts{};
+        unsigned int sampleCmpToDiscardOptimizationSlot{};
+        unsigned int numSample{};
+        unsigned int numBB{};
+        unsigned int numLoopInsts{};
+        unsigned int numOfLoop{};
+        unsigned int numInsts{};    //<! measured after optimization, used as a compiler heuristic
+        unsigned int numAllocaInsts{};
+        unsigned int numPsInputs{};
+        unsigned int numGlobalInsts{};
+        unsigned int numLocalInsts{};
+        unsigned int numSamplesVaryingResource{}; //<! measured before CodeGen for scheduling heuristic
     };
 
     struct SSimplePushInfo
@@ -439,7 +433,8 @@ namespace IGC
         SIMD_SKIP_PERF,      // 7: skip this SIMD due to performance concern (dx12 + discard, MRT, etc) or other reasons.
         SIMD_SKIP_ML,        // 8: skip this SIMD due to ML engine prediction.
         SIMD_FORCE_CONTENT,  // 9: force this simd due to shader content (simd32 if WaveActive, barriers + interlocks)
-        SIMD_INFO_RESERVED   // 10: *** If new entry is added, make sure it still fits in m_SIMDInfo ***
+        SIMD_FORCE_HINT,     // 10: force this simd by hint(s) (now for WaveSize only)
+        SIMD_INFO_RESERVED   // 11: *** If new entry is added, make sure it still fits in m_SIMDInfo ***
     };
 
     enum SIMDInfoOffset
@@ -447,6 +442,7 @@ namespace IGC
         SIMD8_OFFSET = 0,
         SIMD16_OFFSET = SIMD_INFO_RESERVED,
         SIMD32_OFFSET = SIMD_INFO_RESERVED*2,
+        DUAL_SIMD8_OFFSET = SIMD_INFO_RESERVED * 3,
     };
 
     struct SKernelProgram
@@ -484,7 +480,7 @@ namespace IGC
 
         SSimplePushInfo simplePushInfoArr[g_c_maxNumberOfBufferPushed];
 
-        uint64_t    SIMDInfo;
+        uint64_t    SIMDInfo = 0;
         void* m_StagingCtx;
         bool m_RequestStage2;
     };
@@ -526,8 +522,6 @@ namespace IGC
         // TODO: need to separate out bindless and raygen into two structs
         // for both DX and Vulkan.
 
-        void*         ThreadPayloadData = nullptr;
-        unsigned int  TotalDataLength   = 0;
         // dynamically select between the 1D and 2D layout at runtime based
         // on the size of the dispatch.
         uint32_t      DimX1D            = 0;
@@ -547,6 +541,10 @@ namespace IGC
 
         // Shader has LSC store messages with non-default L1 cache control
         bool HasLscStoresWithNonDefaultL1CacheControls = false;
+
+        // This is just diagnostic information to track whether we picked
+        // the retry shader
+        bool BetterThanPrev = false;
     };
 
     struct SRayTracingShadersGroup
@@ -741,6 +739,8 @@ namespace IGC
     public:
         RetryManager();
         ~RetryManager();
+        RetryManager(const RetryManager&) = delete;
+        RetryManager& operator=(const RetryManager&) = delete;
 
         bool AdvanceState();
         bool AllowLICM(llvm::Function* F = nullptr) const;
@@ -753,10 +753,12 @@ namespace IGC
         bool AllowLargeURBWrite(llvm::Function* F = nullptr) const;
         bool AllowConstantCoalescing(llvm::Function* F = nullptr) const;
         bool AllowLargeGRF(llvm::Function* F = nullptr) const;
+        bool ForceIndirectCallsInSyncRT() const;
         bool AllowLoadSinking(llvm::Function* F = nullptr) const;
         void SetFirstStateId(int id);
         bool IsFirstTry() const;
         bool IsLastTry() const;
+        bool Trigger2xGRFRetry() const;
         unsigned GetRetryId() const;
         unsigned GetPerFuncRetryStateId(llvm::Function* F) const;
 
@@ -889,9 +891,12 @@ namespace IGC
         Float_DenormMode    m_floatDenormMode16 = FLOAT_DENORM_FLUSH_TO_ZERO;
         Float_DenormMode    m_floatDenormMode32 = FLOAT_DENORM_FLUSH_TO_ZERO;
         Float_DenormMode    m_floatDenormMode64 = FLOAT_DENORM_FLUSH_TO_ZERO;
+        Float_DenormMode    m_floatDenormModeBFTF = FLOAT_DENORM_FLUSH_TO_ZERO;
 
         PushConstantMode m_pushConstantMode = PushConstantMode::DEFAULT;
 
+        static constexpr uint32_t DEFAULT_TOTAL_GRF_NUM = 128;
+
         SInstrTypes m_instrTypes = {};
         SInstrTypes m_instrTypesAfterOpts = {};
         // The module contains global variables with private address space.
@@ -913,6 +918,8 @@ namespace IGC
         bool m_ForceEarlyZMathCheck = false;
         // Adding multiversioning to partially redundant samples, if AIL is on.
         bool m_enableSampleMultiversioning = false;
+        // Re-enabling SIMD16 for compute shader if spill oversizes on SIMD32
+        bool m_fallbackCSSIMD16 = false;
 
         bool m_src1RemovedForBlendOpt = false;
         llvm::AssemblyAnnotationWriter* annotater = nullptr;
@@ -952,8 +959,9 @@ namespace IGC
         void* gtpin_init = nullptr;
         bool m_hasLegacyDebugInfo = false;
         bool m_hasEmu64BitInsts = false;
-        bool m_hasDPDivSqrtEmu = false;
         bool m_hasDPEmu = false;
+        bool m_hasDPDivSqrtEmu = false;
+        bool m_hasDPConvEmu = false;
 
         // Flag for staged compilation
         CG_FLAG_t m_CgFlag = FLAG_CG_ALL_SIMDS;
@@ -977,10 +985,9 @@ namespace IGC
         std::vector<int> m_gsNonDefaultIdxMap;
         std::vector<int> m_psIdxMap;
         DWORD LtoUsedMask = 0;
-        uint64_t m_SIMDInfo;
+        uint64_t m_SIMDInfo = 0;
         uint32_t HdcEnableIndexSize = 0;
         std::vector<RoutingIndex> HdcEnableIndexValues;
-        uint32_t m_NumGRFPerThread = 0;
 
         // Flag per function/kernel informing about if it has
         // expensive loops and needs trigger retry compilation
@@ -997,7 +1004,7 @@ namespace IGC
         bool m_checkFastFlagPerInstructionInCustomUnsafeOptPass = false;
         // Map to store global offsets in original global buffer
         std::map<std::string, uint64_t> inlineProgramScopeGlobalOffsets;
-
+        std::vector<std::string> entry_names;
     private:
         //For storing error message
         std::stringstream oclErrorMessage;
@@ -1012,6 +1019,7 @@ namespace IGC
         /// input: IGC MetaData Utils
         IGC::IGCMD::MetaDataUtils* m_pMdUtils = nullptr;
         IGC::ModuleMetaData* modMD = nullptr;
+        uint32_t m_NumGRFPerThread = 0;
 
         virtual void setFlagsPerCtx();
 
@@ -1055,12 +1063,21 @@ namespace IGC
         CodeGenContext(CodeGenContext&) = delete;
         CodeGenContext& operator =(CodeGenContext&) = delete;
 
+        // TODO: Right now CodeGenContext::print method must be manually updated for each
+        // new member added. Modify the printer to automatically support new members based
+        // on some "printable" metadata available with member's definition.
+        // Possible solution: TableGen.
+        void print(llvm::raw_ostream& stream) const;
+
         void initLLVMContextWrapper(bool createResourceDimTypes = true);
         llvm::LLVMContext* getLLVMContext() const;
         IGC::IGCMD::MetaDataUtils* getMetaDataUtils() const;
         IGCLLVM::Module* getModule() const;
+        std::vector<std::string> getEntryNames() const;
 
         void setModule(llvm::Module* m);
+        void setEntryNames(llvm::Module* m);
+        void clearEntryNames();
         // Several clients explicitly delete module without resetting module to null.
         // This causes the issue later when the dtor is invoked (trying to delete a
         // dangling pointer again). This function is used to replace any explicit
@@ -1073,11 +1090,13 @@ namespace IGC
         void checkDPEmulationEnabled();
         virtual void InitVarMetaData();
         virtual ~CodeGenContext();
+        CodeGenContext(const CodeGenContext&) = delete;
+        CodeGenContext& operator=(const CodeGenContext&) = delete;
         void clear();
         void clearMD();
-        void EmitError(std::ostream &OS, const char* errorstr, const llvm::Value *context) const;
+        void EmitMessage(std::ostream &OS, const char* errorstr, const llvm::Value *context) const;
         void EmitError(const char* errorstr, const llvm::Value *context);
-        void EmitWarning(const char* warningstr);
+        void EmitWarning(const char* warningstr, const llvm::Value *context = nullptr);
         inline bool HasError() const { return !this->oclErrorMessage.str().empty(); }
         inline bool HasWarning() const { return !this->oclWarningMessage.str().empty(); }
         inline const std::string GetWarning() { return this->oclWarningMessage.str(); }
@@ -1090,6 +1109,7 @@ namespace IGC
         virtual uint32_t getExpGRFSize() const;
         virtual uint32_t getNumGRFPerThread(bool returnDefault = true);
         virtual void setNumGRFPerThread(uint32_t value) { m_NumGRFPerThread = value; }
+        virtual bool isAutoGRFSelectionEnabled() const { return false; };
         virtual bool forceGlobalMemoryAllocation() const;
         virtual bool allocatePrivateAsGlobalBuffer() const;
         virtual bool noLocalToGenericOptionEnabled() const;
@@ -1100,6 +1120,7 @@ namespace IGC
         virtual uint32_t getIntelScratchSpacePrivateMemoryMinimalSizePerThread() const;
         virtual bool enableZEBinary() const;
         bool isPOSH() const;
+        virtual bool isBufferBoundsChecking() const;
 
         UserAddrSpaceMD& getUserAddrSpaceMD() {
             IGC_ASSERT(llvmCtxWrapper);
@@ -1131,6 +1152,10 @@ namespace IGC
                 }
                 break;
 
+            case ShaderDispatchMode::DUAL_SIMD8:
+                offset = DUAL_SIMD8_OFFSET;
+                break;
+
             default:
                 break;
             }
@@ -1140,17 +1165,22 @@ namespace IGC
         void SetSIMDInfo(SIMDInfoBit bit, SIMDMode simd, ShaderDispatchMode mode)
         {
             unsigned int offset = GetSIMDInfoOffset(simd, mode);
-            m_SIMDInfo |= 1ULL << (bit + offset);
+            unsigned int shift = bit + offset;
+            IGC_ASSERT(shift < 64);
+            m_SIMDInfo |= 1ULL << shift;
         }
 
         void ClearSIMDInfo(SIMDMode simd, ShaderDispatchMode mode)
         {
             unsigned int offset = GetSIMDInfoOffset(simd, mode);
+            IGC_ASSERT(offset < 64);
             m_SIMDInfo &= ~(0xffULL << offset);
         }
 
         uint64_t GetSIMDInfo() { return m_SIMDInfo; }
 
+        SIMDMode GetSIMDMode();
+
         virtual llvm::Optional<SIMDMode> knownSIMDSize() const {
             return llvm::None;
         }
