@@ -37,8 +37,6 @@ namespace IGC
 
         CustomSafeOptPass();
 
-        ~CustomSafeOptPass() {}
-
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
         {
             AU.addRequired<CodeGenContextWrapper>();
@@ -76,6 +74,8 @@ namespace IGC
         void visitXor(llvm::Instruction& XorInstr);
         void visitShuffleIndex(llvm::CallInst* I);
         void visitSelectInst(llvm::SelectInst& S);
+        void mergeDotAddToDp4a(llvm::CallInst* I);
+
         //
         // IEEE Floating point arithmetic is not associative.  Any pattern
         // match that changes the order or paramters is unsafe.
@@ -113,8 +113,6 @@ namespace IGC
 
         TrivialLocalMemoryOpsElimination();
 
-        ~TrivialLocalMemoryOpsElimination() {}
-
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
         {
             AU.addRequired<CodeGenContextWrapper>();
@@ -152,8 +150,6 @@ namespace IGC
 
         GenSpecificPattern();
 
-        ~GenSpecificPattern() {}
-
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
         {
             AU.setPreservesCFG();
@@ -177,6 +173,7 @@ namespace IGC
         void visitZExtInst(llvm::ZExtInst& I);
         void visitCastInst(llvm::CastInst& I);
         void visitIntToPtr(llvm::IntToPtrInst& I);
+        void visitMul(llvm::BinaryOperator& I);
         void visitSDiv(llvm::BinaryOperator& I);
         void visitTruncInst(llvm::TruncInst& I);
         void visitBitCastInst(llvm::BitCastInst& I);
@@ -198,8 +195,6 @@ namespace IGC
 
         FCmpPaternMatch();
 
-        ~FCmpPaternMatch() {}
-
         virtual bool runOnFunction(llvm::Function& F) override;
 
         virtual llvm::StringRef getPassName() const override
@@ -217,8 +212,6 @@ namespace IGC
 
         IGCConstProp(bool enableSimplifyGEP = false);
 
-        ~IGCConstProp() {}
-
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
         {
             AU.addRequired<llvm::TargetLibraryInfoWrapperPass>();
@@ -235,14 +228,13 @@ namespace IGC
         }
 
     private:
-        llvm::Module* module;
+        llvm::Module* module = nullptr;
         llvm::Constant* replaceShaderConstant(llvm::Instruction* inst);
         llvm::Constant* ConstantFoldCmpInst(llvm::CmpInst* inst);
         llvm::Constant* ConstantFoldExtractElement(llvm::ExtractElementInst* inst);
         llvm::Constant* ConstantFoldCallInstruction(llvm::CallInst* inst);
         bool simplifyAdd(llvm::BinaryOperator* BO);
         bool simplifyGEP(llvm::GetElementPtrInst* GEP);
-        bool m_enableMathConstProp;
         bool m_enableSimplifyGEP;
         const llvm::DataLayout* m_TD;
         llvm::TargetLibraryInfo* m_TLI;
