@@ -15,7 +15,6 @@ SPDX-License-Identifier: MIT
 
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/Config/llvm-config.h"
-#include "WrapperLLVM/Utils.h"
 
 #include "llvm/Analysis/CodeMetrics.h"
 #include "llvm/Analysis/LoopInfo.h"
@@ -32,12 +31,6 @@ using namespace IGC;
 
 namespace llvm {
 
-    char DummyPass::ID = 0;
-    void initializeDummyPassPass(PassRegistry& Registry);
-    DummyPass::DummyPass() : ImmutablePass(ID) {
-        initializeDummyPassPass(*PassRegistry::getPassRegistry());
-    }
-
     bool GenIntrinsicsTTIImpl::isLoweredToCall(const Function* F)
     {
         if (GenISAIntrinsic::isIntrinsic(F))
@@ -91,6 +84,8 @@ namespace llvm {
             llvm::BasicBlock* BB = const_cast<llvm::BasicBlock*>(&*BBI);
             for (auto II = BB->begin(); II != BB->end(); II++)
             {
+                if (IGCLLVM::isDebugOrPseudoInst(*II))
+                    continue;
                 if (llvm::GenIntrinsicInst* pIntrinsic = llvm::dyn_cast<llvm::GenIntrinsicInst>(II))
                 {
                     if (CheckSendMsg && isSendMessage(pIntrinsic))
@@ -292,9 +287,9 @@ namespace llvm {
 
             // Find at least one base address, such that all loads
             // from it can be replaced by registers
-            for (auto LIIterator : LoadInstructions) {
+            for (const auto& LIIterator : LoadInstructions) {
                 bool Found = true;
-                for (auto LI : LIIterator.second)
+                for (const auto& LI : LIIterator.second)
                     Found &= canReplaceWithRegisters(LI, L, SE);
                 if (Found) {
                     UP.UpperBound = true;
@@ -310,7 +305,7 @@ namespace llvm {
         if (!ExitingBlock || !L->isLoopExiting(ExitingBlock))
             ExitingBlock = L->getExitingBlock();
         if (ExitingBlock)
-            TripCount = IGCLLVM::getref(SE).getSmallConstantTripCount(L, ExitingBlock);
+            TripCount = SE.getSmallConstantTripCount(L, ExitingBlock);
 
         // Do not enable partial unrolling if the loop counter is float. It can cause precision issue.
         if (ExitingBlock) {
@@ -423,7 +418,7 @@ namespace llvm {
         int instCount = instructionList.size();
 
         // Check if the specific basic block has block read or write.
-        auto hasBlockReadWrite = [](BasicBlock* BB) -> bool {
+        auto hasBlockReadWrite = [](BasicBlock* BB) {
             for (auto I = BB->begin(), E = BB->end(); I != E; ++I)
                 if (auto GII = dyn_cast<GenIntrinsicInst>(I))
                     switch (GII->getIntrinsicID()) {
@@ -594,6 +589,20 @@ namespace llvm {
         , const User* U
 #endif
     ) {
+        // The extra cost of speculative execution for math intrinsics
+        if (auto *II = dyn_cast_or_null<IntrinsicInst>(U)) {
+            if (Intrinsic::ID IID = II->getIntrinsicID()) {
+                switch (IID) {
+                    case Intrinsic::cos:
+                    case Intrinsic::sin:
+                    case Intrinsic::sqrt:
+                        return TTI::TCC_Expensive;
+                    default:
+                        break;
+                }
+            }
+        }
+
         IGC::CodeGenContext* CGC = this->ctx;
         if (!CGC->enableFunctionCall() && !GenISAIntrinsic::isIntrinsic(F) &&
             !F->isIntrinsic()) {
@@ -626,6 +635,20 @@ namespace llvm {
     llvm::InstructionCost GenIntrinsicsTTIImpl::getUserCost(const User* U, ArrayRef<const Value*> Operands, TTI::TargetCostKind CostKind)
 #endif
     {
+        // The extra cost of speculative execution for math intrinsics
+        if (auto *II = dyn_cast_or_null<IntrinsicInst>(U)) {
+            if (Intrinsic::ID IID = II->getIntrinsicID()) {
+                switch (IID) {
+                    case Intrinsic::cos:
+                    case Intrinsic::sin:
+                    case Intrinsic::sqrt:
+                        return TTI::TCC_Expensive;
+                    default:
+                        break;
+                }
+            }
+        }
+
         const Function* F = dyn_cast<Function>(U);
         if (F != nullptr)
         {
@@ -651,6 +674,3 @@ namespace llvm {
 #endif
 
 } // namespace llvm
-// Register the basic pass.
-INITIALIZE_PASS(DummyPass, "gen-tti-dummy-pass",
-    "Dummy Pass for GenTTIImpl", false, true)
