@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -24,6 +24,7 @@ void initializeAlignmentAnalysisPass(llvm::PassRegistry&);
 void initializePreBIImportAnalysisPass(llvm::PassRegistry&);
 void initializeBIImportPass(llvm::PassRegistry&);
 void initializeBlockCoalescingPass(llvm::PassRegistry&);
+void initializeBlockMemOpAddrScalarizationPassPass(llvm::PassRegistry&);
 void initializeBreakConstantExprPass(llvm::PassRegistry&);
 void initializeBuiltinCallGraphAnalysisPass(llvm::PassRegistry&);
 void initializeBuiltinsConverterPass(llvm::PassRegistry&);
@@ -62,6 +63,7 @@ void initializeGASResolvingPass(llvm::PassRegistry&);
 void initializeGASRetValuePropagatorPass(llvm::PassRegistry&);
 void initializeStaticGASResolutionPass(llvm::PassRegistry&);
 void initializeLowerGPCallArgPass(llvm::PassRegistry&);
+void initializeGenerateBlockMemOpsPassPass(llvm::PassRegistry&);
 void initializeGenericAddressAnalysisPass(llvm::PassRegistry&);
 void initializeGenericAddressDynamicResolutionPass(llvm::PassRegistry&);
 void initializeGenFDIVEmulationPass(llvm::PassRegistry&);
@@ -91,8 +93,6 @@ void initializeInsertGenericPtrArithmeticMetadataPass(llvm::PassRegistry&);
 void initializeImageFuncResolutionPass(llvm::PassRegistry&);
 void initializeImageFuncsAnalysisPass(llvm::PassRegistry&);
 void initializeImage3dToImage2darrayPass(llvm::PassRegistry&);
-void initializeImplicitGlobalIdPass(llvm::PassRegistry&);
-void initializeCleanImplicitIdsPass(llvm::PassRegistry&);
 void initializeInlineLocalsResolutionPass(llvm::PassRegistry&);
 void initializeInlineUnmaskedFunctionsPassPass(llvm::PassRegistry&);
 void initializeInsertBranchOptPass(llvm::PassRegistry&);
@@ -104,6 +104,7 @@ void initializeLegalizeResourcePointerPass(llvm::PassRegistry&);
 void initializeLegalizeFunctionSignaturesPass(llvm::PassRegistry&);
 void initializeLiveVarsAnalysisPass(llvm::PassRegistry&);
 void initializeLogicalAndToBranchPass(llvm::PassRegistry&);
+void initializeLowerByValAttributePass(llvm::PassRegistry&);
 void initializeLowerGEPForPrivMemPass(llvm::PassRegistry&);
 void initializeLowerImplicitArgIntrinsicsPass(llvm::PassRegistry&);
 void initializeLowPrecisionOptPass(llvm::PassRegistry&);
@@ -137,6 +138,7 @@ void initializeResolveOCLAtomicsPass(llvm::PassRegistry&);
 void initializeResolveSampledImageBuiltinsPass(llvm::PassRegistry&);
 void initializeResourceAllocatorPass(llvm::PassRegistry&);
 void initializeRewriteLocalSizePass(llvm::PassRegistry&);
+void initializeFPRoundingModeCoalescingPass(llvm::PassRegistry&);
 void initializeRuntimeValueLegalizationPassPass(llvm::PassRegistry&);
 void initializeRuntimeValueVectorExtractPassPass(llvm::PassRegistry&);
 void initializeSampleCmpToDiscardPass(llvm::PassRegistry&);
@@ -169,6 +171,7 @@ void initializeLSCCacheOptimizationPassPass(llvm::PassRegistry&);
 void initializeBindlessKernelArgLoweringPassPass(llvm::PassRegistry&);
 void initializeSplitPreparePassPass(llvm::PassRegistry&);
 void initializeSplitAsyncPassPass(llvm::PassRegistry&);
+void initializePrepareSyncRTHelpersPass(llvm::PassRegistry&);
 void initializeSyncHandlingPassPass(llvm::PassRegistry&);
 void initializeBindlessInlineDataPassPass(llvm::PassRegistry&);
 void initializeRayTracingConstantCoalescingPassPass(llvm::PassRegistry&);
@@ -187,6 +190,8 @@ void initializeTraceRayInlineLoweringPassPass(llvm::PassRegistry&);
 void initializeRayTracingAddressSpaceAAWrapperPassPass(llvm::PassRegistry&);
 void initializeRayTracingMemDSEPassPass(llvm::PassRegistry&);
 void initializeOverrideTMaxPassPass(llvm::PassRegistry&);
+void initializeTranslateToProgrammableOffsetsPassPass(llvm::PassRegistry&);
+void initializeDynamicRayManagementPassPass(llvm::PassRegistry&);
 void initializeVectorBitCastOptPass(llvm::PassRegistry&);
 void initializeVectorPreProcessPass(llvm::PassRegistry&);
 void initializeVectorProcessPass(llvm::PassRegistry&);
@@ -202,8 +207,9 @@ void initializePingPongTexturesAnalysisPass(llvm::PassRegistry&);
 void initializePingPongTexturesOptPass(llvm::PassRegistry&);
 void initializeSampleMultiversioningPass(llvm::PassRegistry&);
 void initializeLoopCanonicalizationPass(llvm::PassRegistry&);
+void initializeLoopSplitWidePHIsPass(llvm::PassRegistry&);
 void initializeLoopHoistConstantPass(llvm::PassRegistry&);
-void initializeDisableLICMForSpecificLoopsPass(llvm::PassRegistry&);
+void initializeSpecialCasesDisableLICMPass(llvm::PassRegistry&);
 void initializeMemOptPass(llvm::PassRegistry&);
 void initializeLdStCombinePass(llvm::PassRegistry&);
 void initializeBIFTransformsPass(llvm::PassRegistry&);
@@ -235,9 +241,11 @@ void initializeUnreachableHandlingPass(llvm::PassRegistry&);
 void initializeBreakdownIntrinsicPassPass(llvm::PassRegistry&);
 void initializeCatchAllLineNumberPass(llvm::PassRegistry&);
 void initializeDebugInfoPassPass(llvm::PassRegistry&);
+void initializeIGCLivenessAnalysisPass(llvm::PassRegistry&);
+void initializeIGCRegisterPressurePrinterPass(llvm::PassRegistry&);
+void initializeIGCFunctionExternalRegPressureAnalysisPass(llvm::PassRegistry&);
 void initializePromoteConstantStructsPass(llvm::PassRegistry&);
 void initializeLowerInvokeSIMDPass(llvm::PassRegistry&);
-void initializeImplicitGIDRestoringPass(llvm::PassRegistry&);
 void initializeRemoveCodeAssumptionsPass(llvm::PassRegistry&);
 void initializePromoteBoolsPass(llvm::PassRegistry&);
 void initializeResolveConstExprCallsPass(llvm::PassRegistry&);
@@ -245,3 +253,6 @@ void initializeHandleSpirvDecorationMetadataPass(llvm::PassRegistry&);
 void initializeNontemporalLoadsAndStoresInAssertPass(llvm::PassRegistry&);
 void initializeHandleDevicelibAssertPass(llvm::PassRegistry&);
 void initializeStackOverflowDetectionPassPass(llvm::PassRegistry &);
+void initializeBufferBoundsCheckingPass(llvm::PassRegistry&);
+void initializeBufferBoundsCheckingPatcherPass(llvm::PassRegistry&);
+void initializeBfloatFuncsResolutionPass(llvm::PassRegistry &);
