@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -14,6 +14,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPush.hpp"
 #include <llvmWrapper/IR/IRBuilder.h>
 #include "llvmWrapper/IR/Attributes.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include <llvm/IR/Function.h>
 #include <llvmWrapper/IR/Instructions.h>
 #include <llvmWrapper/IR/CallSite.h>
@@ -186,7 +187,7 @@ static std::string updatedMangleName(const std::string& FuncName, const std::str
 static std::string updateSPIRmangleName(StringRef FuncName, const MangleSubstTy& MangleSubst)
 {
     std::string NewNameStr = FuncName.str();
-    for (auto Key : MangleSubst)
+    for (const auto& Key : MangleSubst)
     {
         auto Mangle = Key.first.str();
         auto NewMangle = Key.second.str();
@@ -331,7 +332,7 @@ std::unique_ptr<llvm::Module> BIImport::Construct(Module& M, CLElfLib::CElfReade
     unsigned numOfHeaders = pElfReader->GetElfHeader()->NumSectionHeaderEntries;
     std::vector<std::unique_ptr<llvm::Module>> elf_index(numOfHeaders);
 
-    std::function<void(Function*)> Explore = [&](Function* pRoot) -> void
+    std::function<void(Function*)> Explore = [&](Function* pRoot)
     {
         TFunctionsVec calledFuncs;
         GetCalledFunctions(pRoot, calledFuncs);
@@ -535,67 +536,6 @@ void BIImport::fixSPIRFunctionsReturnType(Module& M)
         F->eraseFromParent();
 }
 
-// The built-in definition returns i32, however, at this point the function call
-// that has been added for round_to_tf32() call returns a float (as the orig
-// matrix type was float).  So we need to: 1) Change the return type of the
-// function declaration to int so that it matches the builtin definition; 2)
-// Cast the returned value of the function back to float so that the previous
-// users of the return value are happy.
-void fixRoundToTF32ReturnType(Module &M) {
-    SmallPtrSet<Function *, 8> funcsToRemove;
-    for (auto &F : M) {
-        if (!F.isDeclaration())
-            continue;
-        auto FuncName = F.getName();
-
-        if (!FuncName.contains("OpRoundFToTF32INTEL") ||
-            FuncName.contains("_old"))
-            continue;
-        if (!F.getReturnType()->isFloatTy())
-            continue;
-
-        FunctionType *FT = F.getFunctionType();
-
-        FunctionType *NewFT = FunctionType::get(
-            Type::getInt32Ty(M.getContext()), FT->params(), false);
-        auto *NewF =
-            Function::Create(NewFT, F.getLinkage(), FuncName + ".cloned", M);
-
-        SmallPtrSet<CallInst *, 16> Calls;
-
-        for (auto user : F.users())
-            if (CallInst *CI = dyn_cast<CallInst>(user))
-                Calls.insert(CI);
-
-        for (auto CI : Calls) {
-            IRBuilder<> builder(CI);
-
-            SmallVector<Value *, 4> Args;
-            for (auto &Arg : CI->args())
-                Args.push_back(Arg);
-
-            auto *newCall = builder.CreateCall(NewF, Args);
-            newCall->setCallingConv(CI->getCallingConv());
-            newCall->setAttributes(CI->getAttributes());
-            // Convert the value back so that previous users of
-            // the return value are happy
-            auto *converted = builder.CreateBitCast(newCall, CI->getType());
-
-            CI->replaceAllUsesWith(converted);
-            CI->eraseFromParent();
-        }
-
-        std::string originalName = FuncName.str();
-        F.setName(FuncName + "_old");
-        NewF->setName(originalName);
-
-        funcsToRemove.insert(&F);
-    }
-
-    for (auto *F : funcsToRemove)
-        F->eraseFromParent();
-}
-
 // Older Clang versions generate invalid bitcast instructions for explicit
 // C-style casts with specified address space. For example:
 //   %0 = bitcast i8 addrspace(1)* %mem to i32 addrspace(4)*
@@ -631,8 +571,7 @@ void BIImport::fixInvalidBitcasts(llvm::Module &M)
 
             // New bitcast
             PointerType *FinalBitCastType =
-                PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(DstPtrType),
-                                 SrcPtrType->getAddressSpace());
+                IGCLLVM::getWithSamePointeeType(DstPtrType, SrcPtrType->getAddressSpace());
             BitCastInst *NewBitCastI = new BitCastInst(
                 BitCastI->getOperand(0), FinalBitCastType, "bcast", BitCastI);
 
@@ -655,7 +594,6 @@ bool BIImport::runOnModule(Module& M)
     }
 
     fixSPIRFunctionsReturnType(M);
-    fixRoundToTF32ReturnType(M);
 
     for (auto& F : M)
     {
@@ -691,7 +629,7 @@ bool BIImport::runOnModule(Module& M)
         }
     }
 
-    std::function<void(Function*)> Explore = [&](Function* pRoot) -> void
+    std::function<void(Function*)> Explore = [&](Function* pRoot)
     {
         TFunctionsVec calledFuncs;
         GetCalledFunctions(pRoot, calledFuncs);
@@ -857,7 +795,7 @@ bool BIImport::runOnModule(Module& M)
                     FunctionType* funcTy = FunctionType::get(builder.getVoidTy(), { argBuffer->getType() }, false);
                     Value* funcPtr = builder.CreatePointerCast(CI->getArgOperand(0), PointerType::get(funcTy, 0));
                     // Replace builtin with the function call
-                    CallInst* callFunc = builder.CreateCall(funcPtr, argBuffer);
+                    CallInst* callFunc = builder.CreateCall(funcTy, funcPtr, argBuffer);
                     callFunc->setCallingConv(llvm::CallingConv::SPIR_FUNC);
                     CI->replaceAllUsesWith(callFunc);
                     InstToRemove.push_back(CI);
@@ -931,11 +869,11 @@ bool BIImport::runOnModule(Module& M)
                         Function* callerF = CI->getParent()->getParent();
                         // Assume the caller has explicit subgroup size set, otherwise we cannot determine which variant to use
                         FunctionInfoMetaDataHandle funcInfoMD = pMdUtils->getFunctionsInfoItem(callerF);
-                        unsigned subgroup_size = (unsigned)funcInfoMD->getSubGroupSize()->getSIMD_size();
+                        unsigned subgroup_size = (unsigned)funcInfoMD->getSubGroupSize()->getSIMDSize();
                         IGC_ASSERT(subgroup_size != 0);
 
                         // Parse each variant string in the table, stop at the first one that matches subgroup_size
-                        for (auto var : VariantsTable)
+                        for (const auto& var : VariantsTable)
                         {
                             // We only need to get the SIMD size from the string
                             auto [symStr, fName, vecLen] = IGC::ParseVectorVariantFunctionString(var);
@@ -949,8 +887,8 @@ bool BIImport::runOnModule(Module& M)
                     IGCLLVM::IRBuilder<> builder(CI);
                     // Load function address from the table index
                     Value* FP = builder.CreateGEP(FPTablePtr, builder.getInt32(tableIndex));
-                    FP = builder.CreateLoad(FP);
-                    IGC_ASSERT(FP->getType()->isPointerTy() && IGCLLVM::getNonOpaquePtrEltTy(FP->getType())->isFunctionTy());
+                    FP = builder.CreateLoad(cast<llvm::GetElementPtrInst>(FP)->getResultElementType(), FP);
+                    IGC_ASSERT(FP->getType()->isPointerTy());
                     // Call the loaded function address
                     SmallVector<Value*, 8> Args;
                     for (unsigned i = 1; i < IGCLLVM::getNumArgOperands(CI); i++)
@@ -1037,22 +975,23 @@ void BIImport::removeFunctionBitcasts(Module& M)
                         {
                             pDstFunc = Function::Create(pInstCall->getFunctionType(), funcTobeChanged->getLinkage(), funcTobeChanged->getName(), &M);
                             if (pDstFunc->arg_size() != funcTobeChanged->arg_size()) continue;
-                            // Need to copy the attributes over too.
-                            AttributeList FuncAttrs = funcTobeChanged->getAttributes();
-                            pDstFunc->setAttributes(FuncAttrs);
-
                             // Go through and convert function arguments over, remembering the mapping.
                             Function::arg_iterator itSrcFunc = funcTobeChanged->arg_begin();
                             Function::arg_iterator eSrcFunc = funcTobeChanged->arg_end();
                             llvm::Function::arg_iterator itDest = pDstFunc->arg_begin();
 
-                            // Fix incorrect address space caused by CloneFunctionInto later
-                            // for example, CloneFunctionInto causes incorrect LLVM IR, like below
+                            // Fix incorrect address space or incorrect pointer type caused by CloneFunctionInto later
+                            // 1. AddressSpaceCast example: CloneFunctionInto causes incorrect LLVM IR, like below
                             //     %arrayidx.le.i = getelementptr inbounds i8, i8 addrspace(1)* %8, i64 %conv.le.i
                             //     %9 = load i8, i8 addrspace(4)* %arrayidx.le.i, align 1, !tbaa !309
                             // Address space should match for %arrayidx.le.i, so we insert necessary
-                            // address space casts, which should be eliminated later
-                            SmallVector<Instruction *, 5> ascInsts;
+                            // address space casts, which should be eliminated later by other passes
+                            // 2. incorrect type example:
+                            //     %0 = load i16, %"class.sycl::_V1::ext::oneapi::bfloat16" addrspace(4)* %x, align 2
+                            // Load value type should match pointer type for %x, so we insert necessary bitcast:
+                            //     %x.bcast = bitcast %"class.sycl::_V1::ext::oneapi::bfloat16" addrspace(4)* %x to i16 addrspace(4)*
+                            //     %0 = load i16, i16 addrspace(4)* %x.bcast, align 2
+                            SmallVector<Instruction *, 5> castInsts;
 
                             for (; itSrcFunc != eSrcFunc; ++itSrcFunc, ++itDest)
                             {
@@ -1061,17 +1000,23 @@ void BIImport::removeFunctionBitcasts(Module& M)
                                 Type *srcType = (*itSrcFunc).getType();
                                 Value *destVal = &(*itDest);
                                 Type *destType = destVal->getType();
-                                if (srcType->isPointerTy() && destType->isPointerTy() &&
-                                    srcType->getPointerAddressSpace() != destType->getPointerAddressSpace())
+                                if (srcType->isPointerTy() && destType->isPointerTy())
                                 {
-                                    AddrSpaceCastInst *newASC = new AddrSpaceCastInst(destVal, srcType, destVal->getName() + "cast");
-                                    ascInsts.push_back(newASC);
-                                    operandMap[&(*itSrcFunc)] = newASC;
-                                }
-                                else
-                                {
-                                    operandMap[&(*itSrcFunc)] = &(*itDest);
+                                    if (srcType->getPointerAddressSpace() != destType->getPointerAddressSpace())
+                                    {
+                                        AddrSpaceCastInst *newASC = new AddrSpaceCastInst(destVal, srcType, destVal->getName() + ".ascast");
+                                        castInsts.push_back(newASC);
+                                        destVal = newASC;
+                                    }
+                                    if (!IGCLLVM::isOpaqueOrPointeeTypeEquals(srcType, destType))
+                                    {
+                                        BitCastInst *newBT = new BitCastInst(destVal, srcType, destVal->getName() + ".bcast");
+                                        castInsts.push_back(newBT);
+                                        destVal = newBT;
+                                    }
                                 }
+
+                                operandMap[&(*itSrcFunc)] = destVal;
                             }
 
                             // Clone the body of the function into the dest function.
@@ -1084,9 +1029,13 @@ void BIImport::removeFunctionBitcasts(Module& M)
                                 Returns,
                                 "");
 
+                            // Need to copy the attributes over too.
+                            AttributeList FuncAttrs = funcTobeChanged->getAttributes();
+                            pDstFunc->setAttributes(FuncAttrs);
+
                             // get first instruction in function and insert addressspacecast before it
                             Instruction *firstInst = &(*pDstFunc->begin()->getFirstInsertionPt());
-                            for (Instruction *valToInsert : ascInsts)
+                            for (Instruction *valToInsert : castInsts)
                                 valToInsert->insertBefore(firstInst);
 
                             pDstFunc->setCallingConv(funcTobeChanged->getCallingConv());
@@ -1100,6 +1049,8 @@ void BIImport::removeFunctionBitcasts(Module& M)
                         auto newCI = CallInst::Create(pDstFunc, Args, "", pInstCall);
                         newCI->takeName(pInstCall);
                         newCI->setCallingConv(pInstCall->getCallingConv());
+                        newCI->setAttributes(pInstCall->getAttributes());
+                        newCI->setDebugLoc(pInstCall->getDebugLoc());
                         pInstCall->replaceAllUsesWith(newCI);
                         pInstCall->dropAllReferences();
                         if (constExpr->use_empty())
@@ -1122,19 +1073,6 @@ void BIImport::removeFunctionBitcasts(Module& M)
 
 void BIImport::InitializeBIFlags(Module& M)
 {
-    auto MD = *(getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData());
-    auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
-
-    /// @brief  Adds initialization to a global variable according to given value.
-    ///         If the given global variable does not exist, does nothing.
-    auto initializeVarWithValue = [&M](StringRef varName, uint32_t value)
-    {
-        GlobalVariable* gv = M.getGlobalVariable(varName);
-        if (gv == nullptr)
-            return;
-        gv->setInitializer(ConstantInt::get(Type::getInt32Ty(M.getContext()), value));
-    };
-
     /// @brief Set global variable linkage to extrnal. This is needed for
     ///        variables that will become relocations.
     auto makeVarExternal = [&M](StringRef varName)
@@ -1145,60 +1083,7 @@ void BIImport::InitializeBIFlags(Module& M)
         gv->setLinkage(GlobalValue::ExternalLinkage);
     };
 
-    bool isFlushDenormToZero =
-        ((pCtx->m_floatDenormMode32 == FLOAT_DENORM_FLUSH_TO_ZERO) ||
-            MD.compOpt.DenormsAreZero);
-    initializeVarWithValue("__FlushDenormals", isFlushDenormToZero ? 1 : 0);
-    initializeVarWithValue("__DashGSpecified", MD.compOpt.DashGSpecified ? 1 : 0);
-    initializeVarWithValue("__FastRelaxedMath", MD.compOpt.RelaxedBuiltins ? 1 : 0);
-    initializeVarWithValue("__MadEnable", MD.compOpt.MadEnable ? 1 : 0);
-    initializeVarWithValue("__OptDisable", MD.compOpt.OptDisable ? 1 : 0);
-    bool isUseMathWithLUTEnabled = false;
-    if (IGC_IS_FLAG_ENABLED(UseMathWithLUT))
-    {
-        isUseMathWithLUTEnabled = true;
-    }
-    initializeVarWithValue("__UseMathWithLUT", isUseMathWithLUTEnabled ? 1 : 0);
-    initializeVarWithValue("__UseNativeFP32GlobalAtomicAdd", pCtx->platform.hasFP32GlobalAtomicAdd() ? 1 : 0);
-    initializeVarWithValue("__UseNativeFP16AtomicMinMax", pCtx->platform.hasFP16AtomicMinMax() ? 1 : 0);
-    initializeVarWithValue("__HasInt64SLMAtomicCAS", pCtx->platform.hasInt64SLMAtomicCAS() ? 1 : 0);
-    initializeVarWithValue("__UseNativeFP64GlobalAtomicAdd", pCtx->platform.hasFP64GlobalAtomicAdd() ? 1 : 0);
-    initializeVarWithValue("__UseNative64BitIntBuiltin", pCtx->platform.hasNoFullI64Support() ? 0 : 1);
-    initializeVarWithValue("__HasThreadPauseSupport", pCtx->platform.hasThreadPauseSupport() ? 1 : 0);
-    initializeVarWithValue("__UseNative64BitFloatBuiltin", pCtx->platform.hasNoFP64Inst() ? 0 : 1);
-    initializeVarWithValue("__hasHWLocalThreadID", pCtx->platform.hasHWLocalThreadID() ? 1 : 0);
-    initializeVarWithValue("__CRMacros",
-        pCtx->platform.hasCorrectlyRoundedMacros() ? 1 : 0);
-
-    if (StringRef(pCtx->getModule()->getTargetTriple()).size() > 0)
-    {
-        initializeVarWithValue("__APIRS", false);
-    }
-
-    bool useHighAccuracyMathFuncs = false;
-    if (pCtx->type == ShaderType::OPENCL_SHADER)
-    {
-        OpenCLProgramContext *OCLContext = static_cast<OpenCLProgramContext*>(pCtx);
-
-        initializeVarWithValue("__IsSPIRV", OCLContext->isSPIRV());
-
-        float profilingTimerResolution = OCLContext->getProfilingTimerResolution();
-        initializeVarWithValue("__ProfilingTimerResolution", *reinterpret_cast<int*>(&profilingTimerResolution));
-
-        useHighAccuracyMathFuncs = OCLContext->m_InternalOptions.UseHighAccuracyMathFuncs;
-
-        initializeVarWithValue("__UseLSC", pCtx->platform.hasLSC());
-        initializeVarWithValue("__ForceL1Prefetch", IGC_IS_FLAG_ENABLED(ForcePrefetchToL1Cache) ? 1 : 0);
-    }
-
-    initializeVarWithValue("__EnableSWSrgbWrites", IGC_GET_FLAG_VALUE(cl_khr_srgb_image_writes));
-    initializeVarWithValue("__UseHighAccuracyMath", useHighAccuracyMathFuncs);
-
     makeVarExternal("__SubDeviceID");
-    initializeVarWithValue("__MaxHWThreadIDPerSubDevice", pCtx->platform.GetGTSystemInfo().ThreadCount);
-
-
-    initializeVarWithValue("__JointMatrixLoadStoreOpt", IGC_GET_FLAG_VALUE(JointMatrixLoadStoreOpt));
 }
 
 extern "C" llvm::ModulePass* createBuiltInImportPass(
@@ -1246,6 +1131,23 @@ bool PreBIImportAnalysis::runOnModule(Module& M)
     {
         Function* pFunc = &(*I);
 
+        // To make the behavior predictable, fix the workgroup XYZ walk order here.
+        if (IGC_IS_FLAG_ENABLED(ForceXYZworkGroupWalkOrder))
+        {
+            MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+            ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
+            if (isEntryFunc(pMdUtils, pFunc)) {
+                uint32_t WGSize = IGCMetaDataHelper::getThreadGroupSize(*pMdUtils, pFunc);
+                uint32_t WGSizeHint = IGCMetaDataHelper::getThreadGroupSizeHint(*pMdUtils, pFunc);
+                if (WGSize != 1 && WGSizeHint != 1)
+                {
+                    modMD->FuncMD[pFunc].workGroupWalkOrder.dim0 = 0;
+                    modMD->FuncMD[pFunc].workGroupWalkOrder.dim1 = 1;
+                    modMD->FuncMD[pFunc].workGroupWalkOrder.dim2 = 2;
+                }
+            }
+        }
+
         StringRef funcName = pFunc->getName();
         bool isFuncNameToSearch = (funcName == OCL_GET_GLOBAL_OFFSET ||
             funcName == OCL_GET_LOCAL_ID ||
@@ -1370,7 +1272,7 @@ bool PreBIImportAnalysis::runOnModule(Module& M)
         std::string sinPiBuiltinName = "__builtin_spirv_OpenCL_sinpi_f32";
         std::string cosPiBuiltinName = "__builtin_spirv_OpenCL_cospi_f32";
 #endif
-        auto violatesPromotionConds = [&](Instruction* inst) -> bool{
+        auto violatesPromotionConds = [&](Instruction* inst) {
             IGC_ASSERT(inst->getOpcode() == Instruction::FMul || inst->getOpcode() == Instruction::Load);
             if (inst && inst->hasNUsesOrMore(2)) {
                 Value::use_iterator instUse = inst->use_begin();
@@ -1495,11 +1397,9 @@ bool PreBIImportAnalysis::runOnModule(Module& M)
         }
     }
 
-    for (auto InstTuple : InstToModify)
+    for (const auto &InstTuple : InstToModify)
     {
-      auto inst = std::get<0>(InstTuple);
-      auto value = std::get<1>(InstTuple);
-      auto srcPos = std::get<2>(InstTuple);
+      auto [inst, value, srcPos] = InstTuple;
       inst->setOperand(
           srcPos,
           ConstantFP::get(
@@ -1507,7 +1407,7 @@ bool PreBIImportAnalysis::runOnModule(Module& M)
           (float) value));
     }
 
-    for (auto CIPair : CallToReplace)
+    for (const auto& CIPair : CallToReplace)
     {
       auto oldCI = CIPair.first;
       auto newCI = CIPair.second;
