@@ -77,6 +77,12 @@ void CCommand::replaceGenISACallInst(GenISAIntrinsic::ID intrinsicName, ArrayRef
     m_pCallInst->replaceAllUsesWith(newCall);
 }
 
+void CCommand::emitError(const char *ErrorStr, const Value *Context)
+{
+    m_pCodeGenContext->EmitError(ErrorStr, Context);
+    m_bHasError = true;
+}
+
 void CImagesBI::prepareZeroOffsets()
 {
     m_args.push_back(m_pIntZero); // offsetU
@@ -122,6 +128,9 @@ void CImagesBI::prepareCoords(Dimension Dim, Value* Coord, Value* Zero)
     case DIM_1D:
         //  no need to extract since in 1 Dim Coord isn't a vector
         break;
+    case DIM_UNKNOWN:
+        emitError("Unexpected dimension", NULL);
+        break;
     }
 }
 
@@ -129,45 +138,62 @@ void CImagesBI::prepareCoords(Dimension Dim, Value* Coord, Value* Zero)
 
 void CImagesBI::prepareColor(Value* Color)
 {
-    Value* TmpColor;
+    Value* TmpColor = Color;
+    auto *VTy = dyn_cast<IGCLLVM::FixedVectorType>(Color->getType());
     if (Color->getType()->getScalarType() != m_pFloatType)
     {
         // GenISA_typedwrite intrinsic expect to get the color as float,
         // therefore we do bitcast that should disappear in the final code.
+        Type *CastTy = VTy ? IGCLLVM::FixedVectorType::get(m_pFloatType, static_cast<unsigned>(VTy->getNumElements())) : m_pFloatType;
         Instruction* tmp = BitCastInst::Create(
             Instruction::BitCast,
             Color,
-            IGCLLVM::FixedVectorType::get(m_pFloatType, 4),
+            CastTy,
             "floatColor",
             m_pCallInst);
         tmp->setDebugLoc(m_DL);
         TmpColor = tmp;
     }
-    else
+
+    Value* ColorX = VTy ? ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_X), "ColorX", m_pCallInst) : TmpColor; // color x
+    if (VTy)
     {
-        TmpColor = Color;
+        cast<Instruction>(ColorX)->setDebugLoc(m_DL);
     }
 
-    Instruction* ColorX = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_X), "ColorX", m_pCallInst); // color x
-    Instruction* ColorY = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_Y), "ColorY", m_pCallInst); // color y
-    Instruction* ColorZ = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_Z), "ColorZ", m_pCallInst); // color z
-    Instruction* ColorW = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_W), "ColorW", m_pCallInst); // color w
-    ColorX->setDebugLoc(m_DL);
-    ColorY->setDebugLoc(m_DL);
-    ColorZ->setDebugLoc(m_DL);
-    ColorW->setDebugLoc(m_DL);
+    Value* ColorY = nullptr;
+    Value* ColorZ = nullptr;
+    Value* ColorW = nullptr;
+    if (VTy)
+    {
+        unsigned NumElts = static_cast<unsigned>(VTy->getNumElements());
+        IGC_ASSERT(NumElts == 2 || NumElts == 4);
+        if (NumElts >= 2)
+        {
+            ColorY = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_Y), "ColorY", m_pCallInst); // color y
+            cast<Instruction>(ColorY)->setDebugLoc(m_DL);
+        }
+        if (NumElts == 4)
+        {
+            ColorZ = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_Z), "ColorZ", m_pCallInst); // color z
+            ColorW = ExtractElementInst::Create(TmpColor, ConstantInt::get(m_pIntType, COORD_W), "ColorW", m_pCallInst); // color w
+            cast<Instruction>(ColorZ)->setDebugLoc(m_DL);
+            cast<Instruction>(ColorW)->setDebugLoc(m_DL);
+        }
+    }
 
+    Value *Undef = UndefValue::get(m_pFloatType);
     m_args.push_back(ColorX);
-    m_args.push_back(ColorY);
-    m_args.push_back(ColorZ);
-    m_args.push_back(ColorW);
-
+    m_args.push_back(ColorY ? ColorY : Undef);
+    m_args.push_back(ColorZ ? ColorZ : Undef);
+    m_args.push_back(ColorW ? ColorW : Undef);
 }
 
 void CImagesBI::prepareImageBTI()
 {
-    Argument* pImg = nullptr;
+    Value* pImg = nullptr;
     ConstantInt* imageIndex = CImagesUtils::getImageIndex(m_pParamMap, m_pCallInst, 0, pImg);
+    IGC_ASSERT(isa<Argument>(pImg) || isa<LoadInst>(pImg));
     unsigned int addrSpace = EncodeAS4GFXResource(*imageIndex, RESOURCE);
     Value* img = ConstantPointerNull::get(PointerType::get(pImg->getType(), addrSpace));
     m_args.push_back(img); // BTI
@@ -183,11 +209,22 @@ void CImagesBI::prepareImageBTI()
     }
 }
 
+void CImagesBI::preparePairedResource()
+{
+    Value* pImg = nullptr;
+    ConstantInt* imageIndex = CImagesUtils::getImageIndex(m_pParamMap, m_pCallInst, 0, pImg);
+    IGC_ASSERT(isa<Argument>(pImg) || isa<LoadInst>(pImg));
+    unsigned int addrSpace = EncodeAS4GFXResource(*imageIndex, RESOURCE);
+    Type* ptrTy = llvm::PointerType::get(m_pFloatType, addrSpace);
+    Value* pairedResource = UndefValue::get(ptrTy);
+    m_args.push_back(pairedResource);
+}
+
 void CImagesBI::verifyCommand()
 {
     if (m_IncorrectBti)
     {
-        m_pCodeGenContext->EmitError("Inconsistent use of image!", NULL);
+        emitError("Inconsistent use of image!", NULL);
     }
 }
 
@@ -212,14 +249,30 @@ Argument* CImagesBI::CImagesUtils::findImageFromBufferPtr(const MetaDataUtils& M
     return nullptr;
 }
 
+static bool isBindlessImageLoad(Value *v)
+{
+    auto *load = dyn_cast<LoadInst>(v);
+    if (!load)
+        return false;
+
+    auto *ptrOp = load->getPointerOperand()->stripPointerCasts();
+    if (!isa<Argument>(ptrOp) && !isa<GetElementPtrInst>(ptrOp) && !isa<AllocaInst>(ptrOp) && !isa<CallInst>(ptrOp))
+        return false;
+
+    return load->getType()->isIntegerTy(64);
+};
+
 ConstantInt* CImagesBI::CImagesUtils::getImageIndex(
     ParamMap* pParamMap,
     CallInst* pCallInst,
     unsigned int paramIndex,
-    Argument*& imageParam)
+    Value*& imageParam)
 {
     ConstantInt* imageIndex = nullptr;
-    imageParam = cast<Argument>(ValueTracker::track(pCallInst, paramIndex));
+
+    imageParam = ValueTracker::track(pCallInst, paramIndex, nullptr, nullptr, isBindlessImageLoad);
+    IGC_ASSERT(imageParam);
+    IGC_ASSERT(isa<Argument>(imageParam) || isa<LoadInst>(imageParam));
     int i = (*pParamMap)[imageParam].index;
     imageIndex = ConstantInt::get(Type::getInt32Ty(pCallInst->getContext()), i);
     return imageIndex;
@@ -227,13 +280,15 @@ ConstantInt* CImagesBI::CImagesUtils::getImageIndex(
 
 BufferType CImagesBI::CImagesUtils::getImageType(ParamMap* pParamMap, CallInst* pCallInst, unsigned int paramIndex)
 {
-    Argument* imageParam = cast<Argument>(ValueTracker::track(pCallInst, paramIndex));
-    return (*pParamMap)[imageParam].type;
+    Value *imageParam = ValueTracker::track(pCallInst, paramIndex, nullptr, nullptr, isBindlessImageLoad);
+    IGC_ASSERT(imageParam);
+    IGC_ASSERT(isa<Argument>(imageParam) || isa<LoadInst>(imageParam));
+    return isa<LoadInst>(imageParam) ? BufferType::BINDLESS : (*pParamMap)[imageParam].type;
 }
 
 void CImagesBI::createGetBufferPtr()
 {
-    Argument* pImg = nullptr;
+    Value* pImg = nullptr;
     ConstantInt* imageIndex = CImagesUtils::getImageIndex(m_pParamMap, m_pCallInst, 0, pImg);
     BufferType bufType = CImagesUtils::getImageType(m_pParamMap, m_pCallInst, 0);
     unsigned int addressSpace = IGC::EncodeAS4GFXResource(*imageIndex, bufType);
@@ -245,7 +300,9 @@ void CImagesBI::createGetBufferPtr()
         auto modMD = m_pCodeGenContext->getModuleMetaData();
         if (modMD->UseBindlessImage)
         {
-            Value* basePointer = BitCastInst::CreatePointerCast(pImg, ptrTy, "bindless_img", m_pCallInst);
+            Value* basePointer = isa<IntegerType>(pImg->getType()) ?
+                BitCastInst::CreateBitOrPointerCast(pImg, ptrTy, "bindless_img", m_pCallInst) :
+                BitCastInst::CreatePointerCast(pImg, ptrTy, "bindless_img", m_pCallInst);
             m_args.push_back(basePointer);
             return;
         }
@@ -293,7 +350,7 @@ bool CImagesBI::derivedFromInt(const Value* pVal)
 {
     SmallPtrSet<const Value*, 16> visitedPHIs;
 
-    std::function<bool(const Value*)> go = [&](const Value* val) -> bool
+    std::function<bool(const Value*)> go = [&](const Value* val)
     {
         if (val->getType()->getScalarType()->isIntegerTy() || isa<UndefValue>(val))
         {
@@ -365,15 +422,45 @@ class COCL_sample : public CImagesBI
 public:
     COCL_sample(ParamMap* paramMap, InlineMap* inlineMap, int* nextSampler, Dimension Dim, MetaDataUtils* pMdUtils, ModuleMetaData* modMD) : CImagesBI(paramMap, inlineMap, nextSampler, Dim), m_pMdUtils(pMdUtils), m_modMD(modMD) {}
 
-    ConstantInt* getSamplerIndex(void)
+    Value* getSamplerValue(void)
     {
         ConstantInt* samplerIndex = nullptr;
-        Value* samplerParam = ValueTracker::track(m_pCallInst, 1, m_pMdUtils, m_modMD);
+        auto isBindlessSampler = [](Value *v)
+        {
+            // Bindless sampler is computed in ResolveSampledImageBuiltins pass.
+            if (auto *I = dyn_cast<BinaryOperator>(v))
+            {
+                if (I->getOpcode() != BinaryOperator::Or || !I->getType()->isIntegerTy(64))
+                    return false;
+                if (auto *C = dyn_cast<ConstantInt>(I->getOperand(1)))
+                {
+                    return C->isOne();
+                }
+            }
+            return false;
+        };
+        Value* samplerParam = ValueTracker::track(m_pCallInst, 1, m_pMdUtils, m_modMD, isBindlessSampler);
         if (!samplerParam) {
-            m_pCodeGenContext->EmitError("There are instructions that use a sampler, but no sampler found in the kernel!", m_pCallInst);
+            emitError("There are instructions that use a sampler, but no sampler found in the kernel!", m_pCallInst);
             return nullptr;
         }
 
+        auto modMD = m_pCodeGenContext->getModuleMetaData();
+
+        // If bindless image is preferred, map the bindless pointer
+        if (modMD->UseBindlessImage)
+        {
+            // If sampler is argument, look up index in the parameter map.
+            int i = isa<Argument>(samplerParam) ? (*m_pParamMap)[samplerParam].index : 0;
+            samplerIndex = ConstantInt::get(m_pIntType, i);
+            unsigned int addressSpace = IGC::EncodeAS4GFXResource(*samplerIndex, BufferType::BINDLESS_SAMPLER);
+            Type* ptrTy = llvm::PointerType::get(m_pFloatType, addressSpace);
+            Value* bindlessSampler = isa<IntegerType>(samplerParam->getType()) ?
+                BitCastInst::CreateBitOrPointerCast(samplerParam, ptrTy, "bindless_sampler", m_pCallInst) :
+                BitCastInst::CreatePointerCast(samplerParam, ptrTy, "bindless_sampler", m_pCallInst);
+            return bindlessSampler;
+        }
+
         // Argument samplers are looked up in the parameter map
         if (isa<Argument>(samplerParam))
         {
@@ -568,13 +655,17 @@ public:
         }
     }
 
-    bool prepareSamplerIndex()
+    bool prepareSamplerValue()
     {
-        ConstantInt* samplerIndex = getSamplerIndex();
-        if (!samplerIndex) return false;
-        unsigned int addrSpace = EncodeAS4GFXResource(*samplerIndex, SAMPLER);
-        Value* sampler = ConstantPointerNull::get(PointerType::get(samplerIndex->getType(), addrSpace));
-        m_args.push_back(sampler);
+        Value* samplerValue = getSamplerValue();
+        if (!samplerValue) return false;
+        if (isa<ConstantInt>(samplerValue))
+        {
+            unsigned int addrSpace = EncodeAS4GFXResource(*samplerValue, SAMPLER);
+            samplerValue = ConstantPointerNull::get(PointerType::get(samplerValue->getType(), addrSpace));
+        }
+
+        m_args.push_back(samplerValue);
         return true;
     }
 
@@ -637,18 +728,21 @@ public:
         case DIM_1D_ARRAY:
             //  no need to extract since in 1 Dim gradient is not a vector.
             break;
+        case DIM_UNKNOWN:
+            emitError("Unexpected dimension", NULL);
+            break;
         }
     }
 
 protected:
-    MetaDataUtils* m_pMdUtils;
-    ModuleMetaData* m_modMD;
-    Value* m_gradXX;
-    Value* m_gradXY;
-    Value* m_gradXZ;
-    Value* m_gradYX;
-    Value* m_gradYY;
-    Value* m_gradYZ;
+    MetaDataUtils* m_pMdUtils = nullptr;
+    ModuleMetaData* m_modMD = nullptr;
+    Value* m_gradXX = nullptr;
+    Value* m_gradXY = nullptr;
+    Value* m_gradXZ = nullptr;
+    Value* m_gradYX = nullptr;
+    Value* m_gradYY = nullptr;
+    Value* m_gradYZ = nullptr;
 };
 
 class COCL_sample_l : public COCL_sample
@@ -665,9 +759,10 @@ public:
         m_args.push_back(CoordY);
         m_args.push_back(CoordZ);
         m_args.push_back(m_pFloatZero); // ai (?)
+        preparePairedResource();
         createGetBufferPtr();
-        bool samplerIndexFound = prepareSamplerIndex();
-        if (!samplerIndexFound) return;
+        bool samplerValueFound = prepareSamplerValue();
+        if (!samplerValueFound) return;
 
         prepareZeroOffsets();
         Type* types[] = {
@@ -675,6 +770,7 @@ public:
             m_pFloatType,
             m_args[5]->getType(),
             m_args[6]->getType(),
+            m_args[7]->getType(),
         };
         replaceGenISACallInst(GenISAIntrinsic::GenISA_sampleLptr, types);
     }
@@ -701,14 +797,16 @@ public:
         m_args.push_back(m_gradYZ);
         m_args.push_back(m_pFloatZero); // ai (?)
         m_args.push_back(m_pFloatZero); // minLOD (?)
+        preparePairedResource();
         prepareImageBTI();
-        prepareSamplerIndex();
+        prepareSamplerValue();
         prepareZeroOffsets();
         Type* types[] = {
             m_pCallInst->getType(),
             m_pFloatType,
             m_args[11]->getType(),
             m_args[12]->getType(),
+            m_args[13]->getType(),
         };
         replaceGenISACallInst(GenISAIntrinsic::GenISA_sampleDptr, types);
     }
@@ -727,11 +825,13 @@ public:
         m_args.push_back(CoordY);
         m_args.push_back(m_pCallInst->getOperand(2)); // LOD
         m_args.push_back(CoordZ);
+        preparePairedResource();
         createGetBufferPtr();
         prepareZeroOffsets();
         Type* types[] = {
             m_pCallInst->getType(),
             m_args[4]->getType(),
+            m_args[5]->getType(),
         };
         replaceGenISACallInst(GenISAIntrinsic::GenISA_ldptr, types);
     }
@@ -750,11 +850,13 @@ public:
         m_args.push_back(CoordY);
         m_args.push_back(m_pCallInst->getOperand(2)); // LOD
         m_args.push_back(CoordZ);
+        preparePairedResource();
         createGetBufferPtr();
         prepareZeroOffsets();
         Type* types[] = {
             IGCLLVM::FixedVectorType::get(m_pFloatType, 4),
             m_args[4]->getType(),
+            m_args[5]->getType(),
         };
         replaceGenISACallInst(GenISAIntrinsic::GenISA_ldptr, types);
     }
@@ -868,6 +970,46 @@ public:
     }
 };
 
+class CGetImageProperty : public CImagesBI
+{
+    // Image property that needs to be recieved
+    const ImageProperty m_Prop;
+    CGetImageProperty() = delete;
+    CGetImageProperty(ParamMap* paramMap, ImageProperty prop) : CImagesBI(paramMap), m_Prop(prop) {}
+
+public:
+    static std::unique_ptr<CGetImageProperty> create(ParamMap* paramMap, ImageProperty prop)
+    {
+        return std::unique_ptr<CGetImageProperty>(new CGetImageProperty(paramMap, prop));
+    }
+
+    void createIntrinsic()
+    {
+        createGetBufferPtr();
+        LLVM3DBuilder<> builder(*m_pCtx, m_pCodeGenContext->platform.getPlatformInfo());
+        builder.SetInsertPoint(m_pCallInst);
+        Value* info = builder.Create_resinfo(builder.getInt32(0), m_args[0]);
+        Value* prop = nullptr;
+        switch (m_Prop)
+        {
+        case WIDTH:
+            prop = builder.CreateExtractElement(info, builder.getInt32(0));
+            break;
+        case HEIGHT:
+        case ARRAY_SIZE_1D:
+            prop = builder.CreateExtractElement(info, builder.getInt32(1));
+            break;
+        case DEPTH:
+        case ARRAY_SIZE_2D:
+            prop = builder.CreateExtractElement(info, builder.getInt32(2));
+            break;
+        default:
+            emitError("Unsupported bindless image property requested.", NULL);
+        }
+        m_pCallInst->replaceAllUsesWith(prop);
+    }
+};
+
 class CSimpleIntrinMapping : public CCommand
 {
     // id - ID of the intrinsic to be replaced by the call
@@ -1050,7 +1192,7 @@ public:
         // Push images like src image, fwd ref image, and bwd ref image.
         IGC_ASSERT((num_images >= 2) && (num_images <= 3));
         for (uint i = 0; i < num_images; i++) {
-            Argument* pImg = nullptr;
+            Value* pImg = nullptr;
             m_args.push_back(CImagesBI::CImagesUtils::getImageIndex(m_pParamMap, m_pCallInst, i + 3, pImg));
         }
 
@@ -1092,7 +1234,7 @@ public:
 
         // Push images like src image, fwd ref image, and bwd ref image.
         for (int i = 0; i < m_numImgArgs; i++) {
-            Argument* pImg = nullptr;
+            Value* pImg = nullptr;
             m_args.push_back(CImagesBI::CImagesUtils::getImageIndex(m_pParamMap, m_pCallInst, i + 1, pImg));
         }
 
@@ -1257,14 +1399,33 @@ CBuiltinsResolver::CBuiltinsResolver(CImagesBI::ParamMap* paramMap, CImagesBI::I
     m_CommandMap["__builtin_IB_OCL_3d_sample_dui"] = initSamplerClass<COCL_sample_d>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_3D, pMdUtils, modMD);
 
     // Write Image
-    m_CommandMap["__builtin_IB_write_1d_ui"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D);
-    m_CommandMap["__builtin_IB_write_1darr_ui"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D_ARRAY);
-    m_CommandMap["__builtin_IB_write_2d_ui"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D);
-    m_CommandMap["__builtin_IB_write_2darr_ui"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D_ARRAY);
-    m_CommandMap["__builtin_IB_write_3d_ui"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_3D);
+    m_CommandMap["__builtin_IB_write_1d_u1i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D);
+    m_CommandMap["__builtin_IB_write_1d_u2i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D);
+    m_CommandMap["__builtin_IB_write_1d_u4i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D);
+    m_CommandMap["__builtin_IB_write_1darr_u1i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D_ARRAY);
+    m_CommandMap["__builtin_IB_write_1darr_u2i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D_ARRAY);
+    m_CommandMap["__builtin_IB_write_1darr_u4i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_1D_ARRAY);
+    m_CommandMap["__builtin_IB_write_2d_u1i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D);
+    m_CommandMap["__builtin_IB_write_2d_u2i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D);
+    m_CommandMap["__builtin_IB_write_2d_u4i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D);
+    m_CommandMap["__builtin_IB_write_2darr_u1i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D_ARRAY);
+    m_CommandMap["__builtin_IB_write_2darr_u2i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D_ARRAY);
+    m_CommandMap["__builtin_IB_write_2darr_u4i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D_ARRAY);
+    m_CommandMap["__builtin_IB_write_3d_u1i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_3D);
+    m_CommandMap["__builtin_IB_write_3d_u2i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_3D);
+    m_CommandMap["__builtin_IB_write_3d_u4i"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_3D);
     m_CommandMap["__builtin_IB_write_2d_f"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D);
     m_CommandMap["__builtin_IB_write_2darr_f"] = initImageClass<CWrite>(paramMap, inlineMap, nextSampler, CImagesBI::Dimension::DIM_2D_ARRAY);
 
+    // Get Image properties only supported here for bindless mode
+    if (modMD->compOpt.UseBindlessMode && !modMD->compOpt.UseLegacyBindlessMode){
+        m_CommandMap["__builtin_IB_get_image_width"] = CGetImageProperty::create(paramMap, CImagesBI::ImageProperty::WIDTH);
+        m_CommandMap["__builtin_IB_get_image_height"] = CGetImageProperty::create(paramMap, CImagesBI::ImageProperty::HEIGHT);
+        m_CommandMap["__builtin_IB_get_image_depth"] = CGetImageProperty::create(paramMap, CImagesBI::ImageProperty::DEPTH);
+        m_CommandMap["__builtin_IB_get_image1d_array_size"] = CGetImageProperty::create(paramMap, CImagesBI::ImageProperty::ARRAY_SIZE_1D);
+        m_CommandMap["__builtin_IB_get_image2d_array_size"] = CGetImageProperty::create(paramMap, CImagesBI::ImageProperty::ARRAY_SIZE_2D);
+    }
+
     //convert Built-ins
     m_CommandMap["__builtin_IB_frnd_ne"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_ROUNDNE, false);
     m_CommandMap["__builtin_IB_ftoh_rtn"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_ftof_rtn, true);
@@ -1326,6 +1487,7 @@ CBuiltinsResolver::CBuiltinsResolver(CImagesBI::ParamMap* paramMap, CImagesBI::I
     m_CommandMap["__builtin_IB_hw_thread_id"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_hw_thread_id, false);
     m_CommandMap["__builtin_IB_slice_id"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_slice_id, false);
     m_CommandMap["__builtin_IB_subslice_id"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_subslice_id, false);
+    m_CommandMap["__builtin_IB_logical_subslice_id"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_logical_subslice_id, false);
     m_CommandMap["__builtin_IB_dual_subslice_id"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_dual_subslice_id, false);
     m_CommandMap["__builtin_IB_eu_id"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_eu_id, false);
     m_CommandMap["__builtin_IB_get_sr0"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_getSR0, false);
@@ -1560,6 +1722,8 @@ CBuiltinsResolver::CBuiltinsResolver(CImagesBI::ParamMap* paramMap, CImagesBI::I
     m_CommandMap["__builtin_IB_get_stack_pointer"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_getStackPointer, false);
     m_CommandMap["__builtin_IB_get_stack_size_per_thread"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_getStackSizePerThread, false);
 
+    m_CommandMap["__devicelib_exit"] = CSimpleIntrinMapping::create(GenISAIntrinsic::GenISA_thread_exit, false);
+
     m_KnownBuiltins = {
         //resolved later in GenericAddressDynamicResolution pass:
         "__builtin_IB_to_local",
@@ -1579,5 +1743,5 @@ bool CBuiltinsResolver::resolveBI(CallInst* Inst)
     m_CommandMap[calleeName]->execute(Inst, m_CodeGenContext);
     m_CommandMap[calleeName]->verifyCommand();
 
-    return !m_CodeGenContext->HasError();
+    return !m_CommandMap[calleeName]->hasError();
 }
