@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -132,8 +132,7 @@ auto AlignmentAnalysis::getAlignValue(Value* V) const
             // If the globalvariable uses the default alignment, pull it from the datalayout
             if (!align)
             {
-                Type* gvType = GV->getType();
-                return m_DL->getABITypeAlignment(IGCLLVM::getNonOpaquePtrEltTy(gvType));
+                return m_DL->getABITypeAlignment(GV->getValueType());
             }
             else
             {
@@ -225,8 +224,7 @@ alignment_t AlignmentAnalysis::visitAllocaInst(AllocaInst& I)
     // If the alloca uses the default alignment, pull it from the datalayout
     if (!newAlign)
     {
-        Type* allocaType = I.getType();
-        newAlign = m_DL->getABITypeAlignment(IGCLLVM::getNonOpaquePtrEltTy(allocaType));
+        newAlign = m_DL->getABITypeAlignment(I.getAllocatedType());
     }
 
     return newAlign;
@@ -362,7 +360,6 @@ alignment_t AlignmentAnalysis::visitGetElementPtrInst(GetElementPtrInst& I)
     // The alignment can never be better than the alignment of the base pointer
     alignment_t newAlign = getAlignValue(I.getPointerOperand());
 
-    Type* Ty = I.getPointerOperand()->getType()->getScalarType();
     gep_type_iterator GTI = gep_type_begin(&I);
     for (auto op = I.op_begin() + 1, opE = I.op_end(); op != opE; ++op, ++GTI)
     {
@@ -371,11 +368,10 @@ alignment_t AlignmentAnalysis::visitGetElementPtrInst(GetElementPtrInst& I)
         {
             auto Field = int_cast<unsigned>((cast<Constant>(*op))->getUniqueInteger().getZExtValue());
             offset = int_cast<alignment_t>(m_DL->getStructLayout(StTy)->getElementOffset(Field));
-            Ty = StTy->getElementType(Field);
         }
         else
         {
-            Ty = GTI.getIndexedType();
+            Type* Ty = GTI.getIndexedType();
             auto multiplier = int_cast<alignment_t>(m_DL->getTypeAllocSize(Ty));
             offset = multiplier * getAlignValue(*op);
         }
