@@ -40,6 +40,7 @@ ImageFuncResolution::ImageFuncResolution() : FunctionPass(ID), m_implicitArgs()
 bool ImageFuncResolution::runOnFunction(Function& F) {
     const MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     m_implicitArgs = ImplicitArgs(F, pMdUtils);
+    m_pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
     m_changed = false;
     visit(F);
     return m_changed;
@@ -52,6 +53,9 @@ void ImageFuncResolution::visitCallInst(CallInst& CI)
         return;
     }
 
+    bool isImplicitImageArgs = !m_pCtx->getModuleMetaData()->compOpt.UseBindlessMode ||
+                                    m_pCtx->getModuleMetaData()->compOpt.UseLegacyBindlessMode;
+
     Value* imageRes = nullptr;
 
     // Add appropriate sequence and image dimension func
@@ -59,14 +63,29 @@ void ImageFuncResolution::visitCallInst(CallInst& CI)
 
     if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE_HEIGHT))
     {
+        if (!isImplicitImageArgs)
+        {
+            IGC_ASSERT_MESSAGE(false, "Getting Image Height from implicit args is supported only in bindful mode");
+            return;
+        }
         imageRes = getImageHeight(CI);
     }
     else if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE_WIDTH))
     {
+        if (!isImplicitImageArgs)
+        {
+            IGC_ASSERT_MESSAGE(false, "Getting Image Width from implicit args is supported only in bindful mode");
+            return;
+        }
         imageRes = getImageWidth(CI);
     }
     else if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE_DEPTH))
     {
+        if (!isImplicitImageArgs)
+        {
+            IGC_ASSERT_MESSAGE(false, "Getting Image Depth from implicit args is supported only in bindful mode");
+            return;
+        }
         imageRes = getImageDepth(CI);
     }
     else if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE_NUM_MIP_LEVELS))
@@ -85,8 +104,13 @@ void ImageFuncResolution::visitCallInst(CallInst& CI)
     {
         imageRes = getImplicitImageArg(CI, ImplicitArg::IMAGE_SRGB_CHANNEL_ORDER);
     }
-    else if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE_ARRAY_SIZE))
+    else if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE1D_ARRAY_SIZE) || funcName.equals(ImageFuncsAnalysis::GET_IMAGE2D_ARRAY_SIZE))
     {
+        if (!isImplicitImageArgs)
+        {
+            IGC_ASSERT_MESSAGE(false, "Getting Image Array Size from implicit args is supported only in bindful mode");
+            return;
+        }
         imageRes = getImageArraySize(CI);
     }
     else if (funcName.equals(ImageFuncsAnalysis::GET_IMAGE_NUM_SAMPLES))
@@ -181,136 +205,89 @@ Value* ImageFuncResolution::getImageNumSamples(CallInst& CI)
     return arg;
 }
 
-Value* ImageFuncResolution::getSamplerAddressMode(CallInst& CI)
+template<ImplicitArg::ArgType ArgTy>
+Value* ImageFuncResolution::getSamplerProperty(CallInst& CI)
 {
     MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
-
-    Value* sampler = ValueTracker::track(&CI, 0, pMdUtils, modMD);
-    IGC_ASSERT_MESSAGE(sampler != nullptr, "Sampler untraceable for ImplicitArg::SAMPLER_ADDRESS");
-    if (isa<Argument>(sampler))
+    if (Value* sampler = ValueTracker::track(&CI, 0, pMdUtils, modMD))
     {
-        Argument* arg = getImplicitImageArg(CI, ImplicitArg::SAMPLER_ADDRESS);
-        return arg;
-    }
-    else
-    {
-        llvm::Function* pFunc = CI.getParent()->getParent();
-
-        IGC_ASSERT_MESSAGE(isa<ConstantInt>(sampler), "Sampler must be a constant integer");
-        InlineSamplerState samplerStateAddressMode{ cast<ConstantInt>(sampler)->getZExtValue() };
-        uint64_t samplerVal = 0;
-        uint samplerValue = int_cast<unsigned int>(cast<ConstantInt>(sampler)->getZExtValue());
-        if (modMD->FuncMD.find(pFunc) != modMD->FuncMD.end())
+        if (isa<Argument>(sampler))
         {
-            FunctionMetaData funcMD = modMD->FuncMD[pFunc];
-            ResourceAllocMD resAllocMD = funcMD.resAllocMD;
-            for (auto i = resAllocMD.inlineSamplersMD.begin(), e = resAllocMD.inlineSamplersMD.end(); i != e; i++)
+            if (m_implicitArgs.isImplicitArgExist(ArgTy))
             {
-                IGC::InlineSamplersMD inlineSamplerMD = *i;
-                if (samplerValue == inlineSamplerMD.m_Value)
-                {
-                    InlineSamplerState samplerState{ static_cast<uint64_t>(samplerValue) };
-                    samplerVal = inlineSamplerMD.addressMode;
-                }
+                Argument* arg = getImplicitImageArg(CI, ArgTy);
+                return arg;
             }
         }
-        return ConstantInt::get(CI.getType(), samplerVal);
+        else
+        {
+            llvm::Function* pFunc = CI.getFunction();
+
+            IGC_ASSERT_MESSAGE(isa<ConstantInt>(sampler), "Sampler must be a constant integer");
+            uint64_t samplerVal = 0;
+            if (modMD->FuncMD.find(pFunc) != modMD->FuncMD.end())
+            {
+                FunctionMetaData funcMD = modMD->FuncMD[pFunc];
+                ResourceAllocMD resAllocMD = funcMD.resAllocMD;
+                uint samplerValue = int_cast<unsigned int>(cast<ConstantInt>(sampler)->getZExtValue());
+                for (auto i = resAllocMD.inlineSamplersMD.begin(), e = resAllocMD.inlineSamplersMD.end(); i != e; i++)
+                {
+                    IGC::InlineSamplersMD inlineSamplerMD = *i;
+                    if (samplerValue == inlineSamplerMD.m_Value)
+                    {
+                        InlineSamplerState samplerState{ static_cast<uint64_t>(samplerValue) };
+                        if constexpr (ArgTy == ImplicitArg::SAMPLER_ADDRESS)
+                        {
+                            samplerVal = inlineSamplerMD.addressMode;
+                        }
+                        else if constexpr (ArgTy == ImplicitArg::SAMPLER_NORMALIZED)
+                        {
+                            samplerVal = inlineSamplerMD.NormalizedCoords;
+                        }
+                        else if constexpr (ArgTy == ImplicitArg::SAMPLER_SNAP_WA)
+                        {
+                            bool anyAddressModeClamp =
+                                inlineSamplerMD.TCXAddressMode == iOpenCL::SAMPLER_TEXTURE_ADDRESS_MODE_BORDER ||
+                                inlineSamplerMD.TCYAddressMode == iOpenCL::SAMPLER_TEXTURE_ADDRESS_MODE_BORDER ||
+                                inlineSamplerMD.TCZAddressMode == iOpenCL::SAMPLER_TEXTURE_ADDRESS_MODE_BORDER;
+                            bool anyMapFilterModeNearest =
+                                inlineSamplerMD.MagFilterType == iOpenCL::SAMPLER_MAPFILTER_POINT ||
+                                inlineSamplerMD.MinFilterType == iOpenCL::SAMPLER_MAPFILTER_POINT;
+                            bool snapWARequired = anyAddressModeClamp &&
+                                anyMapFilterModeNearest &&
+                                !inlineSamplerMD.NormalizedCoords;
+                            samplerVal = snapWARequired ? -1 : 0;
+                        }
+                        else
+                        {
+                            llvm_unreachable("unexpected sampler property");
+                        }
+                    }
+                }
+            }
+            return ConstantInt::get(CI.getType(), samplerVal);
+        }
     }
+
+    // TODO: For now disable WA if unable to trace sampler argument.
+    // Will need to rework WA to add support for indirect sampler case.
+    return ConstantInt::get(CI.getType(), 0);
+}
+
+Value* ImageFuncResolution::getSamplerAddressMode(CallInst& CI)
+{
+    return getSamplerProperty<ImplicitArg::SAMPLER_ADDRESS>(CI);
 }
 
 Value* ImageFuncResolution::getSamplerNormalizedCoords(CallInst& CI)
 {
-    MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
-    ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
-    Value* sampler = ValueTracker::track(&CI, 0, pMdUtils, modMD);
-    if (sampler == nullptr)
-    {
-        // TODO: For now disable WA if unable to trace sampler argument.
-        // Will need to rework WA to add support for indirect sampler case.
-        return ConstantInt::get(CI.getType(), 0);
-    }
-    else if (isa<Argument>(sampler))
-    {
-        Argument* arg = getImplicitImageArg(CI, ImplicitArg::SAMPLER_NORMALIZED);
-        return arg;
-    }
-    else
-    {
-        llvm::Function* pFunc = CI.getParent()->getParent();
-        IGC_ASSERT_MESSAGE(isa<ConstantInt>(sampler), "Sampler must be a constant integer");
-
-        uint64_t samplerVal = 0;
-        uint samplerValue = int_cast<unsigned int>(cast<ConstantInt>(sampler)->getZExtValue());
-
-        if (modMD->FuncMD.find(pFunc) != modMD->FuncMD.end())
-        {
-            FunctionMetaData funcMD = modMD->FuncMD[pFunc];
-            ResourceAllocMD resAllocMD = funcMD.resAllocMD;
-            for (auto i = resAllocMD.inlineSamplersMD.begin(), e = resAllocMD.inlineSamplersMD.end(); i != e; ++i)
-            {
-                IGC::InlineSamplersMD inlineSamplerMD = *i;
-                if (samplerValue == inlineSamplerMD.m_Value)
-                {
-                    InlineSamplerState samplerState{ static_cast<uint64_t>(samplerValue) };
-                    samplerVal = inlineSamplerMD.NormalizedCoords;
-                }
-            }
-        }
-        return ConstantInt::get(CI.getType(), samplerVal);
-    }
+    return getSamplerProperty<ImplicitArg::SAMPLER_NORMALIZED>(CI);
 }
 
 Value* ImageFuncResolution::getSamplerSnapWARequired(CallInst& CI)
 {
-    MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
-    ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
-    Value* sampler = ValueTracker::track(&CI, 0, pMdUtils, modMD);
-    if (sampler == nullptr)
-    {
-        // TODO: For now disable WA if unable to trace sampler argument.
-        // Will need to rework WA to add support for indirect sampler case.
-        return ConstantInt::get(CI.getType(), 0);
-    }
-    else if (isa<Argument>(sampler))
-    {
-        Argument* arg = getImplicitImageArg(CI, ImplicitArg::SAMPLER_SNAP_WA);
-        return arg;
-    }
-    else
-    {
-        IGC_ASSERT_MESSAGE(isa<ConstantInt>(sampler), "Sampler must be a constant integer");
-
-        llvm::Function* pFunc = CI.getParent()->getParent();
-
-        bool snapWARequired = false;
-        uint samplerVal = int_cast<unsigned int>(cast<ConstantInt>(sampler)->getZExtValue());
-
-        if (modMD->FuncMD.find(pFunc) != modMD->FuncMD.end())
-        {
-            FunctionMetaData funcMD = modMD->FuncMD[pFunc];
-            ResourceAllocMD resAllocMD = funcMD.resAllocMD;
-            for (auto i = resAllocMD.inlineSamplersMD.begin(), e = resAllocMD.inlineSamplersMD.end(); i != e; ++i)
-            {
-                InlineSamplersMD inlineSamplerMD = *i;
-                if (samplerVal == inlineSamplerMD.m_Value)
-                {
-                    InlineSamplerState samplerState{ static_cast<uint64_t>(samplerVal) };
-                    bool anyAddressModeClamp =
-                        inlineSamplerMD.TCXAddressMode == iOpenCL::SAMPLER_TEXTURE_ADDRESS_MODE_BORDER ||
-                        inlineSamplerMD.TCYAddressMode == iOpenCL::SAMPLER_TEXTURE_ADDRESS_MODE_BORDER ||
-                        inlineSamplerMD.TCZAddressMode == iOpenCL::SAMPLER_TEXTURE_ADDRESS_MODE_BORDER;
-                    bool anyMapFilterModeNearest =
-                        inlineSamplerMD.MagFilterType == iOpenCL::SAMPLER_MAPFILTER_POINT ||
-                        inlineSamplerMD.MinFilterType == iOpenCL::SAMPLER_MAPFILTER_POINT;
-                    snapWARequired = anyAddressModeClamp &&
-                        anyMapFilterModeNearest &&
-                        !inlineSamplerMD.NormalizedCoords;
-                }
-            }
-        }
-        return ConstantInt::get(CI.getType(), snapWARequired ? -1 : 0);
-    }
+    return getSamplerProperty<ImplicitArg::SAMPLER_SNAP_WA>(CI);
 }
 
 Argument* ImageFuncResolution::getImplicitImageArg(CallInst& CI, ImplicitArg::ArgType argType) {
@@ -324,8 +301,5 @@ Argument* ImageFuncResolution::getImplicitImageArg(CallInst& CI, ImplicitArg::Ar
     IGC_ASSERT_MESSAGE(pFunc->arg_size() >= numImplicitArgs, "Function arg size does not match meta data args.");
     unsigned int implicitArgIndexInFunc = pFunc->arg_size() - numImplicitArgs + implicitArgIndex;
 
-    Function::arg_iterator arg = pFunc->arg_begin();
-    for (unsigned int i = 0; i < implicitArgIndexInFunc; ++i, ++arg);
-
-    return &(*arg);
+    return std::next(pFunc->arg_begin(), implicitArgIndexInFunc);
 }
