@@ -16,6 +16,11 @@ SPDX-License-Identifier: MIT
 #include <llvm/IR/InstVisitor.h>
 #include "common/LLVMWarningsPop.hpp"
 
+namespace IGC
+{
+    class CodeGenContext;
+}
+
 namespace IGC
 {
     /// @brief  ImageFuncsAnalysis pass used for analyzing which OpenCL image dimension functions
@@ -45,6 +50,7 @@ namespace IGC
         {
             AU.setPreservesCFG();
             AU.addRequired<MetaDataUtilsWrapper>();
+            AU.addRequired<CodeGenContextWrapper>();
         }
 
         /// @brief  Main entry point.
@@ -75,7 +81,8 @@ namespace IGC
         static const llvm::StringRef GET_IMAGE_CHANNEL_DATA_TYPE;
         static const llvm::StringRef GET_IMAGE_CHANNEL_ORDER;
         static const llvm::StringRef GET_IMAGE_SRGB_CHANNEL_ORDER;
-        static const llvm::StringRef GET_IMAGE_ARRAY_SIZE;
+        static const llvm::StringRef GET_IMAGE1D_ARRAY_SIZE;
+        static const llvm::StringRef GET_IMAGE2D_ARRAY_SIZE;
         static const llvm::StringRef GET_IMAGE_NUM_SAMPLES;
         static const llvm::StringRef GET_SAMPLER_ADDRESS_MODE;
         static const llvm::StringRef GET_SAMPLER_NORMALIZED_COORDS;
@@ -94,7 +101,11 @@ namespace IGC
         ///        one for each image).
         ImplicitArg::ArgMap m_argMap;
         /// @brief  MetaData utils used to generate LLVM metadata
-        IGCMD::MetaDataUtils* m_pMDUtils;
+        IGCMD::MetaDataUtils* m_pMDUtils = nullptr;
+
+        /// @brief If true, implicit image args for information like width, height, etc.
+        ///        will be added to m_argMap
+        bool m_addImplicitImageArgs{};
     };
 
 } // namespace IGC
