@@ -269,6 +269,8 @@ KernelArg::ArgType KernelArg::calcArgType(const ImplicitArg& arg) const
         // fall through until LOCAL_ID_Z
     case ImplicitArg::LOCAL_ID_Z:
         return KernelArg::ArgType::IMPLICIT_LOCAL_IDS;
+    case ImplicitArg::BUFFER_SIZE:
+        return KernelArg::ArgType::IMPLICIT_BUFFER_SIZE;
     case ImplicitArg::STAGE_IN_GRID_ORIGIN:
         return KernelArg::ArgType::IMPLICIT_STAGE_IN_GRID_ORIGIN;
     case ImplicitArg::STAGE_IN_GRID_SIZE:
@@ -360,7 +362,6 @@ KernelArg::ArgType KernelArg::calcArgType(const ImplicitArg& arg) const
 
     case ImplicitArg::ASSERT_BUFFER_POINTER:
         return KernelArg::ArgType::IMPLICIT_ASSERT_BUFFER;
-
     default:
         return KernelArg::ArgType::NOT_TO_ALLOCATE;
     }
@@ -802,7 +803,6 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_STAGE_IN_GRID_ORIGIN,
             KernelArg::ArgType::IMPLICIT_STAGE_IN_GRID_SIZE,
             KernelArg::ArgType::IMPLICIT_ENQUEUED_LOCAL_WORK_SIZE,
-
             KernelArg::ArgType::IMPLICIT_BINDLESS_OFFSET,
 
             KernelArg::ArgType::IMPLICIT_IMAGE_HEIGHT,
@@ -843,6 +843,8 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::RT_STACK_ID,
             KernelArg::ArgType::IMPLICIT_LOCAL_IDS,
 
+            KernelArg::ArgType::IMPLICIT_BUFFER_SIZE,
+
             KernelArg::ArgType::IMPLICIT_ARG_BUFFER,
             KernelArg::ArgType::IMPLICIT_ASSERT_BUFFER,
 
@@ -924,7 +926,6 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_STAGE_IN_GRID_ORIGIN,
             KernelArg::ArgType::IMPLICIT_STAGE_IN_GRID_SIZE,
             KernelArg::ArgType::IMPLICIT_ENQUEUED_LOCAL_WORK_SIZE,
-
             KernelArg::ArgType::IMPLICIT_BINDLESS_OFFSET,
 
             KernelArg::ArgType::IMPLICIT_ARG_BUFFER,
@@ -964,6 +965,8 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_LOCAL_MEMORY_STATELESS_WINDOW_SIZE,
             KernelArg::ArgType::IMPLICIT_PRIVATE_MEMORY_STATELESS_SIZE,
 
+            KernelArg::ArgType::IMPLICIT_BUFFER_SIZE,
+
             KernelArg::ArgType::STRUCT,
             KernelArg::ArgType::SAMPLER,
             KernelArg::ArgType::IMAGE_1D,
