@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -8,6 +8,7 @@ SPDX-License-Identifier: MIT
 
 #include "AdaptorCommon/ImplicitArgs.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/LocalBuffers/InlineLocalsResolution.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryToSLM.hpp"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
@@ -45,8 +46,6 @@ InlineLocalsResolution::InlineLocalsResolution() :
     initializeInlineLocalsResolutionPass(*PassRegistry::getPassRegistry());
 }
 
-const unsigned int InlineLocalsResolution::VALID_LOCAL_HIGH_BITS = 0x10000000;
-
 static bool useAsPointerOnly(Value* V) {
     IGC_ASSERT_MESSAGE(V->getType()->isPointerTy(), "Expect the input value is a pointer!");
 
@@ -520,8 +519,7 @@ void InlineLocalsResolution::computeOffsetList(Module& M, llvm::MapVector<Functi
             offsetMap[F][G] = (offset & 0xFFFF);
 
             // And the total size after this local is added
-            PointerType* ptrType = dyn_cast<PointerType>(G->getType());
-            Type* varType = IGCLLVM::getNonOpaquePtrEltTy(ptrType);
+            Type* varType = G->getValueType();
             if (G == m_pGV)
             {
                 // it is GetMemPoolPtr usage
