@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2021 Intel Corporation
+Copyright (C) 2020-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -40,7 +40,6 @@ IGC_INITIALIZE_PASS_END(PrivateMemoryToSLM, PASS_FLAG, PASS_DESCRIPTION, PASS_CF
 namespace IGC
 {
     char PrivateMemoryToSLM::ID = 0;
-    const unsigned int PrivateMemoryToSLM::VALID_LOCAL_HIGH_BITS = 0x10000000;
     const unsigned int PrivateMemoryToSLM::SLM_LOCAL_VARIABLE_ALIGNMENT = 4;
     const unsigned int PrivateMemoryToSLM::SLM_LOCAL_SIZE_ALIGNMENT = 32;
 
@@ -191,10 +190,9 @@ namespace IGC
 
             // Calculate an offset for new SLM variables.
             unsigned int offset = 0;
-            for (auto offsets : ModuleMD->FuncMD[F].localOffsets)
+            for (const auto &offsets : ModuleMD->FuncMD[F].localOffsets)
             {
-                PointerType* ptrType = dyn_cast<PointerType>(offsets.m_Var->getType());
-                Type* varType = IGCLLVM::getNonOpaquePtrEltTy(ptrType);
+                Type* varType = offsets.m_Var->getValueType();
                 offset = iSTD::Align(offset, IGCLLVM::getPreferredAlignValue(&DL, offsets.m_Var));
                 offset += (unsigned int) DL.getTypeAllocSize(varType);
             }
@@ -230,7 +228,7 @@ namespace IGC
 
                 if (m_ForceAll || isForcedBuffer)
                 {
-                    Type* origType = IGCLLVM::getNonOpaquePtrEltTy(pAI->getType());
+                    Type* origType = pAI->getAllocatedType();
                     bool isArray = origType->isArrayTy();
                     Type* eltType = isArray ? origType->getArrayElementType() : origType;
                     uint64_t numEltsPerThread = isArray ? origType->getArrayNumElements() : 1;
