@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -13,6 +13,7 @@ SPDX-License-Identifier: MIT
 #include "common/Stats.hpp"
 #include "common/secure_string.h"
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/Support/Alignment.h"
 #include <llvm/IR/Function.h>
@@ -38,6 +39,14 @@ IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
 IGC_INITIALIZE_PASS_DEPENDENCY(AssumptionCacheTracker)
 IGC_INITIALIZE_PASS_END(StatelessToStateful, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
 
+static cl::opt<TargetAddressing> targetAddressingMode(
+    "target-addressing-mode", cl::init(TargetAddressing::BINDFUL), cl::Hidden,
+    cl::values(
+        clEnumValN(TargetAddressing::BINDFUL, "bindful", "Set bindful as target addressing mode"),
+        clEnumValN(TargetAddressing::BINDLESS, "bindless", "Set bindless as target addressing mode")
+    ),
+    cl::desc("Set target addressing for stateful promotion"));
+
 // This pass turns a global/constants address space (stateless) load/store into a stateful a load/store.
 //
 // The conservative approach is to search for any directly positively-indexed kernels argument, such as:
@@ -129,15 +138,11 @@ IGC_INITIALIZE_PASS_END(StatelessToStateful, PASS_FLAG, PASS_DESCRIPTION, PASS_C
 
 char StatelessToStateful::ID = 0;
 
-StatelessToStateful::StatelessToStateful()
+StatelessToStateful::StatelessToStateful() : FunctionPass(ID), m_targetAddressing(targetAddressingMode) {}
+
+StatelessToStateful::StatelessToStateful(TargetAddressing addressing)
     : FunctionPass(ID),
-    m_hasBufferOffsetArg(false),
-    m_hasOptionalBufferOffsetArg(false),
-    m_hasPositivePointerOffset(false),
-    m_ACT(nullptr),
-    m_pImplicitArgs(nullptr),
-    m_pKernelArgs(nullptr),
-    m_changed(false)
+    m_targetAddressing(addressing)
 {
     initializeStatelessToStatefulPass(*PassRegistry::getPassRegistry());
 }
@@ -221,7 +226,7 @@ Argument* StatelessToStateful::getBufferOffsetArg(Function* F, uint32_t ArgNumbe
 // The final instruction of the expansion is returned in 'offset'
 //
 bool StatelessToStateful::getOffsetFromGEP(
-    Function* F, SmallVector<GetElementPtrInst*, 4> GEPs,
+    Function* F, const SmallVector<GetElementPtrInst*, 4>& GEPs,
     uint32_t argNumber, bool isImplicitArg, Value*& offset)
 {
     Module* M = F->getParent();
@@ -266,7 +271,6 @@ bool StatelessToStateful::getOffsetFromGEP(
 
         IGC_ASSERT_MESSAGE(PtrTy, "Only accept scalar pointer!");
 
-        Type* Ty = PtrTy;
         gep_type_iterator GTI = gep_type_begin(GEP);
         for (auto OI = GEP->op_begin() + 1, E = GEP->op_end(); OI != E; ++OI, ++GTI)
         {
@@ -283,11 +287,10 @@ bool StatelessToStateful::getOffsetFromGEP(
                     PointerValue = BinaryOperator::CreateAdd(PointerValue, OffsetValue, "", GEP);
                     cast<llvm::Instruction>(PointerValue)->setDebugLoc(GEP->getDebugLoc());
                 }
-                Ty = StTy->getElementType(Field);
             }
             else
             {
-                Ty = GTI.getIndexedType();
+                Type* Ty = GTI.getIndexedType();
                 if (const ConstantInt * CI = dyn_cast<ConstantInt>(Idx))
                 {
                     if (!CI->isZero())
@@ -538,7 +541,7 @@ bool StatelessToStateful::isUntypedAtomic(const GenISAIntrinsic::ID intrinID)
         intrinID == GenISAIntrinsic::GenISA_fcmpxchgatomicrawA64);
 }
 
-unsigned StatelessToStateful::encodeStatefulAddrspace(unsigned uavIndex)
+unsigned StatelessToStateful::encodeBindfulAddrspace(unsigned uavIndex)
 {
     auto int32Ty = Type::getInt32Ty(m_Module->getContext());
     auto resourceNumber = ConstantInt::get(int32Ty, uavIndex);
@@ -555,8 +558,37 @@ void StatelessToStateful::promoteIntrinsic(InstructionInfo& II)
     Module* M = m_F->getParent();
     const DebugLoc& DL = I->getDebugLoc();
     GenISAIntrinsic::ID const intrinID = I->getIntrinsicID();
+    PointerType* pTy = IGCLLVM::getWithSamePointeeType(dyn_cast<PointerType>(II.ptr->getType()), II.getStatefulAddrSpace());
+
+    if (m_targetAddressing == TargetAddressing::BINDLESS)
+    {
+        Argument* srcOffset = m_pImplicitArgs->getNumberedImplicitArg(*m_F, ImplicitArg::BINDLESS_OFFSET, II.getBaseArgIndex());
+        auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, srcOffset, pTy, "", I);
+        if (intrinID == GenISAIntrinsic::GenISA_simdBlockRead)
+        {
+            Function* newBlockReadFunc = GenISAIntrinsic::getDeclaration(M,
+                GenISAIntrinsic::GenISA_simdBlockReadBindless,
+                { I->getType(), newBasePtr->getType(), Type::getInt32Ty(M->getContext())});
+            Instruction* newBlockRead = CallInst::Create(newBlockReadFunc, { newBasePtr, II.offset }, "", I);
+            newBlockRead->setDebugLoc(I->getDebugLoc());
+            I->replaceAllUsesWith(newBlockRead);
+            I->eraseFromParent();
+        }
+        else if (intrinID == GenISAIntrinsic::GenISA_simdBlockWrite)
+        {
+            Function* newBlockWriteFunc = GenISAIntrinsic::getDeclaration(M,
+                GenISAIntrinsic::GenISA_simdBlockWriteBindless,
+                { newBasePtr->getType(), I->getOperand(1)->getType(), Type::getInt32Ty(M->getContext())});
+            Instruction* newBlockWrite = CallInst::Create(newBlockWriteFunc, { newBasePtr, I->getOperand(1), II.offset }, "", I);
+            newBlockWrite->setDebugLoc(I->getDebugLoc());
+            I->replaceAllUsesWith(newBlockWrite);
+            I->eraseFromParent();
+        }
+        return;
+    }
+
+    IGC_ASSERT(m_targetAddressing == TargetAddressing::BINDFUL);
 
-    PointerType* pTy = PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(II.ptr->getType()), II.getStatefulAddrSpace());
     Instruction* statefulPtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
     Instruction* statefulInst = nullptr;
 
@@ -580,6 +612,36 @@ void StatelessToStateful::promoteIntrinsic(InstructionInfo& II)
             { pTy,I->getOperand(1)->getType() });
         statefulInst = CallInst::Create(pFunc, args, "", I);
     }
+    else if (intrinID == GenISAIntrinsic::GenISA_LSCStoreCmask)
+    {
+        SmallVector<Value*, 6> args;
+        args.push_back(statefulPtr);
+        args.push_back(I->getOperand(1));
+        args.push_back(I->getOperand(2));
+        args.push_back(I->getOperand(3));
+        args.push_back(I->getOperand(4));
+        args.push_back(I->getOperand(5));
+        Function* pFunc = GenISAIntrinsic::getDeclaration(
+            M,
+            intrinID,
+            { pTy, I->getOperand(2)->getType() });
+        statefulInst = CallInst::Create(pFunc, args, "", I);
+    }
+    else if (intrinID == GenISAIntrinsic::GenISA_LSCLoadCmask)
+    {
+        Function* pCurrInstFunc = I->getCalledFunction();
+        SmallVector<Value*, 5> args;
+        args.push_back(statefulPtr);
+        args.push_back(I->getOperand(1));
+        args.push_back(I->getOperand(2));
+        args.push_back(I->getOperand(3));
+        args.push_back(I->getOperand(4));
+        Function* pFunc = GenISAIntrinsic::getDeclaration(
+            M,
+            intrinID,
+            { pCurrInstFunc->getReturnType(), pTy });
+        statefulInst = CallInst::Create(pFunc, args, "", I);
+    }
     else if (isUntypedAtomic(intrinID))
     {
         if (intrinID == GenISAIntrinsic::GenISA_intatomicrawA64 ||
@@ -614,31 +676,51 @@ void StatelessToStateful::promoteLoad(InstructionInfo& II)
     LoadInst* I = cast<LoadInst>(II.statelessInst);
     PointerType* pTy = PointerType::get(I->getType(), II.getStatefulAddrSpace());
 
-    Instruction* statefulPtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
-    Instruction* pLoad = new LoadInst(
-        IGCLLVM::getNonOpaquePtrEltTy(statefulPtr->getType()),
-        statefulPtr,
-        "",
-        I->isVolatile(),
-        IGCLLVM::getAlign(*I), I->getOrdering(), I->getSyncScopeID(),
-        I);
-
     const DebugLoc& DL = I->getDebugLoc();
-    statefulPtr->setDebugLoc(DL);
-    pLoad->setDebugLoc(DL);
 
-    Value* ptr = I->getPointerOperand();
-    PointerType* ptrType = dyn_cast<PointerType>(ptr->getType());
-    if (ptrType && ptrType->getAddressSpace() == ADDRESS_SPACE_CONSTANT)
+    if (m_targetAddressing == TargetAddressing::BINDLESS)
     {
-        LLVMContext& context = I->getContext();
-        MDString* const metadataName = MDString::get(context, "invariant.load");
-        MDNode* node = MDNode::get(context, metadataName);
-        pLoad->setMetadata(LLVMContext::MD_invariant_load, node);
-    }
+        Argument* srcOffset = m_pImplicitArgs->getNumberedImplicitArg(*m_F, ImplicitArg::BINDLESS_OFFSET, II.getBaseArgIndex());
+        auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, srcOffset, pTy, "", I);
+        auto bindlessLoad = IGC::CreateLoadRawIntrinsic(I, cast<Instruction>(newBasePtr), II.offset);
 
-    I->replaceAllUsesWith(pLoad);
-    I->eraseFromParent();
+        newBasePtr->setDebugLoc(DL);
+        bindlessLoad->setDebugLoc(DL);
+
+        I->replaceAllUsesWith(bindlessLoad);
+        I->eraseFromParent();
+    }
+    else if (m_targetAddressing == TargetAddressing::BINDFUL)
+    {
+        auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
+        auto bindfulLoad = new LoadInst(
+            I->getType(),
+            newBasePtr,
+            "",
+            I->isVolatile(),
+            IGCLLVM::getAlign(*I), I->getOrdering(), I->getSyncScopeID(),
+            I);
+
+        newBasePtr->setDebugLoc(DL);
+        bindfulLoad->setDebugLoc(DL);
+
+        Value* ptr = I->getPointerOperand();
+        PointerType* ptrType = dyn_cast<PointerType>(ptr->getType());
+        if (ptrType && ptrType->getAddressSpace() == ADDRESS_SPACE_CONSTANT)
+        {
+            LLVMContext& context = I->getContext();
+            MDString* const metadataName = MDString::get(context, "invariant.load");
+            MDNode* node = MDNode::get(context, metadataName);
+            bindfulLoad->setMetadata(LLVMContext::MD_invariant_load, node);
+        }
+
+        I->replaceAllUsesWith(bindfulLoad);
+        I->eraseFromParent();
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(false, "Unsupported addressing!");
+    }
 }
 
 void StatelessToStateful::promoteStore(InstructionInfo& II)
@@ -648,19 +730,38 @@ void StatelessToStateful::promoteStore(InstructionInfo& II)
     Value* dataVal = I->getValueOperand();
     PointerType* pTy = PointerType::get(dataVal->getType(), II.getStatefulAddrSpace());
 
-    Instruction* statefulPtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
-    Instruction* pStore = new StoreInst(
-        dataVal,
-        statefulPtr,
-        I->isVolatile(),
-        IGCLLVM::getAlign(*I), I->getOrdering(), I->getSyncScopeID(),
-        I);
-
     const DebugLoc& DL = I->getDebugLoc();
-    statefulPtr->setDebugLoc(DL);
-    pStore->setDebugLoc(DL);
 
-    I->eraseFromParent();
+    if (m_targetAddressing == TargetAddressing::BINDLESS)
+    {
+        Argument* srcOffset = m_pImplicitArgs->getNumberedImplicitArg(*m_F, ImplicitArg::BINDLESS_OFFSET, II.getBaseArgIndex());
+        auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, srcOffset, pTy, "", I);
+        auto bindlessStore = IGC::CreateStoreRawIntrinsic(I, cast<Instruction>(newBasePtr), II.offset);
+
+        newBasePtr->setDebugLoc(DL);
+        bindlessStore->setDebugLoc(DL);
+
+        I->eraseFromParent();
+    }
+    else if (m_targetAddressing == TargetAddressing::BINDFUL)
+    {
+        auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
+        auto bindfulStore = new StoreInst(
+            dataVal,
+            newBasePtr,
+            I->isVolatile(),
+            IGCLLVM::getAlign(*I), I->getOrdering(), I->getSyncScopeID(),
+            I);
+
+        newBasePtr->setDebugLoc(DL);
+        bindfulStore->setDebugLoc(DL);
+
+        I->eraseFromParent();
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(false, "Unsupported addressing!");
+    }
 }
 
 void StatelessToStateful::promoteInstruction(StatelessToStateful::InstructionInfo& InstInfo)
@@ -696,21 +797,34 @@ void StatelessToStateful::promote()
     {
         IGC_ASSERT(bufferPos < maxPromotionCount);
 
-        ArgAllocMD* argAlloc = &resAllocMD->argAllocMDList[baseArgIndex];
-
-        // If the support for dynamic BTIs allocation is disabled, then BTIs are pre-assigned
-        // in ResourceAllocator pass for all resources independently whether they are
-        // accessed through stateful addressing model or not.
-        if (ctx->platform.supportDynamicBTIsAllocation())
+        unsigned statefullAddrspace = 0;
+        if (m_targetAddressing == TargetAddressing::BINDLESS)
         {
-            argAlloc->type = ResourceTypeEnum::UAVResourceType;
-            argAlloc->indexType = resAllocMD->uavsNumType + bufferPos;
+            statefullAddrspace =
+                IGC::EncodeAS4GFXResource(
+                    *UndefValue::get(Type::getInt32Ty(m_Module->getContext())),
+                    IGC::BINDLESS);
+        }
+        else
+        {
+            ArgAllocMD* argAlloc = &resAllocMD->argAllocMDList[baseArgIndex];
+
+            // If the support for dynamic BTIs allocation is disabled, then BTIs are pre-assigned
+            // in ResourceAllocator pass for all resources independently whether they are
+            // accessed through stateful addressing model or not.
+            if (ctx->platform.supportDynamicBTIsAllocation())
+            {
+                argAlloc->type = ResourceTypeEnum::UAVResourceType;
+                argAlloc->indexType = resAllocMD->uavsNumType + bufferPos;
+            }
+
+            statefullAddrspace = encodeBindfulAddrspace(argAlloc->indexType);
         }
 
-        unsigned statefullAddrspace = encodeStatefulAddrspace(argAlloc->indexType);
         for (auto &instInfo : instsToPromote)
         {
             instInfo.setStatefulAddrspace(statefullAddrspace);
+            instInfo.setBaseArgIndex(baseArgIndex);
             promoteInstruction(instInfo);
         }
         bufferPos++;
@@ -760,6 +874,19 @@ void StatelessToStateful::visitCallInst(CallInst& I)
 
         addToPromotionMap(I, ptr);
     }
+    else if (intrinID == GenISAIntrinsic::GenISA_LSCLoadCmask ||
+        intrinID == GenISAIntrinsic::GenISA_LSCStoreCmask)
+    {
+        Value* ptr = Inst->getOperand(0);
+        PointerType* ptrTy = dyn_cast<PointerType>(ptr->getType());
+        // If not global, skip.
+        if (ptrTy->getPointerAddressSpace() != ADDRESS_SPACE_GLOBAL)
+        {
+            return;
+        }
+
+        addToPromotionMap(I, ptr);
+    }
 
     // check if there's non-kernel-arg load/store
     if (IGC_IS_FLAG_ENABLED(DumpHasNonKernelArgLdSt)) {
@@ -774,6 +901,7 @@ void StatelessToStateful::visitCallInst(CallInst& I)
             case GenISAIntrinsic::GenISA_LSCLoad:
             case GenISAIntrinsic::GenISA_LSCLoadBlock:
             case GenISAIntrinsic::GenISA_LSCPrefetch:
+            case GenISAIntrinsic::GenISA_LSCLoadCmask:
                 return true;
             default:
                 break;
@@ -787,6 +915,7 @@ void StatelessToStateful::visitCallInst(CallInst& I)
             case GenISAIntrinsic::GenISA_LSCStore:
             case GenISAIntrinsic::GenISA_LSCStoreBlock:
             case GenISAIntrinsic::GenISA_simdBlockWrite:
+            case GenISAIntrinsic::GenISA_LSCStoreCmask:
                 return true;
             default:
                 break;
@@ -924,7 +1053,7 @@ void StatelessToStateful::finalizeArgInitialValue(Function* F)
     Type* int32Ty = Type::getInt32Ty(M->getContext());
     Value* ZeroValue = ConstantInt::get(int32Ty, 0);
 
-    for (auto II : m_argsInfo)
+    for (const auto &II : m_argsInfo)
     {
         const KernelArg* kernelArg = II.first;
         int mapVal = II.second;
