@@ -160,21 +160,6 @@ bool ScalarizeFunction::runOnFunction(Function& F)
         if (Value * val = dyn_cast<Value>(*index))
         {
             UndefValue* undefVal = UndefValue::get((*index)->getType());
-
-            if (MDNode* pEIMD = (*index)->getMetadata("implicitGlobalID"))
-            {
-                // Compute thread and group identification instructions must have 'Output' attribute
-                // added later during compilation. The implicitGlobalID metadata attached to this
-                // instruction must be assigned to a new instruction, which replaces this instruction.
-                // Unfortunatelly, replaceAllUsesWith() will not ensure such propagation.
-                Instruction* pNewInst = dyn_cast_or_null<llvm::Instruction>(undefVal);
-                if (pNewInst)
-                {
-                    IGC_ASSERT_MESSAGE(pNewInst, "Missing implicit global ID instruction");
-                    Instruction* instr = dyn_cast<Instruction>(*index);
-                    pNewInst->copyMetadata(*instr);
-                }
-            }
             (val)->replaceAllUsesWith(undefVal);
         }
         IGC_ASSERT_MESSAGE((*index)->use_empty(), "Unable to remove used instruction");
@@ -325,6 +310,9 @@ void ScalarizeFunction::dispatchInstructionToScalarize(Instruction* I)
 
     switch (I->getOpcode())
     {
+    case Instruction::FNeg:
+        scalarizeInstruction(dyn_cast<UnaryOperator>(I));
+        break;
     case Instruction::Add:
     case Instruction::Sub:
     case Instruction::Mul:
@@ -430,6 +418,59 @@ void ScalarizeFunction::recoverNonScalarizableInst(Instruction* Inst)
     }
 }
 
+void ScalarizeFunction::scalarizeInstruction(UnaryOperator* UI)
+{
+    V_PRINT(scalarizer, "\t\tUnary instruction\n");
+    IGC_ASSERT_MESSAGE(UI, "instruction type dynamic cast failed");
+    IGCLLVM::FixedVectorType* instType = dyn_cast<IGCLLVM::FixedVectorType>(UI->getType());
+    // Do not apply if optnone function
+    if (UI->getFunction()->hasOptNone())
+        return;
+
+    // Only need handling for vector binary ops
+    if (!instType) return;
+
+    // Prepare empty SCM entry for the instruction
+    SCMEntry* newEntry = getSCMEntry(UI);
+
+    // Get additional info from instruction
+    unsigned numElements = int_cast<unsigned>(instType->getNumElements());
+
+    // Obtain scalarized argument
+    SmallVector<Value*, MAX_INPUT_VECTOR_WIDTH>operand0;
+    bool op0IsConst;
+
+    obtainScalarizedValues(operand0, &op0IsConst, UI->getOperand(0), UI);
+
+    // If argument is constant, don't bother Scalarizing inst
+    if (op0IsConst) return;
+
+    // Generate new (scalar) instructions
+    SmallVector<Value*, MAX_INPUT_VECTOR_WIDTH>newScalarizedInsts;
+    newScalarizedInsts.resize(numElements);
+    for (unsigned dup = 0; dup < numElements; dup++)
+    {
+        Value* Val = UnaryOperator::Create(
+            UI->getOpcode(),
+            operand0[dup],
+            UI->getName(),
+            UI
+        );
+        if (UnaryOperator* UO = dyn_cast<UnaryOperator>(Val)) {
+            // Copy fast math flags if any.
+            if (isa<FPMathOperator>(UO))
+                UO->setFastMathFlags(UI->getFastMathFlags());
+        }
+        newScalarizedInsts[dup] = Val;
+    }
+
+    // Add new value/s to SCM
+    updateSCMEntryWithValues(newEntry, &(newScalarizedInsts[0]), UI, true);
+
+    // Remove original instruction
+    m_removedInsts.insert(UI);
+}
+
 void ScalarizeFunction::scalarizeInstruction(BinaryOperator* BI)
 {
     V_PRINT(scalarizer, "\t\tBinary instruction\n");
@@ -807,27 +848,6 @@ void ScalarizeFunction::scalarizeInstruction(ExtractElementInst* EI)
     if (static_cast<unsigned int>(scalarIndex) < (unsigned)valueVType->getNumElements())
     {
         IGC_ASSERT_MESSAGE(NULL != operand[static_cast<unsigned int>(scalarIndex)], "SCM error");
-
-        if (IGC_IS_FLAG_ENABLED(UseOffsetInLocation))
-        {
-            // Metadata "implicitGlobalID" must be propagated to a new instruction as a WA
-            // for missing meta data preservation in this pass. When a general fix is applied
-            // then instructions below for this specific propagation must be removed.
-            Value* pNewVal = operand[static_cast<unsigned int>(scalarIndex)];
-
-            if (MDNode* pEIMD = EI->getMetadata("implicitGlobalID"))
-            {
-                // Compute thread and group identification instructions must have 'Output' attribute
-                // added later during compilation. The implicitGlobalID metadata attached to this
-                // instruction must be assigned to a new instruction, which replaces this instruction.
-                // Unfortunatelly, replaceAllUsesWith() will not ensure such propagation.
-                Instruction* pNewInst = dyn_cast_or_null<llvm::Instruction>(pNewVal);
-                IGC_ASSERT_MESSAGE(pNewInst, "Missing implicit global ID instruction");
-
-                pNewInst->copyMetadata(*EI);
-            }
-        }
-
         // Replace all users of this inst, with the extracted scalar value
         EI->replaceAllUsesWith(operand[static_cast<unsigned int>(scalarIndex)]);
     }
@@ -1326,7 +1346,7 @@ void ScalarizeFunction::resolveDeferredInstructions()
     llvm::MapVector<Value*, Value*> dummyToScalarMap;
 
     // lambda to check if a value is a dummy instruction
-    auto isDummyValue = [this](Value* val) -> bool
+    auto isDummyValue = [this](Value* val)
     {
         auto* call = dyn_cast<CallInst>(val);
         if (!call) return false;
@@ -1421,7 +1441,7 @@ void ScalarizeFunction::resolveDeferredInstructions()
         }
     }
 
-    for ( auto entry : dummyToScalarMap )
+    for (const auto &entry : dummyToScalarMap)
     {
         // Replace and erase all dummy instructions (don't use eraseFromParent as the dummy is not in the function)
         Instruction *dummyInst = cast<Instruction>(entry.first);
