@@ -277,6 +277,9 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
 {
     if (!V) return nullptr;
 
+    // Continue to find from other users if current leaf is invalid.
+    auto isValidLeaf = [](Value *leaf) { return leaf && !isa<ConstantPointerNull>(leaf); };
+
     for (auto U : V->users())
     {
         if (visitedValues.find(U) != visitedValues.end()) continue;
@@ -308,7 +311,7 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
                 continue;
 
             unsigned reducedIndices = numIndices - 1;
-            if (auto leaf = findAllocaValue(GEP, depth - reducedIndices))
+            if (auto leaf = findAllocaValue(GEP, depth - reducedIndices); isValidLeaf(leaf))
             {
                 IGC_ASSERT(gepIndices.size() >= reducedIndices);
                 gepIndices.resize(gepIndices.size() - reducedIndices);
@@ -317,7 +320,7 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
         }
         else if (auto* CI = dyn_cast<CastInst>(U))
         {
-            if (auto leaf = findAllocaValue(CI, depth))
+            if (auto leaf = findAllocaValue(CI, depth); isValidLeaf(leaf))
                 return leaf;
         }
         else if (auto* CI = dyn_cast<CallInst>(U))
@@ -336,7 +339,7 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
                         Function* F = CI->getCalledFunction();
                         unsigned OpNo = OP.getOperandNo();
                         IGC_ASSERT(F->arg_size() > OpNo);
-                        if (auto leaf = findAllocaValue(F->arg_begin() + OpNo, depth))
+                        if (auto leaf = findAllocaValue(F->arg_begin() + OpNo, depth); isValidLeaf(leaf))
                         {
                             callInsts.push_back(CI);
                             return leaf;
@@ -356,7 +359,7 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
             // We cannot ignore load if alloca type is a pointer.
             if (LI->getType()->isPointerTy())
             {
-                if (auto leaf = findAllocaValue(LI, depth))
+                if (auto leaf = findAllocaValue(LI, depth); isValidLeaf(leaf))
                     return leaf;
             }
         }
@@ -370,7 +373,7 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
                 // through another alloca and we need to continue tracing it.
                 if (StoredValue->getType()->isPointerTy())
                 {
-                    if (auto leaf = findAllocaValue(ST->getPointerOperand(), depth))
+                    if (auto leaf = findAllocaValue(ST->getPointerOperand(), depth); isValidLeaf(leaf))
                         return leaf;
                 }
             }
@@ -388,7 +391,7 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
 Value* ValueTracker::trackValue(Value* I)
 {
     Value* baseValue = I;
-    auto isFinalValue = [this](auto V) { return callInsts.empty() && workList.empty() && (V == nullptr || llvm::isa<Argument>(V) || llvm::isa<ConstantInt>(V)); };
+    auto isFinalValue = [this](auto V) { return callInsts.empty() && workList.empty() && (V == nullptr || llvm::isa<Argument>(V) || llvm::isa<ConstantInt>(V) || (m_predicate && m_predicate(V))); };
 
     while (true)
     {
@@ -415,6 +418,10 @@ Value* ValueTracker::trackValue(Value* I)
         }
         else if (auto* I = dyn_cast<AllocaInst>(baseValue))
         {
+            // To track alloca, all indices must be constant.
+            if (std::any_of(gepIndices.begin(), gepIndices.end(), [](const auto& it) { return !isa<ConstantInt>(it); }))
+                return nullptr;
+
             // As alloca has been found, proceed with the second step of the algorithm.
             baseValue = findAllocaValue(I, gepIndices.size());
         }
@@ -444,9 +451,6 @@ Value* ValueTracker::trackValue(Value* I)
         }
         else if (auto* I = dyn_cast<GetElementPtrInst>(baseValue))
         {
-            if (!I->hasAllConstantIndices())
-                return nullptr;
-
             for (unsigned int i = I->getNumIndices(); i > 1; --i)
                 gepIndices.push_back(cast<ConstantInt>(I->getOperand(i)));
 
@@ -494,6 +498,12 @@ Value* ValueTracker::trackValue(Value* I)
             phiVisited.insert(std::make_pair(I, baseValue));
             return baseValue;
         }
+        else if (auto *BO = dyn_cast<BinaryOperator>(baseValue))
+        {
+            Value *Op0 = BO->getOperand(0);
+            Value *Op1 = BO->getOperand(1);
+            baseValue = isa_and_nonnull<Constant>(Op0) ? Op1 : (isa_and_nonnull<Constant>(Op1) ? Op0 : nullptr);
+        }
         else
         {
             baseValue = nullptr;
@@ -508,9 +518,21 @@ Value* ValueTracker::track(
     CallInst* pCallInst,
     const uint index,
     const MetaDataUtils* pMdUtils,
-    const IGC::ModuleMetaData* pModMD)
+    const IGC::ModuleMetaData* pModMD,
+    function_ref<bool(Value *)> predicate)
 {
-    ValueTracker VT(pCallInst->getParent()->getParent(), pMdUtils, pModMD);
+    ValueTracker VT(pCallInst->getParent()->getParent(), pMdUtils, pModMD, predicate);
     Value* baseValue = pCallInst->getOperand(index);
     return VT.trackValue(baseValue);
 }
+
+Value* ValueTracker::track(
+    Value* value,
+    Function* function,
+    const MetaDataUtils* pMdUtils,
+    const IGC::ModuleMetaData* pModMD,
+    function_ref<bool(Value *)> predicate)
+{
+    ValueTracker VT(function, pMdUtils, pModMD, predicate);
+    return VT.trackValue(value);
+}
