@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2019-2021 Intel Corporation
+Copyright (C) 2019-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -10,6 +10,7 @@ SPDX-License-Identifier: MIT
 #include "AdaptorCommon/ImplicitArgs.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include <llvm/IR/Constants.h>
 #include <llvm/IR/Function.h>
 #include "common/LLVMWarningsPop.hpp"
@@ -103,12 +104,7 @@ void PromoteStatelessToBindless::GetAccessInstToSrcPointerMap(Instruction* inst,
             case GenISAIntrinsic::GenISA_simdBlockRead:
             case GenISAIntrinsic::GenISA_simdBlockWrite:
                 break;
-            case GenISAIntrinsic::GenISA_intatomicrawA64:
-                // Ignore a buffer in this intrinsic, keep it stateless.
-                canPromoteAccess = false;
-                break;
             default:
-                IGC_ASSERT_MESSAGE(0, "Unsupported Instruction");
                 canPromoteAccess = false;
                 break;
             }
@@ -182,7 +178,7 @@ void PromoteStatelessToBindless::PromoteStatelessToBindlessBuffers(Function& F)
 
     bool supportDynamicBTIsAllocation = ctx->platform.supportDynamicBTIsAllocation();
 
-    for (auto iter : m_SrcPtrToAccessMap)
+    for (auto &iter : m_SrcPtrToAccessMap)
     {
         Argument* srcPtr = cast<Argument>(iter.first);
 
@@ -215,7 +211,7 @@ void PromoteStatelessToBindless::PromoteStatelessToBindlessBuffers(Function& F)
         }
 
         // Loop through all access instructions for srcPtr
-        for (auto insts : iter.second)
+        for (auto &insts : iter.second)
         {
             Instruction* accessInst = cast<Instruction>(insts.first);
             Instruction* addrUsedInst = cast<Instruction>(insts.second);
@@ -230,7 +226,7 @@ void PromoteStatelessToBindless::PromoteStatelessToBindlessBuffers(Function& F)
             Value* resourcePtr = IGC::GetBufferOperand(accessInst);
             IGC_ASSERT(resourcePtr);
             unsigned bindlessAS = IGC::EncodeAS4GFXResource(*UndefValue::get(builder.getInt32Ty()), IGC::BINDLESS);
-            PointerType* basePointerType = PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(resourcePtr->getType()), bindlessAS);
+            PointerType* basePointerType = IGCLLVM::getWithSamePointeeType(dyn_cast<PointerType>(resourcePtr->getType()), bindlessAS);
             Value* bufferOffset = builder.CreatePtrToInt(resourcePtr, builder.getInt32Ty());
 
             Value* basePointer = nullptr;
