@@ -1,3 +1,4 @@
+
 /*========================== begin_copyright_notice ============================
 
 Copyright (C) 2017-2021 Intel Corporation
@@ -45,6 +46,7 @@ See LICENSE.TXT for details.
 
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/Support/Alignment.h"
+#include "usc_gen7_types.h"
 
 namespace llvm {
 /// IntrinsicInst - A useful wrapper class for inspecting calls to intrinsic
@@ -69,8 +71,8 @@ protected:
 public:
     /// getIntrinsicID - Return the intrinsic ID of this intrinsic.
     ///
-    GenISAIntrinsic::ID getIntrinsicID() const {
-        return GenISAIntrinsic::getIntrinsicID(getCalledFunction());
+    GenISAIntrinsic::ID getIntrinsicID(bool useContextWrapper = true) const {
+        return GenISAIntrinsic::getIntrinsicID(getCalledFunction(), useContextWrapper);
     }
 
     bool isGenIntrinsic(GenISAIntrinsic::ID intrinID) const {
@@ -272,6 +274,13 @@ public:
         {
         case GenISAIntrinsic::GenISA_gather4Cptr:
         case GenISAIntrinsic::GenISA_gather4POCptr:
+        case GenISAIntrinsic::GenISA_gather4ICptr:
+        case GenISAIntrinsic::GenISA_gather4ICPOptr:
+        case GenISAIntrinsic::GenISA_gather4LCptr:
+        case GenISAIntrinsic::GenISA_gather4LCPOptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedCptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedICptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedLCptr:
             return true;
         default:
             break;
@@ -279,10 +288,101 @@ public:
         return false;
     }
 
+    bool hasBias() const
+    {
+        switch (getIntrinsicID())
+        {
+        case GenISAIntrinsic::GenISA_gather4Bptr:
+        case GenISAIntrinsic::GenISA_gather4BPOptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedBptr:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+    unsigned int getBiasIndex() const
+    {
+        return hasBias() ? 0 : std::numeric_limits<unsigned int>::max();
+    }
 
+    Value* getBiasValue() const
+    {
+        unsigned int index = getBiasIndex();
+        return hasBias() ? getOperand(index) : nullptr;
+    }
+
+    bool hasLod() const
+    {
+        switch (getIntrinsicID())
+        {
+        case GenISAIntrinsic::GenISA_gather4Lptr:
+        case GenISAIntrinsic::GenISA_gather4LPOptr:
+        case GenISAIntrinsic::GenISA_gather4LCptr:
+        case GenISAIntrinsic::GenISA_gather4LCPOptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedLptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedLCptr:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+
+    unsigned int getLodIndex() const
+    {
+        unsigned int shift = hasRef() ? 1 : 0;
+        return hasLod() ? shift : std::numeric_limits<unsigned int>::max();
+    }
+
+    Value* getLodValue() const
+    {
+        unsigned int index = getLodIndex();
+        return hasLod() ? getOperand(index) : nullptr;
+    }
+
+    bool hasImmediateOffsets() const
+    {
+        return true;
+    }
+
+    unsigned int getImmediateOffsetsIndex() const
+    {
+        return hasImmediateOffsets() ?
+            getNumOperands() - 5 : std::numeric_limits<unsigned int>::max();
+    }
+
+    Value* getImmediateOffsetsValue(unsigned int coordIndex) const
+    {
+        unsigned int operandCoordIndex = getImmediateOffsetsIndex() + coordIndex;
+        return hasImmediateOffsets() ? getOperand(operandCoordIndex) : nullptr;
+    }
+
+    bool IsDerivative() const
+    {
+        switch (getIntrinsicID())
+        {
+        case GenISAIntrinsic::GenISA_gather4Iptr:
+        case GenISAIntrinsic::GenISA_gather4IPOptr:
+        case GenISAIntrinsic::GenISA_gather4Bptr:
+        case GenISAIntrinsic::GenISA_gather4BPOptr:
+        case GenISAIntrinsic::GenISA_gather4ICptr:
+        case GenISAIntrinsic::GenISA_gather4ICPOptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedBptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedIptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedICptr:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+
+    inline unsigned int getPairedTextureIndex() const { return getNumOperands() - 8; }
     inline unsigned int getTextureIndex() const { return getNumOperands() - 7; }
     inline unsigned int getSamplerIndex() const { return getNumOperands() - 6; }
     inline Value* getTextureValue() const { return getOperand(getTextureIndex()); }
+    inline Value* getPairedTextureValue() const { return getOperand(getPairedTextureIndex()); }
     inline Value* getSamplerValue() const { return getOperand(getSamplerIndex()); }
 
     static inline bool classof(const GenIntrinsicInst *I) {
@@ -291,6 +391,23 @@ public:
         case GenISAIntrinsic::GenISA_gather4Cptr:
         case GenISAIntrinsic::GenISA_gather4POptr:
         case GenISAIntrinsic::GenISA_gather4POCptr:
+        case GenISAIntrinsic::GenISA_gather4Iptr:
+        case GenISAIntrinsic::GenISA_gather4IPOptr:
+        case GenISAIntrinsic::GenISA_gather4Bptr:
+        case GenISAIntrinsic::GenISA_gather4BPOptr:
+        case GenISAIntrinsic::GenISA_gather4Lptr:
+        case GenISAIntrinsic::GenISA_gather4LPOptr:
+        case GenISAIntrinsic::GenISA_gather4ICptr:
+        case GenISAIntrinsic::GenISA_gather4ICPOptr:
+        case GenISAIntrinsic::GenISA_gather4LCptr:
+        case GenISAIntrinsic::GenISA_gather4LCPOptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedLptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedBptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedIptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedCptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedICptr:
+        case GenISAIntrinsic::GenISA_gather4POPackedLCptr:
             return true;
         default: return false;
         }
@@ -339,9 +456,11 @@ public:
     inline unsigned int getTextureIndex() const { return getNumOperands() - 5; }
     inline unsigned int getImmediateOffsetsIndex() const
     {
-        return 5;
+        return 6;
     }
+    inline unsigned int getPairedTextureIndex() const { return hasPairedTextureArg() ? getNumOperands() - 6 : UINT_MAX;  }
     inline Value* getTextureValue() const { return getOperand(getTextureIndex()); }
+    inline Value* getPairedTextureValue() const { return hasPairedTextureArg() ? getOperand(getPairedTextureIndex()) : nullptr; }
     inline Value* getCoordinateValue(unsigned int i) const { return getOperand(getCoordinateIndex(i)); }
     inline void   setCoordinateValue(unsigned int i, Value* val) { setOperand(getCoordinateIndex(i), val); }
 
@@ -361,6 +480,7 @@ public:
         case GenISAIntrinsic::GenISA_ldptr:
         case GenISAIntrinsic::GenISA_ldmsptr:
         case GenISAIntrinsic::GenISA_ldmcsptr:
+        case GenISAIntrinsic::GenISA_ldlptr:
             return true;
         default: return false;
         }
@@ -369,6 +489,16 @@ public:
     static inline bool classof(const Value *V) {
         return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
     }
+
+    bool hasPairedTextureArg() const {
+        switch (getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_ldptr:
+        case GenISAIntrinsic::GenISA_ldlptr:
+            return true;
+        default:
+            return false;
+        }
+    }
 };
 
 class LdMSIntrinsic : public SamplerLoadIntrinsic {
@@ -450,6 +580,11 @@ public:
         case GenISAIntrinsic::GenISA_sampleDCptr:
         case GenISAIntrinsic::GenISA_sampleLCptr:
         case GenISAIntrinsic::GenISA_sampleBCptr:
+        case GenISAIntrinsic::GenISA_sampleCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleBCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleDCMlodptr:
+        case GenISAIntrinsic::GenISA_samplePOCptr:
+        case GenISAIntrinsic::GenISA_samplePOLCptr:
             return true;
         default:
             break;
@@ -463,6 +598,8 @@ public:
         {
         case GenISAIntrinsic::GenISA_sampleBptr:
         case GenISAIntrinsic::GenISA_sampleBCptr:
+        case GenISAIntrinsic::GenISA_sampleBCMlodptr:
+        case GenISAIntrinsic::GenISA_samplePOBptr:
             return true;
         default:
             break;
@@ -488,6 +625,8 @@ public:
         {
         case GenISAIntrinsic::GenISA_sampleLptr:
         case GenISAIntrinsic::GenISA_sampleLCptr:
+        case GenISAIntrinsic::GenISA_samplePOLptr:
+        case GenISAIntrinsic::GenISA_samplePOLCptr:
             return true;
         default:
             break;
@@ -530,6 +669,12 @@ public:
         return getOperand(index);
     }
 
+    inline Value* getPairedTextureValue() const
+    {
+        unsigned int index = getPairedTextureIndex();
+        return hasPairedTextureArg() ? getOperand(index) : nullptr;
+    }
+
     inline Value* getTextureValue() const
     {
         unsigned int index = getTextureIndex();
@@ -547,6 +692,14 @@ public:
         return index;
     }
 
+    inline unsigned int getPairedTextureIndex() const
+    {
+        unsigned int index = IsLODInst() ? getNumOperands() - 4 :
+            hasPairedTextureArg() ? getNumOperands() - 7 : UINT_MAX;
+
+        return index;
+    }
+
     inline int getSamplerIndex() const
     {
         unsigned int index = getNumOperands() - 5;
@@ -591,6 +744,12 @@ public:
         case GenISAIntrinsic::GenISA_sampleBptr:
         case GenISAIntrinsic::GenISA_sampleCptr:
         case GenISAIntrinsic::GenISA_sampleBCptr:
+        case GenISAIntrinsic::GenISA_sampleMlodptr:
+        case GenISAIntrinsic::GenISA_sampleCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleBCMlodptr:
+        case GenISAIntrinsic::GenISA_samplePOptr:
+        case GenISAIntrinsic::GenISA_samplePOBptr:
+        case GenISAIntrinsic::GenISA_samplePOCptr:
         case GenISAIntrinsic::GenISA_lodptr:
             return true;
         default:
@@ -612,6 +771,16 @@ public:
         case GenISAIntrinsic::GenISA_sampleBCptr:
         case GenISAIntrinsic::GenISA_lodptr:
         case GenISAIntrinsic::GenISA_sampleKillPix:
+        case GenISAIntrinsic::GenISA_sampleMlodptr:
+        case GenISAIntrinsic::GenISA_sampleCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleBCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleDCMlodptr:
+        case GenISAIntrinsic::GenISA_samplePOptr:
+        case GenISAIntrinsic::GenISA_samplePOBptr:
+        case GenISAIntrinsic::GenISA_samplePOLptr:
+        case GenISAIntrinsic::GenISA_samplePOCptr:
+        case GenISAIntrinsic::GenISA_samplePODptr:
+        case GenISAIntrinsic::GenISA_samplePOLCptr:
             return true;
         default: return false;
         }
@@ -621,6 +790,32 @@ public:
         return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
     }
 
+    bool hasPairedTextureArg() const {
+        switch (getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_sampleptr:
+        case GenISAIntrinsic::GenISA_sampleBptr:
+        case GenISAIntrinsic::GenISA_sampleCptr:
+        case GenISAIntrinsic::GenISA_sampleDptr:
+        case GenISAIntrinsic::GenISA_sampleDCptr:
+        case GenISAIntrinsic::GenISA_sampleLptr:
+        case GenISAIntrinsic::GenISA_sampleLCptr:
+        case GenISAIntrinsic::GenISA_sampleBCptr:
+        case GenISAIntrinsic::GenISA_sampleMlodptr:
+        case GenISAIntrinsic::GenISA_sampleCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleBCMlodptr:
+        case GenISAIntrinsic::GenISA_sampleDCMlodptr:
+        case GenISAIntrinsic::GenISA_samplePOptr:
+        case GenISAIntrinsic::GenISA_samplePOBptr:
+        case GenISAIntrinsic::GenISA_samplePOLptr:
+        case GenISAIntrinsic::GenISA_samplePOCptr:
+        case GenISAIntrinsic::GenISA_samplePODptr:
+        case GenISAIntrinsic::GenISA_samplePOLCptr:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
 };
 
 class LdRawIntrinsic : public GenIntrinsicInst {
@@ -811,6 +1006,8 @@ public:
     static inline bool classof(const GenIntrinsicInst *I) {
         GenISAIntrinsic::ID ID = I->getIntrinsicID();
         if (ID == GenISAIntrinsic::GenISA_intatomictyped ||
+            ID == GenISAIntrinsic::GenISA_floatatomictyped ||
+            ID == GenISAIntrinsic::GenISA_fcmpxchgatomictyped ||
             ID == GenISAIntrinsic::GenISA_icmpxchgatomictyped )
         {
             return true;
@@ -850,6 +1047,51 @@ public:
 };
 
 
+class UrbWriteIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_URBWrite;
+    }
+
+    static inline bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+    enum class Argument
+    {
+        Offset,
+        Mask,
+        ValueBeg,
+        Value0 = ValueBeg,
+        Value1,
+        Value2,
+        Value3,
+        Value4,
+        Value5,
+        Value6,
+        Value7,
+        ValueEnd,
+        Count = ValueEnd
+    };
+
+    static constexpr uint32_t scNumVals = static_cast<uint32_t>(Argument::ValueEnd) - static_cast<uint32_t>(Argument::ValueBeg);
+
+    Value* getOffset() const {
+        return getOperand(static_cast<uint32_t>(Argument::Offset));
+    }
+
+    Value* getMask() const {
+        return getOperand(static_cast<uint32_t>(Argument::Mask));
+    }
+
+    Value* getValue(uint32_t index) const {
+        IGC_ASSERT(scNumVals > index);
+        return getOperand(static_cast<uint32_t>(Argument::ValueBeg) + index);
+    }
+};
+
 class SGVIntrinsic : public GenIntrinsicInst {
 public:
     // Methods for support type inquiry through isa, cast, and dyn_cast:
@@ -1029,6 +1271,7 @@ public:
     Value* getTMax() const { return getOperand(15); }
     void setTMax(Value* V) { setOperand(15, V); }
     Value* getPayload() const { return getOperand(16); }
+    Value* getComparisonValue() const { return getOperand(17); }
 };
 
 class CallShaderHLIntrinsic : public ContinuationHLIntrinsic {
@@ -1074,6 +1317,7 @@ public:
     void setGlobalBufferPointer(Value* V) { return setOperand(0, V); }
 
     Value* getPayload()             const { return getOperand(1); }
+
 };
 
 class TraceRayAsyncIntrinsic : public TraceRayIntrinsic {
@@ -1100,6 +1344,7 @@ public:
     static inline bool classof(const Value *V) {
         return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
     }
+
 };
 
 class BTDIntrinsic : public GenIntrinsicInst {
@@ -1117,6 +1362,7 @@ public:
     Value* getGlobalBufferPointer() const { return getOperand(0); }
     Value* getStackID()             const { return getOperand(1); }
     Value* getShaderRecordAddress() const { return getOperand(2); }
+
 };
 
 class StackIDReleaseIntrinsic : public GenIntrinsicInst {
@@ -1282,47 +1528,6 @@ public:
     }
 };
 
-// Common methods for X and Y components
-class TileIntrinsic : public GenIntrinsicInst {
-public:
-    Value* getTID() const { return getOperand(0); }
-    uint32_t getTileXDim() const {
-        return (uint32_t)cast<ConstantInt>(getOperand(1))->getZExtValue();
-    }
-    uint32_t getSubtileXDim() const {
-        return (uint32_t)cast<ConstantInt>(getOperand(2))->getZExtValue();
-    }
-    uint32_t getSubtileYDim() const {
-        return (uint32_t)cast<ConstantInt>(getOperand(3))->getZExtValue();
-    }
-};
-
-class TileXIntrinsic : public TileIntrinsic {
-public:
-    // Methods for support type inquiry through isa, cast, and dyn_cast:
-    static inline bool classof(const GenIntrinsicInst* I) {
-        GenISAIntrinsic::ID ID = I->getIntrinsicID();
-        return ID == GenISAIntrinsic::GenISA_TileXOffset;
-    }
-
-    static inline bool classof(const Value* V) {
-        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
-    }
-};
-
-class TileYIntrinsic : public TileIntrinsic {
-public:
-    // Methods for support type inquiry through isa, cast, and dyn_cast:
-    static inline bool classof(const GenIntrinsicInst* I) {
-        GenISAIntrinsic::ID ID = I->getIntrinsicID();
-        return ID == GenISAIntrinsic::GenISA_TileYOffset;
-    }
-
-    static inline bool classof(const Value* V) {
-        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
-    }
-};
-
 class SWStackPtrIntrinsic : public GenIntrinsicInst {
 public:
     // Methods for support type inquiry through isa, cast, and dyn_cast:
@@ -1366,6 +1571,31 @@ public:
     Value* getFlags()  const { return getOperand(0); }
 };
 
+class RayQueryCheckIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_RayQueryCheck;
+    }
+
+    static inline bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+};
+
+class RayQueryReleaseIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_RayQueryRelease;
+    }
+
+    static inline bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+};
 
 class RayQueryInstrisicBase : public GenIntrinsicInst
 {
@@ -1435,6 +1665,7 @@ public:
     Value* getTMin() const { return getOperand(10); }
     Value* getTMax() const { return getOperand(11); }
     void setTMax(Value* V) { setOperand(11, V); }
+    Value* getComparisonValue() const { return getOperand(12); }
 };
 
 class TraceRaySyncProceedHLIntrinsic : public RayQueryInstrisicBase {
@@ -1646,6 +1877,24 @@ public:
     }
 };
 
+class LocalRootSignatureValueIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst *I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_LocalRootSignatureValue;
+    }
+
+    static inline bool classof(const Value *V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+    Value* getByteOffset() const { return getOperand(0); }
+    uint32_t getRootSigSize() const {
+        return int_cast<uint32_t>(getImm64Operand(1));
+    }
+};
+
 class StaticConstantPatchIntrinsic : public GenIntrinsicInst {
 public:
     // Methods for support type inquiry through isa, cast, and dyn_cast:
@@ -1664,6 +1913,39 @@ public:
         return constantVal->getAsString();
     }
 };
+class PreemptionEnableIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_PreemptionEnable;
+    }
+
+    static inline bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+    Value* getFlag() const {
+        return getOperand(0);
+    }
+};
+
+class PreemptionDisableIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_PreemptionDisable;
+    }
+
+    static inline bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+    Value* getFlag() const {
+        return getOperand(0);
+    }
+};
 
 class FPBinaryOperatorIntrinsic : public GenIntrinsicInst {
 public:
@@ -1719,8 +2001,53 @@ public:
     {
         return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
     }
+};
 
+class SaveInReservedArgSpaceIntrinsic : public GenIntrinsicInst
+{
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I)
+    {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_SaveInReservedArgSpace;
+    }
 
+    static inline bool classof(const Value* V)
+    {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+    IGC::ARG_SPACE_RESERVATION_SLOTS getSlot()
+    {
+        return static_cast<IGC::ARG_SPACE_RESERVATION_SLOTS>(cast<ConstantInt>(getOperand(0))->getZExtValue());
+    }
+
+    Value* getData()
+    {
+        return getOperand(1);
+    }
+};
+
+class ReadFromReservedArgSpaceIntrinsic : public GenIntrinsicInst
+{
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I)
+    {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_ReadFromReservedArgSpace;
+    }
+
+    static inline bool classof(const Value* V)
+    {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+    IGC::ARG_SPACE_RESERVATION_SLOTS getSlot()
+    {
+        return static_cast<IGC::ARG_SPACE_RESERVATION_SLOTS>(cast<ConstantInt>(getOperand(0))->getZExtValue());
+    }
 };
 
 template <class X, class Y>
