@@ -773,7 +773,7 @@ namespace TC
         const char * end = optValue + strlen(optValue);
 
         // parse
-        std::string invalidFormatMessage = "Invalid format of -cl-std option, expected -cl-std=CLMAJOR.MINOR";
+        const std::string invalidFormatMessage = "Invalid format of -cl-std option, expected -cl-std=CLMAJOR.MINOR";
         auto isNumeric = [](char v) { return (v >= '0') && (v <= '9'); };
         if (false == ((end - optValue >= 5) && (optValue[0] == 'C') && (optValue[1] == 'L') && isNumeric(optValue[2])
             && (optValue[3] == '.') && isNumeric(optValue[4])
@@ -1332,6 +1332,10 @@ namespace TC
                             (strcmp(pParam, "-cl-intel-256-GRF-per-thread") == 0) || //temporary options
                             (strcmp(pParam, "-ze-opt-256-GRF-per-thread") == 0) || //temporary options
                             (strcmp(pParam, "-ze-opt-large-register-file") == 0) || //temporary options
+                            (strcmp(pParam, "-cl-intel-large-grf-kernel") == 0) ||
+                            (strcmp(pParam, "-ze-opt-large-grf-kernel") == 0) ||
+                            (strcmp(pParam, "-cl-intel-regular-grf-kernel") == 0) ||
+                            (strcmp(pParam, "-ze-opt-regular-grf-kernel") == 0) ||
                             (strcmp(pParam, "-ze-opt-disable-recompilation") == 0) || //temporary options
                             (strcmp(pParam, "-cl-intel-num-thread-per-eu") == 0) || //temporary options
                             (strncmp(pParam, "-cl-intel-reqd-eu-thread-count", 30) == 0) ||
@@ -1365,7 +1369,15 @@ namespace TC
                             (strcmp(pParam, "-ze-gtpin-scratch-area-size") == 0) || //used by GTPin
                             (strcmp(pParam, "-ze-opt-enable-auto-large-GRF-mode") == 0) ||
                             (strcmp(pParam, "-cl-intel-enable-auto-large-GRF-mode") == 0) ||
-                            (strcmp(pParam, "-ze-skip-fde") == 0);
+                            (strcmp(pParam, "-ze-skip-fde") == 0) ||
+                            (strcmp(pParam, "-ze-no-fusedCallWA") == 0) ||
+                            (strcmp(pParam, "-ze-disable-compaction") == 0) ||
+                            (strcmp(pParam, "-cl-poison-unsupported-fp64-kernels") == 0) || //used by fp64 poisoning
+                            (strcmp(pParam, "-ze-poison-unsupported-fp64-kernels") == 0) || //used by fp64 poisoning
+                            (strcmp(pParam, "-cl-fp64-gen-emu") == 0) || //used by fp64 emulation
+                            (strcmp(pParam, "-ze-fp64-gen-emu") == 0) || //used by fp64 emulation
+                            (strcmp(pParam, "-cl-fp64-gen-conv-emu") == 0) || //used by fp64 conversion emulation
+                            (strcmp(pParam, "-ze-fp64-gen-conv-emu") == 0); //used by fp64 conversion emulation
 
                         if (isCommonOption)
                         {
@@ -1402,6 +1414,15 @@ namespace TC
                                 // Next token is N, so ignore it
                                 ignoreNextToken = true;
                             }
+                            else if (
+                                (strcmp(pParam, "-cl-intel-large-grf-kernel") == 0) ||
+                                (strcmp(pParam, "-ze-opt-large-grf-kernel") == 0) ||
+                                (strcmp(pParam, "-cl-intel-regular-grf-kernel") == 0) ||
+                                (strcmp(pParam, "-ze-opt-regular-grf-kernel") == 0))
+                            {
+                                // Next token is kernel name substring, ignore it
+                                ignoreNextToken = true;
+                            }
                         }
                         // Check for Intel OpenCL CPU options
                         // OCL Kernel Profiler requires "-g" to create debug information for instrumented kernels.
@@ -1980,7 +2001,7 @@ namespace TC
         }
 
         unsigned int i;
-        for (i = 0; i < sizeof(g_cClangTranslationCodes) / sizeof(g_cClangTranslationCodes[0]); i++)
+        for (i = 0; i < std::size(g_cClangTranslationCodes); i++)
         {
             // Some quick checks to ensure that the input and output types
             // are compatible with this translation blocks
@@ -1991,7 +2012,7 @@ namespace TC
             }
         }
 
-        if (i >= sizeof(g_cClangTranslationCodes) / sizeof(g_cClangTranslationCodes[0]))
+        if (i >= std::size(g_cClangTranslationCodes))
         {
             return false;
         }
