@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021-2023 Intel Corporation
+Copyright (C) 2021-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -53,6 +53,7 @@ _Noreturn void __cm_cl_trap();
 // FIXME: For legacy issues 64-bit pointer is always returned.
 uint64_t __cm_cl_assert_buffer();
 uint64_t __cm_cl_printf_buffer();
+uint64_t __cm_cl_sync_buffer();
 
 int __cm_cl_printf_format_index(__constant const char *str);
 // DPC++ tend to place constant strings in global address space.
@@ -77,28 +78,28 @@ void __cm_cl_scatter(vector_impl<T, width> data, int addrspace,
                      vector_impl<char, width> mask);
 
 // vector BTI atomic
-template <typename T, int width>
+template <typename T, int width, int cachelevels>
 vector_impl<T, width> __cm_cl_vector_atomic_bti(
     vector_impl<char, width> mask, char opcode, char addrsize, char elementsize,
-    char l1cachecontrol, char l3cachecontrol, int bti,
+    vector_impl<char, cachelevels> cachecontrols, int bti,
     vector_impl<int, width> index, short scale, int offset,
     vector_impl<T, width> src1, vector_impl<T, width> src2,
     vector_impl<T, width> passthru);
 
 // vector SLM atomic
-template <typename T, int width>
+template <typename T, int width, int cachelevels>
 vector_impl<T, width> __cm_cl_vector_atomic_slm(
     vector_impl<char, width> mask, char opcode, char addrsize, char elementsize,
-    char l1cachecontrol, char l3cachecontrol, int base,
+    vector_impl<char, cachelevels> cachecontrols, int base,
     vector_impl<int, width> index, short scale, int offset,
     vector_impl<T, width> src1, vector_impl<T, width> src2,
     vector_impl<T, width> passthru);
 
 // vector UGM atomic
-template <typename T, int width>
+template <typename T, int width, int cachelevels>
 vector_impl<T, width> __cm_cl_vector_atomic_ugm(
     vector_impl<char, width> mask, char opcode, char addrsize, char elementsize,
-    char l1cachecontrol, char l3cachecontrol, long base,
+    vector_impl<char, cachelevels> cachecontrols, long base,
     vector_impl<long, width> index, short scale, int offset,
     vector_impl<T, width> src1, vector_impl<T, width> src2,
     vector_impl<T, width> passthru);
@@ -297,6 +298,11 @@ inline __global void *printf_buffer() {
   return reinterpret_cast<__global void *>(ptr);
 }
 
+inline __global uint8_t *sync_buffer() {
+  auto ptr = static_cast<uintptr_t>(__cm_cl_sync_buffer());
+  return reinterpret_cast<__global uint8_t *>(ptr);
+}
+
 inline int printf_format_index(__constant const char *str) {
   return __cm_cl_printf_format_index(str);
 }
