@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021-2023 Intel Corporation
+Copyright (C) 2021-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -234,7 +234,12 @@ Type &getTypeFromBuiltinOperand(CallInst &BiCall, int OpIdx) {
   Value &BiOp = *BiCall.getArgOperand(OpIdx);
   switch (BuiltinOperandKind[BiID][OpIdx]) {
   case OperandKind::Output:
-    return *IGCLLVM::getNonOpaquePtrEltTy(BiOp.getType());
+    switch (BiID) {
+    case BuiltinID::AddCarry:
+      return *BiCall.getType();
+    default:
+      llvm_unreachable("Unexpected builtin with an output operand");
+    }
   case OperandKind::Input:
   case OperandKind::Constant:
     return *BiOp.getType();
@@ -345,12 +350,7 @@ Value &createMainInst<BuiltinID::Gather>(const std::vector<Value *> &Operands,
       cast<ConstantInt>(Operands[GatherOperand::Alignment])->getZExtValue());
   auto *MaskV = IRB.CreateTrunc(Operands[GatherOperand::Mask], MaskVTy);
   auto *PassthruV = Operands[GatherOperand::Passthru];
-
-#if LLVM_VERSION_MAJOR > 12
   auto *GatherV = IRB.CreateMaskedGather(RetVTy, PtrV, Align, MaskV, PassthruV);
-#else
-  auto *GatherV = IRB.CreateMaskedGather(PtrV, Align, MaskV, PassthruV);
-#endif
 
   return *GatherV;
 }
@@ -659,6 +659,10 @@ static AtomicRMWInst::BinOp getLLVMAtomicBinOpFromCMCL(CMCLOperation Op) {
     return AtomicRMWInst::Xor;
   case CMCLOperation::Andl:
     return AtomicRMWInst::And;
+  case CMCLOperation::Fadd:
+    return AtomicRMWInst::FAdd;
+  case CMCLOperation::Fsub:
+    return AtomicRMWInst::FSub;
   }
 }
 
