@@ -28,8 +28,6 @@ SPDX-License-Identifier: MIT
 #include "vc/Support/ShaderDump.h"
 #include "vc/Support/ShaderOverride.h"
 
-#include "llvmWrapper/Support/MemoryBuffer.h"
-
 #include "Probe/Assertion.h"
 
 #include "inc/common/sku_wa.h"
@@ -37,6 +35,7 @@ SPDX-License-Identifier: MIT
 #include <llvm/Pass.h>
 #include <llvm/PassRegistry.h>
 #include <llvm/ADT/StringSet.h>
+#include <llvm/Support/MemoryBuffer.h>
 
 #include <limits>
 #include <memory>
@@ -113,9 +112,6 @@ struct GenXBackendOptions {
   // Disable induction variable simplification
   bool DisableIndvarsOpt = false;
 
-  // use new Prolog/Epilog Insertion pass vs old CisaBuilder machinery
-  bool UseNewStackBuilder = true;
-
   FunctionControl FCtrl = FunctionControl::Default;
 
   // Non-owning pointer to workaround table.
@@ -174,7 +170,13 @@ struct GenXBackendOptions {
   unsigned InteropSubgroupSize = 16;
 
   // Run auxiliary checker/fixup for GV access clobbering cases.
-  bool CheckGVClobbering = false;
+  bool CheckGVClobbering =
+#ifdef NDEBUG
+      false
+#else
+      true
+#endif
+      ;
 
   // Compile until vISA stage only.
   bool EmitVisaOnly = false;
@@ -323,8 +325,6 @@ public:
 
   bool disableIndvarsOpt() const { return Options.DisableIndvarsOpt; }
 
-  bool useNewStackBuilder() const { return Options.UseNewStackBuilder; }
-
   unsigned getStatelessPrivateMemSize() const {
     return Options.StatelessPrivateMemSize;
   }
