@@ -65,6 +65,18 @@ enum class BuiltinFunctionKind {
   PostLegalization,
 };
 
+// GenX IR may have different sets of validity invariants for different stages
+// in pipeline.
+enum class GenXVerifyStage {
+#define GENX_VERIFY_STAGE(c_, s_, d_) c_,
+#include "GenXVerifyStages.inc.h"
+#undef GENX_VERIFY_STAGE
+  // --- Add stages above, in respective optimization pipeline order ---
+  End_,
+  Default_ = static_cast<int>(GenXVerifyStage::End_) -
+             1 // Must be set to the latest one.
+};
+
 FunctionPass *createGenXPrinterPass(raw_ostream &O, const std::string &Banner);
 ModulePass *createGenXGroupPrinterPass(raw_ostream &O,
                                        const std::string &Banner);
@@ -80,6 +92,7 @@ ModulePass *createGenXEarlySimdCFConformancePass();
 FunctionPass *createGenXPredToSimdCFPass();
 FunctionPass *createGenXReduceIntSizePass();
 FunctionPass *createGenXInlineAsmLoweringPass();
+FunctionPass *createGenXBFloatLoweringPass();
 FunctionPass *createGenXLoweringPass();
 FunctionPass *createGenXVectorCombinerPass();
 FunctionPass *createGenXLowerAggrCopiesPass();
@@ -119,6 +132,8 @@ ModulePass *createGenXLiveRangesWrapperPass();
 ModulePass *createGenXRematerializationWrapperPass();
 ModulePass *createGenXCoalescingWrapperPass();
 ModulePass *createGenXGVClobberCheckerPass();
+ModulePass *
+createGenXVerifyPass(GenXVerifyStage PipelineStage = GenXVerifyStage::Default_);
 ModulePass *createGenXAddressCommoningWrapperPass();
 ModulePass *createGenXArgIndirectionWrapperPass();
 FunctionPass *createGenXTidyControlFlowPass();
@@ -133,12 +148,14 @@ ModulePass *createGenXStructSplitterPass();
 FunctionPass *createGenXPredRegionLoweringPass();
 FunctionPass *createGenXDebugLegalizationPass();
 FunctionPass *createGenXFixInvalidFuncNamePass();
+ModulePass *createGenXLegalizeGVLoadUsesPass();
 ModulePass *createGenXGASCastWrapperPass();
 FunctionPass *createGenXGASDynamicResolutionPass();
 ModulePass *createGenXInitBiFConstantsPass();
 FunctionPass *createGenXGlobalUniformAnalysisPass();
 ModulePass *createGenXBuiltinFunctionsPass(BuiltinFunctionKind Kind);
 FunctionPass *createGenXLegacyToLscTranslatorPass();
+ModulePass *createGenXSLMResolution();
 
 namespace genx {
 
