@@ -73,9 +73,6 @@ static cl::opt<bool>
 static cl::opt<bool> OptStricterOword(
     "vc-i64emu-strict-report-oword", cl::init(false), cl::Hidden,
     cl::desc("strict check will break on 64-bit oword reads/writes"));
-static cl::opt<bool> OptStricterAlloc(
-    "vc-i64emu-strict-report-alloc", cl::init(false), cl::Hidden,
-    cl::desc("strict check will break on 64-bit alloc"));
 static cl::opt<bool>
     OptStricterConst("vc-i64emu-strict-const", cl::init(false), cl::Hidden,
                      cl::desc("strict check will break on 64-bit constanti"));
@@ -107,7 +104,7 @@ struct OpType {
   Type *FirstArgType;
 };
 static std::function<bool(const OpType &, const OpType &)> OpTypeComparator =
-    [](const OpType &ot1, const OpType &ot2) -> bool {
+    [](const OpType &ot1, const OpType &ot2) {
   if (ot1.Opcode < ot2.Opcode)
     return true;
   if (ot2.Opcode < ot1.Opcode)
@@ -1194,6 +1191,7 @@ Value *GenXEmulate::Emu64Expander::visitGenxAddSat(CallInst &CI) {
     IGC_ASSERT_MESSAGE(0, "unknown intrinsic passed to saturation add emu");
   }
 
+  IGC_ASSERT_EXIT(Result);
   if (Result->getType() != CI.getType()) {
     auto TruncID = (IID == GenXIntrinsic::genx_uuadd_sat)
                        ? GenXIntrinsic::genx_uutrunc_sat
@@ -1643,8 +1641,6 @@ bool GenXEmulate::Emu64Expander::hasStrictEmulationRequirement(
   case GenXIntrinsic::genx_oword_ld:
   case GenXIntrinsic::genx_oword_ld_unaligned:
     return OptStricterOword;
-  case GenXIntrinsic::genx_alloca:
-    return OptStricterAlloc;
   }
 
   switch (Inst->getOpcode()) {
