@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2023 Intel Corporation
+Copyright (C) 2023-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -50,6 +50,7 @@ private:
   Value *translateSVMGatherScatter(CallInst &CI) const;
   Value *translateQuadGatherScatter(CallInst &CI) const;
   Value *translateAtomic(CallInst &CI) const;
+  Value *translateMediaLoadStore(CallInst &CI) const;
 
   const GenXSubtarget *ST = nullptr;
 };
@@ -212,6 +213,12 @@ void GenXLegacyToLscTranslator::visitCallInst(CallInst &CI) {
   case GenXIntrinsic::genx_svm_block_st:
     NewCI = translateOWordLoadStore(CI);
     break;
+  case GenXIntrinsic::genx_media_ld:
+  case GenXIntrinsic::genx_media_st:
+    if (!ST->translateMediaBlockMessages())
+      return;
+    NewCI = translateMediaLoadStore(CI);
+    break;
   }
 
   if (!NewCI) {
@@ -294,11 +301,14 @@ Value *GenXLegacyToLscTranslator::translateOWordLoadStore(CallInst &CI) const {
   auto *NewVTy = IGCLLVM::FixedVectorType::get(Builder.getInt32Ty(), NumDWords);
 
   auto *PredTy = IGCLLVM::FixedVectorType::get(Builder.getInt1Ty(), 1);
+  auto *CacheOpts = ConstantDataVector::getSplat(
+      ST->getNumCacheLevels(), Builder.getInt8(LSC_CACHING_DEFAULT));
 
-  SmallVector<Type *, 3> Types;
+  SmallVector<Type *, 4> Types;
   if (!Data)
     Types.push_back(NewVTy);
   Types.push_back(PredTy);
+  Types.push_back(CacheOpts->getType());
   Types.push_back(Addr->getType());
   if (Data)
     Types.push_back(NewVTy);
@@ -309,8 +319,7 @@ Value *GenXLegacyToLscTranslator::translateOWordLoadStore(CallInst &CI) const {
       Builder.getInt8(AddrSize),
       Builder.getInt8(LSC_DATA_SIZE_32b),
       Builder.getInt8(getLSCElementsPerAddress(NumDWords)),
-      Builder.getInt8(0), // L1 cache control (default)
-      Builder.getInt8(0), // L3 cache control (default)
+      CacheOpts,
       Base,
       Addr,
       Builder.getInt16(1), // Address scale
@@ -403,21 +412,24 @@ Value *GenXLegacyToLscTranslator::translateGatherScatter(CallInst &CI) const {
   auto *NumBlocks = cast<ConstantInt>(CI.getArgOperand(NumBlocksIndex));
   auto ElementSize = getLSCElementSize(1 << NumBlocks->getZExtValue());
 
-  SmallVector<Type *, 3> Types;
+  auto *CacheOpts = ConstantDataVector::getSplat(
+      ST->getNumCacheLevels(), Builder.getInt8(LSC_CACHING_DEFAULT));
+
+  SmallVector<Type *, 4> Types;
   if (IsLoad)
     Types.push_back(DataVTy);
   Types.push_back(PredVTy);
+  Types.push_back(CacheOpts->getType());
   Types.push_back(AddrVTy);
   if (!IsLoad)
     Types.push_back(DataVTy);
 
-  SmallVector<Value *, 11> Args = {
+  SmallVector<Value *, 10> Args = {
       Pred,
       Builder.getInt8(LSC_ADDR_SIZE_32b),
       Builder.getInt8(ElementSize),
       Builder.getInt8(LSC_DATA_ELEMS_1),
-      Builder.getInt8(0), // L1 cache control (default)
-      Builder.getInt8(0), // L3 cache control (default)
+      CacheOpts,
       Base,
       Addr,
       Builder.getInt16(1), // Address scale
@@ -479,21 +491,24 @@ GenXLegacyToLscTranslator::translateSVMGatherScatter(CallInst &CI) const {
   auto *VTy = IGCLLVM::FixedVectorType::get(ETy, SimdWidth * ElementsPerLane);
   Data = Builder.CreateBitCast(Data, VTy);
 
-  SmallVector<Type *, 3> Types;
+  auto *CacheOpts = ConstantDataVector::getSplat(
+      ST->getNumCacheLevels(), Builder.getInt8(LSC_CACHING_DEFAULT));
+
+  SmallVector<Type *, 4> Types;
   if (IsLoad)
     Types.push_back(VTy);
   Types.push_back(PredVTy);
+  Types.push_back(CacheOpts->getType());
   Types.push_back(AddrVTy);
   if (!IsLoad)
     Types.push_back(VTy);
 
-  SmallVector<Value *, 11> Args = {
+  SmallVector<Value *, 10> Args = {
       Pred,
       Builder.getInt8(LSC_ADDR_SIZE_64b),
       Builder.getInt8(ElementSize),
       Builder.getInt8(getLSCElementsPerAddress(ElementsPerLane)),
-      Builder.getInt8(0),  // L1 cache control (default)
-      Builder.getInt8(0),  // L3 cache control (default)
+      CacheOpts,
       Builder.getInt64(0), // Address base
       Addr,
       Builder.getInt16(1), // Address scale
@@ -606,21 +621,24 @@ GenXLegacyToLscTranslator::translateQuadGatherScatter(CallInst &CI) const {
   if (SourceIndex >= 0)
     Data = Builder.CreateBitCast(CI.getArgOperand(SourceIndex), DataVTy);
 
-  SmallVector<Type *, 3> Types;
+  auto *CacheOpts = ConstantDataVector::getSplat(
+      ST->getNumCacheLevels(), Builder.getInt8(LSC_CACHING_DEFAULT));
+
+  SmallVector<Type *, 4> Types;
   if (IsLoad)
     Types.push_back(DataVTy);
   Types.push_back(PredVTy);
+  Types.push_back(CacheOpts->getType());
   Types.push_back(AddrVTy);
   if (!IsLoad)
     Types.push_back(DataVTy);
 
-  SmallVector<Value *, 11> Args = {
+  SmallVector<Value *, 10> Args = {
       Pred,
       Builder.getInt8(AddrSize),
       Builder.getInt8(LSC_DATA_SIZE_32b),
       Builder.getInt8(ChannelMask),
-      Builder.getInt8(0), // L1 cache control (default)
-      Builder.getInt8(0), // L3 cache control (default)
+      CacheOpts,
       Base,
       Addr,
       Builder.getInt16(1), // Address scale
@@ -721,6 +739,8 @@ Value *GenXLegacyToLscTranslator::translateAtomic(CallInst &CI) const {
 
   auto *Ty = CI.getType();
   auto AddrSize = LSC_ADDR_SIZE_32b;
+  auto DataSize =
+      Ty->isIntOrIntVectorTy(64) ? LSC_DATA_SIZE_64b : LSC_DATA_SIZE_32b;
 
   int ArgN = 0;
   auto *Pred = CI.getArgOperand(ArgN++);
@@ -754,13 +774,15 @@ Value *GenXLegacyToLscTranslator::translateAtomic(CallInst &CI) const {
   if (IGCLLVM::getNumArgOperands(&CI) == ArgN + 1)
     Passthru = CI.getArgOperand(ArgN);
 
-  SmallVector<Value *, 13> Args = {
+  auto *CacheOpts = ConstantDataVector::getSplat(
+      ST->getNumCacheLevels(), Builder.getInt8(LSC_CACHING_DEFAULT));
+
+  SmallVector<Value *, 12> Args = {
       Pred,
       Builder.getInt8(Opcode),
       Builder.getInt8(AddrSize),
-      Builder.getInt8(LSC_DATA_SIZE_32b),
-      Builder.getInt8(0), // L1 cache control (default)
-      Builder.getInt8(0), // L3 cache control (default)
+      Builder.getInt8(DataSize),
+      CacheOpts,
       Base,
       Addr,
       Builder.getInt16(1), // Address scale
@@ -771,7 +793,79 @@ Value *GenXLegacyToLscTranslator::translateAtomic(CallInst &CI) const {
   };
 
   auto *Func = vc::InternalIntrinsic::getInternalDeclaration(
-      CI.getModule(), NewIID, {Ty, Pred->getType(), Addr->getType()});
+      CI.getModule(), NewIID,
+      {Ty, Pred->getType(), CacheOpts->getType(), Addr->getType()});
+  auto *I = Builder.CreateCall(Func, Args);
+  LLVM_DEBUG(dbgs() << "New intrinsic generated: " << *I);
+  return I;
+}
+
+Value *GenXLegacyToLscTranslator::translateMediaLoadStore(CallInst &CI) const {
+  LLVM_DEBUG(dbgs() << "Translate intrinsic: " << CI);
+  IGC_ASSERT(ST->translateMediaBlockMessages());
+
+  IRBuilder<> Builder(&CI);
+  auto IID = vc::getAnyIntrinsicID(&CI);
+
+  IGC_ASSERT(IID == GenXIntrinsic::genx_media_ld ||
+             IID == GenXIntrinsic::genx_media_st);
+  auto IsLoad = IID == GenXIntrinsic::genx_media_ld;
+  auto NewIID = IsLoad ? GenXIntrinsic::genx_lsc_load2d_typed_bti
+                       : GenXIntrinsic::genx_lsc_store2d_typed_bti;
+
+  auto *Modifier = cast<ConstantInt>(CI.getArgOperand(0));
+  auto *BTI = CI.getArgOperand(1);
+  auto *Plane = cast<ConstantInt>(CI.getArgOperand(2));
+  auto *BlockWidth = cast<ConstantInt>(CI.getArgOperand(3));
+  auto *AddrX = CI.getArgOperand(4);
+  auto *AddrY = CI.getArgOperand(5);
+  Value *Data = nullptr;
+  IGCLLVM::FixedVectorType *VTy = nullptr;
+
+  if (IsLoad) {
+    VTy = cast<IGCLLVM::FixedVectorType>(CI.getType());
+  } else {
+    Data = CI.getArgOperand(6);
+    VTy = cast<IGCLLVM::FixedVectorType>(Data->getType());
+  }
+
+  if (Modifier->getZExtValue() != 0) {
+    LLVM_DEBUG(dbgs() << "Modifiers are not supported for media block "
+                         "intrinsic translations: "
+                      << CI);
+    return nullptr;
+  }
+  if (Plane->getZExtValue() != 0) {
+    LLVM_DEBUG(dbgs() << "Non-zero plane is not supported for media block "
+                         "intrinsic translations: "
+                      << CI);
+    return nullptr;
+  }
+
+  auto *ETy = VTy->getElementType();
+  unsigned ESize = ETy->getScalarSizeInBits() / ByteBits;
+  auto DataSize = ESize * VTy->getNumElements();
+
+  unsigned Width = BlockWidth->getZExtValue();
+  unsigned RoundedWidth = roundedVal(Width, 4u);
+  unsigned Height = DataSize / RoundedWidth;
+  IGC_ASSERT(Width > 0 && Width <= 64);
+  IGC_ASSERT(Width % ESize == 0);
+  IGC_ASSERT(DataSize % RoundedWidth == 0);
+
+  SmallVector<Value *, 8> Args = {
+      Builder.getInt8(0), // L1 cache control (default)
+      Builder.getInt8(0), // L3 cache control (default)
+      BTI,
+      Builder.getInt32(Height),
+      Builder.getInt32(Width / ESize),
+      AddrX,
+      AddrY,
+  };
+  if (!IsLoad)
+    Args.push_back(Data);
+
+  auto *Func = GenXIntrinsic::getGenXDeclaration(CI.getModule(), NewIID, {VTy});
   auto *I = Builder.CreateCall(Func, Args);
   LLVM_DEBUG(dbgs() << "New intrinsic generated: " << *I);
   return I;
