@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2023 Intel Corporation
+Copyright (C) 2023-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -138,7 +138,7 @@ void GenXLoadStoreLegalization::visitCallInst(CallInst &CI) {
   LLVM_DEBUG(dbgs() << "Trying: " << CI << "\n");
 
   const auto ExecSize = vc::InternalIntrinsic::getMemorySimdWidth(&CI);
-  const auto VectorSize =
+  const uint64_t VectorSize =
       vc::InternalIntrinsic::getMemoryVectorSizePerLane(&CI);
 
   const auto MaxWidth = getMaxLegalSimdWidth(CI);
@@ -183,7 +183,7 @@ Value *GenXLoadStoreLegalization::splitMemoryOperation(Value *InsertTo,
                                                        unsigned SplitWidth,
                                                        unsigned &Index) const {
   const auto ExecSize = vc::InternalIntrinsic::getMemorySimdWidth(&CI);
-  const auto VectorSize =
+  const uint64_t VectorSize =
       vc::InternalIntrinsic::getMemoryVectorSizePerLane(&CI);
 
   auto *Func = getMemoryIntrinsic(CI, SplitWidth);
@@ -195,7 +195,10 @@ Value *GenXLoadStoreLegalization::splitMemoryOperation(Value *InsertTo,
       auto *VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
       if (!VTy)
         return Arg;
-      bool IsSOA = VTy->getNumElements() == VectorSize * ExecSize;
+      auto NumElements = VTy->getNumElements();
+      bool IsSOA = NumElements == VectorSize * ExecSize;
+      if (NumElements != ExecSize && !IsSOA)
+        return Arg;
       return createExtractFromSOAValue(&CI, Arg, IsSOA ? VectorSize : 1, Index,
                                        SplitWidth);
     });
@@ -216,7 +219,7 @@ Value *GenXLoadStoreLegalization::extendMemoryOperation(Value *InsertTo,
                                                         unsigned ExtendWidth,
                                                         unsigned Index) const {
   const auto ExecSize = vc::InternalIntrinsic::getMemorySimdWidth(&CI);
-  const auto VectorSize =
+  const uint64_t VectorSize =
       vc::InternalIntrinsic::getMemoryVectorSizePerLane(&CI);
   const auto RestSize = ExecSize - Index;
   if (RestSize == 0)
@@ -228,11 +231,16 @@ Value *GenXLoadStoreLegalization::extendMemoryOperation(Value *InsertTo,
   IRBuilder<> Builder(&CI);
 
   SmallVector<Value *, 13> Args;
-  llvm::transform(CI.args(), std::back_inserter(Args), [&](Value *Arg) {
+  llvm::transform(CI.args(), std::back_inserter(Args), [&](Value *Arg) -> Value* {
     auto *VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
     if (!VTy)
       return Arg;
-    bool IsSOA = VTy->getNumElements() == VectorSize * ExecSize;
+
+    auto NumElements = VTy->getNumElements();
+    bool IsSOA = NumElements == VectorSize * ExecSize;
+    if (NumElements != ExecSize && !IsSOA)
+      return Arg;
+
     if (Index > 0)
       Arg = createExtractFromSOAValue(&CI, Arg, IsSOA ? VectorSize : 1, Index,
                                       RestSize);
@@ -242,10 +250,36 @@ Value *GenXLoadStoreLegalization::extendMemoryOperation(Value *InsertTo,
     auto *ETy = VTy->getElementType();
     auto *InsTy = IGCLLVM::FixedVectorType::get(
         ETy, IsSOA ? VectorSize * ExtendWidth : ExtendWidth);
-    Value *Insert = ETy->isIntegerTy(1) ? Constant::getNullValue(InsTy)
-                                        : UndefValue::get(InsTy);
-    return createInsertToSOAValue(&CI, Insert, Arg, IsSOA ? VectorSize : 1, 0,
-                                  RestSize);
+    if (!ETy->isIntegerTy(1))
+      return createInsertToSOAValue(&CI, UndefValue::get(InsTy), Arg,
+                                    IsSOA ? VectorSize : 1, 0, RestSize);
+    if (RestSize == 4 || RestSize == 8 || RestSize == 16 || !isa<CmpInst>(Arg))
+      return createInsertToSOAValue(&CI, Constant::getNullValue(InsTy), Arg,
+                                    IsSOA ? VectorSize : 1, 0, RestSize);
+
+    // If a predicate is illegally sized, it will cause problems later in
+    // GenXLegalization pass because wrpredregion must follow offset alignment
+    // restrictions. In case when the illegal predicate is result of cmp
+    // instruction we can also extend this instruction instead of just
+    // writing its result into a legal-sized predicate.
+    auto *Cmp = cast<CmpInst>(Arg);
+    auto *OpVTy = cast<IGCLLVM::FixedVectorType>(Cmp->getOperand(0)->getType());
+    auto *NewOpTy = IGCLLVM::FixedVectorType::get(
+        OpVTy->getElementType(),
+        IsSOA ? VectorSize * ExtendWidth : ExtendWidth);
+    auto *NewOp0 = createInsertToSOAValue(&CI, Constant::getNullValue(NewOpTy),
+                                          Cmp->getOperand(0),
+                                          IsSOA ? VectorSize : 1, 0, RestSize);
+    auto *NewOp1 = createInsertToSOAValue(
+        &CI,
+        Cmp->isTrueWhenEqual() ? Constant::getAllOnesValue(NewOpTy)
+                               : Constant::getNullValue(NewOpTy),
+        Cmp->getOperand(1), IsSOA ? VectorSize : 1, 0, RestSize);
+    auto *NewCmp =
+        CmpInst::Create(Cmp->getOpcode(), Cmp->getPredicate(), NewOp0, NewOp1,
+                        Cmp->getName() + ".extended", &CI);
+    NewCmp->setDebugLoc(Cmp->getDebugLoc());
+    return NewCmp;
   });
 
   Value *Res = Builder.CreateCall(Func, Args);
@@ -269,8 +303,8 @@ GenXLoadStoreLegalization::getMemoryIntrinsic(CallInst &CI,
                                               unsigned NewWidth) const {
   const auto IID = vc::InternalIntrinsic::getInternalIntrinsicID(&CI);
   IGC_ASSERT_EXIT(IID != vc::InternalIntrinsic::not_internal_intrinsic);
-  const auto ExecSize = vc::InternalIntrinsic::getMemorySimdWidth(&CI);
-  const auto VectorSize =
+  const uint64_t ExecSize = vc::InternalIntrinsic::getMemorySimdWidth(&CI);
+  const uint64_t VectorSize =
       vc::InternalIntrinsic::getMemoryVectorSizePerLane(&CI);
 
   SmallVector<Type *, 3> OverloadedTypes;
@@ -296,9 +330,13 @@ GenXLoadStoreLegalization::getMemoryIntrinsic(CallInst &CI,
     auto *ArgVTy = cast<IGCLLVM::FixedVectorType>(ArgTy);
     auto *ETy = ArgVTy->getElementType();
     auto NumElements = ArgVTy->getNumElements();
-    IGC_ASSERT(NumElements == ExecSize || NumElements == VectorSize * ExecSize);
+    bool IsSOA = NumElements == VectorSize * ExecSize;
+
+    if (NumElements != ExecSize && !IsSOA) {
+      OverloadedTypes.push_back(ArgTy);
+      continue;
+    }
 
-    auto IsSOA = NumElements == VectorSize * ExecSize;
     auto Width = IsSOA ? VectorSize * NewWidth : NewWidth;
     auto *VTy = IGCLLVM::FixedVectorType::get(ETy, Width);
     OverloadedTypes.push_back(VTy);
