@@ -61,18 +61,17 @@ void GenXSubtarget::initSubtargetFeatures(StringRef CPU, StringRef FS) {
                  .Case("XeHP", XeHP)
                  .Case("XeHPG", XeHPG)
                  .Case("XeLPG", XeLPG)
+                 .Case("XeLPGPlus", XeLPGPlus)
                  .Case("XeHPC", XeHPC)
+                 .Case("XeHPCVG", XeHPCVG)
+                 .Case("Xe2", Xe2)
                  .Default(Invalid);
 
   std::string CPUName(CPU);
   if (CPUName.empty() || TargetId == Invalid)
     report_fatal_error("Undefined or blank arch passed");
 
-  ParseSubtargetFeatures(CPUName,
-#if LLVM_VERSION_MAJOR >= 12
-                         /*TuneCPU=*/CPUName,
-#endif
-                         FS);
+  ParseSubtargetFeatures(CPUName, /*TuneCPU=*/CPUName, FS);
   if (EnforceLongLongEmulation)
     EmulateLongLong = true;
   if (EnforceDivRem32Emulation)
@@ -81,11 +80,7 @@ void GenXSubtarget::initSubtargetFeatures(StringRef CPU, StringRef FS) {
 
 GenXSubtarget::GenXSubtarget(const Triple &TT, const std::string &CPU,
                              const std::string &FS)
-    : GenXGenSubtargetInfo(TT, CPU,
-#if LLVM_VERSION_MAJOR >= 12
-                           /*TuneCPU=*/CPU,
-#endif
-                           FS),
+    : GenXGenSubtargetInfo(TT, CPU, /*TuneCPU=*/CPU, FS),
       TargetTriple(TT) {
   initSubtargetFeatures(CPU, FS);
 }
@@ -97,9 +92,12 @@ uint32_t GenXSubtarget::getMaxThreadsNumPerSubDevice() const {
   case XeHP:
   case XeHPG:
   case XeLPG:
-    return 1 << 12;
+  case XeLPGPlus:
   case XeHPC:
+  case XeHPCVG:
     return 1 << 12;
+  case Xe2:
+    return 1 << 13;
   }
   return 0;
 }
@@ -109,7 +107,8 @@ ArrayRef<std::pair<int, int>> GenXSubtarget::getThreadIdReservedBits() const {
   switch (TargetId) {
   case GenXSubtarget::XeHP:
   case GenXSubtarget::XeHPG:
-  case GenXSubtarget::XeLPG: {
+  case GenXSubtarget::XeLPG:
+  case GenXSubtarget::XeLPGPlus: {
     // [13:11] Slice ID.
     // [10:9] Dual - SubSlice ID
     // [8] SubSlice ID.
@@ -121,7 +120,8 @@ ArrayRef<std::pair<int, int>> GenXSubtarget::getThreadIdReservedBits() const {
     static const std::pair<int, int> Bits[] = {{6, 1}, {3, 1}};
     return Bits;
   }
-  case GenXSubtarget::XeHPC: {
+  case GenXSubtarget::XeHPC:
+  case GenXSubtarget::XeHPCVG: {
     // [14:12] Slice ID.
     // [11:9] SubSlice ID
     // [8] : EUID[2]
@@ -132,6 +132,17 @@ ArrayRef<std::pair<int, int>> GenXSubtarget::getThreadIdReservedBits() const {
     static const std::pair<int, int> Bits[] = {{6, 2}, {3, 1}};
     return Bits;
   }
+  case GenXSubtarget::Xe2: {
+    // [15:11] Slice ID.
+    // [10] : Reserved MBZ
+    // [9:8] SubSlice ID
+    // [7] : Reserved MBZ
+    // [6:4] : EUID
+    // [3] : Reserved MBZ
+    // [2:0] : TID
+    static const std::pair<int, int> Bits[] = {{10, 1}, {7, 1}, {3, 1}};
+    return Bits;
+  }
   default:
     // All other platforms have pre-defined Thread ID register
     return {};
@@ -149,14 +160,16 @@ ArrayRef<std::pair<int, int>> GenXSubtarget::getSubsliceIdBits() const {
   switch (TargetId) {
   case GenXSubtarget::XeHP:
   case GenXSubtarget::XeHPG:
-  case GenXSubtarget::XeLPG: {
+  case GenXSubtarget::XeLPG:
+  case GenXSubtarget::XeLPGPlus: {
     // [13:11] Slice ID.
     // [10:9] Dual - SubSlice ID
     // [8] SubSlice ID.
     static const std::pair<int, int> Bits[] = {{8, 6}};
     return Bits;
   }
-  case GenXSubtarget::XeHPC: {
+  case GenXSubtarget::XeHPC:
+  case GenXSubtarget::XeHPCVG: {
     // [14:12] Slice ID.
     // [11:9] SubSlice ID
     static const std::pair<int, int> Bits[] = {{9, 6}};
@@ -171,20 +184,27 @@ ArrayRef<std::pair<int, int>> GenXSubtarget::getEUIdBits() const {
   switch (TargetId) {
   case GenXSubtarget::XeHP:
   case GenXSubtarget::XeHPG:
-  case GenXSubtarget::XeLPG: {
+  case GenXSubtarget::XeLPG:
+  case GenXSubtarget::XeLPGPlus: {
     // [7] : EUID[2]
     // [6] : Reserved
     // [5:4] EUID[1:0]
     static const std::pair<int, int> Bits[] = {{4, 2}, {7, 1}};
     return Bits;
   }
-  case GenXSubtarget::XeHPC: {
+  case GenXSubtarget::XeHPC:
+  case GenXSubtarget::XeHPCVG: {
     // [8] : EUID[2]
     // [7:6] : Reserved
     // [5:4] EUID[1:0]
     static const std::pair<int, int> Bits[] = {{4, 2}, {8, 1}};
     return Bits;
   }
+  case GenXSubtarget::Xe2: {
+    // [6:4] : EUID
+    static const std::pair<int, int> Bits[] = {{4, 3}};
+    return Bits;
+  }
   default:
     // All other platforms have pre-defined Thread ID register
     return {};
@@ -217,10 +237,16 @@ TARGET_PLATFORM GenXSubtarget::getVisaPlatform() const {
     return TARGET_PLATFORM::Xe_DG2;
   case XeLPG:
     return TARGET_PLATFORM::Xe_MTL;
+  case XeLPGPlus:
+    return TARGET_PLATFORM::Xe_ARL;
   case XeHPC:
     if (!partialI64Emulation())
       return TARGET_PLATFORM::Xe_PVC;
+    LLVM_FALLTHROUGH;
+  case XeHPCVG:
     return TARGET_PLATFORM::Xe_PVCXT;
+  case Xe2:
+    return TARGET_PLATFORM::Xe2;
   default:
     return TARGET_PLATFORM::GENX_NONE;
   }
