@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2023 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -55,7 +55,10 @@ public:
     XeHP,
     XeHPG,
     XeLPG,
+    XeLPGPlus,
     XeHPC,
+    XeHPCVG,
+    Xe2,
     Invalid,
   };
 
@@ -103,6 +106,10 @@ private:
 
   // Currenly used for PVC B-stepping (some i64 operations are unsupported)
   bool PartialI64Emulation = false;
+
+  // True if there is no legacy dataport shared function.
+  bool NoLegacyDataport = false;
+
   // Some targets do not support i64 ops natively, we have an option to emulate
   bool EmulateLongLong = false;
 
@@ -112,8 +119,8 @@ private:
   // True if it is profitable to use native DxD->Q multiplication
   bool UseMulDDQ = false;
 
-  // True if codegenerating for OCL runtime.
-  bool OCLRuntime = false;
+  // True if codegenerating for OCL runtime (set by default since CMRT removed)
+  bool OCLRuntime = true;
 
   // True if subtarget supports switchjmp visa instruction
   bool HasSwitchjmp = false;
@@ -155,6 +162,12 @@ private:
   /// True if subtarget supports LSC messages
   bool HasLSCMessages = false;
 
+  /// True if subtarget supports typed LSC messages
+  bool HasLSCTypedMessages = false;
+
+  /// True if subtarget supports constant offset for LSC message address
+  bool HasLSCOffset = false;
+
   /// True if subtarget supports half SIMD LSC messages
   bool HasHalfSIMDLSC = false;
 
@@ -167,6 +180,9 @@ private:
   /// Has L3 flush on GPU-scope invalidate.
   bool HasL3FlushOnGPUScopeInvalidate = false;
 
+  /// Has denormal control for BF16 and TF32 types on DPAS
+  bool HasSystolicDenormControl = false;
+
   /// True if Vx1 and VxH indirect addressing are allowed for Byte datatypes
   bool HasMultiIndirectByteRegioning = false;
 
@@ -212,6 +228,9 @@ private:
   /// Max supported SLM size (in kbytes)
   int MaxSLMSize = 64;
 
+  // Number of elements in Address Register
+  unsigned AddressRegisterElements = 16;
+
   // Shows which surface should we use for stack
   PreDefined_Surface StackSurf;
 
@@ -244,15 +263,9 @@ public:
   // can take up.
   unsigned getLSCMaxDataRegisters() const { return 8; }
 
-  bool isOCLRuntime() const { return OCLRuntime; }
-
   // ParseSubtargetFeatures - Parses features string setting specified
   // subtarget options.  Definition of function is auto generated by tblgen.
-  void ParseSubtargetFeatures(StringRef CPU,
-#if LLVM_VERSION_MAJOR >= 12
-                              StringRef TuneCPU,
-#endif
-                              StringRef FS);
+  void ParseSubtargetFeatures(StringRef CPU, StringRef TuneCPU, StringRef FS);
 
   // \brief Initialize the features for the GenX target.
   void initSubtargetFeatures(StringRef CPU, StringRef FS);
@@ -271,14 +284,24 @@ public:
 
   bool hasLSCMessages() const { return HasLSCMessages; }
 
+  bool hasLSCTypedMessages() const { return HasLSCTypedMessages; }
+
+  bool hasLSCOffset() const { return HasLSCOffset; }
+
   bool translateLegacyMessages() const {
-    return (HasLSCMessages && TranslateLegacyMessages);
+    return HasLSCMessages && TranslateLegacyMessages;
+  }
+
+  bool translateMediaBlockMessages() const {
+    return HasLSCTypedMessages && TranslateLegacyMessages;
   }
 
   bool hasHalfSIMDLSC() const { return HasHalfSIMDLSC; }
 
   bool partialI64Emulation() const { return PartialI64Emulation; }
 
+  bool noLegacyDataport() const { return NoLegacyDataport; }
+
   /// * hasPackedFloat - true if packed float immediate vector operands are
   /// supported
   bool hasPackedFloat() const { return HasPackedFloat; }
@@ -391,6 +414,8 @@ public:
   // predefined variable instead of sr0, returns *true* for such ones.
   bool getsHWTIDFromPredef() const { return GetsHWTIDFromPredef; }
 
+  bool hasSystolicDenormControl() const { return HasSystolicDenormControl; }
+
   uint32_t getMaxThreadsNumPerSubDevice() const;
   ArrayRef<std::pair<int, int>> getThreadIdReservedBits() const;
 
@@ -403,6 +428,13 @@ public:
   /// bit fields for ThreadID (from lsb to msb).
   ArrayRef<std::pair<int, int>> getThreadIdBits() const;
 
+  unsigned getNumCacheLevels() const { return 2; }
+
+  // Address Register size in elements.
+  unsigned getAddressRegisterElements() const {
+    return AddressRegisterElements;
+  }
+
   // Generic helper functions...
   const Triple &getTargetTriple() const { return TargetTriple; }
 
