@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2023 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -70,16 +70,54 @@ static cl::opt<std::string>
     FGDumpsPrefix("vc-fg-dump-prefix", cl::init(""), cl::Hidden,
                   cl::desc("prefix to use for FG dumps"));
 
+static cl::opt<bool>
+    SkipOCLRuntimeInfo("vc-skip-ocl-runtime-info", cl::init(false), cl::Hidden,
+                  cl::desc("skip GenXOCLRuntimeInfo in addPassesToEmitFile for llc tests"));
+
 static cl::opt<bool> EmitVLoadStore(
     "genx-emit-vldst", cl::init(true), cl::Hidden,
     cl::desc("Emit load/store intrinsic calls for pass-by-ref arguments"));
 
+static cl::opt<bool> PeelLoopsDpasNullAcc(
+    "vc-peel-loops-dpas-null-acc", cl::init(true), cl::Hidden,
+    cl::desc("Peel first iteration of a loop with dpas instructions, when dpas "
+             "accumulator operand is zero-initialized"));
+static cl::opt<unsigned> PeelLoopDpasNullAccMaxBlocks(
+    "vc-peel-loops-dpas-null-acc-max-blocks", cl::init(16), cl::Hidden,
+    cl::desc("Max number of a loop basic blocks to peel, when the loop has "
+             "dpas instructions with zero-initialized accumulator operand"));
+static cl::opt<unsigned> PeelLoopDpasNullAccMaxInstr(
+    "vc-peel-loops-dpas-null-acc-max-instr", cl::init(192), cl::Hidden,
+    cl::desc("Max number of a loop instructions to peel, when the loop has "
+             "dpas instructions with zero-initialized accumulator operand"));
+static cl::opt<unsigned> PeelLoopDpasNullAccMin(
+    "vc-peel-loops-dpas-null-acc-min-dpas-instr", cl::init(4), cl::Hidden,
+    cl::desc("Minimal number of dpas instructions with zero-initialized "
+             "accumulator operand to peel the loop with first iteration"));
+
 // There's another copy of DL string in clang/lib/Basic/Targets.cpp
 static std::string getDL(bool Is64Bit) {
   return Is64Bit ? "e-p:64:64-p3:32:32-p6:32:32-i64:64-n8:16:32:64"
-                 : "e-p:32:32-p3:32:32-p6:32:32-i64:64-n8:16:32";
+                 : "e-p:32:32-p3:32:32-p6:32:32-i64:64-n8:16:32:64";
 }
 
+namespace {
+bool isDpasAccumulator(const Value *V) {
+  if (!V->hasOneUse())
+    return false;
+
+  const auto *User = V->user_back();
+  const auto IID = vc::getAnyIntrinsicID(User);
+
+  if (IID != GenXIntrinsic::genx_dpas && IID != GenXIntrinsic::genx_dpas2)
+    return true;
+
+  const auto *CI = cast<CallInst>(User);
+  return CI->getArgOperand(0) == V;
+}
+
+} // namespace
+
 namespace llvm {
 
 //===----------------------------------------------------------------------===//
@@ -94,6 +132,7 @@ void initializeGenXPasses(PassRegistry &registry) {
   initializeGenXCisaBuilderPass(registry);
   initializeGenXCoalescingWrapperPass(registry);
   initializeGenXGVClobberCheckerPass(registry);
+  initializeGenXVerifyPass(registry);
   initializeGenXDeadVectorRemovalPass(registry);
   initializeGenXDepressurizerWrapperPass(registry);
   initializeGenXEarlySimdCFConformancePass(registry);
@@ -112,6 +151,7 @@ void initializeGenXPasses(PassRegistry &registry) {
   // initializeGenXLivenessWrapperPass(registry);
   initializeGenXLivenessWrapperPass(registry);
   initializeGenXLowerAggrCopiesPass(registry);
+  initializeGenXBFloatLoweringPass(registry);
   initializeGenXLoweringPass(registry);
   initializeGenXModulePass(registry);
   initializeGenXNumberingWrapperPass(registry);
@@ -130,6 +170,7 @@ void initializeGenXPasses(PassRegistry &registry) {
   initializeGenXPromoteArrayPass(registry);
   initializeGenXBackendConfigPass(registry);
   initializeGenXImportOCLBiFPass(registry);
+  initializeGenXBIFFlagCtrlResolutionPass(registry);
   initializeGenXSimplifyPass(registry);
   initializeCMABIPass(registry);
   initializeCMLowerVLoadVStorePass(registry);
@@ -156,6 +197,7 @@ void initializeGenXPasses(PassRegistry &registry) {
   initializeGenXInlineAsmLoweringPass(registry);
   initializeGenXDebugLegalizationPass(registry);
   initializeGenXFixInvalidFuncNamePass(registry);
+  initializeGenXLegalizeGVLoadUsesPass(registry);
   initializeGenXGASCastWrapperPass(registry);
   initializeGenXGASDynamicResolutionPass(registry);
   initializeGenXInitBiFConstantsPass(registry);
@@ -163,17 +205,126 @@ void initializeGenXPasses(PassRegistry &registry) {
   initializeGenXFinalizerPass(registry);
   initializeGenXBuiltinFunctionsPass(registry);
   initializeGenXLegacyToLscTranslatorPass(registry);
+  initializeGenXSLMResolutionPass(registry);
+  initializeGenXTypeLegalizationPass(registry);
   // WRITE HERE MORE PASSES IF IT'S NEEDED;
 }
 
-#if LLVM_VERSION_MAJOR >= 15
-TargetTransformInfo GenXTargetMachine::getTargetTransformInfo(const Function &F) const
-#else
-TargetTransformInfo GenXTargetMachine::getTargetTransformInfo(const Function& F)
-#endif
+TargetTransformInfo GenXTargetMachine::getTargetTransformInfo(
+  const Function &F) LLVM_GET_TTI_API_QUAL
 {
-  GenXTTIImpl GTTI(F.getParent()->getDataLayout(), *BC);
-  return TargetTransformInfo(GTTI);
+  GenXTTIImpl GTTI(F.getParent()->getDataLayout(), *BC, Subtarget);
+  return TargetTransformInfo(std::move(GTTI));
+}
+void GenXTTIImpl::getUnrollingPreferences(
+    Loop *L, ScalarEvolution &SE,
+    TargetTransformInfo::UnrollingPreferences &UP,
+    OptimizationRemarkEmitter *ORE) {
+  if (BC.ignoreLoopUnrollThresholdOnPragma() &&
+      GetUnrollMetadataForLoop(L, "llvm.loop.unroll.enable"))
+    UP.Threshold = UP.PartialThreshold = std::numeric_limits<unsigned>::max();
+
+  if (unsigned VCUnrollThreshold = BC.getLoopUnrollThreshold()) {
+    UP.Threshold = VCUnrollThreshold;
+    UP.PartialThreshold = VCUnrollThreshold;
+    UP.Partial = true;
+  }
+
+  if (GetUnrollMetadataForLoop(L, "llvm.loop.unroll.full")) {
+    UP.Threshold = std::numeric_limits<unsigned>::max();
+    UP.Partial = false;
+    UP.Runtime = false;
+  }
+}
+
+void GenXTTIImpl::getPeelingPreferences(
+    Loop *L, ScalarEvolution &SE,
+    TargetTransformInfo::PeelingPreferences &PP) const {
+  if (!PeelLoopsDpasNullAcc || ST.hasFusedEU())
+    return;
+
+  // Don't peel the loop if user explicitly asked to unroll
+  if (auto *LoopID = L->getLoopID();
+      LoopID && (GetUnrollMetadata(LoopID, "llvm.loop.unroll.enable") ||
+                 GetUnrollMetadata(LoopID, "llvm.loop.unroll.full") ||
+                 GetUnrollMetadata(LoopID, "llvm.loop.unroll.count")))
+    return;
+
+  // Only analyze the inner-most loops
+  if (!L->getSubLoops().empty())
+    return;
+
+  // Limit loop size to the specific number of basic blocks
+  if (L->getNumBlocks() > PeelLoopDpasNullAccMaxBlocks)
+    return;
+
+  // Match the following two cases:
+  // 1. Dpas accumulator is a phi node with a zero value came from the pre-loop
+  // basic block.
+  //   %acc = phi <N x Ty> [ zeroinitializer, %entry ], [ %dst, %loop ]
+  //   %dst = call @llvm.genx.dpas(%acc, ...)
+  // 2. Dpas accumulator is a rdregion taking a piece of a large vector. The
+  // large vector is a phi node with a zero value taken from the pre-loop basic
+  // block.
+  //   %phi = phi <N x ty>
+  //   %acc = call @llvm.genx.rdregion(%phi, ...)
+  //   %dst = call @llvm.genx.dpas(%acc, ...)
+  //   %wrregion = call @llvm.genx.wrregion(%phi, %dst, ...)
+
+  const auto PhiNodes = L->getHeader()->phis();
+  const uint64_t NumDpasZeroAcc = std::accumulate(
+      std::begin(PhiNodes), std::end(PhiNodes), 0ull,
+      [this](uint64_t Acc, const auto &Phi) {
+        // Check vector Phi nodes
+        const auto *PhiVTy = dyn_cast<IGCLLVM::FixedVectorType>(Phi.getType());
+        if (!PhiVTy || Phi.getNumIncomingValues() != 2)
+          return Acc;
+
+        // Check if one of the Phi inputs is constant zero
+        bool IsZeroAcc = llvm::any_of(Phi.incoming_values(), [](const auto &V) {
+          const auto *C = dyn_cast<Constant>(&V);
+          return C && C->isZeroValue();
+        });
+        if (!IsZeroAcc)
+          return Acc;
+
+        // Simple case. The only user of the Phi node must be dpas intrinsic.
+        if (isDpasAccumulator(&Phi))
+          return Acc + 1;
+
+        // If vector decomposition is disabled, we unable to simplify the region
+        // access chain. So, it doesn't make sense to peel the loop.
+        if (ST.disableVectorDecomposition())
+          return Acc;
+
+        if (!Phi.hasNUses(2))
+          return Acc;
+
+        // Find the rdregion used as a dpas accumulator
+        const auto It = llvm::find_if(Phi.users(), [](const Value *V) {
+          return GenXIntrinsic::isRdRegion(V) && isDpasAccumulator(V);
+        });
+        if (It == Phi.user_end())
+          return Acc;
+
+        // Assume that the whole phi node value is used as an accumulator source
+        // for multiple similar dpas instructions.
+        const auto *DpasVTy = cast<IGCLLVM::FixedVectorType>((*It)->getType());
+        return Acc + (PhiVTy->getNumElements() / DpasVTy->getNumElements());
+      });
+
+  if (NumDpasZeroAcc < PeelLoopDpasNullAccMin)
+    return;
+
+  // Don't peel too large loops
+  const auto Blocks = L->getBlocks();
+  const auto NumInstr = std::accumulate(
+      std::begin(Blocks), std::end(Blocks), 0u,
+      [](unsigned Acc, const auto *BB) { return Acc + BB->size(); });
+  if (NumInstr > PeelLoopDpasNullAccMaxInstr)
+    return;
+
+  PP.PeelCount = 1;
 }
 
 } // namespace llvm
@@ -318,6 +469,9 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createGenXInitBiFConstantsPass());
   vc::addPass(PM, createGenXFixInvalidFuncNamePass());
 
+  /// .. include:: GenXRawSendRipper.cpp
+  vc::addPass(PM, createGenXRawSendRipperPass());
+
   // Install GenX-specific TargetTransformInfo for passes such as
   // LowerAggrCopies and InfoAddressSpace
   vc::addPass(PM, createTargetTransformInfoWrapperPass(getTargetIRAnalysis()));
@@ -328,11 +482,7 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createSROAPass());
   vc::addPass(PM, createEarlyCSEPass());
   vc::addPass(PM, createLowerExpectIntrinsicPass());
-#if LLVM_VERSION_MAJOR >= 12
   vc::addPass(PM, createCFGSimplificationPass(SimplifyCFGOptions().hoistCommonInsts(true)));
-#else
-  vc::addPass(PM, createCFGSimplificationPass());
-#endif
   vc::addPass(PM, createInstructionCombiningPass());
   vc::addPass(PM, createCFGSimplificationPass());
 
@@ -352,8 +502,12 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createPromoteMemoryToRegisterPass());
   // All passes which modify the LLVM IR are now complete; run the verifier
   // to ensure that the IR is valid.
-  if (!DisableVerify)
+
+  if (!DisableVerify) {
     vc::addPass(PM, createVerifierPass());
+    vc::addPass(PM, createGenXVerifyPass(GenXVerifyStage::PostIrAdaptors));
+  }
+
   // Run passes to generate vISA.
 
   /// BasicAliasAnalysis
@@ -395,6 +549,8 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createGenXReduceIntSizePass());
   /// .. include:: GenXGlobalValueLowering.cpp
   vc::addPass(PM, createGenXGlobalValueLoweringPass());
+  /// .. include:: GenXSLMResolution.cpp
+  vc::addPass(PM, createGenXSLMResolution());
 
   /// .. include:: GenXStackUsage.cpp
   vc::addPass(PM, createGenXStackUsagePass());
@@ -467,11 +623,16 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
                         BuiltinFunctionKind::PreLegalization));
     vc::addPass(PM, createAlwaysInlinerLegacyPass());
   }
-
+  /// .. include:: GenXBFloatLowering.cpp
+  vc::addPass(PM, createGenXBFloatLoweringPass());
   /// .. include:: GenXLowering.cpp
   vc::addPass(PM, createGenXLoweringPass());
-  if (!DisableVerify)
+
+  if (!DisableVerify) {
     vc::addPass(PM, createVerifierPass());
+    vc::addPass(PM, createGenXVerifyPass(GenXVerifyStage::PostGenXLowering));
+  }
+
   /// .. include:: GenXRegionCollapsing.cpp
   vc::addPass(PM, createGenXRegionCollapsingPass());
   /// EarlyCSE
@@ -483,8 +644,12 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createEarlyCSEPass());
   /// .. include:: GenXPatternMatch.cpp
   vc::addPass(PM, createGenXPatternMatchPass(PatternMatchKind::PreLegalization));
-  if (!DisableVerify)
+
+  if (!DisableVerify) {
     vc::addPass(PM, createVerifierPass());
+    vc::addPass(PM, createGenXVerifyPass(GenXVerifyStage::PostGenXLowering));
+  }
+
   /// .. include:: GenXExtractVectorizer.cpp
   vc::addPass(PM, createGenXExtractVectorizerPass());
   /// .. include:: GenXVectorCombiner.cpp
@@ -528,8 +693,13 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   /// .. include:: GenXConstants.cpp
   /// .. include:: GenXVectorDecomposer.h
   vc::addPass(PM, createGenXPostLegalizationPass());
-  if (!DisableVerify)
+
+  if (!DisableVerify) {
     vc::addPass(PM, createVerifierPass());
+    vc::addPass(PM,
+                createGenXVerifyPass(GenXVerifyStage::PostGenXLegalization));
+  }
+
   /// EarlyCSE
   /// --------
   /// This is a standard LLVM pass, run at this point in the GenX backend.
@@ -539,8 +709,13 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createEarlyCSEPass());
   /// .. include:: GenXPatternMatch.cpp
   vc::addPass(PM, createGenXPatternMatchPass(PatternMatchKind::PostLegalization));
-  if (!DisableVerify)
+
+  if (!DisableVerify) {
     vc::addPass(PM, createVerifierPass());
+    vc::addPass(PM,
+                createGenXVerifyPass(GenXVerifyStage::PostGenXLegalization));
+  }
+
   /// LICM
   /// ----
   /// This is a standard LLVM pass to hoist/sink the loop invariant code after
@@ -607,25 +782,32 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createGenXNumberingWrapperPass());
   /// .. include:: GenXLiveRanges.cpp
   vc::addPass(PM, createGenXLiveRangesWrapperPass());
-  /// .. include:: GenXGVClobberChecker.cpp
-  if (BackendConfig.checkGVClobbering())
-    vc::addPass(PM, createGenXGVClobberCheckerPass());
   /// .. include:: GenXCoalescing.cpp
   vc::addPass(PM, createGenXCoalescingWrapperPass());
   /// .. include:: GenXAddressCommoning.cpp
   vc::addPass(PM, createGenXAddressCommoningWrapperPass());
+
   /// .. include:: GenXArgIndirection.cpp
   vc::addPass(PM, createGenXArgIndirectionWrapperPass());
   /// .. include:: GenXTidyControlFlow.cpp
   vc::addPass(PM, createGenXTidyControlFlowPass());
+
+  if (BackendConfig.checkGVClobbering())
+    /// .. include:: GenXGVClobberChecker.cpp
+    vc::addPass(PM, createGenXGVClobberCheckerPass());
+
   /// .. include:: GenXVisaRegAlloc.h
   auto *RegAlloc = createGenXVisaRegAllocWrapperPass();
   vc::addPass(PM, RegAlloc);
   if (BackendConfig.enableRegAllocDump())
     vc::addPass(PM, createGenXModuleAnalysisDumperPass(RegAlloc, FGDumpsPrefix,
                                                        ".regalloc"));
-  if (!DisableVerify)
+  if (!DisableVerify) {
     vc::addPass(PM, createVerifierPass());
+    vc::addPass(PM,
+                createGenXVerifyPass(GenXVerifyStage::PostGenXLegalization));
+  }
+
   /// .. include:: GenXCisaBuilder.cpp
   vc::addPass(PM, createGenXCisaBuilderPass());
   vc::addPass(PM, createGenXFinalizerPass());
@@ -637,7 +819,7 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   // Explicit construction can be omitted because adding of extractor
   // pass will create runtime info analysis. Leaving it exlicit for
   // clarity.
-  if (Subtarget.isOCLRuntime())
+  if (!SkipOCLRuntimeInfo)
     vc::addPass(PM, new GenXOCLRuntimeInfo());
 
   return false;
@@ -661,12 +843,18 @@ void GenXTargetMachine::adjustPassManager(PassManagerBuilder &PMBuilder) {
   // Packetize.
   auto AddPacketize = [](const PassManagerBuilder &Builder,
                          PassManagerBase &PM) {
+    PM.add(createGenXLegalizeGVLoadUsesPass());
+#ifndef NDEBUG
+    PM.add(createGenXVerifyPass(GenXVerifyStage::PostSPIRVReader));
+#endif
     PM.add(createGenXTranslateIntrinsicsPass());
     PM.add(createGenXTranslateSPIRVBuiltinsPass());
     PM.add(createAlwaysInlinerLegacyPass());
     PM.add(createAlwaysInlinerLegacyPass());
     PM.add(createGenXPrintfResolutionPass());
     PM.add(createGenXImportOCLBiFPass());
+    PM.add(createGenXBIFFlagCtrlResolutionPass());
+    PM.add(createGenXTypeLegalizationPass());
     PM.add(createGenXPacketizePass());
     PM.add(createAlwaysInlinerLegacyPass());
     PM.add(createGenXPrintfLegalizationPass());
@@ -674,11 +862,7 @@ void GenXTargetMachine::adjustPassManager(PassManagerBuilder &PMBuilder) {
     PM.add(createPromoteMemoryToRegisterPass());
     PM.add(createInferAddressSpacesPass());
     PM.add(createEarlyCSEPass(true));
-#if LLVM_VERSION_MAJOR >= 12
     PM.add(createCFGSimplificationPass(SimplifyCFGOptions().hoistCommonInsts(true)));
-#else
-    PM.add(createCFGSimplificationPass());
-#endif
     PM.add(createInstructionCombiningPass());
     PM.add(createDeadCodeEliminationPass());
     PM.add(createSROAPass());
@@ -740,17 +924,15 @@ void GenXTargetMachine::adjustPassManager(PassManagerBuilder &PMBuilder) {
                            AddLowerLoadStore);
   }
 
-  if (Subtarget.isOCLRuntime()) {
-    auto AddIndirect = [](const PassManagerBuilder &Builder,
-                          PassManagerBase &PM) {
-      PM.add(createGenXCloneIndirectFunctionsPass());
-      PM.add(createGenXTrampolineInsertionPass());
-    };
-    PMBuilder.addExtension(PassManagerBuilder::EP_ModuleOptimizerEarly,
-                           AddIndirect);
-    PMBuilder.addExtension(PassManagerBuilder::EP_EnabledOnOptLevel0,
-                           AddIndirect);
-  }
+  auto AddIndirect = [](const PassManagerBuilder &Builder,
+                        PassManagerBase &PM) {
+    PM.add(createGenXCloneIndirectFunctionsPass());
+    PM.add(createGenXTrampolineInsertionPass());
+  };
+  PMBuilder.addExtension(PassManagerBuilder::EP_ModuleOptimizerEarly,
+                         AddIndirect);
+  PMBuilder.addExtension(PassManagerBuilder::EP_EnabledOnOptLevel0,
+                         AddIndirect);
 
   // Have to internalize functions before CM implicit parameters as all
   // implicit parameters cannot be supported at once for external functions
@@ -784,16 +966,14 @@ void GenXTargetMachine::adjustPassManager(PassManagerBuilder &PMBuilder) {
   PMBuilder.addExtension(PassManagerBuilder::EP_EnabledOnOptLevel0, AddCMABI);
 
   // BTI assignment.
-  if (Subtarget.isOCLRuntime()) {
-    auto AddBTIAssign = [](const PassManagerBuilder &Builder,
-                           PassManagerBase &PM) {
-      PM.add(createGenXBTIAssignmentPass());
-    };
-    PMBuilder.addExtension(PassManagerBuilder::EP_ModuleOptimizerEarly,
-                           AddBTIAssign);
-    PMBuilder.addExtension(PassManagerBuilder::EP_EnabledOnOptLevel0,
-                           AddBTIAssign);
-  }
+  auto AddBTIAssign = [](const PassManagerBuilder &Builder,
+                         PassManagerBase &PM) {
+    PM.add(createGenXBTIAssignmentPass());
+  };
+  PMBuilder.addExtension(PassManagerBuilder::EP_ModuleOptimizerEarly,
+                         AddBTIAssign);
+  PMBuilder.addExtension(PassManagerBuilder::EP_EnabledOnOptLevel0,
+                         AddBTIAssign);
 
   // CM kernel argument offset.
   auto AddCMKernelArgOffset = [this](const PassManagerBuilder &Builder,
