@@ -825,19 +825,20 @@ Value *GenXPacketize::packetizeLLVMIntrinsic(Instruction *pInst) {
   }
 
   // override certain intrinsics
-  Value *pNewCall;
   switch (id) {
+  default:
+    break;
   case Intrinsic::log2:
-    pNewCall = B->VLOG2PS(packetizedArgs[0]);
+    if (!pInst->hasApproxFunc())
+      return B->VLOG2PS(packetizedArgs[0]);
     break;
   case Intrinsic::exp2:
-    pNewCall = B->VEXP2PS(packetizedArgs[0]);
+    if (!pInst->hasApproxFunc())
+      return B->VEXP2PS(packetizedArgs[0]);
     break;
-  default: {
-    Function *newF = getVectorIntrinsic(M, id, vectorArgTys);
-    pNewCall = CallInst::Create(newF, packetizedArgs, "", pCall);
-  }
   }
+  Function *newF = getVectorIntrinsic(M, id, vectorArgTys);
+  auto *pNewCall = CallInst::Create(newF, packetizedArgs, "", pCall);
   return pNewCall;
 }
 
@@ -1725,12 +1726,7 @@ Value *GenXPacketize::packetizeInstruction(Instruction *pInst) {
       SmallVector<DbgVariableIntrinsic *, 1> DbgUsers;
       llvm::findDbgUsers(DbgUsers, pInst);
       for (auto *DII : DbgUsers) {
-#if LLVM_VERSION_MAJOR >= 13
         DII->replaceVariableLocationOp(pInst, pResult);
-#else
-        DII->setOperand(0, llvm::MetadataAsValue::get(DII->getContext(),
-                                                      llvm::ValueAsMetadata::get(pResult)));
-#endif
       }
     }
 
