@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021-2023 Intel Corporation
+Copyright (C) 2021-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -256,6 +256,26 @@ static void assertValidPrintCall(const CallInst &CI) {
   (void)CI;
 }
 
+// Format string operand is expected to be GEP. In some cases bitcast is
+// present instead. It has to be substituted with an equivalent GEP
+static void fixFormatStringOperand(CallInst &CI) {
+  auto *FmtOp = CI.getOperand(0);
+  if (isConstantStringFirstElementGEP(*FmtOp))
+    return;
+  auto *BCO = dyn_cast<BitCastOperator>(FmtOp);
+  if (!BCO)
+    return;
+  auto *Ptr = BCO->getOperand(0);
+  if (!isConstantString(*Ptr))
+    return;
+  auto *GV = cast<GlobalVariable>(Ptr);
+  auto *Zero = Constant::getNullValue(Type::getInt32Ty(Ptr->getContext()));
+  Constant* Indices[2] = { Zero, Zero };
+  auto *GEP =
+      ConstantExpr::getGetElementPtr(GV->getValueType(), GV, Indices);
+  CI.setOperand(0, GEP);
+}
+
 // Returns pair of format string size (including '\0') and argument information.
 static std::pair<int, PrintfArgInfoSeq>
 analyzeFormatString(const Value &FmtStrOp) {
@@ -269,6 +289,11 @@ analyzeFormatString(const Value &FmtStrOp) {
 // Marks strings passed as "%s" arguments in printf.
 // Recursive function, long instruction chains aren't expected.
 static void markStringArgument(Value &Arg) {
+  if (isConstantString(Arg)) {
+    auto &String = cast<GlobalVariable>(Arg);
+    String.addAttribute(PrintfStringVariable);
+    return;
+  }
   if (isa<GEPOperator>(Arg)) {
     auto *String = getConstStringGVFromOperandOptional(Arg);
     if (!String)
@@ -288,6 +313,11 @@ static void markStringArgument(Value &Arg) {
     markStringArgument(*SI.getTrueValue());
     return;
   }
+  if (isa<BitCastOperator>(Arg)) {
+    auto &BCO = cast<BitCastOperator>(Arg);
+    markStringArgument(*BCO.getOperand(0));
+    return;
+  }
   if (isCastToGenericAS(Arg))
     return markStringArgument(
         *cast<IGCLLVM::AddrSpaceCastOperator>(Arg).getPointerOperand());
@@ -314,6 +344,7 @@ static void markPrintfStrings(CallInst &OrigPrintf,
 
 void GenXPrintfResolution::handlePrintfCall(CallInst &OrigPrintf) {
   assertValidPrintCall(OrigPrintf);
+  fixFormatStringOperand(OrigPrintf);
   auto [FmtStrSize, ArgsInfo] =
       analyzeFormatString(*OrigPrintf.getArgOperand(0));
   if (ArgsInfo.size() != IGCLLVM::getNumArgOperands(&OrigPrintf) - 1)
@@ -407,7 +438,7 @@ void GenXPrintfResolution::updatePrintfImplDeclarations(Module &M) {
   std::transform(
       std::begin(PrintfImplFunc::Name), std::end(PrintfImplFunc::Name),
       PrintfImplDecl.begin(),
-      [&M](const char *Name) -> FunctionCallee { return M.getFunction(Name); });
+      [&M](const char *Name) { return M.getFunction(Name); });
 }
 
 using ArgsInfoStorage = std::array<unsigned, ArgsInfoVector::Size>;
