@@ -115,6 +115,11 @@ enum IIT_Info {
 #include "vc/InternalIntrinsics/InternalIntrinsicDescription.gen"
 #undef GET_INTRINSIC_GENERATOR_GLOBAL
 
+static Intrinsic::IITDescriptor getVector(unsigned Width) {
+  using namespace Intrinsic;
+  return IITDescriptor::getVector(Width, false);
+}
+
 static void
 DecodeIITType(unsigned &NextElt, ArrayRef<unsigned char> Infos,
               SmallVectorImpl<Intrinsic::IITDescriptor> &OutputTable) {
@@ -170,39 +175,39 @@ DecodeIITType(unsigned &NextElt, ArrayRef<unsigned char> Infos,
     OutputTable.push_back(IITDescriptor::get(IITDescriptor::Integer, 128));
     return;
   case IIT_V1:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 1));
+    OutputTable.push_back(getVector(1));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V2:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 2));
+    OutputTable.push_back(getVector(2));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V4:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 4));
+    OutputTable.push_back(getVector(4));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V8:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 8));
+    OutputTable.push_back(getVector(8));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V16:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 16));
+    OutputTable.push_back(getVector(16));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V32:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 32));
+    OutputTable.push_back(getVector(32));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V64:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 64));
+    OutputTable.push_back(getVector(64));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V512:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 512));
+    OutputTable.push_back(getVector(512));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_V1024:
-    OutputTable.push_back(IITDescriptor::get(IITDescriptor::Vector, 1024));
+    OutputTable.push_back(getVector(1024));
     DecodeIITType(NextElt, Infos, OutputTable);
     return;
   case IIT_PTR:
@@ -365,6 +370,7 @@ static Type *DecodeFixedType(ArrayRef<Intrinsic::IITDescriptor> &Infos,
       return IGCLLVM::FixedVectorType::get(EltTy, VTy->getNumElements());
     }
     IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
+    break;
   }
   case IITDescriptor::PtrToArgument: {
     Type *Ty = Tys[D.getArgumentNumber()];
@@ -716,6 +722,14 @@ bool InternalIntrinsic::isInternalMemoryIntrinsic(InternalIntrinsic::ID id) {
   case InternalIntrinsic::lsc_store_quad_bss:
   case InternalIntrinsic::lsc_store_quad_slm:
   case InternalIntrinsic::lsc_store_quad_ugm:
+  case InternalIntrinsic::lsc_load_quad_tgm:
+  case InternalIntrinsic::lsc_prefetch_quad_tgm:
+  case InternalIntrinsic::lsc_store_quad_tgm:
+  case InternalIntrinsic::lsc_load_block_2d_ugm:
+  case InternalIntrinsic::lsc_load_block_2d_ugm_transposed:
+  case InternalIntrinsic::lsc_load_block_2d_ugm_vnni:
+  case InternalIntrinsic::lsc_prefetch_block_2d_ugm:
+  case InternalIntrinsic::lsc_store_block_2d_ugm:
     return true;
   }
 
@@ -743,6 +757,9 @@ bool InternalIntrinsic::isMemoryBlockIntrinsic(const llvm::Instruction *I) {
   case InternalIntrinsic::lsc_store_quad_bss:
   case InternalIntrinsic::lsc_store_quad_slm:
   case InternalIntrinsic::lsc_store_quad_ugm:
+  case InternalIntrinsic::lsc_load_quad_tgm:
+  case InternalIntrinsic::lsc_prefetch_quad_tgm:
+  case InternalIntrinsic::lsc_store_quad_tgm:
     return false;
   }
 
@@ -794,6 +811,9 @@ InternalIntrinsic::getMemoryVectorSizePerLane(const llvm::Instruction *I) {
 
     IGC_ASSERT_UNREACHABLE();
   }
+  case InternalIntrinsic::lsc_load_quad_tgm:
+  case InternalIntrinsic::lsc_prefetch_quad_tgm:
+  case InternalIntrinsic::lsc_store_quad_tgm:
   case InternalIntrinsic::lsc_load_quad_bti:
   case InternalIntrinsic::lsc_load_quad_bss:
   case InternalIntrinsic::lsc_load_quad_slm:
@@ -837,15 +857,31 @@ InternalIntrinsic::getMemoryRegisterElementSize(const llvm::Instruction *I) {
   switch (IID) {
   default:
     break;
+  // All typed intrinsics work with 32-bit elements
+  case InternalIntrinsic::lsc_load_quad_tgm:
+  case InternalIntrinsic::lsc_prefetch_quad_tgm:
+  case InternalIntrinsic::lsc_store_quad_tgm:
+    return 32;
   case InternalIntrinsic::lsc_atomic_bti:
   case InternalIntrinsic::lsc_atomic_slm:
   case InternalIntrinsic::lsc_atomic_ugm:
     ElementSizeIndex = 3;
     break;
+  case InternalIntrinsic::lsc_load_block_2d_ugm:
+  case InternalIntrinsic::lsc_load_block_2d_ugm_transposed:
+  case InternalIntrinsic::lsc_load_block_2d_ugm_vnni:
+  case InternalIntrinsic::lsc_prefetch_block_2d_ugm:
+  case InternalIntrinsic::lsc_store_block_2d_ugm:
+    ElementSizeIndex = 1;
+    break;
   }
 
   auto *ElementSize = cast<ConstantInt>(I->getOperand(ElementSizeIndex));
   switch (ElementSize->getZExtValue()) {
+  case LSC_DATA_SIZE_8b:
+    return 8;
+  case LSC_DATA_SIZE_16b:
+    return 16;
   case LSC_DATA_SIZE_8c32b:
   case LSC_DATA_SIZE_16c32b:
   case LSC_DATA_SIZE_32b:
@@ -857,3 +893,23 @@ InternalIntrinsic::getMemoryRegisterElementSize(const llvm::Instruction *I) {
   }
   IGC_ASSERT_UNREACHABLE();
 }
+
+int InternalIntrinsic::getMemoryCacheControlOperandIndex(
+    const llvm::Instruction *I) {
+  auto IID = getInternalIntrinsicID(I);
+  if (!isInternalMemoryIntrinsic(IID))
+    return -1;
+
+  switch (IID) {
+  case InternalIntrinsic::lsc_load_block_2d_ugm:
+  case InternalIntrinsic::lsc_load_block_2d_ugm_transposed:
+  case InternalIntrinsic::lsc_load_block_2d_ugm_vnni:
+  case InternalIntrinsic::lsc_prefetch_block_2d_ugm:
+  case InternalIntrinsic::lsc_store_block_2d_ugm:
+    return 2;
+  default:
+    break;
+  }
+
+  return 4;
+}
