@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2018-2021 Intel Corporation
+Copyright (C) 2018-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -166,11 +166,12 @@ namespace IGCLLVM
 #endif
 
 #if LLVM_VERSION_MAJOR >= 11
-      inline llvm::CallInst* CreateCall(llvm::Value* Callee, llvm::ArrayRef<llvm::Value*> Args = llvm::None,
-                                           const llvm::Twine& Name = "", llvm::MDNode* FPMathTag = nullptr) {
+
+        inline llvm::CallInst* CreateCall(llvm::Value* Callee, llvm::ArrayRef<llvm::Value*> Args = llvm::None,
+                                          const llvm::Twine& Name = "", llvm::MDNode* FPMathTag = nullptr) {
             return llvm::IRBuilder<T, InserterTyDef()>::CreateCall(
-                                                           llvm::cast<llvm::FunctionType>(IGCLLVM::getNonOpaquePtrEltTy(Callee->getType())), Callee,
-            Args, Name, FPMathTag);
+                llvm::cast<llvm::Function>(Callee)->getFunctionType(),
+                Callee, Args, Name, FPMathTag);
         }
 
         inline llvm::CallInst *
@@ -178,10 +179,9 @@ namespace IGCLLVM
                    llvm::ArrayRef<llvm::OperandBundleDef> OpBundles,
                    const llvm::Twine &Name = "",
                    llvm::MDNode *FPMathTag = nullptr) {
-          return llvm::IRBuilder<T, InserterTyDef()>::CreateCall(
-              llvm::cast<llvm::FunctionType>(
-                  IGCLLVM::getNonOpaquePtrEltTy(Callee->getType())),
-              Callee, Args, OpBundles, Name, FPMathTag);
+            return llvm::IRBuilder<T, InserterTyDef()>::CreateCall(
+                llvm::cast<llvm::Function>(Callee)->getFunctionType(),
+                Callee, Args, OpBundles, Name, FPMathTag);
         }
 
         inline llvm::CallInst *
@@ -189,24 +189,30 @@ namespace IGCLLVM
                    llvm::ArrayRef<llvm::Value *> Args = llvm::None,
                    const llvm::Twine &Name = "",
                    llvm::MDNode *FPMathTag = nullptr) {
-          return llvm::IRBuilder<T, InserterTyDef()>::CreateCall(FTy, Callee, Args,
-                                                          Name, FPMathTag);
+            return llvm::IRBuilder<T, InserterTyDef()>::CreateCall(FTy, Callee, Args, Name, FPMathTag);
         }
 #endif
 
 #if LLVM_VERSION_MAJOR >= 13
+
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::LoadInst* CreateLoad(llvm::Value* Ptr, const char *Name)
         {
             llvm::Type* ptrType = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType());
             return llvm::IRBuilder<T, InserterTyDef()>::CreateLoad(ptrType, Ptr, Name);
         }
 
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::LoadInst* CreateLoad(llvm::Value* Ptr, const llvm::Twine &Name = "")
         {
             llvm::Type* ptrType = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType());
             return llvm::IRBuilder<T, InserterTyDef()>::CreateLoad(ptrType, Ptr, Name);
         }
 
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::LoadInst* CreateLoad(llvm::Value* Ptr, bool isVolatile, const llvm::Twine &Name = "")
         {
             llvm::Type* ptrType = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType());
@@ -215,6 +221,8 @@ namespace IGCLLVM
 
         using llvm::IRBuilder<T, InserterTyDef()>::CreateLoad;
 
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         /// Provided to resolve 'CreateAlignedLoad(Ptr, Align, "...")'
         /// correctly, instead of converting the string to 'bool' for the isVolatile
         /// parameter.
@@ -224,28 +232,38 @@ namespace IGCLLVM
             return llvm::IRBuilder<T, InserterTyDef()>::CreateAlignedLoad(ptrType, Ptr, Align, Name);
         }
 
+        /// Provided to resolve 'CreateAlignedLoad(ptrType, Ptr, Align, "...")'
+        /// correctly, instead of converting the string to 'bool' for the isVolatile
+        /// parameter.
+        inline llvm::LoadInst* CreateAlignedLoad(llvm::Type* ptrType, llvm::Value* Ptr, IGCLLVM::Align Align, const llvm::Twine& Name = "")
+        {
+            return llvm::IRBuilder<T, InserterTyDef()>::CreateAlignedLoad(ptrType, Ptr, Align, Name);
+        }
+
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::LoadInst* CreateAlignedLoad(llvm::Value* Ptr, IGCLLVM::Align Align, const llvm::Twine& Name = "")
         {
             llvm::Type* ptrType = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType());
             return llvm::IRBuilder<T, InserterTyDef()>::CreateAlignedLoad(ptrType, Ptr, Align, Name);
         }
 
+        /// Provided to resolve 'CreateAlignedLoad(ptrType, Ptr, Align, "...")' correctly.
+        inline llvm::LoadInst* CreateAlignedLoad(llvm::Type* ptrType, llvm::Value* Ptr, IGCLLVM::Align Align, bool isVolatile, const llvm::Twine& Name = "")
+        {
+            return llvm::IRBuilder<T, InserterTyDef()>::CreateAlignedLoad(ptrType, Ptr, Align, isVolatile, Name);
+        }
+
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::LoadInst* CreateAlignedLoad(llvm::Value* Ptr, IGCLLVM::Align Align, bool isVolatile, const llvm::Twine& Name = "")
         {
             llvm::Type* ptrType = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType());
             return llvm::IRBuilder<T, InserterTyDef()>::CreateAlignedLoad(ptrType, Ptr, Align, isVolatile, Name);
         }
 
-        inline llvm::Value* CreateConstGEP1_32(
-            llvm::Value* Ptr,
-            unsigned Idx0,
-            const llvm::Twine& Name = "")
-        {
-            return llvm::IRBuilder<T, InserterTyDef()>::CreateConstGEP1_32(IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType()), Ptr, Idx0, Name);
-        }
-
-        using llvm::IRBuilder<T, InserterTyDef()>::CreateConstGEP1_32;
-
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::Value* CreateInBoundsGEP(llvm::Value *Ptr, llvm::ArrayRef<llvm::Value*> IdxList,
                            const llvm::Twine &Name = "") {
             llvm::Type *Ty = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType()->getScalarType());
@@ -254,6 +272,8 @@ namespace IGCLLVM
 
         using llvm::IRBuilder<T, InserterTyDef()>::CreateInBoundsGEP;
 
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         inline llvm::Value* CreateGEP(llvm::Value* Ptr, llvm::ArrayRef<llvm::Value*> IdxList,
             const llvm::Twine& Name = "") {
             llvm::Type* Ty = IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType()->getScalarType());
@@ -262,6 +282,8 @@ namespace IGCLLVM
 
         using llvm::IRBuilder<T, InserterTyDef()>::CreateGEP;
 
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         llvm::CallInst *CreateMaskedGather(llvm::Value *Ptrs, Align Alignment, llvm::Value *Mask,
                                      llvm::Value *PassThru, const llvm::Twine &Name) {
           auto *PtrsTy = llvm::cast<llvm::FixedVectorType>(Ptrs->getType());
@@ -287,6 +309,8 @@ namespace IGCLLVM
           return createAtomicRMW(*this, Op, Ptr, Val, Ordering, SSID);
         }
 
+        // This wrapper function is deprecated because it uses typed pointer and
+        // should no longer be used in LLVM 14+ compatible code.
         llvm::CallInst *CreateMaskedLoad(llvm::Value *Ptr, Align Alignment, llvm::Value *Mask,
                                    llvm::Value *PassThru, const llvm::Twine &Name) {
           auto *PtrTy = llvm::cast<llvm::PointerType>(Ptr->getType());
@@ -298,6 +322,8 @@ namespace IGCLLVM
 #endif
 
 #if LLVM_VERSION_MAJOR >= 14
+        // This wrapper function is deprecated because it uses getNonOpaquePointerElementType() function
+        // and should no longer be used in LLVM 14+ compatible code.
         llvm::Value* CreatePtrDiff(llvm::Value *LHS, llvm::Value *RHS, const llvm::Twine &Name = "") {
           auto *PtrTy = llvm::cast<llvm::PointerType>(LHS->getType());
           llvm::Type *Ty = PtrTy->getNonOpaquePointerElementType();
@@ -313,22 +339,6 @@ namespace IGCLLVM
             return llvm::IRBuilder<T, InserterTyDef()>::CreateFreeze(V, Name);
 #endif
         }
-
-        inline llvm::Value* CreateConstInBoundsGEP2_64(
-            llvm::Value* Ptr,
-            uint64_t Idx0,
-            uint64_t Idx1,
-            const llvm::Twine& Name = "")
-        {
-#if LLVM_VERSION_MAJOR <= 10
-            return llvm::IRBuilder<T, InserterTyDef()>::CreateConstInBoundsGEP2_64(Ptr, Idx0, Idx1, Name);
-#else
-            return llvm::IRBuilder<T, InserterTyDef()>::CreateConstInBoundsGEP2_64(IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType()), Ptr, Idx0, Idx1, Name);
-#endif
-        }
-
-        using llvm::IRBuilder<T, InserterTyDef()>::CreateConstInBoundsGEP2_64;
-
     };
 
     template <typename T = llvm::ConstantFolder,
