@@ -249,6 +249,7 @@ namespace IGC
         bool doSyncDispatchRays = false;
 
         RTMemoryStyle MemStyle = RTMemoryStyle::Xe;
+        RayDispatchInlinedDataStyle GlobalDataStyle = RayDispatchInlinedDataStyle::Xe;
     };
 
     // Info specific to each raytracing shader
@@ -325,6 +326,8 @@ namespace IGC
         bool localIDPresent = false;
         bool groupIDPresent = false;
         int privateMemoryPerWI = 0;
+        // Bytes reserved at the beginning of the frame to write old FP there.
+        unsigned int prevFPOffset = 0;
         bool globalIDPresent = false;
         // This is true if the function has any sync raytracing functionality
         bool hasSyncRTCalls = false;
@@ -361,6 +364,7 @@ namespace IGC
     struct CompOptions
     {
         bool DenormsAreZero                             = false;
+        bool BFTFDenormsAreZero                         = false;
         bool CorrectlyRoundedDivSqrt                    = false;
         bool OptDisable                                 = false;
         bool MadEnable                                  = false;
@@ -379,6 +383,7 @@ namespace IGC
         bool UnsafeMathOptimizations                    = false;
         bool disableCustomUnsafeOpts                    = false;
         bool disableReducePow                           = false;
+        bool disableSqrtOpt                             = false;
         bool FiniteMathOnly                             = false;
         bool FastRelaxedMath                            = false;
         bool DashGSpecified                             = false;
@@ -405,6 +410,8 @@ namespace IGC
         bool UseLegacyBindlessMode                      = true;
         bool disableMathRefactoring                     = false;
         bool atomicBranch                               = false;
+        bool spillCompression                           = false;
+        bool DisableEarlyOut                            = false;
         bool ForceInt32DivRemEmu                        = false;
         bool ForceInt32DivRemEmuSP                      = false;
         bool DisableFastestSingleCSSIMD                 = false;
@@ -415,11 +422,13 @@ namespace IGC
         bool UseStatelessforPrivateMemory               = false;
         bool EnableTakeGlobalAddress                    = false;
         bool IsLibraryCompilation                       = false;
+        unsigned LibraryCompileSIMDSize                 = 0;
         bool FastVISACompile                            = false;
         bool MatchSinCosPi                              = false;
         bool ExcludeIRFromZEBinary                      = false;
         bool EmitZeBinVISASections                      = false;
         bool FP64GenEmulationEnabled                    = false;
+        bool FP64GenConvEmulationEnabled                = false;
 
         //when true, compiler disables the Remat optimization for compute shaders
         bool allowDisableRematforCS                     = false;
@@ -439,8 +448,12 @@ namespace IGC
         bool DisableFDivToFMulInvOpt                    = false;
         bool initializePhiSampleSourceWA                = false;
         bool WaDisableSubspanUseNoMaskForCB             = false;
+        bool DisableLoosenSimd32Occu                    = false;
 
         unsigned FastestS1Options                       = 0;  // FCEXP_NO_EXPRIMENT. Can't access the enum here for some reason.
+        bool DisableFastestForWaveIntrinsicsCS          = false;
+        bool ForceLinearWalkOnLinearUAV                 = false;
+        bool DisableLscSamplerRouting                   = false;
     };
 
     enum class ThreadIDLayout
@@ -462,11 +475,12 @@ namespace IGC
         unsigned int forceTotalGRFNum = 0; // 0 means not forced
         unsigned int VISAPreSchedRPThreshold = 0; // 0 means use the default
         unsigned int SetLoopUnrollThreshold = 0; // 0 means use the default
+        bool forceSpillCompression = false;
         bool allowLowerSimd = false;
         bool disableSimd32Slicing = false;
         bool disableSplitOnSpill = false;
         bool enableNewSpillCostFunction = false;
-        bool forcedVISAPreRAScheduler = false;
+        bool forceVISAPreSched = false;
         bool forceUniformBuffer = false;
         bool forceUniformSurfaceSampler = false;
         // disables dispatch along y and tiled order optimizations
@@ -478,7 +492,11 @@ namespace IGC
         // force enable tile y optimization
         bool forceTileYWalk = false;
         // enable atomic branch optimization
-        bool atomicBranch = false;
+        unsigned int atomicBranch = 0;
+        // enable spill compression
+        bool spillCompression = false;
+        // disable early out
+        bool disableEarlyOut = false;
         // enable compute walk order optimization
         bool walkOrderEnabled = false;
         unsigned int walkOrderOverride = 0;
@@ -605,6 +623,14 @@ namespace IGC
         unsigned int rtSyncSurfPtrOffset = 0;
     };
 
+    enum InlineProgramScopeBufferType
+    {
+        Constants,
+        ConstantStrings,
+        Globals,
+        Count
+    };
+
     struct InlineProgramScopeBuffer
     {
         int alignment = 0;
@@ -680,8 +706,7 @@ namespace IGC
         std::vector<InlineResInfo> inlineResInfoData;
         ImmConstantInfo immConstant;
         llvm::SetVector<llvm::GlobalVariable*> stringConstants;
-        std::vector<InlineProgramScopeBuffer> inlineConstantBuffers;
-        std::vector<InlineProgramScopeBuffer> inlineGlobalBuffers;
+        std::array<InlineProgramScopeBuffer, InlineProgramScopeBufferType::Count> inlineBuffers = {};
         std::vector<PointerProgramBinaryInfo> GlobalPointerProgramBinaryInfos;
         std::vector<PointerProgramBinaryInfo> ConstantPointerProgramBinaryInfos;
         std::vector<PointerAddressRelocInfo> GlobalBufferAddressRelocInfo;
@@ -734,7 +759,13 @@ namespace IGC
         std::array<uint64_t, NUM_SHADER_RESOURCE_VIEW_SIZE> m_ShaderResourceViewMcsMask{};
         unsigned int computedDepthMode = 0; //Defaults to 0 meaning depth mode is off
         bool isHDCFastClearShader = false;
+
+        std::array<uint32_t, NUM_ARG_SPACE_RESERVATION_SLOTS> argRegisterReservations{};
+
+        uint8_t SIMD16_SpillThreshold = 0;
+        uint8_t SIMD32_SpillThreshold = 0;
     };
+
     void serialize(const IGC::ModuleMetaData &moduleMD, llvm::Module* module);
     void deserialize(IGC::ModuleMetaData &deserializedMD, const llvm::Module* module);
 
