@@ -382,43 +382,64 @@ static bool ReadIGCEnv(
 /*****************************************************************************\
 ReadIGCRegistry
 \*****************************************************************************/
-static bool ReadIGCRegistry(
+bool ReadIGCRegistry(
     const char*  pName,
     void*        pValue,
     unsigned int size ,
-    std::string registrykeypath)
+    bool readFromEnv)
 {
     // All platforms can retrieve settings from environment
-    if( ReadIGCEnv( pName, pValue, size ) )
+    if( readFromEnv && ReadIGCEnv( pName, pValue, size ))
     {
         return true;
     }
 
 #if defined _WIN32
     LONG success = ERROR_SUCCESS;
-    HKEY uscKey;
-
-    success = RegOpenKeyExA(
-        HKEY_LOCAL_MACHINE,
-        registrykeypath.c_str(),
-        0,
-        KEY_READ,
-        &uscKey );
-
-    if( ERROR_SUCCESS == success )
+    static std::vector<std::string> registrykeypaths;
+    if (registrykeypaths.empty())
     {
-        DWORD dwSize = size;
-        success = RegQueryValueExA(
-            uscKey,
-            pName,
-            NULL,
-            NULL,
-            (LPBYTE)pValue,
-            &dwSize );
-
-        RegCloseKey( uscKey );
+        registrykeypaths.push_back(IGC_REGISTRY_KEY);
+        std::vector<DEVINST> drivers;
+        std::list<std::string> registrypaths;
+        GetIntelDriverPaths(drivers);
+        for (auto driverInfo : drivers)
+        {
+            std::string driverStoreRegKeyPath = getNewRegistryPath(driverInfo);
+            std::string registryKeyPath = "SYSTEM\\ControlSet001\\Control\\Class\\" + driverStoreRegKeyPath + "\\IGC";
+            registrykeypaths.push_back(registryKeyPath);
+        }
+    }
+
+    for (const std::string& registrykeypath : registrykeypaths)
+    {
+        HKEY uscKey;
+        success = RegOpenKeyExA(
+            HKEY_LOCAL_MACHINE,
+            registrykeypath.c_str(),
+            0,
+            KEY_READ,
+            &uscKey);
+
+        if (ERROR_SUCCESS == success)
+        {
+            DWORD dwSize = size;
+            success = RegQueryValueExA(
+                uscKey,
+                pName,
+                NULL,
+                NULL,
+                (LPBYTE)pValue,
+                &dwSize);
+
+            RegCloseKey(uscKey);
+        }
+
+        if (ERROR_SUCCESS == success)
+        {
+            return true;
+        }
     }
-    return ( ERROR_SUCCESS == success );
 #endif // defined _WIN32
 
     return false;
@@ -719,6 +740,23 @@ parseHashType(llvm::StringRef line)
     return { std::make_pair(*Ty, line.substr(Loc + 1)) };
 }
 
+static
+std::optional<std::pair<bool, llvm::StringRef>>
+detectEntryPoints(llvm::StringRef &line)
+{
+    using namespace llvm;
+    size_t Loc = line.find_first_of(':');
+    if (Loc == StringRef::npos)
+        return {};
+    auto key = line.substr(0, Loc);
+
+    bool isEntryPoint = false;
+    if (key == "entry_point") {
+        isEntryPoint = true;
+    }
+
+    return { std::make_pair(isEntryPoint, line.substr(Loc + 1)) };
+}
 // parses this syntax:
 // Each hash may be optionally prefixed with "0x" (e.g., 0xaaaaaaaaaaaaaaaa)
 // hash:abcdabcdabcdabcd-ffffffffffffffff,aaaaaaaaaaaaaaaa
@@ -746,7 +784,7 @@ static void ParseHashRange(llvm::StringRef line, std::vector<HashRange>& ranges)
         StringRef start = token.substr(0, dash);
         StringRef end = (dash == StringRef::npos) ?
             start : token.substr(dash + 1);
-        HashRange range;
+        HashRange range = {};
         range.start = parseAsInt(start);
         range.end   = parseAsInt(end);
         range.Ty    = HashType;
@@ -755,6 +793,27 @@ static void ParseHashRange(llvm::StringRef line, std::vector<HashRange>& ranges)
     } while (!vString.empty());
 }
 
+// parses this syntax:
+// each seperated by comma
+// entry_point:MicropolyRasterize, CS
+static void ParseEntryPoint(llvm::StringRef line, std::vector<EntryPoint>& entry_points)
+{
+    auto Result = detectEntryPoints(line);
+    if (!Result)
+        return;
+    auto [isEntryPoint, vString] = *Result;
+    if (!isEntryPoint)
+        return;
+    // re-initialize entries
+    entry_points.clear();
+    do {
+        auto [token, RHS] = vString.split(',');
+        EntryPoint entry_point {};
+        entry_point.entry_point_name = token.str();
+        entry_points.push_back(entry_point);
+        vString = RHS;
+    } while (!vString.empty());
+}
 static void setIGCKeyOnHash(
     std::vector<HashRange>& hashes, const unsigned value,
     SRegKeyVariableMetaData* var)
@@ -765,7 +824,15 @@ static void setIGCKeyOnHash(
     var->Set();
     var->m_Value = value;
 }
-
+static void setIGCKeyOnEntryPoints(
+    std::vector<EntryPoint>& entry_points, const unsigned value,
+    SRegKeyVariableMetaData* var)
+{
+    // hashes can be empty if the var is not set via Options.txt
+    var->entry_points.insert(var->entry_points.end(), entry_points.begin(), entry_points.end());
+    var->Set();
+    var->m_Value = value;
+}
 // Implicitly set the subkeys
 static void setImpliedIGCKeys()
 {
@@ -810,6 +877,7 @@ static void setImpliedIGCKeys()
     IGC_SET_IMPLIED_REGKEY(DisableRayTracingOptimizations, 1, RematThreshold, 0);
     IGC_SET_IMPLIED_REGKEY(DisableRayTracingOptimizations, 1, DisableDPSE, true);
     IGC_SET_IMPLIED_REGKEY(DisableRayTracingOptimizations, 1, DisableSWStackOffsetElision, true);
+    IGC_SET_IMPLIED_REGKEY(DisableRayTracingOptimizations, 1, DisableInvalidateRTStackAfterLastRead, true);
 
     IGC_SET_IMPLIED_REGKEY(ForceRTRetry, 1, RetryManagerFirstStateId, 1);
 
@@ -855,12 +923,13 @@ void setImpliedRegkey(SRegKeyVariableMetaData& name,
     if (set)
     {
         setIGCKeyOnHash(name.hashes, subvalue, &subname);
+        setIGCKeyOnEntryPoints(name.entry_points, subvalue, &subname);
     }
 }
 
 static void declareIGCKey(
     const std::string& line, const char* dataType, const char* regkeyName,
-    std::vector<HashRange>& hashes, SRegKeyVariableMetaData* regKey)
+    std::vector<HashRange>& hashes, std::vector<EntryPoint>& entry_points, SRegKeyVariableMetaData* regKey)
 {
     bool isSet = false;
     debugString value = { 0 };
@@ -868,7 +937,8 @@ static void declareIGCKey(
     if (isSet && !hashes.empty())
     {
         std::cout << std::endl << "** hashes ";
-        for (size_t i = 0; i < hashes.size(); i++) {
+        for (size_t i = 0; i < hashes.size(); i++)
+        {
             memcpy_s(hashes[i].m_string, sizeof(value), value, sizeof(value));
             regKey->hashes.push_back(hashes[i]);
             if (hashes[i].end == hashes[i].start)
@@ -878,6 +948,21 @@ static void declareIGCKey(
         }
         std::cout << std::endl;
 
+        std::cout << "** regkey " << line << std::endl;
+        regKey->Set();
+        memcpy_s(regKey->m_string, sizeof(value), value, sizeof(value));
+    }
+    if (isSet && !entry_points.empty())
+    {
+        std::cout << std::endl << "** entry point ";
+        for (auto entry_point : entry_points)
+        {
+            strcpy_s(entry_point.m_string,sizeof(value), value);
+            regKey->entry_points.push_back(entry_point);
+            std::cout << entry_point.entry_point_name << ", ";
+        }
+        std::cout << std::endl;
+
         std::cout << "** regkey " << line << std::endl;
         regKey->Set();
         memcpy_s(regKey->m_string, sizeof(value), value, sizeof(value));
@@ -889,6 +974,7 @@ static void LoadDebugFlagsFromFile()
     std::ifstream input(GetOptionFile());
     std::string line;
     std::vector<HashRange> hashes;
+    std::vector<EntryPoint> entry_points;
 
     if (input.is_open())
         std::cout << std::endl << "** DebugFlags " << GetOptionFile() << " is opened" << std::endl;
@@ -897,11 +983,14 @@ static void LoadDebugFlagsFromFile()
         if (line.empty() || line.front() == '#')
             continue;
         ParseHashRange(line, hashes);
+        ParseEntryPoint(line, entry_points);
 #define DECLARE_IGC_REGKEY(dataType, regkeyName, defaultValue, description, releaseMode)         \
 {                                                                                   \
-    declareIGCKey(line, #dataType, #regkeyName, hashes, &(g_RegKeyList.regkeyName));\
+    declareIGCKey(line, #dataType, #regkeyName, hashes, entry_points, &(g_RegKeyList.regkeyName));\
 }
 #include "igc_regkeys.h"
+DECLARE_IGC_REGKEY(bool, ShaderDumpEnable, false, "dump LLVM IR, visaasm, and GenISA", true)
+
 #undef DECLARE_IGC_REGKEY
 
     }
@@ -916,6 +1005,7 @@ static void LoadDebugFlagsFromString(const char* input)
     std::istringstream istream(input);
     std::string line;
     std::vector<HashRange> hashes;
+    std::vector<EntryPoint>entry_points;
 
     std::cout << std::endl << "** DebugFlags " << input << " is applied" << std::endl;
 
@@ -923,9 +1013,10 @@ static void LoadDebugFlagsFromString(const char* input)
         if (line.empty())
             continue;
         ParseHashRange(line, hashes);
+        ParseEntryPoint(line, entry_points);
 #define DECLARE_IGC_REGKEY(dataType, regkeyName, defaultValue, description, releaseMode)         \
 {                                                                                   \
-    declareIGCKey(line, #dataType, #regkeyName, hashes, &(g_RegKeyList.regkeyName));\
+    declareIGCKey(line, #dataType, #regkeyName, hashes, entry_points, &(g_RegKeyList.regkeyName));\
 }
 #include "igc_regkeys.h"
 #undef DECLARE_IGC_REGKEY
@@ -951,15 +1042,33 @@ void appendToOptionsLogFile(std::string const &message)
 }
 
 static thread_local ShaderHash g_CurrentShaderHash;
+static thread_local std::vector<std::string> g_CurrentEntryPointNames;
 void SetCurrentDebugHash(const ShaderHash& hash)
 {
     g_CurrentShaderHash = hash;
 }
 
+// ray tracing shader can have multiple access point
+void SetCurrentEntryPoints(const std::vector<std::string>& entry_points)
+{
+    g_CurrentEntryPointNames = entry_points;
+}
+
+void ClearCurrentEntryPoints()
+{
+    g_CurrentEntryPointNames.clear();
+}
+
 bool CheckHashRange(SRegKeyVariableMetaData& varname)
 {
-    if (varname.hashes.empty())
-        return true;
+    if (varname.hashes.empty()) {
+        if (varname.entry_points.empty()) {
+            return true;
+        }
+        else {
+            return false;
+        }
+    }
     if (!g_CurrentShaderHash.is_set())
     {
         std::string msg = "Warning: hash not calculated yet; IGC_GET_FLAG_VALUE(" + std::string(varname.GetName()) + ") returned default value";
@@ -986,10 +1095,71 @@ bool CheckHashRange(SRegKeyVariableMetaData& varname)
     return false;
 }
 
+bool CheckEntryPoint(SRegKeyVariableMetaData& varname)
+{
+    //return false;
+    if (varname.entry_points.empty())
+    {
+        if (varname.hashes.empty())
+        {
+            return true;
+        }
+        else
+        {
+            return false;
+        }
+    }
+
+    if (g_CurrentEntryPointNames.size() == 0)
+    {
+        std::string msg = "Warning: entry point not set yet; IGC_GET_FLAG_VALUE(" + std::string(varname.GetName()) + ") returned default value";
+        appendToOptionsLogFile(msg);
+    }
+    //looping entry point recorded by regkey metadata
+    for (auto& it : varname.entry_points)
+    {
+        //looping each entry point of current shader
+        for (std::string& CurrEntryPoint : g_CurrentEntryPointNames)
+        {
+            if (CurrEntryPoint == it.entry_point_name)
+            {
+                varname.m_Value = it.m_Value;
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+/*****************************************************************************\
+
+Function:
+    GetRegistryKeyValue
+
+Description:
+    Reads a registry variable from the registry
+
+Input:
+    key
+    buffer
+    bufferSize
+
+Output:
+    read status
+
+\*****************************************************************************/
+extern "C" bool IGC_DEBUG_API_CALL GetRegistryKeyValue(const char* key, void* buffer, uint32_t bufferSize)
+{
+    bool isSet = ReadIGCRegistry(
+        key,
+        buffer,
+        bufferSize);
+    return isSet;
+}
+
 static void LoadFromRegKeyOrEnvVarOrOptions(
     const std::string& options = "",
-    bool* RegFlagNameError = nullptr,
-    const std::string& registrykeypath = IGC_REGISTRY_KEY)
+    bool* RegFlagNameError = nullptr)
 {
     SRegKeyVariableMetaData* pRegKeyVariable = (SRegKeyVariableMetaData*)&g_RegKeyList;
     constexpr unsigned NUM_REGKEY_ENTRIES =
@@ -1001,10 +1171,10 @@ static void LoadFromRegKeyOrEnvVarOrOptions(
         std::string nameWithEqual = name;
         nameWithEqual = nameWithEqual + "=";
 
-        bool isSet = ReadIGCRegistry(
+        bool isSet = GetRegistryKeyValue(
             name,
             &value,
-            sizeof(value), registrykeypath);
+            sizeof(value));
 
         if (isSet)
         {
@@ -1094,18 +1264,6 @@ void LoadRegistryKeys(const std::string& options, bool *RegFlagNameError)
     if(!flagsSet)
     {
         flagsSet = true;
-        // dump out IGC.xml for the registry manager
-#if defined(_WIN64) || defined(_WIN32)
-        std::vector<DEVINST> drivers;
-        std::list<std::string> registrypaths;
-        GetIntelDriverPaths(drivers);
-        for (auto driverInfo : drivers)
-        {
-            std::string driverStoreRegKeyPath = getNewRegistryPath(driverInfo);
-            std::string registryKeyPath = "SYSTEM\\ControlSet001\\Control\\Class\\" + driverStoreRegKeyPath + "\\IGC";
-            LoadFromRegKeyOrEnvVarOrOptions(options, RegFlagNameError, registryKeyPath);
-        }
-#endif
         LoadFromRegKeyOrEnvVarOrOptions(options, RegFlagNameError);
 #if !defined(_DEBUG)
         if (IGC_IS_FLAG_ENABLED(EnableDebugging))
