@@ -8,6 +8,7 @@
 
 # -*- Python -*-
 
+import subprocess
 import lit.formats
 import lit.util
 
@@ -39,13 +40,39 @@ llvm_config.with_environment('LD_LIBRARY_PATH', [config.ocloc_lib_dir,
                                                  config.cclang_lib_dir], append_path=True)
 
 
-tool_dirs = [config.ocloc_dir, config.llvm_tools_dir, config.spirv_as_dir]
+tool_dirs = [config.ocloc_dir, config.llvm_tools_dir, config.spirv_as_dir, config.llvm_spirv_dir]
 
-if llvm_config.add_tool_substitutions([ToolSubst('ocloc', unresolved='break')], tool_dirs) is False:
+if llvm_config.add_tool_substitutions([ToolSubst('ocloc', unresolved='break')], tool_dirs) :
+  ocloc_path = llvm_config.config.substitutions[-1][1]
+else :
   lit_config.note('Did not find ocloc in %s, ocloc will be used from system paths' % tool_dirs)
+  ocloc_path = 'ocloc'
 
 llvm_config.add_tool_substitutions([ToolSubst('llvm-dwarfdump')], tool_dirs)
-llvm_config.add_tool_substitutions([ToolSubst('opt')], tool_dirs)
+llvm_config.add_tool_substitutions([ToolSubst('llvm-as')], tool_dirs)
+llvm_config.add_tool_substitutions([ToolSubst('not')], tool_dirs)
+
+
+def get_available_devices(tool_path, ld_path):
+    set_of_devices = set()
+    # ocloc executable is expected to be present for these tests
+    ocloc_cmd = subprocess.Popen(
+        [tool_path, 'compile', '--help'], stdout=subprocess.PIPE, env={'LD_LIBRARY_PATH': ld_path})
+    ocloc_out = ocloc_cmd.stdout.read().decode()
+    ocloc_cmd.wait()
+    for line in ocloc_out.split('\n') :
+        if '<device_type> can be:' in line :
+            fields = line.strip().split(':')
+            for dev in fields[1].split(','):
+               acronym = dev.strip()
+               if acronym and not 'ip version' in acronym :
+                  set_of_devices.add(acronym + '-supported')
+    return set_of_devices
+
+# Get supported acronyms for ocloc
+devices = get_available_devices(ocloc_path, config.environment['LD_LIBRARY_PATH'])
+lit_config.note('Supported device acronyms(to be used with REQUIRES:): \n%s' % ' '.join(sorted(devices)))
+config.available_features.update(devices)
 
 if not config.regkeys_disabled:
   config.available_features.add('regkeys')
@@ -54,9 +81,16 @@ if config.spirv_as_enabled:
   config.available_features.add('spirv-as')
   llvm_config.add_tool_substitutions([ToolSubst('spirv-as', unresolved='fatal')], tool_dirs)
 
+if config.llvm_spirv_enabled:
+  config.available_features.add('llvm-spirv')
+  llvm_config.add_tool_substitutions([ToolSubst('llvm-spirv', unresolved='fatal')], tool_dirs)
+
 if config.is32b == "1":
   config.available_features.add('sys32')
 
+if config.debug_build:
+  config.available_features.add('debug')
+
 if config.use_khronos_spirv_translator_in_sc == "1":
   config.available_features.add('khronos-translator')
   config.available_features.add('khronos-translator-' + config.llvm_version_major)
