@@ -63,6 +63,8 @@ public:
     BBList.clear();
     callees.clear();
   }
+  FuncInfo(const FuncInfo&) = delete;
+  FuncInfo& operator=(const FuncInfo&) = delete;
 
   void clear() {
     BBSet.clear();
@@ -88,6 +90,12 @@ public:
   void addCallee(FuncInfo *fn) { callees.push_back(fn); }
   std::list<FuncInfo *> &getCallees() { return callees; }
 
+  // const version that does O(n) lookup.
+  bool contains(G4_BB *bb) const {
+    auto it = std::find(BBList.begin(), BBList.end(), bb);
+    return it != BBList.end();
+  }
+
   bool contains(G4_BB *bb) {
     if (BBSet.size() != BBList.size()) {
       BBSet.clear();
@@ -120,7 +128,7 @@ public:
   unsigned getPostID() const { return postID; }
   void setPostID(unsigned id) { postID = id; }
 
-  void dump() const;
+  void dump(std::ostream &os = std::cerr) const;
 }; // FuncInfo
 
 //
@@ -350,7 +358,7 @@ public:
     return dcl == pseudoVCEDcl;
   }
   bool isPseudoVCADcl(const G4_Declare *dcl) const {
-    for (auto iter : fcallToPseudoDclMap) {
+    for (const auto &iter : fcallToPseudoDclMap) {
       if (iter.second.VCA == dcl) {
         return true;
       }
@@ -358,7 +366,7 @@ public:
     return false;
   }
   bool isPseudoA0Dcl(const G4_Declare *dcl) const {
-    for (auto iter : fcallToPseudoDclMap) {
+    for (const auto &iter : fcallToPseudoDclMap) {
       if (iter.second.A0 == dcl) {
         return true;
       }
@@ -366,7 +374,7 @@ public:
     return false;
   }
   bool isPseudoFlagDcl(const G4_Declare *dcl) const {
-    for (auto iter : fcallToPseudoDclMap) {
+    for (const auto &iter : fcallToPseudoDclMap) {
       if (iter.second.Flag == dcl) {
         return true;
       }
@@ -380,7 +388,7 @@ public:
     if (isPseudoVCEDcl(dcl)) {
       return true;
     }
-    for (auto iter : fcallToPseudoDclMap) {
+    for (const auto &iter : fcallToPseudoDclMap) {
       if (iter.second.A0 == dcl || iter.second.Flag == dcl ||
           iter.second.VCA == dcl) {
         return true;
@@ -423,8 +431,6 @@ public:
   void preprocess(INST_LIST &instlist);
 
   FlowGraph() = delete;
-  FlowGraph(const FlowGraph &) = delete;
-  FlowGraph &operator=(const FlowGraph &) = delete;
 
   FlowGraph(INST_LIST_NODE_ALLOCATOR &alloc, G4_Kernel *kernel, Mem_Manager &m)
       : traversalNum(0), numBBId(0), reducible(true), doIPA(false),
@@ -434,6 +440,8 @@ public:
         pseudoVCEDcl(NULL), immDom(*kernel), pDom(*kernel), loops(*kernel) {}
 
   ~FlowGraph();
+  FlowGraph(const FlowGraph&) = delete;
+  FlowGraph& operator=(const FlowGraph&) = delete;
 
   void setBuilder(IR_Builder *pBuilder) { builder = pBuilder; }
 
