@@ -193,6 +193,8 @@ public:
         parent(fg), instList(alloc) {}
 
   ~G4_BB() { instList.clear(); }
+  G4_BB(const G4_BB&) = delete;
+  G4_BB& operator=(const G4_BB&) = delete;
 
   void *operator new(size_t sz, Mem_Manager &m) { return m.alloc(sz); }
 
@@ -248,7 +250,7 @@ public:
   void emitInstruction(std::ostream &output, INST_LIST_ITER &it);
   void emitBasicInstruction(std::ostream &output, INST_LIST_ITER &it);
   void emitBasicInstructionComment(std::ostream &output, INST_LIST_ITER &it,
-                                   int *suppressRegs, int *lastRegs);
+                                   int *suppressRegs, int *lastRegs, int32_t pc);
   void emitInstructionSourceLineMapping(std::ostream &output,
                                         INST_LIST_ITER &it);
   void emitBankConflict(std::ostream &output, const G4_INST *inst);
