@@ -81,6 +81,19 @@ typedef std::list<USE_DEF_NODE, USE_DEF_ALLOCATOR> DEF_EDGE_LIST;
 typedef std::list<USE_DEF_NODE, USE_DEF_ALLOCATOR>::iterator DEF_EDGE_LIST_ITER;
 
 namespace vISA {
+#define SWSB_MAX_ALU_DEPENDENCE_DISTANCE 11
+#define SWSB_MAX_ALU_DEPENDENCE_DISTANCE_64BIT 15
+
+enum SB_INST_PIPE {
+  PIPE_NONE = 0,
+  PIPE_INT = 1,
+  PIPE_FLOAT = 2,
+  PIPE_LONG = 3,
+  PIPE_MATH = 4,
+  PIPE_DPAS = 6,
+  PIPE_SEND = 7,
+  PIPE_ALL = 8
+};
 // forward declaration for the binary of an instruction
 class BinInst;
 
@@ -114,7 +127,13 @@ class G4_INST {
 
 protected:
   G4_opcode op;
-  llvm::SmallVector<G4_Operand *, G4_MAX_SRCS> srcs;
+  // Reserve 4 sources should be enough for most of instructions.
+  // TODO: In fact, we would reserve not just 4 but also set the size of "srcs"
+  // to 4. Currently there is some legacy code that presumes srcs[0:3] are
+  // always accessible for updates, and refactoring the code appears to be
+  // a challenge. We probably should still pursue the opportunity to remove that
+  // presumption and make the code cleaner.
+  llvm::SmallVector<G4_Operand *, /*N=*/4> srcs;
   G4_DstRegRegion *dst;
   G4_Predicate *predicate;
   G4_CondMod *mod;
@@ -274,6 +293,11 @@ public:
           G4_Sat s, G4_ExecSize size, G4_DstRegRegion *d, G4_Operand *s0,
           G4_Operand *s1, G4_Operand *s2, G4_Operand *s3, G4_InstOpts opt);
 
+  G4_INST(const IR_Builder &irb, G4_Predicate *prd, G4_opcode o, G4_CondMod *m,
+          G4_Sat s, G4_ExecSize size, G4_DstRegRegion *d, G4_Operand *s0,
+          G4_Operand *s1, G4_Operand *s2, G4_Operand *s3, G4_Operand *s4,
+          G4_InstOpts opt);
+
   G4_INST(const IR_Builder &irb, G4_Predicate *prd, G4_opcode o, G4_CondMod *m,
           G4_Sat s, G4_ExecSize size, G4_DstRegRegion *d, G4_Operand *s0,
           G4_Operand *s1, G4_Operand *s2, G4_Operand *s3, G4_Operand *s4,
@@ -392,7 +416,6 @@ public:
     return (G4_SendDescRaw *)msgDesc;
   }
 
-  virtual bool mayExceedTwoGRF() const { return false; }
   // special instructions(e.g., send) should override
   virtual void computeRightBound(G4_Operand *opnd);
 
@@ -644,6 +667,8 @@ public:
       return Opnd_src2;
     case 3:
       return Opnd_src3;
+    case 4:
+      return Opnd_src4;
     default:
       vISA_ASSERT_UNREACHABLE("bad source id");
       return Opnd_src0;
@@ -700,6 +725,10 @@ public:
   bool isIntegerPipeInstructionXe() const;
   bool isFloatPipeInstructionXe() const;
 
+  int getMaxDepDistance() const;
+  SB_INST_PIPE getInstructionPipeXe() const;
+  SB_INST_PIPE getDistDepPipeXe() const;
+
   void swapDefUse(Gen4_Operand_Number srcIxA = Opnd_src0,
                   Gen4_Operand_Number srcIxB = Opnd_src1);
   void addDefUse(G4_INST *use, Gen4_Operand_Number usePos);
@@ -923,6 +952,25 @@ public:
 
   const IR_Builder &getBuilder() const { return builder; }
 
+  bool isPureBFInst() const {
+    G4_Operand *dst = getDst();
+    if (!dst || dst->getType() != Type_BF)
+      return false;
+    for (int i = 0, numSrcs = getNumSrc(); i < numSrcs; i++) {
+      auto src = getSrc(i);
+      if (src && src->getType() != Type_BF)
+        return false;
+    }
+    return true;
+  }
+
+  bool canSupportPureBF() const {
+    return (op == G4_mov || op == G4_add || op == G4_sel || op == G4_cmp ||
+           op == G4_csel || op == G4_cmpn || op == G4_mul || op == G4_mad ||
+           op == G4_math);
+  }
+
+  virtual bool requireNopAfter() const { return false; }
 private:
   // use inheritDIFrom() instead
   void setLocation(MDLocation *loc) { setMetadata(Metadata::InstLoc, loc); }
@@ -972,12 +1020,25 @@ public:
     return GenPrecisionTable[(int)P].BitSize;
   }
 
+  static bool hasSamePrecision(GenPrecision p1, GenPrecision p2) {
+    if (p1 == p2) {
+      return true;
+    }
+
+    return (((p1 == GenPrecision::U8 || p1 == GenPrecision::S8) &&
+             (p2 == GenPrecision::U8 || p2 == GenPrecision::S8)) ||
+            ((p1 == GenPrecision::U4 || p1 == GenPrecision::S4 ||
+              p1 == GenPrecision::U2 || p1 == GenPrecision::S2) &&
+             (p2 == GenPrecision::U4 || p2 == GenPrecision::S4 ||
+              p2 == GenPrecision::U2 || p2 == GenPrecision::S2)));
+  }
+
   G4_InstDpas(const IR_Builder &builder, G4_opcode o, G4_ExecSize size,
               G4_DstRegRegion *d, G4_Operand *s0, G4_Operand *s1,
-              G4_Operand *s2, G4_Operand *s3, G4_InstOpts opt, GenPrecision a,
-              GenPrecision w, uint8_t sd, uint8_t rc)
+              G4_Operand *s2, G4_Operand *s3, G4_Operand *s4, G4_InstOpts opt,
+              GenPrecision a, GenPrecision w, uint8_t sd, uint8_t rc)
       : G4_INST(builder, nullptr, o, nullptr, g4::NOSAT, size, d, s0, s1, s2,
-                s3, opt),
+                s3, s4, opt),
         Src2Precision(a), Src1Precision(w), SystolicDepth(sd), RepeatCount(rc) {
   }
 
@@ -988,6 +1049,7 @@ public:
   bool isFP16() const { return Src1Precision == GenPrecision::FP16; }
   bool isTF32() const { return Src1Precision == GenPrecision::TF32; }
   bool isInt() const;
+  bool isInt8() const;
   bool is2xInt8() const; // true if it is 2xint8 dpas
 
   uint8_t getOpsPerChan() const;
@@ -996,6 +1058,14 @@ public:
   GenPrecision getSrc1Precision() const { return Src1Precision; }
   GenPrecision getSrc2Precision() const { return Src2Precision; }
 
+  bool hasSameSrc1Precision(GenPrecision p) const {
+    return hasSamePrecision(Src1Precision, p);
+  }
+
+  bool hasSameSrc2Precision(GenPrecision p) const {
+    return hasSamePrecision(Src2Precision, p);
+  }
+
   void setRepeatCount(uint8_t rc) { RepeatCount = rc; }
   // data size per lane (data size per each systolic depth)
   uint32_t getPrecisionSizePerLaneInByte(GenPrecision P) const {
@@ -1009,7 +1079,6 @@ public:
     return getPrecisionSizePerLaneInByte(Src2Precision);
   }
 
-  bool mayExceedTwoGRF() const override { return true; }
   void computeRightBound(G4_Operand *opnd) override;
   void setMayNeedWA(bool b) { mayNeedRSWA = b; }
   bool mayNeedWA() const { return mayNeedRSWA; }
@@ -1032,7 +1101,7 @@ typedef enum {
   MATH_INT_DIV_QUOT = 0xC,
   MATH_INT_DIV_REM = 0xD,
   MATH_INVM = 0xE,
-  MATH_RSQRTM = 0xF
+  MATH_RSQRTM = 0xF,
 } G4_MathOp;
 
 class G4_InstMath : public G4_INST {
@@ -1056,10 +1125,10 @@ public:
   }
   bool isOneSrcMath() const {
     return mathOp == MATH_INV || mathOp == MATH_LOG || mathOp == MATH_EXP ||
-           mathOp == MATH_SQRT || mathOp == MATH_RSQ || mathOp == MATH_SIN ||
-           mathOp == MATH_COS || mathOp == MATH_RSQRTM;
+      mathOp == MATH_SQRT || mathOp == MATH_RSQ || mathOp == MATH_SIN ||
+      mathOp == MATH_COS || mathOp == MATH_RSQRTM
+      ;
   }
-
   G4_MathOp getMathCtrl() const { return mathOp; }
 };
 
@@ -1189,6 +1258,8 @@ public:
     vASSERT(isReturn());
     setOpcode(G4_pseudo_fret);
   }
+
+  bool requireNopAfter() const override;
 }; // G4_InstCF
 
 class G4_InstSend : public G4_INST {
@@ -1223,7 +1294,6 @@ public:
       op = G4_sendsc;
     }
   }
-  bool mayExceedTwoGRF() const override { return true; }
 
   G4_Operand *getMsgDescOperand() const {
     return isSplitSend() ? srcs[2] : srcs[1];
@@ -1257,7 +1327,8 @@ public:
   bool canBeEOT() const {
     bool canEOT = getMsgDesc()->getDstLenRegs() == 0 &&
                   (getMsgDesc()->getSFID() != SFID::NULL_SFID &&
-                   getMsgDesc()->getSFID() != SFID::SAMPLER);
+                   getMsgDesc()->getSFID() != SFID::SAMPLER &&
+                  !getMsgDesc()->isFence());
 
     return canEOT;
   }
