@@ -210,9 +210,9 @@ void *gtPinData::getIndirRefs(unsigned int &size) {
   unsigned int offset = 0;
   write<uint32_t>(buffer, numRanges, offset);
   for (auto &item : indirRefMap) {
-    vISA_ASSERT(offset < size, "Out of bounds");
-    write<uint32_t>(buffer, item.first, offset);
     for (const auto &arg : item.second) {
+      vISA_ASSERT(offset < size, "Out of bounds");
+      write<uint32_t>(buffer, item.first, offset);
       vISA_ASSERT(offset < size, "Out of bounds");
       write<uint16_t>(buffer, arg.first, offset);
       vISA_ASSERT(offset < size, "Out of bounds");
@@ -235,7 +235,8 @@ static void writeBuffer(std::vector<unsigned char> &buffer,
   bufferSize += numBytes;
 }
 
-void *gtPinData::getGTPinInfoBuffer(unsigned &bufferSize) {
+void *gtPinData::getGTPinInfoBuffer(unsigned &bufferSize,
+                                    unsigned int scratchOffset) {
   if (!gtpin_init && !gtpinInitFromL0) {
     bufferSize = 0;
     return nullptr;
@@ -370,7 +371,8 @@ void *gtPinData::getGTPinInfoBuffer(unsigned &bufferSize) {
   if (t.scratch_area_size) {
     gtpin::igc::igc_token_scratch_area_info_t scratchSlotData;
     scratchSlotData.scratch_area_size = t.scratch_area_size;
-    scratchSlotData.scratch_area_offset = nextScratchFree;
+    vISA_ASSERT(scratchOffset >= nextScratchFree, "scratch offset mismatch");
+    scratchSlotData.scratch_area_offset = scratchOffset;
 
     // gtpin scratch slots are beyond spill memory
     scratchSlotData.token = gtpin::igc::GTPIN_IGC_TOKEN_SCRATCH_AREA_INFO;
@@ -426,7 +428,7 @@ void *gtPinData::getGTPinInfoBuffer(unsigned &bufferSize) {
 
       std::ofstream ofInfo;
       std::stringstream ssInfo;
-      ssInfo << std::string(asmName) << ".gtpin_igc_info";
+      ssInfo << asmName << ".gtpin_igc_info";
       ofInfo.open(ssInfo.str(), std::ofstream::binary);
       if (gtpinBuffer) {
         ofInfo.write((const char *)gtpinBuffer, bufferSize);
@@ -461,7 +463,7 @@ G4_Kernel::G4_Kernel(const PlatformInfo &pInfo, INST_LIST_NODE_ALLOCATOR &alloc,
     : platformInfo(pInfo), m_options(options), m_kernelAttrs(anAttr),
       m_function_id(funcId), RAType(RA_Type::UNKNOWN_RA), asmInstCount(0),
       kernelID(0), fg(alloc, this, m), major_version(major),
-      minor_version(minor), grfMode(pInfo.platform, options), stackCall(this) {
+      minor_version(minor), grfMode(pInfo.platform, options) {
   vISA_ASSERT(major < COMMON_ISA_MAJOR_VER || (major == COMMON_ISA_MAJOR_VER &&
                                                minor <= COMMON_ISA_MINOR_VER),
               "CISA version not supported by this JIT-compiler");
@@ -629,19 +631,6 @@ bool G4_Kernel::updateKernelToLargerGRF() {
   return true;
 }
 
-//
-// Updates kernel's related structures to smaller GRF
-//
-bool G4_Kernel::updateKernelToSmallerGRF() {
-  if (numRegTotal == grfMode.getMinGRF())
-    return false;
-
-  // Scale number of GRFs, Acc, SWSB tokens.
-  setKernelParameters(grfMode.moveToSmallerGRF());
-  fg.builder->rebuildPhyRegPool(getNumRegTotal());
-  return true;
-}
-
 //
 // Updates kernel's related structures based on register pressure
 //
@@ -671,12 +660,12 @@ void G4_Kernel::updateKernelByRegPressure(unsigned regPressure) {
 bool G4_Kernel::updateKernelFromNumGRFAttr() {
   unsigned attrNumGRF =
       m_kernelAttrs->getInt32KernelAttr(Attributes::ATTR_NumGRF);
-  if (!grfMode.isValidNumGRFs(attrNumGRF))
+  if (attrNumGRF != 0 && !grfMode.isValidNumGRFs(attrNumGRF))
     return false;
   if (numRegTotal == attrNumGRF)
     return true;
 
-  autoGRFSelection = false;
+  autoGRFSelection = (attrNumGRF == 0);
   // Scale number of GRFs, Acc, SWSB tokens.
   setKernelParameters(attrNumGRF);
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
@@ -756,12 +745,16 @@ static iga_gen_t getIGAPlatform(TARGET_PLATFORM genPlatform) {
     break;
   case Xe_DG2:
   case Xe_MTL:
+  case Xe_ARL:
     platform = IGA_XE_HPG;
     break;
   case Xe_PVC:
   case Xe_PVCXT:
     platform = IGA_XE_HPC;
     break;
+  case Xe2:
+    platform = IGA_XE2;
+    break;
   default:
     break;
   }
@@ -778,16 +771,13 @@ KernelDebugInfo* G4_Kernel::getKernelDebugInfo() {
 }
 
 void StackCallABI::setVersion() {
-  uint32_t ABIVersion = kernel->getuInt32Option(vISA_StackCallABIVer);
-  if (ABIVersion == 1)
-    version = StackCallABIVersion::VER_1;
-  else if (ABIVersion == 2)
-    version = StackCallABIVersion::VER_2;
-  else if (ABIVersion == 3)
-    version = StackCallABIVersion::VER_3;
+  // VISA ABI version 1 is deprecated so default version to use is version 2
+  version = StackCallABIVersion::VER_2;
 }
 
-StackCallABI::StackCallABI(G4_Kernel *k) : kernel(k) {
+void StackCallABI::init(G4_Kernel *k) {
+  vISA_ASSERT(!kernel, "init called multiple times");
+  kernel = k;
   setVersion();
   if (version == StackCallABIVersion::VER_3) {
     vISA_ASSERT(kernel->getGRFSize() == 64, "require 64-byte GRF for ABI v3");
@@ -991,7 +981,7 @@ void G4_Kernel::setKernelParameters(unsigned newGRF) {
     grfMode.setModeByNumThreads(overrideNumThreads);
     overrideGRFNum = 0;
     autoGRFSelection = false;
-  } else if (overrideGRFNum != grfMode.getDefaultGRF()) {
+  } else if (overrideGRFNum > 0) {
     // Forcing a specific number of GRFs
     grfMode.setModeByNumGRFs(overrideGRFNum);
     autoGRFSelection = false;
@@ -1697,26 +1687,35 @@ static BlockOffsets precomputeBlockOffsets(std::ostream &os, G4_Kernel &g4k,
         lastInstSize = kv.getInstSize(currPc);
 
         G4_INST *inst = (*itInst);
-        if (g4k.fg.builder->hasDPASFuseRSWA() && inst->isCachelineAligned()) {
-          iga::Op opcode = kv.getOpcode(currPc);
-          iga::SWSB sw =
-              kv.getSWSBInfo(currPc, iga::SWSB_ENCODE_MODE::ThreeDistPipe);
 
-          while (opcode == iga::Op::SYNC && !sw.hasDist() && !sw.hasSWSB() &&
-                 !sw.hasSpecialToken() && !sw.hasToken()) {
-            // For HW WA.
-            // In which, the sync instruction is used to make instruction
-            // aligned.  However, due to compaction, we don't know the location
-            // of the instruction, the sync instruction insertion has to happen
-            // during encoding, which is unknow for the instruction size of
-            // kernel in the decoding. That's the issue we have to make
-            // these changes.
+        // For HW WA.
+        // In which, vISA may ask IGA to emit some additional instructions.
+        // For example, sync is used to make instruction aligned, and nop is
+        // used to support stepping in debugger.
+        // However, due to compaction, we might not know the exact location of
+        // the instruction, the sync instruction insertion has to happen during
+        // encoding, which is unknown for the instruction size of kernel in the
+        // decoding. That's the issue we have to make these changes.
+        if (inst->isCachelineAligned()) {
+          iga::Op opcode = kv.getOpcode(currPc);
+          // There could be multiple sync.nop instructions emitted by IGA to
+          // make the instruction aligned. Here we continue to advance PC when
+          // seeing sync.nop so that vISA inst and IGA inst could match again.
+          while (opcode == iga::Op::SYNC) {
             currPc += lastInstSize;
             opcode = kv.getOpcode(currPc);
-            sw = kv.getSWSBInfo(currPc, iga::SWSB_ENCODE_MODE::ThreeDistPipe);
             lastInstSize = kv.getInstSize(currPc);
           }
         }
+
+        // When the inst requires an additional nop after it, again we need to
+        // advance PC to consume NOP to make vISA inst and IGA inst match later.
+        if (inst->requireNopAfter()) {
+          currPc += lastInstSize;
+          lastInstSize = kv.getInstSize(currPc);
+          vASSERT(kv.getOpcode(currPc) == iga::Op::NOP);
+        }
+
         currPc += lastInstSize;
       }
     }
@@ -1837,7 +1836,7 @@ void G4_Kernel::emitDeviceAsmInstructionsIga(std::ostream &os,
   // tryPrintLable - check if the given label is already printed with the given
   // pc. Print it if not, and skip it if yes.
   auto tryPrintLabel = [&os, &printedLabels](int32_t label_pc,
-                                             std::string label_name) {
+                                             const std::string& label_name) {
     auto label_pair = std::make_pair(label_pc, label_name);
     // skip if the same label in the set
     if (printedLabels.find(label_pair) != printedLabels.end())
@@ -1935,22 +1934,28 @@ void G4_Kernel::emitDeviceAsmInstructionsIga(std::ostream &os,
         }
       };
 
-      if (fg.builder->hasDPASFuseRSWA() && i->isCachelineAligned()) {
+      // Advance PC when the vISA instruction needs to be cacheline-aligned or
+      // requires a Nop after. See comments in precomputeBlockOffsets for
+      // details.
+      if (i->isCachelineAligned()) {
         iga::Op opcode = kv.getOpcode(pc);
-        iga::SWSB sw = kv.getSWSBInfo(pc, iga::SWSB_ENCODE_MODE::ThreeDistPipe);
-        while (opcode == iga::Op::SYNC && !sw.hasDist() && !sw.hasSWSB() &&
-               !sw.hasSpecialToken() && !sw.hasToken()) {
-          // What's going on here???  Someone please give us a clue.
+        while (opcode == iga::Op::SYNC) {
           formatToInstToStream(pc, os);
           os << "\n";
           pc += kv.getInstSize(pc);
           opcode = kv.getOpcode(pc);
-          sw = kv.getSWSBInfo(pc, iga::SWSB_ENCODE_MODE::ThreeDistPipe);
         }
       }
+      if (i->requireNopAfter()) {
+        formatToInstToStream(pc, os);
+        os << "\n";
+        pc += kv.getInstSize(pc);
+        vASSERT(kv.getOpcode(pc) == iga::Op::NOP);
+      }
+
       formatToInstToStream(pc, os);
 
-      (*itBB)->emitBasicInstructionComment(os, itInst, suppressRegs, lastRegs);
+      (*itBB)->emitBasicInstructionComment(os, itInst, suppressRegs, lastRegs, pc);
       os << "\n";
 
       pc += kv.getInstSize(pc);
@@ -2060,6 +2065,7 @@ GRFMode::GRFMode(const TARGET_PLATFORM platform, Options *op) : options(op) {
   case Xe_XeHPSDV:
   case Xe_DG2:
   case Xe_MTL:
+  case Xe_ARL:
     configs.resize(2);
     // Configurations with <numGRF, numThreads, SWSBTokens, numAcc>
     configs[0] = Config(128, 8, 16, 4);
@@ -2068,6 +2074,7 @@ GRFMode::GRFMode(const TARGET_PLATFORM platform, Options *op) : options(op) {
     break;
   case Xe_PVC:
   case Xe_PVCXT:
+  case Xe2:
     configs.resize(2);
     // Configurations with <numGRF, numThreads, SWSBTokens, numAcc>
     configs[0] = Config(128, 8, 16, 4);
@@ -2082,21 +2089,36 @@ GRFMode::GRFMode(const TARGET_PLATFORM platform, Options *op) : options(op) {
     defaultMode = 0;
   }
   currentMode = defaultMode;
+
+  // Set lower bound GRF
+  unsigned minGRF = op->getuInt32Option(vISA_MinGRFNum);
+  lowerBoundGRF = minGRF > 0 ? minGRF : configs.front().numGRF;
+  vISA_ASSERT(isValidNumGRFs(lowerBoundGRF), "Invalid lower bound for GRF number");
+
+  // Set upper bound GRF
+  unsigned maxGRF = op->getuInt32Option(vISA_MaxGRFNum);
+  upperBoundGRF = maxGRF > 0 ? maxGRF : configs.back().numGRF;
+  vISA_ASSERT(isValidNumGRFs(upperBoundGRF), "Invalid upper bound for GRF number");
 }
 
-unsigned GRFMode::setModeByRegPressure(unsigned maxRP, unsigned largestInputReg) {
+unsigned GRFMode::setModeByRegPressure(unsigned maxRP,
+                                       unsigned largestInputReg) {
   unsigned size = configs.size(), i = 0;
   // find appropiate GRF based on reg pressure
   for (; i < size; i++) {
-    if (configs[i].VRTEnable && maxRP <= configs[i].numGRF &&
-        // Check that we've at least 8 GRFs over and above
-        // those blocked for kernel input. This helps cases
-        // where an 8 GRF variable shows up in entry BB.
-        (largestInputReg + 8) <= configs[i].numGRF) {
+    if (configs[i].VRTEnable && configs[i].numGRF >= lowerBoundGRF &&
+        configs[i].numGRF <= upperBoundGRF) {
       currentMode = i;
-      break;
+      if (maxRP <= configs[i].numGRF &&
+          // Check that we've at least 8 GRFs over and above
+          // those blocked for kernel input. This helps cases
+          // where an 8 GRF variable shows up in entry BB.
+          (largestInputReg + 8) <= configs[i].numGRF)
+        return configs[currentMode].numGRF;
     }
   }
+  // RP is greater than the maximum GRF available, so set the largest GRF
+  // available
   return configs[currentMode].numGRF;
 }
 
