@@ -83,7 +83,9 @@ public:
   gtpin::igc::igc_init_t *getGTPinInit() { return gtpin_init; }
 
   // return igc_info_t format buffer. caller casts it to igc_info_t.
-  void *getGTPinInfoBuffer(unsigned &bufferSize);
+  // scratchOffset argument is passed in as IGC knows how much scratch space
+  // is required for kernels, especially when stack calls are used.
+  void *getGTPinInfoBuffer(unsigned &bufferSize, unsigned int scratchOffset);
 
   void setScratchNextFree(unsigned next);
   unsigned int getScratchNextFree() const;
@@ -178,8 +180,10 @@ public:
   }
 
   unsigned getMaxGRF() const {
-    auto found = std::find_if(configs.rbegin(), configs.rend(),
-                              [](const Config &c) { return c.VRTEnable; });
+    auto found =
+        std::find_if(configs.rbegin(), configs.rend(), [this](const Config &c) {
+          return c.VRTEnable && c.numGRF <= upperBoundGRF;
+        });
     return found->numGRF;
   }
 
@@ -187,7 +191,7 @@ public:
   unsigned getLargerGRF() const {
     // find the first larger mode that's available for VRT
     for (auto i = currentMode + 1; i < configs.size(); ++i) {
-      if (configs[i].VRTEnable)
+      if (configs[i].VRTEnable && configs[i].numGRF <= upperBoundGRF)
         return configs[i].numGRF;
     }
     return configs[currentMode].numGRF;
@@ -205,18 +209,7 @@ public:
   // Move GRF mode to the larger GRF available and return the number
   unsigned moveToLargerGRF() {
     for (auto i = currentMode + 1; i < configs.size(); ++i) {
-      if (configs[i].VRTEnable) {
-        currentMode = i;
-        break;
-      }
-    }
-    return configs[currentMode].numGRF;
-  }
-
-  // Move GRF mode to the smaller GRF available and return the number
-  unsigned moveToSmallerGRF() {
-    for (auto i = currentMode - 1; i >= 0; --i) {
-      if (configs[i].VRTEnable) {
+      if (configs[i].VRTEnable && configs[i].numGRF <= upperBoundGRF) {
         currentMode = i;
         break;
       }
@@ -248,6 +241,8 @@ private:
   std::vector<Config> configs;
   unsigned defaultMode;
   unsigned currentMode;
+  unsigned lowerBoundGRF;
+  unsigned upperBoundGRF;
   Options *options;
 };
 
@@ -359,7 +354,10 @@ private:
   };
 
 public:
-  StackCallABIVersion getVersion() const { return version; }
+  StackCallABIVersion getVersion() const {
+    vISA_ASSERT(kernel != nullptr, "uninitialized class");
+    return version;
+  }
   void setVersion();
 
   SubRegs_Stackcall subRegs;
@@ -367,7 +365,7 @@ public:
   unsigned int argReg = std::numeric_limits<unsigned int>::max();
   unsigned int retReg = std::numeric_limits<unsigned int>::max();
 
-  StackCallABI(G4_Kernel *k);
+  void init(G4_Kernel *k);
 
 private:
   // Following constexprs describe layout of various subregs on entry to a
@@ -588,6 +586,9 @@ public:
             uint32_t funcId, unsigned char major, unsigned char minor);
   ~G4_Kernel();
 
+  G4_Kernel(const G4_Kernel &) = delete;
+  G4_Kernel& operator=(const G4_Kernel &) = delete;
+
   TARGET_PLATFORM getPlatform() const { return platformInfo.platform; }
   PlatformGen getPlatformGeneration() const { return platformInfo.family; }
   const char *getGenxPlatformString() const {
@@ -619,7 +620,10 @@ public:
 
   void *operator new(size_t sz, Mem_Manager &m) { return m.alloc(sz); }
 
-  void setBuilder(IR_Builder *pBuilder) { fg.setBuilder(pBuilder); }
+  void setBuilder(IR_Builder *pBuilder) {
+    fg.setBuilder(pBuilder);
+    stackCall.init(this);
+  }
 
   bool useAutoGRFSelection() const {
     // Register sharing not enabled in presence of stack calls
@@ -688,7 +692,6 @@ public:
   const char *getName() const { return name; }
 
   bool updateKernelToLargerGRF();
-  bool updateKernelToSmallerGRF();
   void updateKernelByRegPressure(unsigned regPressure);
   bool updateKernelFromNumGRFAttr();
 
