@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -258,6 +258,9 @@ void G4Verifier::verifyDstSrcOverlap(G4_INST *inst) {
 
     for (int i = 0; i < inst->getNumSrc(); i++) {
       G4_Operand *src = inst->getSrc(i);
+      if (inst->isDpas() && i != 1) {
+        continue;
+      }
       if (src != NULL && !src->isNullReg() && src->getTopDcl() &&
           (src->getTopDcl()->getRegFile() == G4_GRF ||
            src->getTopDcl()->getRegFile() == G4_INPUT)) {
@@ -287,7 +290,8 @@ void G4Verifier::verifySend(G4_INST *inst) {
     bool checkEoT = inst->isEOT() && kernel.fg.builder->hasEOTGRFBinding();
     if (checkEoT) {
       auto checkEOTSrc = [this](G4_SrcRegRegion *src) {
-        const unsigned int EOTStart = 112 * kernel.numEltPerGRF<Type_UB>();
+        // Send instruction's header and payload must be placed in last 16 GRFs if the EOT flag is set.
+        const unsigned int EOTStart = (kernel.getNumRegTotal()-16) * kernel.numEltPerGRF<Type_UB>();
         if (src->isNullReg()) {
           return true;
         }
@@ -296,10 +300,10 @@ void G4Verifier::verifySend(G4_INST *inst) {
 
       if (kernel.getNumRegTotal() >= 128) {
         vISA_ASSERT(checkEOTSrc(src0),
-                     "src0 for EOT send is not in r112-r127");
+                    "src0 for EOT send is not in last 16 GRFs");
         if (src1 != nullptr) {
           vISA_ASSERT(checkEOTSrc(src1),
-                       "src1 for EOT sends is not in r112-r127");
+                      "src1 for EOT sends is not in last 16 GRFs");
         }
       }
     }
@@ -403,27 +407,23 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
 
     if (opnd == inst->getDst()) {
       if (opnd->isRightBoundSet() && !opnd->isNullReg()) {
-        unsigned int correctRB = ((inst->getMsgDesc()->getDstLenRegs() +
-                                   opnd->asDstRegRegion()->getRegOff()) *
-                                  kernel.numEltPerGRF<Type_UB>()) -
-                                 1;
+        unsigned int correctRB = opnd->asDstRegRegion()->getRegOff() *
+                                     kernel.numEltPerGRF<Type_UB>();
         uint32_t dstLenBytes = inst->getMsgDesc()->getDstLenBytes();
         if (dstLenBytes < kernel.getGRFSize()) {
-          correctRB = opnd->getLeftBound() + dstLenBytes - 1;
-        } else if (opnd->getTopDcl()->getByteSize() <
-                   kernel.numEltPerGRF<Type_UB>()) {
-          correctRB =
-              opnd->getLeftBound() + opnd->getTopDcl()->getByteSize() - 1;
+          correctRB += (dstLenBytes - 1);
+        } else {
+          uint32_t correctDstLenBytes =
+              std::min(opnd->getTopDcl()->getByteSize(), dstLenBytes);
+          correctRB += (correctDstLenBytes - 1);
         }
 
         G4_Declare *parentDcl = opnd->getBase()->asRegVar()->getDeclare();
-        while (parentDcl != NULL) {
+        while (parentDcl) {
           correctRB += parentDcl->getAliasOffset();
           parentDcl = parentDcl->getAliasDeclare();
         }
 
-        correctRB = std::min(correctRB, opnd->getTopDcl()->getByteSize() - 1);
-
         if (opnd->getRightBound() != correctRB) {
           DEBUG_VERBOSE("Right bound mismatch for send inst dst. Orig rb = "
                         << opnd->getRightBound()
@@ -434,31 +434,28 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
           vISA_ASSERT(false, "Right bound mismatch!");
         }
       }
-    } else if (opnd == inst->getSrc(0) || opnd == inst->getSrc(1)) {
+    } else if (opnd == inst->getSrc(0) ||
+               (inst->isSplitSend() && opnd == inst->getSrc(1))) {
       if (opnd->isRightBoundSet()) {
-        int msgLength = (opnd == inst->getSrc(0))
-                            ? inst->getMsgDesc()->getSrc0LenRegs()
-                            : inst->getMsgDesc()->getSrc1LenRegs();
-        unsigned int numBytes = opnd->getTopDcl()->getByteSize();
-        unsigned int correctRB = 0;
-        if (numBytes < kernel.numEltPerGRF<Type_UB>()) {
-          correctRB = opnd->asSrcRegRegion()->getRegOff() *
-                          kernel.numEltPerGRF<Type_UB>() +
-                      numBytes - 1;
+        unsigned int correctRB = opnd->asSrcRegRegion()->getRegOff() *
+                                 kernel.numEltPerGRF<Type_UB>();
+        unsigned int srcBytes = (opnd == inst->getSrc(0))
+                                    ? inst->getMsgDesc()->getSrc0LenBytes()
+                                    : inst->getMsgDesc()->getSrc1LenBytes();
+        if (srcBytes < kernel.numEltPerGRF<Type_UB>()) {
+          correctRB += (srcBytes - 1);
         } else {
-          correctRB = ((msgLength + opnd->asSrcRegRegion()->getRegOff()) *
-                       kernel.numEltPerGRF<Type_UB>()) -
-                      1;
+          uint32_t correctSrcBytes =
+              std::min(opnd->getTopDcl()->getByteSize(), srcBytes);
+          correctRB += (correctSrcBytes - 1);
         }
 
         G4_Declare *parentDcl = opnd->getBase()->asRegVar()->getDeclare();
-        while (parentDcl != NULL) {
+        while (parentDcl) {
           correctRB += parentDcl->getAliasOffset();
           parentDcl = parentDcl->getAliasDeclare();
         }
 
-        correctRB = std::min(correctRB, opnd->getTopDcl()->getByteSize() - 1);
-
         if (opnd->getRightBound() != correctRB) {
           DEBUG_VERBOSE("Right bound mismatch for send inst src0. Orig rb = "
                         << opnd->getRightBound()
@@ -471,6 +468,13 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
       }
     }
   } else {
+    // Only valid ARF type are NULL and Accumulator for ternary instructions
+    if (inst->getNumSrc() == 3) {
+      if (opnd->isAreg() && !opnd->isNullReg() && !opnd->isAccReg() &&
+          !(opnd == inst->getSrc(0) && opnd->isSrReg()))
+        vISA_ASSERT(false, "Not allowed ARF in ternary instruction");
+    }
+
     if (opnd->isSrcRegRegion() && opnd->isRightBoundSet()) {
       G4_SrcRegRegion newRgn(*(opnd->asSrcRegRegion()));
 
@@ -478,8 +482,18 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
       newRgn.computeLeftBound(*kernel.fg.builder);
       newRgn.computeRightBound(execSize);
 
-      if ((opnd->getRightBound() - opnd->getLeftBound()) >
-              (2u * kernel.numEltPerGRF<Type_UB>())) {
+      if (kernel.fg.builder->supportNativeSIMD32() &&
+          inst->getExecSize() == g4::SIMD32 && opnd->getTypeSize() == 8) {
+        bool indirect1x1 = opnd->isIndirect()  &&
+                           !opnd->asSrcRegRegion()->getRegion()->isRegionWH();
+        vISA_ASSERT(!indirect1x1,
+                    "Must not be indirect 1x1 addressing mode for SIMD32 "
+                    "instructions with 64b datatypes");
+        vISA_ASSERT((opnd->getRightBound() - opnd->getLeftBound()) <
+                        (4u * kernel.numEltPerGRF<Type_UB>()),
+                    "Src cannot span more than 4 GRFs!");
+      } else if ((opnd->getRightBound() - opnd->getLeftBound()) >
+                 (2u * kernel.numEltPerGRF<Type_UB>())) {
         if (!(inst->opcode() == G4_pln && inst->getSrc(1) == opnd)) {
           DEBUG_VERBOSE(
               "Difference between left/right bound is greater than 2 GRF for "
@@ -559,9 +573,18 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
       newRgn.computeLeftBound(*kernel.fg.builder);
       newRgn.computeRightBound(execSize);
 
-      if ((opnd->getRightBound() - opnd->getLeftBound()) >
-              (2u * kernel.numEltPerGRF<Type_UB>()) &&
-          (inst->opcode() != G4_madw)) {
+      if (kernel.fg.builder->supportNativeSIMD32() &&
+          inst->getExecSize() == g4::SIMD32 &&
+          (opnd->getTypeSize() == 8)) {
+        vISA_ASSERT(!opnd->isIndirect(),
+                    "Must be direct addressing mode for SIMD32 instructions "
+                    "with 64b datatypes");
+        vISA_ASSERT((opnd->getRightBound() - opnd->getLeftBound()) <
+                        (4u * kernel.numEltPerGRF<Type_UB>()),
+                    "Dst cannot span more than 4 GRFs!");
+      } else if ((opnd->getRightBound() - opnd->getLeftBound()) >
+                     (2u * kernel.numEltPerGRF<Type_UB>()) &&
+                 (inst->opcode() != G4_madw)) {
         DEBUG_VERBOSE(
             "Difference between left/right bound is greater than 2 GRF for dst "
             "region. Single non-send opnd cannot span 2 GRFs. lb = "
@@ -943,8 +966,9 @@ void G4Verifier::verifyDpas(G4_INST *inst) {
   G4_Type s3Ty = src3 ? src3->getType() : Type_UNDEF;
 
   // No source modifier
-  if (src0->hasModifier() || src1->hasModifier() || src2->hasModifier() ||
-      (src3 && src3->hasModifier())) {
+  if (std::any_of(dpasInst->src_begin(), dpasInst->src_end(),
+          [](G4_Operand *opnd) {
+            return opnd && opnd->asSrcRegRegion()->hasModifier(); })) {
     DEBUG_VERBOSE("should not have source modifier");
     inst->emit(std::cerr);
     DEBUG_VERBOSE("\n");
@@ -952,8 +976,10 @@ void G4Verifier::verifyDpas(G4_INST *inst) {
   }
 
   // No indirect register access
-  if (src0->isIndirect() || src1->isIndirect() || src2->isIndirect() ||
-      dst->isIndirect() || (src3 && src3->isIndirect())) {
+  if (dst->isIndirect() ||
+      std::any_of(dpasInst->src_begin(), dpasInst->src_end(),
+          [](G4_Operand *opnd) {
+            return opnd && opnd->asSrcRegRegion()->isIndirect(); })) {
     DEBUG_VERBOSE("no indirect register access supported!");
     inst->emit(std::cerr);
     DEBUG_VERBOSE("\n");
@@ -1003,16 +1029,13 @@ void G4Verifier::verifyDpas(G4_INST *inst) {
     vISA_ASSERT(false, "invalid");
   }
 
-  // region check, enforce <1;1,0> for source region, <1> for dst
-  auto isSrcRegion110 = [](const RegionDesc *RD) -> bool {
-    return RD->vertStride == 1 && RD->width == 1 && RD->horzStride == 0;
-  };
-
-  if (dst->getHorzStride() != 1 ||
-      (!src0->isNullReg() && !isSrcRegion110(src0->getRegion())) ||
-      !isSrcRegion110(src1->getRegion()) ||
-      !isSrcRegion110(src2->getRegion()) ||
-      (src3 && !isSrcRegion110(src3->getRegion()))) {
+  bool invalidRegion = false;
+  invalidRegion =
+      dst->getHorzStride() != 1 ||
+      (!src0->isNullReg() && !src0->getRegion()->isRegion110()) ||
+      !src1->getRegion()->isRegion110() || !src2->getRegion()->isRegion110() ||
+      (src3 && !src3->getRegion()->isRegion110());
+  if (invalidRegion) {
     DEBUG_VERBOSE("src region should be <1;1,0> and dst region <1>!");
     inst->emit(std::cerr);
     DEBUG_VERBOSE("\n");
