@@ -20,6 +20,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/Support/Allocator.h"
 #include "llvm/ADT/DenseSet.h"
+#include "llvm/ADT/SetVector.h"
 #include "common/LLVMWarningsPop.hpp"
 // clang-format on
 
@@ -47,6 +48,7 @@ enum BankConflict {
 };
 
 class VarSplit;
+class SpillAnalysis;
 
 class BankConflictPass {
 private:
@@ -159,9 +161,9 @@ class LiveRange final {
   unsigned numRegNeeded;
   unsigned degree = 0;
   unsigned refCount = 0;
-  unsigned parentLRID;
+  unsigned parentLRID = 0;
   AssignedReg reg;
-  float spillCost;
+  float spillCost = 0.0f;
   BankConflict bc = BANK_CONFLICT_NONE;
 
   union {
@@ -314,13 +316,44 @@ typedef std::map<vISA::G4_Declare *, std::pair<vISA::G4_INST *, unsigned>>
     CALL_DECL_MAP;
 
 namespace vISA {
+typedef struct Interval {
+  G4_INST *start = nullptr;
+  G4_INST *end = nullptr;
+
+  ~Interval() = default;
+  Interval() = default;
+  Interval(const Interval &) = default;
+  Interval &operator=(const Interval &) = default;
+  Interval(G4_INST *s, G4_INST *e) {
+    start = s;
+    end = e;
+  }
+  bool intervalsOverlap(const Interval &second) const;
+} Interval;
+// Used as entry in priority queue for storing default, non-default
+// intervals.
+struct QueueEntry {
+  G4_Declare *dcl;
+  Interval interval;
+
+  ~QueueEntry() = default;
+  QueueEntry() = default;
+  QueueEntry(const QueueEntry &) = default;
+  QueueEntry &operator=(const QueueEntry &) = default;
+  QueueEntry(G4_Declare *d, const Interval &i) {
+    dcl = d;
+    interval = i;
+  }
+};
+using AllIntervals = std::vector<QueueEntry>;
+
 struct criticalCmpForEndInterval {
   GlobalRA &gra;
   criticalCmpForEndInterval(GlobalRA &g);
-  bool operator()(G4_Declare *A, G4_Declare *B) const;
+  bool operator()(const QueueEntry &A, const QueueEntry &B) const;
 };
 struct AugmentPriorityQueue
-    : std::priority_queue<G4_Declare *, std::vector<G4_Declare *>,
+    : std::priority_queue<QueueEntry, std::vector<QueueEntry>,
                           criticalCmpForEndInterval> {
   AugmentPriorityQueue(criticalCmpForEndInterval cmp);
   auto begin() const { return c.begin(); }
@@ -347,14 +380,127 @@ private:
   FCALL_RET_MAP &fcallRetMap;
   CALL_DECL_MAP callDclMap;
   std::unordered_map<FuncInfo *, PhyRegSummary> localSummaryOfCallee;
-  std::vector<G4_Declare *> sortedIntervals;
+  AllIntervals sortedIntervals;
   AugmentPriorityQueue defaultMaskQueue{criticalCmpForEndInterval(gra)};
   AugmentPriorityQueue nonDefaultMaskQueue{criticalCmpForEndInterval(gra)};
   // overlapDclsWithFunc holds default and non-default range live across
   // all call sites of func.
   std::unordered_map<FuncInfo *, MaskDeclares> overlapDclsWithFunc;
   std::unordered_map<G4_Declare *, MaskDeclares> retDeclares;
+  VarReferences refs;
   bool useGenericAugAlign = false;
+  enum class ArgType {
+      Init = 0,
+      // arg may be defined in callee or may be defined once
+      // and used by multiple call sites. This is different
+      // than other types as the def may appear anywhere other
+      // than entry BB. This is a corner case. Live-interval
+      // of such args span complete function wherever the
+      // variable is referenced.
+      Unknown = 1,
+      // Standard arg that's fully defined before each call site
+      // in same BB.
+      DefBeforeEachCall = 2,
+      // Args that are input to kernel or defined once in entry
+      // BB but used throughout program belong to this type.
+      LiveThrough = 3,
+  };
+  enum class RetValType {
+      Init = 0,
+      // retval pattern irregular. Live-interval of such retval
+      // span complete function wherever the variable is referenced.
+      Unknown = 1,
+      // retval pattern regular. It's defined in callee and
+      // used in BB immediate after call.
+      Regular = 2,
+  };
+  class ArgRetValInfo {
+  public:
+    ArgType argType = ArgType::Init;
+    RetValType retValType = RetValType::Init;
+    // Struct to store type of arg/retval and subroutines where it appears.
+    // We use a set because variables such as HWTID are defined once in
+    // kernel and used in several subroutines.
+    std::unordered_set<const FuncInfo *> subroutines;
+  };
+  std::unordered_map<G4_Declare *, ArgRetValInfo> argsRetVal;
+  std::unordered_map<FuncInfo*, std::unordered_set<G4_Declare*>> argsPerSub;
+  std::unordered_map<FuncInfo *, std::unordered_set<G4_Declare *>> retValPerSub;
+  std::unordered_map<G4_Declare *, std::unordered_set<FuncInfo *>> unknownArgRetvalRefs;
+  std::unordered_map<G4_Declare *, std::unordered_set<FuncInfo *>> nonGRFRefs;
+  std::unordered_map<G4_BB *, FuncInfo *> bbToFunc;
+  // Store home function for given variable. Home function is defined as
+  // function that contains explicit def or use of the variable. Each regular
+  // variable has a unique home function. Arg/retval don't have a unique home
+  // function as they're usually defined in caller and used in callee, ie
+  // they're referenced in different functions.
+  //
+  // This vector is indexed by G4_Declare's dclId.
+  //
+  // It's assumed that augmentation doesn't require resizing this vector
+  // despite inserting new SCALL dcls as they're all function local.
+  std::vector<FuncInfo *> homeFunc;
+  // Sadly, we don't have a way to map G4_INST to containing G4_BB.
+  // So we create it in Augmentation using below vector. To get
+  // FuncInfo* for a G4_INST, we dereference instToFunc using
+  // lexical id of the instruction. We use vector instead of a
+  // map because it's used often, is faster than map, and
+  // importantly it's dense so there's no savings to be had with a map.
+  std::vector<FuncInfo *> instToFunc;
+  const bool hasSubroutines = false;
+
+  void populateFuncMaps();
+  void populateHomeFunc();
+  bool isSubroutineArg(G4_Declare *dcl) const {
+    auto it = argsRetVal.find(dcl);
+    if (it == argsRetVal.end())
+      return false;
+    if ((*it).second.argType != ArgType::Init) {
+      vISA_ASSERT((*it).second.retValType == RetValType::Init,
+                  "expecting retval type");
+      return true;
+    }
+    return false;
+  }
+
+  bool isSubroutineRetVal(G4_Declare *dcl) const {
+    auto it = argsRetVal.find(dcl);
+    if (it == argsRetVal.end())
+      return false;
+    if ((*it).second.retValType != RetValType::Init) {
+      vISA_ASSERT((*it).second.argType == ArgType::Init,
+                  "expecting retval type");
+      return true;
+    }
+    return false;
+  }
+
+  template <ArgType Type> bool isArgType(G4_Declare *dcl) const {
+    auto it = argsRetVal.find(dcl);
+    if (it == argsRetVal.end())
+      return false;
+    return (*it).second.argType == Type;
+  }
+
+  template <RetValType Type> bool isRetvalType(G4_Declare *dcl) const {
+    auto it = argsRetVal.find(dcl);
+    if (it == argsRetVal.end())
+      return false;
+    return (*it).second.retValType == Type;
+  }
+
+  bool isLiveThroughArg(G4_Declare *dcl) const;
+  bool isDefBeforeEachCallArg(G4_Declare *dcl) const;
+  bool isUnknownArg(G4_Declare *dcl) const;
+  bool isSingleDefInEntryBB(G4_Declare* dcl) const;
+  bool isUnknownRetVal(G4_Declare *dcl) const;
+  bool isRegularRetVal(G4_Declare *dcl) const;
+  bool isUnknownArgOrRetval(G4_Declare *dcl) const;
+  FuncInfo *computeHomeFunc(G4_Declare *dcl);
+  bool hasUniqueFuncHome(G4_Declare *dcl) const;
+  FuncInfo *getUniqueFuncHome(G4_Declare *dcl) const;
+
+  void verifyHomeLocation();
 
   bool updateDstMaskForGather(G4_INST *inst, std::vector<unsigned char> &mask);
   bool updateDstMaskForGatherRaw(G4_INST *inst,
@@ -385,8 +531,20 @@ private:
                                    G4_Operand *opnd);
   void updateEndIntervalForLocal(G4_Declare *dcl, G4_INST *curInst,
                                  G4_Operand *opnd);
+  void buildUnknownArgRetval();
+  void buildLiveIntervals(FuncInfo* func);
   void buildLiveIntervals();
   void sortLiveIntervals();
+  void startIntervalForLiveIn(FuncInfo *funcInfo, G4_BB *bb);
+  void handleCallSite(G4_BB *curBB, unsigned int &funcCnt);
+  void handleDstOpnd(FuncInfo *funcInfo, G4_BB *curBB, G4_INST *inst);
+  void handleCondMod(FuncInfo* funcInfo, G4_INST *inst);
+  void endIntervalForLiveOut(FuncInfo *funcInfo, G4_BB *bb);
+  void handleSrcOpnd(FuncInfo *funcInfo, G4_BB *curBB, G4_Operand *src);
+  void handlePred(FuncInfo* funcInfo, G4_INST *inst);
+  void handleNonReducibleExtension(FuncInfo *funcInfo);
+  void handleLoopExtension(FuncInfo *funcInfo);
+  void extendVarLiveness(FuncInfo *funcInfo, G4_BB *bb, G4_INST *inst);
   unsigned getEnd(const G4_Declare *dcl) const;
   bool isNoMask(const G4_Declare *dcl, unsigned size) const;
   bool isConsecutiveBits(const G4_Declare *dcl, unsigned size) const;
@@ -408,16 +566,25 @@ private:
                                  const std::vector<bool> &globalVars);
   void addSIMDIntfForRetDclares(G4_Declare *newDcl,
                                 const std::vector<bool> &globalVars);
+  void discoverArgs(FuncInfo *func);
+  void discoverRetVal(FuncInfo *func);
+  ArgType computeArgType(FuncInfo *func, G4_Declare *arg);
+  RetValType computeRetValType(FuncInfo *func, G4_Declare *retVal);
+
+  // Functions used for verification
+  void dumpSortedIntervals();
+  void dumpArgs(SparseBitVector &subArgs);
+  void dumpRetVal(SparseBitVector &subRetVal);
 
 public:
   Augmentation(Interference &i, const LivenessAnalysis &l, GlobalRA &g);
   ~Augmentation();
+  Augmentation(const Augmentation&) = delete;
+  Augmentation& operator=(const Augmentation&) = delete;
 
   void augmentIntfGraph();
 
-  const std::vector<G4_Declare *> &getSortedLiveIntervals() const {
-    return sortedIntervals;
-  }
+  const auto &getSortedLiveIntervals() const { return sortedIntervals; }
 };
 
 // This class stores base matrices used for interference building for graph coloring.
@@ -630,7 +797,8 @@ class Interference {
   // compatible ranges will not be present in sparseIntf set.
   // We store G4_Declare* instead of id is because variables
   // allocated by LRA will not have a valid id.
-  std::map<G4_Declare *, std::vector<G4_Declare *>> compatibleSparseIntf;
+  std::unordered_map<G4_Declare *, llvm::SmallSetVector<G4_Declare *, 16>>
+      compatibleSparseIntf;
 
   GlobalRA &gra;
   G4_Kernel &kernel;
@@ -755,6 +923,8 @@ public:
       delete[] matrix;
     }
   }
+  Interference(const Interference&) = delete;
+  Interference& operator=(const Interference&) = delete;
 
   bool useDenseMatrix() const {
     // The size check is added to prevent offset overflow in
@@ -766,7 +936,7 @@ public:
     return (maxId < denseMatrixLimit) && (size < max);
   }
 
-  const std::vector<G4_Declare *> *
+  const llvm::SmallSetVector<G4_Declare *, 16> *
   getCompatibleSparseIntf(G4_Declare *d) const {
     if (compatibleSparseIntf.size() > 0) {
       auto it = compatibleSparseIntf.find(d);
@@ -778,6 +948,8 @@ public:
     return nullptr;
   }
 
+  size_t numVarsWithWeakEdges() const { return compatibleSparseIntf.size(); }
+
   void init() {
     if (useDenseMatrix()) {
       auto N = (size_t)rowSize * (size_t)maxId;
@@ -821,11 +993,11 @@ public:
 // Used only when -dumpregchart is passed.
 class RegChartDump {
   const GlobalRA &gra;
-  std::vector<G4_Declare *> sortedLiveIntervals;
+  AllIntervals sortedLiveIntervals;
   std::unordered_map<G4_Declare *, std::pair<G4_INST *, G4_INST *>> startEnd;
 
 public:
-  void recordLiveIntervals(const std::vector<G4_Declare *> &dcls);
+  void recordLiveIntervals(const AllIntervals &dcls);
   void dumpRegChart(std::ostream &, const LiveRangeVec &lrs, unsigned numLRs);
 
   RegChartDump(const GlobalRA &g) : gra(g) {}
@@ -887,25 +1059,76 @@ class GraphColor {
   // Reserved GRF count for fail-safe RA
   unsigned reserveSpillGRFCount = 0;
 
+  template<bool Support4GRFAlign>
   unsigned edgeWeightGRF(const LiveRange *lr1, const LiveRange *lr2);
   unsigned edgeWeightARF(const LiveRange *lr1, const LiveRange *lr2);
   static unsigned edgeWeightGRF(bool lr1EvenAlign, bool lr2EvenAlign,
                                 unsigned lr1_nreg, unsigned lr2_nreg) {
+    unsigned sum = lr1_nreg + lr2_nreg;
     if (!lr1EvenAlign) {
-      return lr1_nreg + lr2_nreg - 1;
+      return sum - 1;
     }
 
-    if (!lr2EvenAlign) {
-      unsigned sum = lr1_nreg + lr2_nreg;
+    if (!lr2EvenAlign)
       return sum + 1 - ((sum) % 2);
-    } else if (lr2EvenAlign) {
-      return lr1_nreg + lr2_nreg - 1 + (lr1_nreg % 2) + (lr2_nreg % 2);
-    } else {
-      vISA_ASSERT_UNREACHABLE("should be unreachable");
-      return 0;
+
+    return sum - 1 + (lr1_nreg % 2) + (lr2_nreg % 2);
+  }
+
+  static unsigned edgeWeightWith4GRF(int lr1Align, int lr2Align,
+                                     unsigned lr1_nreg, unsigned lr2_nreg) {
+    if (lr1Align < 4 && lr2Align < 4)
+      return edgeWeightGRF(lr1Align % 2, lr2Align % 2, lr1_nreg, lr2_nreg);
+
+    if (lr2Align == 4) {
+      if (lr1Align < 2)
+        return lr1_nreg + lr2_nreg - 1;
+      if (lr1Align == 2) {
+        // if (lr2_nreg % 2 == 0) -- lr2 size is even
+        // return lr2_nreg + lr1_nreg;
+        // if (lr2_nreg % 2 == 1) -- lr2 size is odd
+        // return lr2_nreg + lr1_nreg + 1;
+
+        return lr1_nreg + lr2_nreg + (lr2_nreg % 2);
+      } else if (lr1Align == 4) {
+        if (lr2_nreg % 4 == 0)
+          // lr2 size is multiple of 4
+          return lr1_nreg + lr2_nreg;
+
+        // if lr2_nreg % 4 == 1 --  lr2 size is 1 + (4*n)
+        // return lr1_nreg + lr2_nreg + 3;
+        // if lr2_nreg % 2 == 0 -- lr2 size is 2 + (4*n)
+        // return lr2_nreg + lr1_nreg + 2;
+        // if lr2_nreg % 4 == 3 -- lr2 size is 3 + (4*n)
+        // return lr2_nreg + lr1_nreg + 1;
+
+        return lr1_nreg + lr2_nreg + 4 - (lr2_nreg % 4);
+      }
+    }
+
+    vISA_ASSERT(lr1Align == 4, "unexpected condition");
+    return edgeWeightWith4GRF(lr2Align, lr1Align, lr2_nreg, lr1_nreg);
+  }
+
+  void inline relax(LiveRange *lr1, unsigned int w) {
+    // relax degree between 2 nodes
+    VISA_DEBUG_VERBOSE({
+      std::cout << "\t relax ";
+      lr1->dump();
+      std::cout << " degree(" << lr1->getDegree() << ") - " << w << "\n";
+    });
+    lr1->subtractDegree(w);
+
+    unsigned availColor = numColor;
+    availColor = numColor - lr1->getNumForbidden();
+
+    if (lr1->getDegree() + lr1->getNumRegNeeded() <= availColor) {
+      unconstrainedWorklist.push_back(lr1);
+      lr1->setActive(false);
     }
   }
 
+  template <bool Support4GRFAlign>
   void computeDegreeForGRF();
   void computeDegreeForARF();
   void computeSpillCosts(bool useSplitLLRHeuristic, const RPE *rpe);
@@ -978,25 +1201,38 @@ struct RAVarInfo {
   BankConflict conflict =
       BANK_CONFLICT_NONE; // used to indicate bank that should be assigned to
                           // dcl if possible
-  G4_INST *startInterval = nullptr;
-  G4_INST *endInterval = nullptr;
+  // Store intervals over which variable is live. Note that
+  // intervals are used only for subroutine arguments. For
+  // all other variables, it's expected that intervals contains
+  // a single entry.
+  std::vector<Interval> intervals;
   std::vector<unsigned char> mask;
   std::vector<const G4_Declare *> subDclList;
   unsigned subOff = 0;
   std::vector<BundleConflict> bundleConflicts;
   G4_SubReg_Align subAlign = G4_SubReg_Align::Any;
-  bool isEvenAlign = false;
+  int augAlignInGRF = 0;
   AugmentationMasks augMask = AugmentationMasks::Undetermined;
 };
 
+struct CustomHash {
+  size_t operator()(const Interval &seg) const {
+    return seg.start->getLexicalId();
+  }
+};
+
+static bool operator==(const Interval &first, const Interval &second) {
+  return first.start->getLexicalId() == second.start->getLexicalId();
+}
+
 class VerifyAugmentation {
 private:
   G4_Kernel *kernel = nullptr;
   GlobalRA *gra = nullptr;
-  std::vector<G4_Declare *> sortedLiveRanges;
-  std::unordered_map<
-      const G4_Declare *,
-      std::tuple<LiveRange *, AugmentationMasks, G4_INST *, G4_INST *>>
+  AllIntervals sortedLiveRanges;
+  std::unordered_map<const G4_Declare *,
+                     std::tuple<LiveRange *, AugmentationMasks,
+                                std::unordered_set<Interval, CustomHash>>>
       masks;
   LiveRangeVec lrs;
   unsigned numVars = 0;
@@ -1004,6 +1240,7 @@ private:
   std::unordered_map<G4_Declare *, LiveRange *> DclLRMap;
   std::unordered_map<G4_BB *, std::string> bbLabels;
   std::vector<std::tuple<G4_BB *, unsigned, unsigned>> BBLexId;
+  CALL_DECL_MAP callDclMap;
 
   static const char *getStr(AugmentationMasks a) {
     if (a == AugmentationMasks::Default16Bit)
@@ -1038,7 +1275,7 @@ public:
     bbLabels.clear();
     BBLexId.clear();
   }
-  void loadAugData(std::vector<G4_Declare *> &s, const LiveRangeVec &l,
+  void loadAugData(AllIntervals &s, const LiveRangeVec &l, const CALL_DECL_MAP& c,
                    unsigned n, const Interference *i, GlobalRA &g);
   void dump(const char *dclName);
   bool isClobbered(LiveRange *lr, std::string &msg);
@@ -1075,7 +1312,6 @@ public:
 };
 
 class GlobalRA {
-
 private:
   std::unordered_set<G4_INST *> EUFusionCallWAInsts;
   bool m_EUFusionCallWANeeded;
@@ -1110,6 +1346,8 @@ public:
   // The pre assigned forbidden register bits for different kinds
   ForbiddenRegs fbdRegs;
 
+  const bool use4GRFAlign = false;
+
 private:
   template <class REGION_TYPE>
   static unsigned getRegionDisp(REGION_TYPE *region, const IR_Builder &irb);
@@ -1123,7 +1361,6 @@ private:
   void detectUndefinedUses(LivenessAnalysis &liveAnalysis, G4_Kernel &kernel);
   void markBlockLocalVar(G4_RegVar *var, unsigned bbId);
   void markBlockLocalVars();
-  void computePhyReg();
   void fixAlignment();
   // Updates `slot1SetR0` and `slot1ResetR0` with hword spill/fill instructions
   // that need to update r0.5 to address slot 1 scratch space.
@@ -1132,6 +1369,7 @@ private:
   void expandFillIntrinsic(G4_BB *);
   void expandSpillFillIntrinsics(unsigned);
   void saveRestoreA0(G4_BB *);
+  void initAddrRegForImmOffUseNonStackCall();
   static const RAVarInfo defaultValues;
   std::vector<RAVarInfo> vars;
   std::vector<G4_Declare *> UndeclaredVars;
@@ -1154,6 +1392,9 @@ private:
   // store instructions that shouldnt be rematerialized.
   std::unordered_set<G4_INST *> dontRemat;
 
+  // [-2^16...2^16) in bytes
+  //   (frame pointer is biased 2^16 so that -2^16 references scratch[0x0])
+  static constexpr unsigned SPILL_FILL_IMMOFF_MAX = 0x10000; // 64k
 
   // map each BB to its local RA GRF usage summary, populated in local RA.
   std::map<G4_BB *, PhyRegSummary *> bbLocalRAMap;
@@ -1196,6 +1437,7 @@ private:
 
   uint32_t numGRFSpill = 0;
   uint32_t numGRFFill = 0;
+  bool canUseLscImmediateOffsetSpillFill = false;
 
   unsigned int numReservedGRFsFailSafe = BoundedRA::NOT_FOUND;
 
@@ -1260,6 +1502,8 @@ public:
   bool useFastRA = false;
   bool useHybridRAwithSpill = false;
   bool useLocalRA = false;
+  uint32_t nextSpillOffset = 0;
+  uint32_t scratchOffset = 0;
 
   InterferenceMatrixStorage intfStorage;
   IncrementalRA incRA;
@@ -1276,6 +1520,13 @@ public:
 
   void setSubRetLoc(const G4_BB *bb, unsigned s) { subretloc[bb] = s; }
 
+  const G4_Declare *getRetDcl(unsigned int retLoc) const {
+    auto it = retDecls.find(retLoc);
+    if (it == retDecls.end())
+        return nullptr;
+    return it->second;
+  }
+
   bool isSubRetLocConflict(G4_BB *bb, std::vector<unsigned> &usedLoc,
                            unsigned stackTop);
   void assignLocForReturnAddr();
@@ -1295,11 +1546,13 @@ public:
     }
 
     const char *name = builder.getNameString(24, "RET__loc%d", retLoc);
-    G4_Declare *dcl = builder.createDeclare(name, G4_GRF, 2, 1, Type_UD);
+    G4_Declare *dcl = builder.createDeclare(
+        name, G4_GRF, builder.getCallRetOpndSize(), 1, Type_UD);
 
     // call destination must still be QWord aligned
-    dcl->setSubRegAlign(Four_Word);
-    setSubRegAlign(dcl, Four_Word);
+    auto callDstAlign = Four_Word;
+    dcl->setSubRegAlign(callDstAlign);
+    setSubRegAlign(dcl, callDstAlign);
 
     retDecls[retLoc] = dcl;
     return dcl;
@@ -1433,26 +1686,89 @@ public:
   }
 
   G4_INST *getStartInterval(const G4_Declare *dcl) const {
-    return getVar(dcl).startInterval;
+    return getLastStartInterval(dcl);
   }
 
   void setStartInterval(const G4_Declare *dcl, G4_INST *inst) {
-    allocVar(dcl).startInterval = inst;
+    setLastStartInterval(dcl, inst);
   }
 
   G4_INST *getEndInterval(const G4_Declare *dcl) const {
-    return getVar(dcl).endInterval;
+    return getLastEndInterval(dcl);
   }
 
   void setEndInterval(const G4_Declare *dcl, G4_INST *inst) {
-    allocVar(dcl).endInterval = inst;
+    setLastEndInterval(dcl, inst);
+  }
+
+  void clearIntervals(G4_Declare *dcl) {
+    auto &intervals = allocVar(dcl).intervals;
+    intervals.clear();
+  }
+
+  // Used to create new interval for subroutine arg/retval
+  void pushBackNewInterval(const G4_Declare* dcl) {
+    auto &intervals = allocVar(dcl).intervals;
+    intervals.push_back({nullptr, nullptr});
+  }
+
+  const Interval getInterval(const G4_Declare *dcl, unsigned int i) const {
+    if (i < getNumIntervals(dcl)) {
+      return getAllIntervals(dcl)[i];
+    }
+    vISA_ASSERT(false, "OOB access");
+    return {};
+  }
+
+  unsigned int getNumIntervals(const G4_Declare *dcl) const {
+    return getAllIntervals(dcl).size();
+  }
+
+  const G4_INST *getIntervalStart(const Interval &interval) const {
+    return interval.start;
+  }
+
+  const G4_INST *getIntervalEnd(const Interval &interval) const {
+    return interval.end;
+  }
+
+  const std::vector<Interval>& getAllIntervals(const G4_Declare *dcl) const {
+    return getVar(dcl).intervals;
+  }
+
+  G4_INST *getLastStartInterval(const G4_Declare *dcl) const {
+    auto& intervals = getVar(dcl).intervals;
+    if (intervals.empty())
+      return nullptr;
+    return intervals.back().start;
+  }
+
+  void setLastStartInterval(const G4_Declare *dcl, G4_INST *inst) {
+    auto &intervals = allocVar(dcl).intervals;
+    if(intervals.empty())
+      intervals.resize(1);
+    intervals.back().start = inst;
+  }
+
+  G4_INST *getLastEndInterval(const G4_Declare *dcl) const {
+    auto &intervals = getVar(dcl).intervals;
+    if (intervals.empty())
+      return nullptr;
+    return intervals.back().end;
+  }
+
+  void setLastEndInterval(const G4_Declare *dcl, G4_INST *inst) {
+    auto &intervals = allocVar(dcl).intervals;
+    if(intervals.empty())
+      intervals.resize(1);
+    intervals.back().end = inst;
   }
 
   const std::vector<unsigned char> &getMask(const G4_Declare *dcl) const {
     return getVar(dcl).mask;
   }
 
-  void setMask(const G4_Declare *dcl, std::vector<unsigned char> m) {
+  void setMask(const G4_Declare *dcl, const std::vector<unsigned char>& m) {
     allocVar(dcl).mask = m;
   }
 
@@ -1488,9 +1804,15 @@ public:
   }
 
   unsigned get_bundle(unsigned baseReg, int offset) const {
+    if (builder.has64bundleSize2GRFPerBank()) {
+      return (((baseReg + offset) % 32) / 4);
+    }
     if (builder.hasPartialInt64Support()) {
       return (((baseReg + offset) % 32) / 2);
     }
+    if (builder.has64bundleSize()) {
+      return (((baseReg + offset) % 16) / 2);
+    }
     return (((baseReg + offset) % 64) / 4);
   }
 
@@ -1501,11 +1823,17 @@ public:
       bankID = ((baseReg + offset) % 4) / 2;
     }
 
+    if (builder.has64bundleSize2GRFPerBank()) {
+      bankID = ((baseReg + offset) % 4) / 2;
+    }
 
     if (builder.hasOneGRFBank16Bundles()) {
       bankID = (baseReg + offset) % 2;
     }
 
+    if (builder.has64bundleSize()) {
+      bankID = (baseReg + offset) % 2;
+    }
     return bankID;
   }
 
@@ -1553,12 +1881,39 @@ public:
     return true;
   }
 
-  bool isEvenAligned(const G4_Declare *dcl) const {
-    return getVar(dcl).isEvenAlign;
+  bool isQuadAligned(const G4_Declare *dcl) const {
+    auto augAlign = getAugAlign(dcl);
+    return augAlign == 4;
   }
 
-  void setEvenAligned(const G4_Declare *dcl, bool e) {
-    allocVar(dcl).isEvenAlign = e;
+  template <bool Support4GRFAlign = true>
+  bool isEvenAligned(const G4_Declare* dcl) const {
+    auto augAlign = getAugAlign(dcl);
+    if constexpr (Support4GRFAlign)
+      return augAlign > 0 && augAlign % 2 == 0;
+    else
+      return (augAlign > 0);
+  }
+
+  int getAugAlign(const G4_Declare *dcl) const {
+    return getVar(dcl).augAlignInGRF;
+  }
+
+  void forceQuadAlign(const G4_Declare *dcl) { setAugAlign(dcl, 4); }
+
+  void resetAlign(const G4_Declare *dcl) { setAugAlign(dcl, 0); }
+
+  // Due to legacy usage, this method takes a boolean that, when set,
+  // causes alignment to be set to Even (2). When boolean flag is
+  // reset, it also resets alignment to Either (0).
+  void setEvenAligned(const G4_Declare *dcl, bool align) {
+    setAugAlign(dcl, align ? 2 : 0);
+  }
+
+  void setAugAlign(const G4_Declare *dcl, int align) {
+    vISA_ASSERT(align <= 2 || use4GRFAlign, "unexpected alignment");
+    vISA_ASSERT(align <= 4, "unsupported alignment");
+    allocVar(dcl).augAlignInGRF = align;
   }
 
   BankAlign getBankAlign(const G4_Declare *) const;
@@ -1573,7 +1928,8 @@ public:
         useLscForNonStackCallSpillFill(
             k.fg.builder->useLscForNonStackSpillFill()),
         useLscForScatterSpill(k.fg.builder->supportsLSC() &&
-                              k.fg.builder->getOption(vISA_scatterSpill)) {
+                              k.fg.builder->getOption(vISA_scatterSpill)),
+        use4GRFAlign(k.fg.builder->supports4GRFAlign()) {
     vars.resize(k.Declares.size());
 
     if (kernel.getOptions()->getOption(vISA_VerifyAugmentation)) {
@@ -1597,8 +1953,9 @@ public:
   static uint32_t getRefCount(int loopNestLevel);
   void updateSubRegAlignment(G4_SubReg_Align subAlign);
   bool isChannelSliced();
-  void evenAlign();
-  bool evenAlignNeeded(G4_Declare *);
+  // Used by LRA/GRA/hybrid RA
+  void augAlign();
+  int getAlignFromAugBucket(G4_Declare *);
   void getBankAlignment(LiveRange *lr, BankAlign &align);
   void printLiveIntervals();
   void reportUndefinedUses(LivenessAnalysis &liveAnalysis, G4_BB *bb,
@@ -1662,6 +2019,7 @@ public:
   void verifyRA(LivenessAnalysis &liveAnalysis);
   void verifySpillFill();
   void resetGlobalRAStates();
+  bool canSkipFDE() const;
 
   void insertPhyRegDecls();
 
@@ -1683,7 +2041,7 @@ public:
   }
 
   void copyAlignment(G4_Declare *dst, G4_Declare *src) {
-    setEvenAligned(dst, isEvenAligned(src));
+    setAugAlign(dst, getAugAlign(src));
     setSubRegAlign(dst, getSubRegAlign(src));
   }
 
@@ -1877,6 +2235,47 @@ public:
   void run();
   void dump();
 };
+
+// Class used to analyze spill/fill decisions
+class SpillAnalysis {
+public:
+  SpillAnalysis() = default;
+  ~SpillAnalysis();
+  SpillAnalysis(const SpillAnalysis &) = delete;
+  SpillAnalysis &operator=(const SpillAnalysis &) = delete;
+
+  void Dump(std::ostream &OS = std::cerr);
+  void DumpHistogram(std::ostream &OS = std::cerr);
+
+  unsigned int GetDistance(G4_Declare *Dcl);
+  void LoadAugIntervals(AllIntervals &, GlobalRA &);
+  void LoadDegree(G4_Declare *Dcl, unsigned int degree);
+
+  void SetLivenessAnalysis(LivenessAnalysis *L) { LA = L; }
+  void SetGraphColor(GraphColor *C) { GC = C; }
+  void SetSpillManager(SpillManagerGRF *S) { SM = S; }
+
+  void Clear();
+
+  void Do(LivenessAnalysis *L, GraphColor *C, SpillManagerGRF *S);
+
+private:
+  VarReferences *Refs = nullptr;
+
+  LivenessAnalysis *LA = nullptr;
+  GraphColor *GC = nullptr;
+  SpillManagerGRF *SM = nullptr;
+
+  std::unordered_map<G4_Declare *, std::pair<G4_INST *, G4_INST *>>
+      AugIntervals;
+  std::unordered_map<G4_Declare *, unsigned int> DclDegree;
+
+  std::vector<G4_BB *> GetLiveBBs(G4_Declare *,
+                                  std::unordered_map<G4_INST *, G4_BB *> &);
+  std::vector<G4_BB *>
+  GetIntervalBBs(G4_INST *Start, G4_INST *End,
+                 std::unordered_map<G4_INST *, G4_BB *> &InstBBMap);
+};
 } // namespace vISA
 
 #endif // __GRAPHCOLOR_H__
