@@ -104,6 +104,7 @@ public:
   unsigned getNodeID() const { return nodeID; };
 
   uint32_t schedTime = 0;
+  SB_INST_PIPE instPipe = PIPE_NONE;
 
   // Number of predecessor nodes not scheduled
   uint16_t predsNotScheduled = 0;
@@ -229,6 +230,64 @@ struct BucketDescr {
       : bucket(Bucket), operand(Operand), mask(Mask) {}
 };
 
+class SendQueue {
+public:
+  typedef std::pair<Node *, unsigned> sendLatencyCounterPair_t;
+  G4_Kernel &kernel;
+  std::list<sendLatencyCounterPair_t> sendCache;
+  unsigned entries = 0;
+
+  SendQueue(G4_Kernel &K) : kernel(K) {
+    entries = kernel.fg.builder->getSizeOfSendQueue();
+    unsigned threadSchedPolicy =
+        kernel.fg.builder->getuint32Option(vISA_threadSchedPolicy);
+    if (threadSchedPolicy) { // round robin policy
+      unsigned HWThreadsPerEU = kernel.getNumThreads();
+      entries = (entries % HWThreadsPerEU) ? (entries / HWThreadsPerEU + 1)
+                                           : (entries / HWThreadsPerEU);
+    }
+  }
+
+  ~SendQueue() = default;
+
+  void push(Node *n, unsigned initDelay) {
+    sendCache.push_back(sendLatencyCounterPair_t(n, initDelay));
+    if (sendCache.size() > entries) {
+      sendCache.erase(sendCache.begin());
+    }
+  }
+
+  void pop() {
+    if (!sendCache.empty()) {
+      sendCache.erase(sendCache.begin());
+    }
+  }
+
+  unsigned size() { return (int)sendCache.size(); }
+
+  Node *front() {
+    if (!sendCache.empty()) {
+      return (*sendCache.begin()).first;
+    } else {
+      return nullptr;
+    }
+  }
+
+  bool isFull() const { return sendCache.size() >= entries; }
+  bool isEmpty() const { return sendCache.empty(); }
+
+  void updateCycle(unsigned currentCycle) {
+    for (auto it = sendCache.begin(); it != sendCache.end();) {
+      if (currentCycle > (*it).second)
+        it = sendCache.erase(it);
+      else
+        ++it;
+    }
+  }
+
+  std::list<sendLatencyCounterPair_t> &getQueueEntries() { return sendCache; }
+};
+
 class DDD {
   std::vector<Node *> allNodes;
   // Used to allocate BucketNode, which is POD.
@@ -264,6 +323,7 @@ public:
   typedef std::pair<Node *, Node *> instrPair_t;
   typedef std::vector<instrPair_t> instrPairVec_t;
   NODE_LIST Roots;
+  Node *latestInstOfEachPipe[PIPE_ALL];
   NodeAlloc NodeAllocator;
   void moveDeps(Node *fromNode, Node *toNode);
   void pairTypedWriteOrURBWriteNodes(G4_BB *bb);
@@ -276,8 +336,8 @@ public:
 private:
   bool hasSameSourceOneDPAS(G4_INST *curInst, G4_INST *nextInst,
                             BitSet &liveDst, BitSet &liveSrc) const;
-  bool hsaSameTypesAllOperands(const G4_INST &curInst,
-                               const G4_INST &nextInst) const;
+  bool DPASHasSameTypesAllOperands(const G4_INST &curInst,
+                                   const G4_INST &nextInst) const;
 
 public:
   DDD(G4_BB *bb, const LatencyTable &lt, G4_Kernel *k, PointsToAnalysis &p);
@@ -360,7 +420,7 @@ public:
 
 private:
   G4_Kernel &kernel;
-  RPE *rpe;
+  RPE *rpe = nullptr;
   Options *m_options;
 };
 
@@ -383,6 +443,16 @@ static bool instCanUse2xDP(G4_INST *inst) {
   return false;
 }
 
+// Restrictions of candidate for 2xSP:
+//    1, Only support mad
+//    2, Instruction must be Type_DF or Type_F
+static bool instCanUse2xSP(G4_INST *inst) {
+  if (inst->opcode() == G4_mad && (inst->getDst()->getType() == Type_DF ||
+                                   inst->getDst()->getType() == Type_F)) {
+    return true;
+  }
+  return false;
+}
 
 class window2xSP {
 public:
@@ -403,6 +473,9 @@ public:
     instList.clear();
   }
 
+  window2xSP(const window2xSP&) = delete;
+  window2xSP& operator=(const window2xSP&) = delete;
+
   ~window2xSP() {
     instGRFSize = 0;
     instType = Type_UNDEF;
@@ -581,6 +654,32 @@ public:
     return true;
   }
 
+  // Check if the instruction can be added into the 2xSP block
+  bool canAddToBlock2xSP(Node *node) const {
+    G4_INST *inst = node->getInstructions()->front();
+
+    if (!instCanUse2xSP(inst)) {
+      return false;
+    }
+
+    // All instructions in the same block have the same type
+    if (instType != Type_UNDEF && inst->getDst()->getType() != instType) {
+      return false;
+    }
+
+    // All instructions in the block have the same GRF size
+    if (!checkGRFSize(node)) {
+      return false;
+    }
+
+    // For non-first blocks, the inst's src0 should have the same registers as
+    // the dst of instructions in previous block
+    if (preBlock && !hasSameOperand(inst)) {
+      return false;
+    }
+
+    return true;
+  }
 
   // Check if block is good or not:
   //     GRF size should be <= ACC number
