@@ -11,6 +11,7 @@ SPDX-License-Identifier: MIT
 #include "BitSet.h"
 #include "G4_BB.hpp"
 #include "G4_Kernel.hpp"
+#include "PointsToAnalysis.h"
 
 using namespace vISA;
 
@@ -295,7 +296,7 @@ void PostDom::run() {
 
       // Common BBs appear in numInstances map with second value == bb->Succs
       // count
-      for (auto commonBBs : numInstances) {
+      for (const auto &commonBBs : numInstances) {
         if (commonBBs.second == bb->Succs.size())
           tmp.insert(commonBBs.first);
       }
@@ -1016,7 +1017,7 @@ void VarReferences::reset() {
 void VarReferences::run() {
   for (auto bb : kernel.fg) {
     for (auto inst : *bb) {
-      if (inst->isPseudoKill())
+      if (!reportPseudoKill && inst->isPseudoKill())
         continue;
       unsigned int lb = 0, rb = 0;
       auto dst = inst->getDst();
@@ -1031,6 +1032,17 @@ void VarReferences::run() {
           }
           auto &Defs = VarRefs[topdcl].first;
           Defs.push_back(std::make_tuple(inst, bb, lb, rb));
+
+          if (p2a && dst->isIndirect()) {
+            auto *pointees = p2a->getAllInPointsTo(topdcl->getRegVar());
+            vISA_ASSERT(pointees, "expecting valid pointee list");
+            for (const auto& pointee : *pointees) {
+              auto &Defs = VarRefs[pointee.var->getDeclare()->getRootDeclare()].first;
+              // lb, rb are both unknown for indirects
+              Defs.push_back(
+                  std::make_tuple(inst, bb, UnknownBound, UnknownBound));
+            }
+          }
         }
       }
 
@@ -1056,6 +1068,16 @@ void VarReferences::run() {
           if (topdcl) {
             auto &Uses = VarRefs[topdcl].second;
             Uses.push_back(std::make_tuple(inst, bb));
+
+            if (p2a && src->isIndirect()) {
+              auto *pointees = p2a->getAllInPointsTo(topdcl->getRegVar());
+              vISA_ASSERT(pointees, "expecting valid pointee list");
+              for (const auto &pointee : *pointees) {
+                auto &Uses =
+                    VarRefs[pointee.var->getDeclare()->getRootDeclare()].second;
+                Uses.push_back(std::make_tuple(inst, bb));
+              }
+            }
           }
         }
       }
