@@ -460,6 +460,13 @@ bool AccSubPass::isAccCandidate(G4_INST *inst, int &lastUse, bool &mustBeAcc0,
     return false;
   }
 
+  if (builder.usesStack() && (dst->getTopDcl() == kernel.fg.getFramePtrDcl() ||
+                              dst->getTopDcl() == kernel.fg.getStackPtrDcl() ||
+                              dst->getTopDcl() == kernel.fg.getStackPtrDcl() ||
+                              dst->getTopDcl() == kernel.fg.pseudoVCEDcl)) {
+    return false;
+  }
+
   if (!IS_TYPE_FLOAT_FOR_ACC(dst->getType())) {
     isAllFloat = false;
   }
@@ -540,11 +547,19 @@ bool AccSubPass::isAccCandidate(G4_INST *inst, int &lastUse, bool &mustBeAcc0,
             }
           }
         }
-        // Q: What's the purpose of this check?
-        if (!IS_TYPE_FLOAT_FOR_ACC(useInst->getSrc(2)->getType()) ||
-            (useInst->getDst() &&
-             !IS_TYPE_FLOAT_FOR_ACC(useInst->getDst()->getType()))) {
-          return false;
+        if (kernel.fg.builder->removedAccRestrictionsAsGRF()) {
+          if (!IS_TYPE_FLOAT_ALL(useInst->getSrc(2)->getType()) ||
+              (useInst->getDst() &&
+               !IS_TYPE_FLOAT_ALL(useInst->getDst()->getType()))) {
+            return false;
+          }
+        } else {
+          // Q: What's the purpose of this check?
+          if (!IS_TYPE_FLOAT_FOR_ACC(useInst->getSrc(2)->getType()) ||
+              (useInst->getDst() &&
+               !IS_TYPE_FLOAT_FOR_ACC(useInst->getDst()->getType()))) {
+            return false;
+          }
         }
         break;
       case Opnd_src1:
@@ -791,7 +806,7 @@ bool AccSubPass::replaceDstWithAcc(G4_INST *inst, int accNum) {
     if (kernel.fg.builder->removedAccRestrictionsAsGRF()) {
       accSrc = builder.createSrcRegRegion(
           oldSrc->getModifier(), Direct, accReg, (short)accNum, 0,
-          builder.getRegionStride1(), oldSrc->getType());
+          oldSrc->getRegion(), oldSrc->getType());
     } else {
       accSrc = builder.createSrcRegRegion(
           oldSrc->getModifier(), Direct, accReg, (short)accNum, 0,
@@ -912,6 +927,7 @@ struct AccAssignment {
   // on inst type/size) we have to spill active intervals that occupy acc0/acc1.
   // the pre-assigned interavl is also pushed to active list
   void handlePreAssignedInterval(AccInterval *interval) {
+    vISA_ASSERT(interval->assignedAcc >= 0, "interval->assignedAcc can't be negative");
     if (!freeAccs[interval->assignedAcc]) {
       spillInterval(interval->assignedAcc);
     }
