@@ -64,7 +64,7 @@ G4_INST *CoalesceSpillFills::generateCoalescedFill(G4_SrcRegRegion *header,
   // Generate split send instruction with specified payload size and offset
   // Construct fillDst
   const char *dclName = kernel.fg.builder->getNameString(
-      32, "COAL_FILL_%d", kernel.Declares.size());
+      32, "COAL_FILL_%" PRIu64, kernel.Declares.size());
   auto fillDcl = kernel.fg.builder->createDeclare(
       dclName, G4_GRF, kernel.numEltPerGRF<Type_UD>(), dclSize, Type_UD,
       DeclareType::CoalescedSpillFill);
@@ -89,71 +89,13 @@ G4_INST *CoalesceSpillFills::generateCoalescedFill(G4_SrcRegRegion *header,
   return fillInst;
 }
 
-void CoalesceSpillFills::copyToOldFills(
-    G4_DstRegRegion *coalescedFillDst,
-    std::list<
-        std::pair<G4_DstRegRegion *, std::pair<unsigned int, unsigned int>>>
-        indFills,
-    INST_LIST_ITER f, G4_BB *bb, int srcCISAOff) {
-  // Copy data from coalesced fill in to older fills.
-  // This way we don't carry entire coalesced payload
-  // till last fill.
-  for (auto oldFill : indFills) {
-    unsigned int numGRFs =
-        (oldFill.first->getRightBound() - oldFill.first->getLeftBound() +
-         kernel.numEltPerGRF<Type_UB>() - 1) /
-        kernel.numEltPerGRF<Type_UB>();
-    unsigned int rowOff = 0;
-    // TODO: Check for > 2 GRF dst
-    while (numGRFs > 0) {
-      G4_ExecSize simdSize = g4::SIMD8;
-
-      unsigned int off = oldFill.second.first;
-      unsigned int size = oldFill.second.second;
-
-      static const int BYTES_PER_HEXWORD = 32;
-
-      unsigned int scratchOff = ((unsigned)coalescedFillDst->getInst()
-                                     ->getMsgDesc()
-                                     ->getOffset()
-                                     ->immOff) /
-                                BYTES_PER_HEXWORD;
-
-      // Scratch msg offset is always equal or lower than individual fills
-      unsigned int offToUse = off - scratchOff + rowOff;
-
-      if (size > g4::SIMD8)
-        simdSize = g4::SIMD16;
-
-      G4_DstRegRegion *movDst = kernel.fg.builder->createDst(
-          oldFill.first->getBase(), rowOff, 0, 1, Type_UD);
-
-      G4_SrcRegRegion *src = kernel.fg.builder->createSrc(
-          coalescedFillDst->getBase(), offToUse, 0,
-          kernel.fg.builder->getRegionStride1(), Type_UD);
-
-      G4_INST *copy = kernel.fg.builder->createMov(simdSize, movDst, src,
-                                                   InstOpt_WriteEnable, false);
-
-      bb->insertBefore(f, copy);
-
-      if (gra.EUFusionNoMaskWANeeded()) {
-        gra.addEUFusionNoMaskWAInst(bb, copy);
-      }
-
-      numGRFs -= simdSize == 8 ? 1 : 2;
-      rowOff += simdSize == 8 ? 1 : 2;
-    }
-  }
-}
-
 G4_Declare *
 CoalesceSpillFills::createCoalescedSpillDcl(unsigned int payloadSize) {
   // Construct spill src
   const char *dclName = nullptr;
   G4_Declare *spillDcl = nullptr;
 
-  dclName = kernel.fg.builder->getNameString(32, "COAL_SPILL_%d",
+  dclName = kernel.fg.builder->getNameString(32, "COAL_SPILL_%" PRIu64,
                                              kernel.Declares.size());
   spillDcl = kernel.fg.builder->createDeclare(
       dclName, G4_GRF, kernel.numEltPerGRF<Type_UD>(), payloadSize, Type_UD,
@@ -179,7 +121,7 @@ void CoalesceSpillFills::coalesceSpills(
 
   std::set<G4_Declare *> declares;
   unsigned int minRow = UINT_MAX;
-  for (auto d : coalesceableSpills) {
+  for (const auto &d : coalesceableSpills) {
     auto src1Opnd = (*d)->getSrc(1)->asSrcRegRegion();
     auto curRow = src1Opnd->getLeftBound() / kernel.numEltPerGRF<Type_UB>();
     declares.insert(src1Opnd->getTopDcl());
@@ -204,7 +146,7 @@ void CoalesceSpillFills::coalesceSpills(
   coalescedSpillSrc->getInst()->inheritDIFrom(leadInst);
 
   if (declares.size() != 1) {
-    for (auto c : coalesceableSpills) {
+    for (const auto &c : coalesceableSpills) {
       unsigned int scratchOffset, scratchSize;
       getScratchMsgInfo((*c), scratchOffset, scratchSize);
 
@@ -218,7 +160,7 @@ void CoalesceSpillFills::coalesceSpills(
   auto f = coalesceableSpills.back();
   f++;
 
-  for (auto spill : coalesceableSpills) {
+  for (const auto &spill : coalesceableSpills) {
     gra.incRA.markForIntfUpdate(
         (*spill)->asSpillIntrinsic()->getPayload()->getTopDcl());
     bb->erase(spill);
@@ -247,7 +189,7 @@ void CoalesceSpillFills::coalesceFills(
   // dclSize could be larger than payload size when
   // 2 variables across scratch writes are coalesced.
   unsigned int dclSize = payloadSize;
-  for (auto c : coalesceableFills) {
+  for (const auto &c : coalesceableFills) {
     unsigned int scratchOffset, scratchSize;
     auto fill = (*c);
     getScratchMsgInfo((*c), scratchOffset, scratchSize);
@@ -276,7 +218,7 @@ void CoalesceSpillFills::coalesceFills(
                             gra.isEvenAligned(leadInst->getDst()->getTopDcl()));
   newFill->inheritDIFrom(leadInst);
 
-  for (auto c : coalesceableFills) {
+  for (const auto &c : coalesceableFills) {
     unsigned int scratchOffset, scratchSize;
     getScratchMsgInfo((*c), scratchOffset, scratchSize);
 
@@ -289,7 +231,7 @@ void CoalesceSpillFills::coalesceFills(
   auto f = coalesceableFills.front();
   f++;
 
-  for (auto fill : coalesceableFills) {
+  for (const auto &fill : coalesceableFills) {
     if (fill == f) {
       f++;
     }
@@ -321,7 +263,7 @@ bool CoalesceSpillFills::fillHeuristic(
   min = 0xffffffff, max = 0;
   G4_Declare *header =
       (*coalesceableFills.front())->asFillIntrinsic()->getHeader()->getTopDcl();
-  for (auto f : coalesceableFills) {
+  for (const auto &f : coalesceableFills) {
     if ((*f)->asFillIntrinsic()->getHeader()->getTopDcl() != header &&
         !(*f)->asFillIntrinsic()->getFP())
       return false;
@@ -364,7 +306,7 @@ bool CoalesceSpillFills::fillHeuristic(
   const int maxDclSize = 128;
 
   std::map<G4_Declare *, std::bitset<maxDclSize>> allRows;
-  for (auto c : coalesceableFills) {
+  for (const auto &c : coalesceableFills) {
     auto topdcl = (*c)->getDst()->getTopDcl();
 
     unsigned int scratchOffset, scratchSize;
@@ -557,7 +499,7 @@ void CoalesceSpillFills::keepConsecutiveSpills(
   G4_Declare *header =
       (*instList.front())->asSpillIntrinsic()->getHeader()->getTopDcl();
 
-  for (auto instIt : instList) {
+  for (const auto &instIt : instList) {
     auto inst = (*instIt);
 
     if (inst->asSpillIntrinsic()->getHeader()->getTopDcl() != header &&
@@ -619,7 +561,7 @@ void CoalesceSpillFills::keepConsecutiveSpills(
               auto curSrc1Row =
                   (*(*spillIt))->getSrc(1)->asSrcRegRegion()->getRegOff();
               bool success = true;
-              for (auto candidate : allowed) {
+              for (const auto &candidate : allowed) {
                 unsigned int candOffset, candSize;
                 getScratchMsgInfo(*candidate, candOffset, candSize);
                 auto prevSrc1Row =
@@ -658,8 +600,8 @@ void CoalesceSpillFills::keepConsecutiveSpills(
     unsigned int slots = maxOffset - minOffset + 1;
     if (slots == 2 || slots == 4) {
       // Insert coalescable spills in order of appearance
-      for (auto origInst : origInstList) {
-        for (auto allowedSpills : allowed) {
+      for (const auto &origInst : origInstList) {
+        for (const auto &allowedSpills : allowed) {
           if (*origInst == *allowedSpills) {
             coalescable.push_back(origInst);
             break;
@@ -678,7 +620,7 @@ void CoalesceSpillFills::keepConsecutiveSpills(
     }
   }
 
-  instList = origInstList;
+  instList = std::move(origInstList);
   for (auto coalIt = coalescable.begin(), instIt = instList.begin();
        coalIt != coalescable.end(); coalIt++) {
     if (*instIt == *coalIt)
@@ -757,7 +699,7 @@ CoalesceSpillFills::analyzeFillCoalescing(std::list<INST_LIST_ITER> &instList,
       fillHeuristic(coalesceableFills, instList, origInstList, min, max);
   if (!heuristic) {
     coalesceableFills.clear();
-    instList = origInstList;
+    instList = std::move(origInstList);
     instList.pop_front();
   }
 
@@ -814,7 +756,7 @@ bool CoalesceSpillFills::overlap(G4_INST *inst1, G4_INST *inst2,
 
 bool CoalesceSpillFills::overlap(G4_INST *inst,
                                  std::list<INST_LIST_ITER> &allInsts) {
-  for (auto sp : allInsts) {
+  for (const auto &sp : allInsts) {
     bool t;
     auto spillInst = (*sp);
     if (overlap(inst, spillInst, t))
@@ -887,7 +829,7 @@ bool CoalesceSpillFills::replaceCoalescedOperands(G4_INST *inst) {
 bool CoalesceSpillFills::allSpillsSameVar(std::list<INST_LIST_ITER> &spills) {
   // Return true if all vars in spills list have same dcl
   G4_Declare *dcl = nullptr;
-  for (auto s : spills) {
+  for (const auto &s : spills) {
     auto topdcl = (*s)->getSrc(1)->getTopDcl();
 
     if (!dcl)
@@ -1287,7 +1229,7 @@ void CoalesceSpillFills::fixSendsSrcOverlap() {
           // force spills. So we simply choose
           // src1 of sends.
           const char *dclName = kernel.fg.builder->getNameString(
-              32, "COPY_%d", kernel.Declares.size());
+              32, "COPY_%zu", kernel.Declares.size());
           G4_Declare *copyDcl = kernel.fg.builder->createDeclare(
               dclName, G4_GRF, kernel.numEltPerGRF<Type_UD>(),
               src1->getTopDcl()->getNumRows(), Type_UD);
@@ -1479,7 +1421,7 @@ void CoalesceSpillFills::removeRedundantSplitMovs() {
           auto baseIt = dstSrcRowMapping.find(dstRowStart);
           if (baseIt != dstSrcRowMapping.end()) {
             auto base = dstSrcRowMapping.find(dstRowStart)->second;
-            for (auto m : dstSrcRowMapping) {
+            for (const auto &m : dstSrcRowMapping) {
               unsigned int curRow = m.first - dstRowStart;
               if (m.second == INT_MAX) {
                 success = false;
@@ -1502,7 +1444,7 @@ void CoalesceSpillFills::removeRedundantSplitMovs() {
                   inst->getSrc(1)->getType());
               inst->setSrc(sendSrc1, 1);
 
-              for (auto c : copies) {
+              for (const auto &c : copies) {
                 auto defDcl = (*c.second)->getDst()->getTopDcl();
                 auto it = movs.find(defDcl);
                 if (it == movs.end()) {
@@ -1546,13 +1488,13 @@ void CoalesceSpillFills::removeRedundantSplitMovs() {
     }
   }
 
-  for (auto mov : movs) {
+  for (const auto &mov : movs) {
     auto dcl = mov.first;
     auto numRefs = mov.second.first;
     auto &allMovs = mov.second.second;
 
     if (numRefs == 0 && !dcl->getAddressed()) {
-      for (auto m : allMovs) {
+      for (const auto &m : allMovs) {
         auto bb = m.first;
         auto iter = m.second;
         bb->erase(iter);
@@ -1878,13 +1820,13 @@ void CoalesceSpillFills::removeRedundantWrites() {
   }
 
   std::map<G4_INST *, std::pair<INST_LIST_ITER, G4_BB *>> spillToRemove;
-  for (auto scratchAccess : scratchOffsetAccess) {
+  for (const auto &scratchAccess : scratchOffsetAccess) {
     if (scratchAccess.second.second.size() == 0 &&
         scratchAccess.second.first.size() > 0) {
       // 0 fills for scratch slot
       // Check whether all spill slots have 0 fills
       // in case spills are coalesced.
-      for (auto spill : scratchAccess.second.first) {
+      for (const auto &spill : scratchAccess.second.first) {
         bool spillRequired = false;
         unsigned int offset, size;
         getScratchMsgInfo(*spill.second, offset, size);
@@ -1908,7 +1850,7 @@ void CoalesceSpillFills::removeRedundantWrites() {
       // 0 spills for scratch slot, non-zero fills
       // Check whether all fill slots have 0 spills
       // in case fills are coalesced.
-      for (auto fill : scratchAccess.second.second) {
+      for (const auto &fill : scratchAccess.second.second) {
         if (*fill.second == gra.getRestoreBE_FPInst())
           continue;
 
@@ -1933,7 +1875,7 @@ void CoalesceSpillFills::removeRedundantWrites() {
     }
   }
 
-  for (auto removeSp : spillToRemove) {
+  for (const auto &removeSp : spillToRemove) {
     G4_BB *bb = removeSp.second.second;
     auto *inst = *removeSp.second.first;
     if ((inst->isSpillIntrinsic() &&
