@@ -59,7 +59,8 @@ public:
   VISAKernelImpl(enum VISA_BUILD_TYPE type, CISA_IR_Builder *cisaBuilder,
                  const char *name, unsigned int funcId)
       : m_mem(4096), m_CISABuilder(cisaBuilder),
-        m_options(cisaBuilder->getOptions()), m_functionId(funcId) {
+        m_options(cisaBuilder->getOptions()), m_functionId(funcId),
+        m_fastPathOpndPool() {
     mBuildOption = m_CISABuilder->getBuilderOption();
     m_magic_number = COMMON_ISA_MAGIC_NUM;
     m_major_version = m_CISABuilder->getMajorVersion();
@@ -115,12 +116,14 @@ public:
   void *alloc(size_t sz) { return m_mem.alloc(sz); }
 
   virtual ~VISAKernelImpl();
+  VISAKernelImpl(const VISAKernelImpl&) = delete;
+  VISAKernelImpl& operator=(const VISAKernelImpl&) = delete;
 
   void *operator new(size_t sz, vISA::Mem_Manager &m) { return m.alloc(sz); };
 
   vISA::Attributes *getKernelAttributes() { return m_kernelAttrs; }
   // Temporary function to move options to attributes!
-  void finalizeAttributes();
+  bool finalizeAttributes();
 
   void setName(const char *n);
   const char *getName() const { return m_name.c_str(); }
@@ -480,7 +483,7 @@ public:
   VISA_BUILDER_API int
   AppendVISACFFunctionCallInst(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
                                VISA_Exec_Size executionSize,
-                               std::string funcName, unsigned char argSize,
+                               const std::string& funcName, unsigned char argSize,
                                unsigned char returnSize) override;
 
   VISA_BUILDER_API int
@@ -489,7 +492,7 @@ public:
                                    VISA_VectorOpnd *funcAddr, uint8_t argSize,
                                    uint8_t returnSize) override;
 
-  VISA_BUILDER_API int AppendVISACFSymbolInst(std::string symbolName,
+  VISA_BUILDER_API int AppendVISACFSymbolInst(const std::string& symbolName,
                                               VISA_VectorOpnd *dst) override;
 
   VISA_BUILDER_API int
@@ -836,6 +839,12 @@ public:
       LSC_DATA_SHAPE_BLOCK2D dataShape, VISA_RawOpnd *dstData,
       VISA_VectorOpnd *src0Addrs[LSC_BLOCK2D_ADDR_PARAMS],
       int xImmOffset, int yImmOffset, VISA_RawOpnd *src1Data) override;
+  VISA_BUILDER_API int AppendVISALscUntypedBlock2DInst(
+      LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
+      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts,
+      LSC_DATA_SHAPE_BLOCK2D dataShape, VISA_RawOpnd *dstData,
+      VISA_VectorOpnd *src0AddrPayload,
+      int xImmOffset, int yImmOffset, VISA_RawOpnd *src1Data) override;
   VISA_BUILDER_API int AppendVISALscTypedLoad(
       LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrModel,
@@ -882,6 +891,21 @@ public:
                                           LSC_SCOPE scope) override;
 
 
+  VISA_BUILDER_API int AppendVISALscTypedBlock2DInst(
+      LSC_OP op, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrModel,
+      LSC_DATA_SHAPE_TYPED_BLOCK2D dataShape2D,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *dstData, VISA_VectorOpnd *blockStartX,
+      VISA_VectorOpnd *blockStartY, int xImmOffset, int yImmOffset,
+      VISA_RawOpnd *src1Data) override;
+
+  VISA_BUILDER_API int AppendVISALscUntypedAppendCounterAtomicInst(
+      LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
+      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts,
+      LSC_ADDR_TYPE addrType, LSC_DATA_SHAPE data,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *dst, VISA_RawOpnd *srcData) override;
+
   VISA_BUILDER_API int
   AppendVISANamedBarrierWait(VISA_VectorOpnd *barrierId) override;
 
@@ -892,7 +916,6 @@ public:
   /********** APPEND INSTRUCTION APIS END   ******************/
 
   /********** APPEND 3D Instructions START ******************/
-
   VISA_BUILDER_API int
   AppendVISA3dSampler(VISASampler3DSubOpCode subOpcode, bool pixelNullMask,
                       bool cpsEnable, bool uniformSampler, VISA_PredOpnd *pred,
@@ -900,6 +923,18 @@ public:
                       VISAChannelMask srcChannel, VISA_VectorOpnd *aoffimmi,
                       VISA_StateOpndHandle *sampler,
                       VISA_StateOpndHandle *surface,
+                      VISA_RawOpnd *pairedSurface,
+                      VISA_RawOpnd *dst, int numMsgSpecificOpnds,
+                      VISA_RawOpnd **opndArray) override;
+
+  VISA_BUILDER_API int
+  AppendVISA3dSampler(VISASampler3DSubOpCode subOpcode, bool pixelNullMask,
+                      bool cpsEnable, bool uniformSampler, VISA_PredOpnd *pred,
+                      VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+                      VISAChannelMask srcChannel, VISA_VectorOpnd *aoffimmi,
+                      VISA_StateOpndHandle *sampler, unsigned int samplerIdx,
+                      VISA_StateOpndHandle *surface, unsigned int surfaceIdx,
+                      VISA_RawOpnd *pairedSurface,
                       VISA_RawOpnd *dst, int numMsgSpecificOpnds,
                       VISA_RawOpnd **opndArray) override;
 
@@ -908,6 +943,17 @@ public:
                    VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
                    VISA_Exec_Size executionSize, VISAChannelMask srcChannel,
                    VISA_VectorOpnd *aoffimmi, VISA_StateOpndHandle *surface,
+                   VISA_RawOpnd *pairedSurface,
+                   VISA_RawOpnd *dst, int numMsgSpecificOpnds,
+                   VISA_RawOpnd **opndArray) override;
+
+  VISA_BUILDER_API int
+  AppendVISA3dLoad(VISASampler3DSubOpCode subOpcode, bool pixelNullMask,
+                   VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+                   VISA_Exec_Size executionSize, VISAChannelMask srcChannel,
+                   VISA_VectorOpnd *aoffimmi,
+                   VISA_StateOpndHandle *surface, unsigned int surfaceIndex,
+                   VISA_RawOpnd *pairedSurface,
                    VISA_RawOpnd *dst, int numMsgSpecificOpnds,
                    VISA_RawOpnd **opndArray) override;
 
@@ -916,14 +962,25 @@ public:
       VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
       VISASourceSingleChannel srcChannel, VISA_VectorOpnd *aoffimmi,
       VISA_StateOpndHandle *sampler, VISA_StateOpndHandle *surface,
+      VISA_RawOpnd *pairedSurface,
+      VISA_RawOpnd *dst, int numMsgSpecificOpnds,
+      VISA_RawOpnd **opndArray) override;
+
+  VISA_BUILDER_API int AppendVISA3dGather4(
+      VISASampler3DSubOpCode subOpcode, bool pixelNullMask, VISA_PredOpnd *pred,
+      VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+      VISASourceSingleChannel srcChannel, VISA_VectorOpnd *aoffimmi,
+      VISA_StateOpndHandle *sampler, unsigned samplerIndex,
+      VISA_StateOpndHandle *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *pairedSurface,
       VISA_RawOpnd *dst, int numMsgSpecificOpnds,
       VISA_RawOpnd **opndArray) override;
 
   VISA_BUILDER_API int
   AppendVISA3dInfo(VISASampler3DSubOpCode subOpcode, VISA_EMask_Ctrl emask,
                    VISA_Exec_Size executionSize, VISAChannelMask srcChannel,
-                   VISA_StateOpndHandle *surface, VISA_RawOpnd *lod,
-                   VISA_RawOpnd *dst) override;
+                   VISA_StateOpndHandle *surface, unsigned int surfaceIndex,
+                   VISA_RawOpnd *lod, VISA_RawOpnd *dst) override;
 
   VISA_BUILDER_API int AppendVISA3dRTWrite(
       VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
@@ -970,8 +1027,8 @@ public:
   /// This get the same information as GetGenRelocEntryBuffer, but in different
   /// foramt
   VISA_BUILDER_API int GetRelocations(RelocListType &relocs) override;
-  VISA_BUILDER_API int GetGTPinBuffer(void *&buffer,
-                                      unsigned int &size) override;
+  VISA_BUILDER_API int GetGTPinBuffer(void *&buffer, unsigned int &size,
+                                      unsigned int scratchOffset) override;
   VISA_BUILDER_API int SetGTPinInit(void *buffer) override;
   VISA_BUILDER_API int GetFreeGRFInfo(void *&buffer,
                                       unsigned int &size) override;
@@ -1008,6 +1065,9 @@ public:
   /// Get vISA asm of the kernel function
   VISA_BUILDER_API std::string getVISAAsm() const override;
 
+  /// set or get current block frequency information
+  VISA_BUILDER_API int encodeBlockFrequency(uint64_t digits, int16_t scale) override;
+
   // Gets the VISA string format for the variable
   VISA_BUILDER_API std::string getVarName(VISA_GenVar *decl) const override;
   VISA_BUILDER_API std::string getVarName(VISA_PredVar *decl) const override;
@@ -1052,7 +1112,9 @@ public:
       bool cpsEnable, bool uniformSampler, VISA_PredOpnd *pred,
       VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
       ChannelMask srcChannel, VISA_VectorOpnd *aoffimmi,
-      VISA_StateOpndHandle *sampler, VISA_StateOpndHandle *surface,
+      VISA_StateOpndHandle *sampler, unsigned int samplerIdx,
+      VISA_StateOpndHandle *surface, unsigned int surfaceIdx,
+      VISA_RawOpnd *pairedSurface,
       VISA_RawOpnd *dst, unsigned int numMsgSpecificOpnds,
       VISA_RawOpnd **opndArray);
 
@@ -1088,7 +1150,7 @@ public:
   void setJitInfo(vISA::FINALIZER_INFO *jitInfo) { m_jitInfo = jitInfo; }
 
   std::string getOutputAsmPath() const { return m_asmName; }
-  void setOutputAsmPath(std::string val) { m_asmName = val; }
+  void setOutputAsmPath(const std::string& val) { m_asmName = val; }
 
   int compileFastPath();
 
