@@ -28,16 +28,10 @@ SPDX-License-Identifier: MIT
 using namespace ::iga;
 
 DEFINE_GED_SOURCE_ACCESSORS_01(GED_ADDR_MODE, AddrMode)
-
-DEFINE_GED_SOURCE_ACCESSORS_012(GED_REG_FILE, RegFile)
 DEFINE_GED_SOURCE_ACCESSORS_01(int32_t, AddrImm)
 DEFINE_GED_SOURCE_ACCESSORS_01(uint32_t, AddrSubRegNum)
 DEFINE_GED_SOURCE_ACCESSORS_01(uint32_t, Width)
-
-//        DEFINE_GED_SOURCE_ACCESSORS_INLINE_012(GED_REG_FILE, RegFile)
 DEFINE_GED_SOURCE_ACCESSORS_012(GED_DATA_TYPE, DataType)
-DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, RegNum)
-DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, SubRegNum)
 DEFINE_GED_SOURCE_ACCESSORS_012(GED_MATH_MACRO_EXT, MathMacroExt)
 DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, VertStride)
 DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, ChanSel)
@@ -45,6 +39,9 @@ DEFINE_GED_SOURCE_ACCESSORS_012(GED_REP_CTRL, RepCtrl)
 DEFINE_GED_SOURCE_ACCESSORS_012(GED_SRC_MOD, SrcMod)
 DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, HorzStride)
 
+DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, RegNum)
+DEFINE_GED_SOURCE_ACCESSORS_012(uint32_t, SubRegNum)
+DEFINE_GED_SOURCE_ACCESSORS_012(GED_REG_FILE, RegFile)
 static void setImmValKind(Type t, ImmVal &val) {
   switch (t) {
   case Type::B:
@@ -191,7 +188,7 @@ Kernel *Decoder::decodeKernel(const void *binary, size_t binarySize,
     auto blockStarts =
         Block::inferBlocks(errorHandler(), kernel->getMemManager(), insts);
     int id = 1;
-    for (auto bitr : blockStarts) {
+    for (const auto &bitr : blockStarts) {
       bitr.second->setID(id++);
       kernel->appendBlock(bitr.second);
     }
@@ -1143,6 +1140,41 @@ void Decoder::decodeSendInfoXeHPG(SendDescodeInfo &sdi) {
   }
 }
 
+void Decoder::decodeSendInfoXe2(SendDescodeInfo &sdi) {
+  // In Xe2 is similar to XeHP/XeHPG/XeHPC
+  //  - Given an Imm ExDesc Src1Len is an EU field
+  //  - When ExDesc is a reg we get the extra field
+  //    ExDescImm to add more bits to the extended descriptor
+  //  - CPS goes away unconditionally (this is now ExDescImm[11])
+  //  - if SFID is ugm then ExBSO holds an ExDescImm bit as well
+  //  - if ExDesc is a reg and SFID is not ugm, ExBSO bit denotes if
+  //    Src1Len in EU field or in ExDesc
+  sdi.sfid = m_subfunc.send;
+  if (sdi.exDesc.isReg()) {
+    GED_DECODE_RAW_TO(ExMsgDescImm, sdi.exImmOffDesc);
+    sdi.hasExBSO = false;
+    // GED gives us the overlaps too (ExBSO and ExDescImm[15]), so
+    // we must mask them out for non-UGM
+    if (m_subfunc.send != SFID::UGM) {
+      // ExDescImm overlaps ExBSO for non-UGM; clear it
+      sdi.exImmOffDesc &= ~(1 << 15); // ExBSO overlaps ExDescImm[15]
+
+      // if ExBSO is set, decode Src1Length from EU field
+      GED_DECODE_RAW(uint32_t, exBSO, ExBSO);
+      sdi.hasExBSO = exBSO != 0;
+      if (sdi.hasExBSO)
+        GED_DECODE_RAW_TO(Src1Length, sdi.src1Len);
+    } else {
+      // for UGM, src1Length must be in EU field
+      GED_DECODE_RAW_TO(Src1Length, sdi.src1Len);
+    }
+       sdi.exImmOffDesc &= ~(0x7 << 16); // ExDesc.AddrSubReg
+  } else {
+    // exDes is Imm, src1.len must be in EU field
+    GED_DECODE_RAW_TO(Src1Length, sdi.src1Len);
+  }
+  decodeMLenRlenFromDesc(sdi.desc, sdi.src0Len, sdi.dstLen);
+}
 
 Instruction *Decoder::decodeSendInstruction(Kernel &kernel) {
   SendDescodeInfo sdi;
@@ -1154,9 +1186,11 @@ Instruction *Decoder::decodeSendInstruction(Kernel &kernel) {
     decodeSendInfoXe(sdi);
   } else if (platform() == Platform::XE_HP) {
     decodeSendInfoXeHP(sdi);
-  } else if (platform() >= Platform::XE_HPG
-  ) {
+  } else if (platform() >= Platform::XE_HPG &&
+             platform() < Platform::XE2) {
     decodeSendInfoXeHPG(sdi);
+  } else if (platform() >= Platform::XE2) {
+    decodeSendInfoXe2(sdi);
   } else {
     IGA_ASSERT_FALSE("unsupported platform");
   }
@@ -1165,6 +1199,7 @@ Instruction *Decoder::decodeSendInstruction(Kernel &kernel) {
   Instruction *inst = kernel.createSendInstruction(
       *m_opSpec, sdi.sfid, fri.pred, fri.reg, decodeExecSize(),
       decodeChannelOffset(), decodeMaskCtrl(),
+      sdi.exImmOffDesc,
       sdi.exDesc, sdi.desc);
 
   if ((m_opSpec->format & OpSpec::Format::SEND_BINARY) ==
