@@ -539,7 +539,6 @@ size_t DepSetBuilder::DpasMacroBuilder::getNumberOfSuppresionGroups(
   return 0;
 }
 
-
 size_t DepSetBuilder::DpasMacroBuilder::formSrcSuppressionBlock(
     InstListIterator startIt, uint32_t srcIdx) {
   // get the candidate block
@@ -743,10 +742,9 @@ const Instruction &DepSetBuilder::DpasMacroBuilder::formMacro(size_t &dpasCnt) {
     return **cur;
   }
 
-  bool formMacroForSrc1 = false;
 
-  if (!formMacroForSrc1)
-    dpasCnt = std::max(dpasCnt, formSrcSuppressionBlock(m_firstDpasIt, 1));
+
+  dpasCnt = std::max(dpasCnt, formSrcSuppressionBlock(m_firstDpasIt, 1));
   dpasCnt = std::max(dpasCnt, formSrcSuppressionBlock(m_firstDpasIt, 2));
 
   if (dpasCnt == 1) {
@@ -823,7 +821,7 @@ void DepSetBuilder::DpasMacroBuilder::setBits(BitSet<> &bit_set,
 void DepSetBuilder::DpasMacroBuilder::setDstSrcBits(
     const SrcRegRangeType &src_range, const DstRegRangeType &dst_range,
     BitSet<> &src_bits, BitSet<> &dst_bits) const {
-  for (auto regs : src_range) {
+  for (const auto &regs : src_range) {
     setBits(src_bits, regs.first, regs.second);
   }
   setBits(dst_bits, dst_range.first, dst_range.second);
@@ -960,7 +958,7 @@ void DepSet::setInputsFlagDep() {
     fByteOff += execOff / 8; // move over by ARF offset
     size_t execSize = static_cast<size_t>(m_instruction->getExecSize());
     size_t addr = (size_t)m_DB.getARF_F_START() + fByteOff;
-    addFBytes(addr, execSize / 8);
+    addFBytes(addr, execSize);
     m_bucketList.push_back(addr / m_DB.getBYTES_PER_BUCKET());
   }
 }
@@ -971,11 +969,11 @@ void DepSet::setInputsSendDescDep() {
   // set up reg footprint for desc and exdesc if they are registers
   auto desc = m_instruction->getMsgDescriptor();
   if (desc.isReg()) {
-    addA_D(desc.reg); // e.g. a0.0
+    addABytesAndBukets(desc.reg.regNum);
   }
   auto exDesc = m_instruction->getExtMsgDescriptor();
   if (exDesc.isReg()) {
-    addA_D(exDesc.reg); // e.g. a0.0
+    addABytesAndBukets(exDesc.reg.regNum);
   }
 }
 
@@ -1096,9 +1094,9 @@ void DepSet::setInputsSrcDep() {
                                    : m_instruction->getSrc1Length();
           // if we can't tell the number of registers
           // (e.g. the descriptor is in a register),
-          // then we must conservatively assume the worst (31)
+          // then we must conservatively assume the worst (32)
           if (nregs < 0)
-            nregs = 31;
+            nregs = 32;
           uint32_t regNum = op.getDirRegRef().regNum;
           for (uint32_t i = 0; i < (uint32_t)nregs; i++) {
             if ((regNum + i) >= m_DB.getGRF_REGS()) {
@@ -1195,7 +1193,7 @@ void DepSet::setOutputsFlagDep() {
                                                                    // (static_cast<int>(m_instruction->getExecSize())
                                                                    // - 1);
     size_t addr = (size_t)m_DB.getARF_F_START() + fByteOff;
-    addFBytes(addr, execSize / 8);
+    addFBytes(addr, execSize);
     m_bucketList.push_back(addr / m_DB.getBYTES_PER_BUCKET());
   }
 }
@@ -1261,7 +1259,7 @@ void DepSet::setOutputsDstDep() {
       // getDstLength return -1 when it's not able to deduce the length
       // we have to be conservative and use the max
       if (nregs < 0)
-        nregs = 31;
+        nregs = 32;
       for (uint32_t i = 0; i < (uint32_t)nregs; i++) {
         uint32_t regNum = op.getDirRegRef().regNum;
         if ((regNum + i) >= m_DB.getGRF_REGS()) {
@@ -1319,8 +1317,10 @@ void DepSet::setOutputsDstDep() {
   default:
     break;
   }
+
 }
 
+
 DepSet *DepSetBuilder::createDstDepSet(const Instruction &i,
                                        const InstIDs &inst_id_counter,
                                        SWSB_ENCODE_MODE enc_mode) {
@@ -1444,7 +1444,8 @@ bool static const isSpecial(RegName rn) {
   // All the other ARF should have explicitly access so no need
   // the special handling on it
   // Others are SP, IP, IM, DBG
-  if (rn == RegName::ARF_CR || rn == RegName::ARF_SR || rn == RegName::ARF_CE) {
+  if (rn == RegName::ARF_CR || rn == RegName::ARF_SR || rn == RegName::ARF_CE ||
+      rn == RegName::ARF_FC) {
     return true;
   }
   return false;
@@ -1651,6 +1652,7 @@ uint32_t DepSet::addressOf(RegName rnm, const RegRef &rr,
   case RegName::ARF_CR:
   case RegName::ARF_SR:
   case RegName::ARF_CE:
+  case RegName::ARF_FC:
     return getAddressOf(m_DB.getARF_SPECIAL_START(),
                         m_DB.getARF_SPECIAL_BYTES_PER_REG());
   default:
@@ -1681,18 +1683,22 @@ void DepSet::addGrfBytes(size_t reg, size_t subRegBytes, size_t num_bytes) {
   bits->set(grf_addr, num_bytes);
 }
 
-void DepSet::addABytes(size_t reg, size_t subregBytes, size_t num_bytes) {
-  IGA_ASSERT(reg >= 0 && reg < m_DB.getARF_A_REGS(),
+void DepSet::addABytesAndBukets(size_t reg) {
+  IGA_ASSERT(reg < m_DB.getARF_A_REGS(),
              "a# register out of bounds");
-  IGA_ASSERT(subregBytes >= 0 && subregBytes < m_DB.getARF_A_BYTES_PER_REG(),
-             "a# sub register byte out of bounds");
-  IGA_ASSERT(subregBytes >= 0 && subregBytes < m_DB.getARF_A_BYTES_PER_REG(),
-             "a# sub register byte out of bounds");
-  size_t addr =
-      m_DB.getARF_A_START() + m_DB.getARF_A_BYTES_PER_REG() * reg + subregBytes;
+  size_t addr =  m_DB.getARF_A_START() + m_DB.getARF_A_BYTES_PER_REG() * reg;
   IGA_ASSERT(addr < (size_t)m_DB.getARF_A_START() + m_DB.getARF_A_LEN(),
              "a# byte address out of bounds");
+
+  // ARF_A's swsb is tracked at per register base. sub-register number
+  // doesn't matter.
+  auto num_bytes = m_DB.getARF_A_BYTES_PER_REG();
   bits->set(addr, num_bytes);
+  // set the corresponding buckets
+  for (auto i = addr / m_DB.getBYTES_PER_BUCKET();
+       i <= (addr + num_bytes - 1) / m_DB.getBYTES_PER_BUCKET(); i++) {
+    addToBucket((uint32_t)i);
+  }
 }
 void DepSet::addFBytes(size_t fByteOff, size_t num_bytes) {
   bits->set(fByteOff, num_bytes);
