@@ -32,6 +32,7 @@ class KernelViewImpl {
 private:
   KernelViewImpl(const KernelViewImpl &k)
       : m_model(*Model::LookupModel(k.m_model.platform)) {}
+  KernelViewImpl& operator=(const KernelViewImpl&) = delete;
 
 public:
   const Model &m_model;
@@ -136,11 +137,11 @@ kv_t *kv_create(iga_gen_t gen_platf, const void *bytes, size_t bytes_len,
          << "\n";
     };
     if (kvImpl->m_errHandler.hasErrors()) {
-      for (auto d : kvImpl->m_errHandler.getErrors()) {
+      for (const auto& d : kvImpl->m_errHandler.getErrors()) {
         fmtDiag("ERROR", d);
       }
     }
-    for (auto d : kvImpl->m_errHandler.getWarnings()) {
+    for (const auto& d : kvImpl->m_errHandler.getWarnings()) {
       fmtDiag("WARNING", d);
     }
     (void)copyOut(errbuf, errbuf_cap, ss);
@@ -323,6 +324,31 @@ uint32_t kv_get_send_descs(const kv_t *kv, int32_t pc, uint32_t *ex_desc,
   return n;
 }
 
+kv_status_t kv_get_send_exdesc_immoff(const kv_t *kv, int32_t pc,
+                                      uint32_t *exdesc_immoff) {
+  if (!kv || !exdesc_immoff)
+    return KV_INVALID_ARGUMENT;
+  const Instruction *inst = ((KernelViewImpl *)kv)->getInstruction(pc);
+  if (!inst) {
+    return KV_INVALID_ARGUMENT;
+  } else if (!inst->getOpSpec().isAnySendFormat()) {
+    return KV_NON_SEND_INSTRUCTION;
+  } else if (inst->getSendFc() != SFID::UGM) {
+    return KV_DESCRIPTOR_INVALID;
+  }
+
+  const auto exDesc = inst->getExtMsgDescriptor();
+  const auto desc = inst->getMsgDescriptor();
+  if (!exDesc.isReg() || !desc.isImm())
+    return KV_DESCRIPTOR_INVALID;
+
+  auto addrType = ((desc.imm >> 29) & 0x3); // Desc[30:29]
+  if (addrType != 1 && addrType != 2)       // BSS/SS
+    return KV_DESCRIPTOR_INVALID;
+
+  *exdesc_immoff = inst->getExtImmOffDescriptor();
+  return KV_SUCCESS;
+}
 
 kv_status_t kv_get_send_indirect_descs(const kv_t *kv, int32_t pc,
                                        uint8_t *ex_desc_reg,
@@ -494,7 +520,9 @@ uint32_t kv_get_message_len(const kv_t *kv, int32_t pc, uint32_t *mLen,
     numSet++;
     return (uint32_t)n;
   };
-  *rLen = setOne(inst->getDstLength());
+
+  auto dstLen = inst->getDstLength();
+  *rLen = setOne(dstLen);
   *mLen = setOne(inst->getSrc0Length());
   *emLen = setOne(inst->getSrc1Length());
 
