@@ -441,6 +441,12 @@ public:
       LSC_DATA_SHAPE_BLOCK2D dataShape, VISA_RawOpnd *dstData,
       VISA_VectorOpnd *src0Addrs[LSC_BLOCK2D_ADDR_PARAMS],
       int xImmOffset, int yImmOffset, VISA_RawOpnd *src1Data) = 0;
+  VISA_BUILDER_API virtual int AppendVISALscUntypedBlock2DInst(
+      LSC_OP subOpcode, VISA_PredOpnd *pred,
+      VISA_Exec_Size execSize, VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts,
+      LSC_DATA_SHAPE_BLOCK2D dataShape, VISA_RawOpnd *dstData,
+      VISA_VectorOpnd *src0AddrPayload,
+      int xImmOffset, int yImmOffset, VISA_RawOpnd *src1Data) = 0;
   ///////////////////////////////////////////////////////////////////////////
   // LSC typed operations
   //
@@ -503,6 +509,21 @@ public:
                                                   LSC_SCOPE scope) = 0;
 
 
+  VISA_BUILDER_API virtual int AppendVISALscTypedBlock2DInst(
+      LSC_OP subOpcode, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrModel,
+      LSC_DATA_SHAPE_TYPED_BLOCK2D dataShape,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *dstData, VISA_VectorOpnd *blockStartX,
+      VISA_VectorOpnd *blockStartY, int xImmOffset, int yImmOffset,
+      VISA_RawOpnd *src1Data) = 0;
+
+  VISA_BUILDER_API virtual int AppendVISALscUntypedAppendCounterAtomicInst(
+      LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
+      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrType,
+      LSC_DATA_SHAPE data,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *dst, VISA_RawOpnd *src1Data) = 0;
+
   ///////////////////////////////////////////////////////////////////////////
   // Named barriers functions
   //
@@ -634,7 +655,7 @@ public:
   VISA_BUILDER_API virtual int
   AppendVISACFFunctionCallInst(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
                                VISA_Exec_Size executionSize,
-                               std::string funcName, unsigned char argSize,
+                               const std::string& funcName, unsigned char argSize,
                                unsigned char returnSize) = 0;
 
   /// AppendVISACFIndirectFuncCallInst -- append an indirect function call to
@@ -650,7 +671,7 @@ public:
   /// into <dst> faddr symbolName dst symbolName is the unique string to
   /// identify the symbol whose address is taken dst must have UD type with
   /// scalar region
-  VISA_BUILDER_API virtual int AppendVISACFSymbolInst(std::string symbolName,
+  VISA_BUILDER_API virtual int AppendVISACFSymbolInst(const std::string& symbolName,
                                                       VISA_VectorOpnd *dst) = 0;
 
   /// AppendVISACFFunctionRetInst -- append a function return instruction to
@@ -998,11 +1019,10 @@ public:
   /********** APPEND MEDIA Instructions END ******************/
 
   /********** APPEND 3D Instructions START ******************/
-
-       /// AppendVISA3dSampler -- create a vISA sampler message
-       /// @subOpcode the sampler op (sample, sample_c, sample_d, etc.)
-       /// @pixelNullMask whether an extra GRF is returned for the null mask bit
-       /// @cpsEnable whether CPS LOD compensation is enabled
+  /// AppendVISA3dSampler -- create a vISA sampler message
+  /// @subOpcode the sampler op (sample, sample_c, sample_d, etc.)
+  /// @pixelNullMask whether an extra GRF is returned for the null mask bit
+  /// @cpsEnable whether CPS LOD compensation is enabled
   /// @uniformSampler whether the sampler state is uniform across all work items
   /// @pred predicate variable
   /// @emask execution mask
@@ -1012,6 +1032,7 @@ public:
   /// [8:11] - U offset
   /// @sampler sampler index
   /// @surface surface index
+  /// @pairedSurface paired surface index
   /// @dst return value.  Type may be either 32-bit or 16-bit
   /// @numMsgSepcificOpnds.  Number of payload arguments
   /// @opndArray.  payload arguments.  All operands must have the same type,
@@ -1022,6 +1043,16 @@ public:
       VISA_Exec_Size executionSize, VISAChannelMask channelMask,
       VISA_VectorOpnd *aoffimmi, VISA_StateOpndHandle *sampler,
       VISA_StateOpndHandle *surface,
+      VISA_RawOpnd *pairedSurface,
+      VISA_RawOpnd *dst, int numMsgSpecificOpnds, VISA_RawOpnd **opndArray) = 0;
+
+  VISA_BUILDER_API virtual int AppendVISA3dSampler(
+      VISASampler3DSubOpCode subOpcode, bool pixelNullMask, bool cpsEnable,
+      bool uniformSampler, VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+      VISA_Exec_Size executionSize, VISAChannelMask channelMask,
+      VISA_VectorOpnd *aoffimmi, VISA_StateOpndHandle *sampler, unsigned int samplerIdx,
+      VISA_StateOpndHandle *surface, unsigned int surfaceIdx,
+      VISA_RawOpnd *pairedSurface,
       VISA_RawOpnd *dst, int numMsgSpecificOpnds, VISA_RawOpnd **opndArray) = 0;
 
   /// AppendVISA3dLoad -- create a vISA sampler load message
@@ -1034,6 +1065,7 @@ public:
   /// @aoffimmi Dx10 _aoffimmi modifier. [0:3] - R offset, [4:7] - V offset,
   /// [8:11] - U offset
   /// @surface surface index
+  /// @pairedSurface paired surface index
   /// @dst return value.  Type may be either 32-bit or 16-bit
   /// @numMsgSepcificOpnds.  Number of payload arguments
   /// @opndArray.  payload arguments.  All operands must have the type UD
@@ -1042,9 +1074,19 @@ public:
                    VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
                    VISA_Exec_Size executionSize, VISAChannelMask channelMask,
                    VISA_VectorOpnd *aoffimmi, VISA_StateOpndHandle *surface,
+                   VISA_RawOpnd *pairedSurface,
                    VISA_RawOpnd *dst, int numMsgSpecificOpnds,
                    VISA_RawOpnd **opndArray) = 0;
 
+  VISA_BUILDER_API virtual int
+  AppendVISA3dLoad(VISASampler3DSubOpCode subOpcode, bool pixelNullMask,
+                   VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+                   VISA_Exec_Size executionSize, VISAChannelMask channelMask,
+                   VISA_VectorOpnd *aoffimmi,
+                   VISA_StateOpndHandle *surface, unsigned int surfaceIndex,
+                   VISA_RawOpnd *pairedSurface,
+                   VISA_RawOpnd *dst, int numMsgSpecificOpnds,
+                   VISA_RawOpnd **opndArray) = 0;
   /// AppendVISA3dGather4 -- create a vISA sampler gather4 message
   /// @subOpcode the gather4 op (gather4, gather4_c, etc.)
   /// @pixelNullMask whether an extra GRF is returned for the null mask bit
@@ -1056,6 +1098,7 @@ public:
   /// [8:11] - U offset
   /// @sampler sampler index
   /// @surface surface index
+  /// @pairedSurface paired surface index
   /// @dst return value.  Type may be either 32-bit or 16-bit
   /// @numMsgSepcificOpnds.  Number of payload arguments
   /// @opndArray.  payload arguments.  All operands must have the same type,
@@ -1065,13 +1108,23 @@ public:
       VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
       VISASourceSingleChannel srcChannel, VISA_VectorOpnd *aoffimmi,
       VISA_StateOpndHandle *sampler, VISA_StateOpndHandle *surface,
+      VISA_RawOpnd *pairedSurface,
+      VISA_RawOpnd *dst, int numMsgSpecificOpnds, VISA_RawOpnd **opndArray) = 0;
+
+  VISA_BUILDER_API virtual int AppendVISA3dGather4(
+      VISASampler3DSubOpCode subOpcode, bool pixelNullMask, VISA_PredOpnd *pred,
+      VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+      VISASourceSingleChannel srcChannel, VISA_VectorOpnd *aoffimmi,
+      VISA_StateOpndHandle *sampler, unsigned int samplerIndex,
+      VISA_StateOpndHandle *surface, unsigned int surfaceIndex,
+      VISA_RawOpnd *pairedSurface,
       VISA_RawOpnd *dst, int numMsgSpecificOpnds, VISA_RawOpnd **opndArray) = 0;
 
   VISA_BUILDER_API virtual int
   AppendVISA3dInfo(VISASampler3DSubOpCode subOpcode, VISA_EMask_Ctrl emask,
                    VISA_Exec_Size executionSize, VISAChannelMask srcChannel,
-                   VISA_StateOpndHandle *surface, VISA_RawOpnd *lod,
-                   VISA_RawOpnd *dst) = 0;
+                   VISA_StateOpndHandle *surface, unsigned int surfaceIndex,
+                   VISA_RawOpnd *lod, VISA_RawOpnd *dst) = 0;
 
   VISA_BUILDER_API virtual int AppendVISA3dRTWrite(
       VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
@@ -1151,8 +1204,11 @@ public:
   VISA_BUILDER_API virtual int SetGTPinInit(void *buffer) = 0;
 
   /// GetGTPinBuffer -- returns data buffer for gtpin (eg, free GRF info)
-  VISA_BUILDER_API virtual int GetGTPinBuffer(void *&buffer,
-                                              unsigned int &size) = 0;
+  /// scratchOffset - passed by IGC. This argument contains offset at which
+  /// gtpin can store its data. IGC computes total scratch space when
+  /// stack calls are used, therefore, this is passed as argument.
+  VISA_BUILDER_API virtual int GetGTPinBuffer(void *&buffer, unsigned int &size,
+                                              unsigned int scratchOffset) = 0;
 
   /// GetFreeGRFInfo -- returns free GRF information for gtpin
   VISA_BUILDER_API virtual int GetFreeGRFInfo(void *&buffer,
@@ -1200,6 +1256,10 @@ public:
 
   /// getVISAAsm -- Get the compiled .visaasm of the kernel.
   VISA_BUILDER_API virtual std::string getVISAAsm() const = 0;
+
+  /// set or get current block frequency information
+  VISA_BUILDER_API virtual int
+  encodeBlockFrequency(uint64_t digits, int16_t scale) = 0;
 };
 
 class VISAFunction : public VISAKernel {
